/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/frame.h>
#include <wx/fileconf.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/dcclient.h>
#include <wx/settings.h>
#include <wx/sizer.h>
#include "BibTexEntryWindow.h"
#include <wx/panel.h>



namespace Khz{namespace BibTex{





BibtexEntryWindow::BibtexEntryWindow(wxWindow *parent, wxString Bibtex_wxString)
   : wxFrame(parent, wxID_ANY, "Bibtex")
    {

	wxPanel *top_panel = new wxPanel(this, wxID_ANY);
    const wxChar* NEWLINE_CHAR = wxTextFile::GetEOL();
    Bibtex_wxString.Replace ("\n", NEWLINE_CHAR); //for windows compapability

    wxTextCtrl *BibtexTextEntry=new wxTextCtrl(top_panel, wxID_ANY,Bibtex_wxString,
                                               wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_READONLY|wxTE_DONTWRAP| wxTE_RICH2);
    wxButton *Close_button=new wxButton(top_panel,wxID_ANY, "Close");




    //settting monospace font
    wxFont font;
    font=BibtexTextEntry->GetFont();
    font.SetFamily (wxFONTFAMILY_TELETYPE );
    BibtexTextEntry->SetFont(font);

    //setting size of the popup window
    wxWindowDC dc(BibtexTextEntry);
    wxSize size = BibtexTextEntry->GetSizeFromTextSize(dc.GetMultiLineTextExtent(Bibtex_wxString));
    int screen_width=wxSystemSettings::GetMetric ( wxSYS_SCREEN_X );
    int screen_height=wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y );
    size.SetHeight(size.GetHeight()+10+Close_button->GetSize().GetHeight());
    size.SetWidth(size.GetWidth()+30);

    wxBoxSizer *only_sizer = new wxBoxSizer( wxVERTICAL );
    only_sizer->Add(BibtexTextEntry, 0, wxEXPAND|wxALL,5);
    only_sizer->AddStretchSpacer ();
        wxBoxSizer *button_sizer=new wxBoxSizer(wxHORIZONTAL);
        button_sizer->AddStretchSpacer ();
        button_sizer->Add(Close_button);
        //wxSizerFlags flag_to_fit_button(0);
       // flag_to_fit_button.Expand();
       // flag_to_fit_two_last_buttons.Border(wxBOTTOM, 5);

    only_sizer->Add(button_sizer,0,wxEXPAND|wxALL,5);
    top_panel->SetSizerAndFit(only_sizer);

	wxBoxSizer *win_sizer= new wxBoxSizer(wxVERTICAL);
	win_sizer->Add(top_panel,1, wxEXPAND|wxALL);
	SetSizerAndFit(win_sizer);


    this->SetClientSize(size);
    size = this->GetSize();
    if(size.GetHeight()>screen_height)
        {
        size.SetHeight(screen_height);
        }
    if(size.GetWidth()>screen_width)
        {
        size.SetWidth(screen_width);
        }
    this->SetSize(size);
	only_sizer->RecalcSizes();
	win_sizer->RecalcSizes();


    this->Show();
    Close_button->Bind(wxEVT_BUTTON, &BibtexEntryWindow::OnClose, this);
    }
void BibtexEntryWindow::OnClose(wxCommandEvent& event)
    {
    this->Close();
    }

}} // end of namespaces
