/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "BibtexTranslator.h"
#include<vector>
#include <boost/regex.hpp>
#include <boost/regex/icu.hpp> //for the regular expresssions unicode support
#include "string_functions.h"
#include <wx/string.h>
#include <iostream>
//@QQQfinished commenting

namespace Khz{ namespace BibTex{


std::string GenBib::GenBibtexEntryFromFields(std::string Author, std::string Title,
                                    std::string Publisher, std::string Year, std::string edition, std::string additional_key)
    {
    //volume sits in the Title of the book, we are going to get it later.
    std::string Volume="";


    //final set of the Authors, like {"Petrov, A.V.", "Ivanov B.M."}
    std::vector<std::string> Author_final; //authors with right initials, for example "petrov, a.v."

    //the bibtex key is, for example, Ivanov:2010:additional_key.
    //so, Second_name_for_key_in_bibtex_entry should be Ivanov (vithout initials.
    std::string Second_name_for_key_in_bibtex_entry="";
    using namespace boost;
    smatch res;

    //#######################################################################################################################
    //
    //                DEALING WITH AUTHORS (separating initials)
    //
    //#######################################################################################################################


    //remove anything in parentneses [for example (ed.)]
    Author = u32regex_replace(Author, make_u32regex("\\(\\S+\\)\\.*", regex_constants::icase),"" ,regex_constants::format_first_only);

    //remove "et.al"
    Author = u32regex_replace(Author, make_u32regex(", et al.\\s*$", regex_constants::icase),"" ,regex_constants::format_first_only);
    Author = u32regex_replace(Author, make_u32regex(",et al.\\s*$", regex_constants::icase),"" ,regex_constants::format_first_only);
    Author = u32regex_replace(Author, make_u32regex(", и др.\\s*$", regex_constants::icase),"" ,regex_constants::format_first_only);
    Author = u32regex_replace(Author, make_u32regex(",и др.\\s*$", regex_constants::icase),"" ,regex_constants::format_first_only);

    //authors in the "Author" string  are separated by comma.
    //Author is something like "Ivanov A.V., Petrov C.D."
    //We want to transform it into "Ivanov, A.V. and Petrov C.D" (see bibtex specs)

    //explode authors, we are going to work with each author separately.
    std::vector<std::string> exploded_author=Khz::Libs::Mstr::explode(",", Author);

    //we are going to get the initials, second name,
    //we are using reg expressions to separate initials from second name
    std::string second_name;
    std::string initials;
    //for each author
    for(unsigned int i=0;i<exploded_author.size();i++)
        {
                    //second name + initials, for example petrov a.v.
        if (        u32regex_search(exploded_author[i],res, make_u32regex("([^\\.]+)\\s+(\\w+\\.\\w+\\.)", regex_constants::icase))        )
            {
            second_name=res.str(1);
            initials=res.str(2);
            }
                    //initials + second name, for example a.v. petrov
        else if(    u32regex_search(exploded_author[i],res, make_u32regex("(\\w+\\.\\w+\\.)\\s+([^\\.]+)", regex_constants::icase))        )
            {
            second_name=res.str(2);
            initials=res.str(1);
            }
                    //second name + initial, for example petrov v.
        else if(    u32regex_search(exploded_author[i],res, make_u32regex("([^\\.]+)\\s+(\\w+\\.)", regex_constants::icase))        )
            {
            second_name=res.str(1);
            initials=res.str(2);
            }
                    // initial + second name, for example v. petrov
        else if(    u32regex_search(exploded_author[i],res, make_u32regex("(\\w+\\.)\\s+([^\\.]+)", regex_constants::icase))        )
            {
            second_name=res.str(2);
            initials=res.str(1);
            }
        else
            {
            //We have not found any initials, assuming that there are no initials.
            //The whole string is second name
            // We need to remove the last dot(if exist)
            if (u32regex_search(exploded_author[i], res, make_u32regex("(.+)\\.\\s*$", regex_constants::icase)))
                {
                second_name=res.str(1);
                }
            else
                {
                second_name=exploded_author[i];
                }
            //initials are empty
            initials="";
            }
        ///////////////////////////////////////////////////////////////////////
        //put current value into the Author_final array.
        //According to bibtex specs initials are separated by comma
        ////////////////////////////////////////////////////////////////////////
        if(initials!="")
            {
            Author_final.push_back(second_name+ ", " + initials);
            }
        else
            {
            Author_final.push_back(second_name);
            }
        ////////////////////////////////////////////////////////////////////////////
        // ok, finished with put current value into the Author_final array.
        //////////////////////////////////////////////////////////////////////////

        //we use the first author as a bibtex key (have not you forgot that we are in cycle?)
        //Before starting this cycle we set Second_name_for_key_in_bibtex_entry=""
        //So, the first cycle run will fill the Second_name_for_key_in_bibtex_entry variable
        //by the second name of the first author.
        //The second cycle run will skip this step.
        if(Second_name_for_key_in_bibtex_entry=="")
            {
            Second_name_for_key_in_bibtex_entry=

            //we want only letters, remove all of the special characters.
            u32regex_replace(second_name, make_u32regex("[\\W_]", regex_constants::icase),"");

            //we will not use any of the non-english symbols as key of the bibtex item.
            Khz::Libs::Mstr::translate_to_en_from_any_ONLY_ALPHABETIC_SYMBOLS(Second_name_for_key_in_bibtex_entry);
            //std::cout << "\n second name is "<< second_name << "\n";
            }
        }


    //#######################################################################################################################
    //
    //                DEALING WITH Title (separating volume)
    //
    //#######################################################################################################################


    //now we can try to get information about volume number
    //If volume part is found, that we cut this part from the title
    //begin~ getting volume info~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        //special case

    if (     u32regex_search(Title, res, make_u32regex("(?:\\.|\\,)\\s+(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4}\\.{0,1}\\,{0,1}\\s{0,9}n.\\d{1,4})\\s*$",regex_constants::icase))           )
        {
        Volume = res.str(1);
        Title =   u32regex_replace(Title, make_u32regex("(?:\\.|\\,)\\s+(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4}\\.{0,1}\\,{0,1}\\s{0,9}n.\\d{1,4})\\s*$", regex_constants::icase),"",regex_constants::format_first_only);
        }


    //beginning of the string, replace with nothing.
    else if (  u32regex_search(Title, res, make_u32regex("^(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+",regex_constants::icase))           )
        {
        Volume = res.str(1);
        Title= u32regex_replace(Title,     make_u32regex("^(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+", regex_constants::icase),"",regex_constants::format_first_only);
        }

    //in the middle of the string, replace with space
    else if (  u32regex_search(Title, res, make_u32regex("\\s+(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+([^\\s]+)",regex_constants::icase))           )
        {
        Volume = res.str(1);
        std::string temp_word=res.str(2);
        Title= u32regex_replace(Title,     make_u32regex("\\s+(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+([^\\s]+)", regex_constants::icase)," "+temp_word,regex_constants::format_first_only);
        }

    //end of the string, replace withe empty string
    else if (u32regex_search(Title, res, make_u32regex("(?:\\.|\\,)\\s+(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s*$",regex_constants::icase))           )
        {
        Volume = res.str(1);
        Title =   u32regex_replace(Title, make_u32regex("(?:\\.|\\,)\\s+(?:vol|том|t|т)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s*$", regex_constants::icase),"",regex_constants::format_first_only);
        }
    //finished vith volume number definition ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //
    //                         NOW FORMING the FINAL BIBTEX ENTRY
    //
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        std::string final_string;
        final_string="@book{"+Second_name_for_key_in_bibtex_entry+":"+Year+":"+additional_key                   +    ",\n";
        final_string=final_string+GenBib::add_spaces_to_end("author")    +"=\""  + Khz::Libs::Mstr::implode(" and ", Author_final) +  "\",\n";
        final_string=final_string+GenBib::add_spaces_to_end("title")     +"=\"{" + Title                                + "}\",\n";
        final_string=final_string+GenBib::add_spaces_to_end("publisher") +"=\""  + Publisher                            +  "\",\n";
        final_string=final_string+GenBib::add_spaces_to_end("year") +"=\""       + Year               +  "\",\n";
    if(Volume!="")
        {
        final_string=final_string+GenBib::add_spaces_to_end("volume") +"=\""     + Volume                               +  "\",\n";
        }
    if(edition!="")
        {
        final_string=final_string+GenBib::add_spaces_to_end("edition") +"=\""     + edition                             +  "\",\n";
        }
        final_string=final_string+"}";
    return final_string;
    }





std::string GenBib::add_spaces_to_end(std::string str, int length)
    {
    wxString temp_str= wxString(str);
    int str_length= temp_str.length();

    std::string spaces="";
    if(length<=str_length)
        {
        return str;
        }
    else
        {
        int num_of_spaces=length-str_length;
        for (int i=0;i<num_of_spaces;i++)
            {
            spaces=spaces+" ";
            }
        }
    return str+spaces;
    }


}}// namespaces
