/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<string>

namespace Khz{ namespace BibTex {


/**
@brief Forms the bibtex entry for a single book from title, author and so on.

See #GenBibtexEntryFromFields
**/
class GenBib
{

public:
    /**
    @param Author book's author
    @param Title book's title
    @param Publisher book's publisher
    @param Year just year
    @param edition just edition
    Forms the bibtex entry for a single book
    @param additional_key The key of the bibtex entry is going to be:
    Author:Year:dditional_key.
    For example, if additional_key is "fhg", than the bibtex entry key is going to be: Ivanov:2009:fhg
    @return the bibtex entry as a multline string, the lines of the string are separated by the "\n" symbol.
    The example of the return value:
    @code
    @book{Adler:2013:3,
    author              ="Adler, F.R. and  Tanner, C.J.",
    title               ="{Urban ecosystems: Ecological Principles for the Built Environment}",
    publisher           ="draft, CUP",
    year                ="2013",
    }
    @endcode

    **/
    static std::string GenBibtexEntryFromFields(std::string Author, std::string Title,
                                    std::string Publisher, std::string Year, std::string edition, std::string additional_key);


private:
    /**
    Additional function that is used inside of the #GenBibtexEntryFromFields function. There is no need to call this function directly.
    Adds spaces to the end of the string to make the length of the string to be equal to <strong class="paramname">length</strong>.
    Returns initial string if the length of the string is equal or larger than <strong class="paramname">length</strong>.
    **/
    static std::string add_spaces_to_end(std::string str, int length=20);

};


}}
