/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/window.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/fileconf.h>
#include <wx/dirdlg.h>
#include <wx/filedlg.h>
#include "Custom_IDs.h"
#include <wx/settings.h>
#include "Configure_tabs_parent.h"
#include "CfgTabAdvanced.h"
#include <wx/msgdlg.h>



namespace Khz{namespace CfgWin{




//@QQQfinished commenting
CfgTabAdvanced::CfgTabAdvanced(wxWindow* parent,  wxString path_to_config)
    : Configure_gui_tabs_parent(parent, path_to_config)
    {
    /////////////////////////////////////////////////////////////////////////////////////////////////
    //create checkbox, hyperlink, inputs, set min and max values for the spinctrl inputs
    //////////////////////////////////////////////////////////////////////////////////////////////
    sort_always_checkbox=new wxCheckBox(this, wxID_ANY, "Sort always");
    CfgTabAdvanced_info_hyperlink=new wxHyperlinkCtrl (this, wxID_ANY, "Details about these options", "click");
    max_num_of_books_to_sort_input = new wxSpinCtrl( this, wxID_ANY);
    max_num_of_books_to_sort_input->SetRange(1,10000000);

    max_cache_size_input = new wxSpinCtrl(this,wxID_ANY);
    max_cache_size_input->SetRange(1,1000000);
    //////////////////////////////////////////////////////////////////////////////////////////////////
    //ok, created
    //////////////////////////////////////////////////////////////////////////////////////////////////




    //get previously stored values from the config file.
    wxString max_cache_size_string =config->Read("ADVANCED/max_size_of_cache",wxString::Format(wxT("%i"),default_max_cache));

    wxString max_num_of_books_to_sort_string=
                config->Read("ADVANCED/max_num_of_books_to_sort", wxString::Format(wxT("%i"),default_max_books_to_sort));
    //ok, got previously stored values.

    //Now we use the previously got values to set the values of the inputs
    if(max_cache_size_string!="")
        {
        max_cache_size_input->SetValue(wxAtoi(max_cache_size_string));
        max_cache_size=wxAtoi(max_cache_size_string);
        }
    else
        {
        max_cache_size_input->SetValue(default_max_cache);
        max_cache_size=default_max_cache;
        }

    if(max_num_of_books_to_sort_string!="")
        {
        max_num_of_books_to_sort=wxAtoi(max_num_of_books_to_sort_string);
        //negative value means "sort always", so we should check the "sort always" checkbox
        if(max_num_of_books_to_sort<=0)
            {
            sort_always_checkbox->SetValue(true);
            max_num_of_books_to_sort_input->Disable();
            max_num_of_books_to_sort_input->SetValue(default_max_books_to_sort);
            }
        else
            {
            max_num_of_books_to_sort_input->SetValue(wxAtoi(max_num_of_books_to_sort_string));
            }
        }
    else
        {
        max_num_of_books_to_sort_input->SetValue(default_max_books_to_sort);
        max_num_of_books_to_sort=default_max_books_to_sort;
        }
    //ok, we set all of the values.






    //create buttons
    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //#############################################################################################






    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //SIZING  (we configure the appearance of the window.)
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////
    int left_right_border=5;
    only_sizer = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer *inputs= new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *max_cache_sizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *max_num_of_books_sizer=new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);

            max_cache_sizer->Add(new wxStaticText( this, wxID_ANY, wxT("Cache size")), 0, wxEXPAND|wxALL);
            max_cache_sizer->Add(max_cache_size_input, 0, wxEXPAND|wxALL);


            max_num_of_books_sizer->Add(new wxStaticText( this, wxID_ANY, wxT("Max number of books to sort")), 0, wxEXPAND|wxALL);
            max_num_of_books_sizer->Add(max_num_of_books_to_sort_input, 0, wxEXPAND|wxALL);
            max_num_of_books_sizer->Add(sort_always_checkbox, 0, wxEXPAND|wxALL);
        inputs->Add(max_cache_sizer, 0, wxALL|wxEXPAND);
        inputs->AddSpacer(40);
        inputs->Add(max_num_of_books_sizer,0, wxALL|wxEXPAND);

        //#################### buttons ##########################################################
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(left_right_border);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //###############################################################################################################


    only_sizer->Add(inputs, 0, wxALL, 5);
    only_sizer->AddSpacer(40);
    only_sizer->Add(CfgTabAdvanced_info_hyperlink, 0, wxALL|wxALIGN_CENTER, 5);
    only_sizer->AddStretchSpacer ();
    only_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);

    SetSizerAndFit(only_sizer);

    //////////////////////////////////////////////////////////////////////////////
    //              SIZING FINISHED
    //////////////////////////////////////////////////////////////////////////////











    //finally, bindings
    Bind(wxEVT_SPINCTRL, &CfgTabAdvanced::on_something_changed, this);

    Bind(wxEVT_TEXT, &CfgTabAdvanced::on_something_changed2, this);


    apply_button->Bind(wxEVT_BUTTON, &CfgTabAdvanced::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabAdvanced::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabAdvanced::on_cancel_pressed, this);
    sort_always_checkbox->Bind(wxEVT_CHECKBOX, &CfgTabAdvanced::on_sort_always_checkbox_clicked, this);
    CfgTabAdvanced_info_hyperlink->Bind(wxEVT_HYPERLINK, &CfgTabAdvanced::on_info_hyperlink_clicked, this);
    }

void CfgTabAdvanced::on_info_hyperlink_clicked(wxHyperlinkEvent& event)
    {
    wxString NEWLINE_CHAR = wxString(wxTextFile::GetEOL());
    wxString msg="\"Cache size\" is the number of the books that kolkhoznik gets from the database by a single query. "
                 "These books are stored in RAM. If cache is too small than performance issues could appear "
                 "as the database access is much slower than the RAM access. It seems that 4000 (default value) "
                 "is sufficient enough."+
                 NEWLINE_CHAR+
                 NEWLINE_CHAR+
                 "\"Max number of books to sort\" is a maximal number of books that kolkhoznik is allowed to sort. "
                 "The reason for this option is performance issues (sorting is a time-consuming operation). "
                 "It seems that 10 000 (default value) is good enough. "
                 "If you want increase the number, you should increase the cache size too.";
        wxMessageBox(msg, wxT("Info"), wxOK | wxICON_INFORMATION, this);
    }


void CfgTabAdvanced::on_sort_always_checkbox_clicked(wxCommandEvent& event)
    {
    //user checked checkbox
    if(sort_always_checkbox->GetValue())
        {
        max_num_of_books_to_sort_input->Disable();
        max_num_of_books_to_sort=-1;
        this->apply_button->Enable();
        this->IsModified=true;
        }
    //user unchecked checkbox
    else
        {
        max_num_of_books_to_sort_input->Enable();
        max_num_of_books_to_sort=default_max_books_to_sort;
        max_num_of_books_to_sort_input->SetValue(default_max_books_to_sort);
        this->apply_button->Enable();
        this->IsModified=true;
        }
    }



void CfgTabAdvanced::write_changes_to_config()
    {
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_RECONSTRUCT_RIGHT_PANEL;
    delete config;
    config =new wxFileConfig( "", "", config_file);
    config->Write( wxT("ADVANCED/max_num_of_books_to_sort"), max_num_of_books_to_sort );
    config->Write( wxT("ADVANCED/max_size_of_cache"), max_cache_size );
    config->Flush();


    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }

void CfgTabAdvanced::on_apply_pressed(wxCommandEvent& event)
    {
    this->write_changes_to_config();
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }

void CfgTabAdvanced::on_ok_pressed(wxCommandEvent& event)
    {
    if(IsModified)  //user have modified something
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    //close the window
    this->send_close_event_manually();
    }


void CfgTabAdvanced::on_cancel_pressed(wxCommandEvent& event)
    {
    this->send_close_event_manually();
    }

void CfgTabAdvanced::restore_values()
    {

    //read values from the config file (user have not saved changes, so config
    // file contains old values
    wxString max_cache_size_string= config->Read( wxT("ADVANCED/max_size_of_cache"), wxString::Format(wxT("%i"),default_max_cache));

    wxString max_num_of_books_to_sort_string=
                config->Read( wxT("ADVANCED/max_num_of_books_to_sort"), wxString::Format(wxT("%i"),default_max_books_to_sort));
    //ok, we got old values


    //now it is time to restore old values (discard any changes that were made by user)
    if(max_cache_size_string!="")
        {
        max_cache_size_input->SetValue(wxAtoi(max_cache_size_string));
        max_cache_size=wxAtoi(max_cache_size_string);
        }
    else
        {
        max_cache_size_input->SetValue(default_max_cache);
        max_cache_size=default_max_cache;
        }

    if(max_num_of_books_to_sort_string!="")
        {
        max_num_of_books_to_sort=wxAtoi(max_num_of_books_to_sort_string);
        if(max_num_of_books_to_sort<=0)
            {
            sort_always_checkbox->SetValue(true);
            max_num_of_books_to_sort_input->Disable();
            max_num_of_books_to_sort_input->SetValue(default_max_books_to_sort);
            }
        else
            {
            max_num_of_books_to_sort_input->SetValue(wxAtoi(max_num_of_books_to_sort_string));
            }
        }
    else
        {
        max_num_of_books_to_sort_input->SetValue(default_max_books_to_sort);
        max_num_of_books_to_sort=default_max_books_to_sort;
        }


    apply_button->Disable();
    IsModified=false;
    }


void CfgTabAdvanced::on_something_changed(wxSpinEvent& event)
    {
    max_cache_size=max_cache_size_input->GetValue();
    max_num_of_books_to_sort=max_num_of_books_to_sort_input->GetValue();
    IsModified=true;
    apply_button->Enable();
    }
void CfgTabAdvanced::on_something_changed2(wxCommandEvent& event)
    {
    max_cache_size=max_cache_size_input->GetValue();
    max_num_of_books_to_sort=max_num_of_books_to_sort_input->GetValue();
    IsModified=true;
    apply_button->Enable();
    }




}}// namespaces
