/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/defs.h>
#include <wx/button.h>
#include <wx/msgdlg.h>
#include <vector>
#include <wx/dialog.h>
#include "Custom_IDs.h"
#include <wx/window.h>
#include <wx/sizer.h>
#include <wx/fileconf.h>
#include <wx/button.h>
#include <wx/listbox.h>
#include "Configure_tabs_parent.h"
#include "CfgTabDefaultApps.h"
#include <wx/stattext.h>
#include <iostream>
#include <wx/textdlg.h>


namespace Khz{namespace CfgWin{



CfgTabDefaultApps::CfgTabDefaultApps(wxWindow* parent, wxString path_to_config)
        : Configure_gui_tabs_parent(parent, path_to_config)
    {
    //############################## create caption
     wxStaticText *descr =
                new wxStaticText( this, wxID_ANY,
                                 wxT("Set the default application for a file type"));
     wxFont font = descr->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        descr->SetFont(font);
    //#################################



    //###############generate listbox of extensions ##################
    gen_array_of_extensions_from_config(); //fills array_of_extensions from the config file


    //create listbox with extensions
    list_of_extensions= new wxListBox (this, wxID_ANY);

    wxArrayString temp_wx_array= wxArrayString();
    for(unsigned int i=0;i<array_of_extensions.size();i++)  //array_of_extensions was created by gen_array_of_extensions_from_config();
        {
        temp_wx_array.Add(array_of_extensions[i]);

        //fill internal variable
        array_of_applications.push_back(config->Read(wxString("DEFAULT_APPS/")+array_of_extensions[i],""));

        }
    if(array_of_extensions.size()!=0)
        {
        //fill lisbox
        list_of_extensions->InsertItems(temp_wx_array, 0);
        }
    //####################################################

    int left_right_border=5;


    //create text inputs (app_name, full_commaind, the "custom command" checkbox
    //##############text entries and checkbox#################
    app_name=new wxTextCtrl(this,wxID_ANY,wxEmptyString, wxDefaultPosition);
    custom_command_switch=new wxCheckBox (this, wxID_ANY, "Custom command");
    full_command=new wxTextCtrl(this,wxID_ANY,wxEmptyString, wxDefaultPosition);
    //##########################################################

    //create buttons
    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //################################################################################

    //creates buttons
    //##########################delete, edit, add buttons######################
    delete_ext_button= new wxButton(this,wxID_ANY, "Delete");;
    edit_extension_button = new wxButton(this,wxID_ANY, "Edit");;
    add_extension_button =  new wxButton(this,wxID_ANY, "Add");;
    //##########################################################################

    //select the first item of the extensions listbox (list_of_extensions)
    if(array_of_extensions.size()!=0)
        {
        list_of_extensions->SetSelection (0);
        }

    //fills this->app_name, this->full_command
    this->put_data_to_text_entries(0);

    //disable buttons if necessary
    if(array_of_extensions.size()==0)
        {
        delete_ext_button->Disable();
        edit_extension_button->Disable();
        }














    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //SIZING  (we configure the appearance of the window.)
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    //###########################new sizers##########################################
     only_sizer = new wxBoxSizer( wxVERTICAL );


     wxBoxSizer *panels = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer *left_panel = new wxBoxSizer( wxVERTICAL );
     wxBoxSizer *right_panel = new wxBoxSizer( wxVERTICAL );

     wxBoxSizer *del_edit_add_buttons = new wxBoxSizer(wxHORIZONTAL);
    //##################################################################################

    left_panel->Add(   new wxStaticText( this, wxID_ANY,  wxT("Extension")),0, wxEXPAND|wxALL   );
    left_panel->Add(list_of_extensions,1, wxEXPAND|wxALL);

    del_edit_add_buttons->Add(add_extension_button, 0, wxEXPAND|wxALL);
    del_edit_add_buttons->Add(edit_extension_button, 0, wxEXPAND|wxALL);
    del_edit_add_buttons->Add(delete_ext_button, 0, wxEXPAND|wxALL);
    left_panel->Add(del_edit_add_buttons,0,wxEXPAND|wxALL);

    right_panel->Add(   new wxStaticText( this, wxID_ANY,  wxT("Application")),0, wxEXPAND|wxALL   );
    right_panel->Add(app_name,0, wxEXPAND|wxALL);
    right_panel->AddSpacer(20);
    right_panel->Add(   new wxStaticText( this, wxID_ANY,  wxT("Full Command (%f will be replaced by filename)")),0, wxEXPAND|wxALL   );
    right_panel->Add(full_command, 0, wxEXPAND|wxALL);
    right_panel->Add(custom_command_switch, 0, wxEXPAND|wxALL);




    panels->Add(left_panel, 0,wxEXPAND|wxALL,5);
    panels->Add(right_panel, 1,wxEXPAND|wxALL,5);

    //#################### buttons ##########################################################
    wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);
    three_last_buttons->AddStretchSpacer ();
    three_last_buttons->Add(cancel_button);
    three_last_buttons->AddSpacer(10);
    three_last_buttons->Add(apply_button);
    three_last_buttons->AddSpacer(10);
    three_last_buttons->Add(ok_button);
    three_last_buttons->AddSpacer(left_right_border);
    wxSizerFlags flag_to_fit_two_last_buttons(0);
    flag_to_fit_two_last_buttons.Expand();
    flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
    //########################################################################################
    only_sizer->Add(descr,0, wxEXPAND|wxALL,14);  //14 is border,
    only_sizer->Add(panels,1, wxEXPAND|wxALL);
    //Sonly_sizer->AddStretchSpacer ();
    only_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);

    SetSizerAndFit(only_sizer);


    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //SIZING  FINISHED
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////















    //finally, bindings
    //########################BINDINGS################################################################
    add_extension_button->Bind( wxEVT_BUTTON, &CfgTabDefaultApps::on_add_new_extension, this);
    delete_ext_button->Bind(wxEVT_BUTTON, &CfgTabDefaultApps::on_delete_extension, this);
    edit_extension_button->Bind(wxEVT_BUTTON, &CfgTabDefaultApps::on_edit_extension, this);
    list_of_extensions->Bind( wxEVT_LISTBOX,  &CfgTabDefaultApps::on_sel_changed, this);
    custom_command_switch->Bind( wxEVT_CHECKBOX, &CfgTabDefaultApps::on_custom_command_switch_checkbox_changed, this);
    app_name->Bind(wxEVT_TEXT, &CfgTabDefaultApps::on_text_changed_in_application_entry, this);
    full_command->Bind(wxEVT_TEXT, &CfgTabDefaultApps::on_text_changed_in_full_command_entry, this);

    apply_button->Bind(wxEVT_BUTTON, &CfgTabDefaultApps::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabDefaultApps::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabDefaultApps::on_cancel_pressed, this);
    //#################################################################################################
    }

void CfgTabDefaultApps::write_changes_to_config()
    {
    //set the reconfigure type for this->send_reconfigure_panels_event();
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_JUST_READ_CONFIG_AGAIN;

    //delete info from the config file (we will write new values soon)
    config->DeleteGroup ("DEFAULT_APPS");
    config->Flush();
    delete config;
    config =new wxFileConfig( "", "", config_file);



    //write new values to the config file
    for(unsigned int i=0;i<array_of_extensions.size();i++)
        {
        wxString temp_str="DEFAULT_APPS/"+array_of_extensions[i];
        config->Write( temp_str, array_of_applications[i] );
        }
    config->Flush();




    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;


    //fills app_name and full_command inputs
    //P.S. I really forgot why we need this, it seems these values remained unupdated without it somehow.
    if(array_of_extensions.size()!=0)
        {
        wxArrayInt selections;
        list_of_extensions->GetSelections (selections);
        if(selections.GetCount()!=1)
            {
            std::cout << "\n some terrible happened in listbox of CfgTabDefaultApps.cpp. Aborted";
            abort();
            }
        int item_num = selections.Item(0);
        this->put_data_to_text_entries(item_num);
        }
    else
        {
        this->put_data_to_text_entries(0);
        }
    }


void CfgTabDefaultApps::on_apply_pressed(wxCommandEvent& event)
    {
    this->write_changes_to_config();
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_JUST_READ_CONFIG_AGAIN;
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }






void CfgTabDefaultApps::on_ok_pressed(wxCommandEvent& event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    //std::cout << "\nclosing config\n";
    this->send_close_event_manually();
    }





void CfgTabDefaultApps::on_cancel_pressed(wxCommandEvent& event)
    {
    this->send_close_event_manually();
    }





void CfgTabDefaultApps::restore_values()
    {
        //###############generate listbox##################
    array_of_extensions.clear();
    array_of_applications.clear();
    gen_array_of_extensions_from_config();

    list_of_extensions->Clear();
    wxArrayString temp_wx_array= wxArrayString();
    for(unsigned int i=0;i<array_of_extensions.size();i++)
        {
        temp_wx_array.Add(array_of_extensions[i]);
        array_of_applications.push_back(config->Read(wxString("DEFAULT_APPS/")+array_of_extensions[i],""));

        }
    if(array_of_extensions.size()!=0)
        {
        list_of_extensions->InsertItems(temp_wx_array, 0);
        list_of_extensions->SetSelection (0);
        }
    //####################################################
    this->put_data_to_text_entries(0);

    apply_button->Disable();
    if(array_of_extensions.size()!=0)
        {
        delete_ext_button->Enable();
        edit_extension_button->Enable();
        }
    IsModified=false;
    }

void CfgTabDefaultApps::gen_array_of_extensions_from_config()
    {
    wxString temp_str;
    long dummy;
     wxString strOldPath = config->GetPath();

    //#################################################################
    config->SetPath("DEFAULT_APPS");

    bool GCont = config->GetFirstEntry(temp_str, dummy);
    while ( GCont )
        {
        array_of_extensions.push_back(temp_str);
        GCont = config->GetNextEntry(temp_str, dummy);
        }

    config->SetPath(strOldPath);
    //#################################################################

    //for(int i=0;i<array_of_extensions.size();i++)
    //    {
    //    std::cout << "\n" << array_of_extensions[i];
    //    }
    }
void CfgTabDefaultApps::on_add_new_extension(wxCommandEvent& event)
    {
        wxTextEntryDialog dialog(this,
                             wxT("Enter new extension\n"),
                             wxT("Add new Extension"),
                             wxT(""),
                             wxOK | wxCANCEL);

    if (dialog.ShowModal() == wxID_OK)
        {
        wxString new_extension=dialog.GetValue();


        //add values to arrays
        array_of_extensions.insert(array_of_extensions.begin(), new_extension);
        array_of_applications.insert(array_of_applications.begin(), "");


        //update listobx
        wxArrayString temp_wx_array= wxArrayString();
        temp_wx_array.Add(new_extension);
        list_of_extensions->InsertItems (temp_wx_array, 0);
        list_of_extensions->SetSelection (0);
        this->put_data_to_text_entries(0);
        apply_button->Enable();
        delete_ext_button->Enable();
        edit_extension_button->Enable();
        IsModified=true;
        }
    }


void CfgTabDefaultApps::on_delete_extension(wxCommandEvent& event)
    {
    wxArrayInt selections;
    list_of_extensions->GetSelections (selections);
    for(unsigned int i=0;i<selections.GetCount();i++)
        {
        int item_num=selections.Item(i);
        list_of_extensions->Delete(item_num);
        array_of_extensions.erase(array_of_extensions.begin()+item_num);
        array_of_applications.erase(array_of_applications.begin()+item_num);
        }


    std::cout << "\n\n\n after delete extensions are\n";
    for(unsigned int i=0;i<array_of_extensions.size();i++)
        {
        std::cout << "\n" << array_of_extensions[i] << "\n";
        }
    if(array_of_extensions.size()!=0)
        {
        list_of_extensions->SetSelection (0);
        }
    this->put_data_to_text_entries(0);
    apply_button->Enable();
    if(list_of_extensions->GetCount()==0)
        {
        delete_ext_button->Disable();
        edit_extension_button->Disable();
        }
    IsModified=true;
    }


void CfgTabDefaultApps::on_edit_extension(wxCommandEvent& event)
    {
    wxArrayInt selections;
    list_of_extensions->GetSelections (selections);
    if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened in listbox of CfgTabDefaultApps.cpp. Aborted";
        abort();
        }
    int item_num = selections.Item(0);


     wxTextEntryDialog dialog(this,
                             wxT("Edit extension\n"),
                             wxT("Edit extension"),
                             list_of_extensions->GetString(item_num),
                             wxOK | wxCANCEL);

    if (dialog.ShowModal() == wxID_OK)
        {
        wxString new_extension=dialog.GetValue();
        list_of_extensions->SetString(item_num, new_extension);
        array_of_extensions[item_num]=new_extension;
        apply_button->Enable();
        IsModified=true;
        }
    }

void CfgTabDefaultApps::put_data_to_text_entries(unsigned int item_num)
    {
    if(item_num >= list_of_extensions->GetCount())
        {
        app_name->ChangeValue("");
        app_name->Disable();
        full_command->ChangeValue("");
        custom_command_switch->SetValue(false);
        full_command->Disable();
        return;
        }



    wxString app_to_open_book= array_of_applications[item_num];
    if( app_to_open_book.Find("%f") ==  wxNOT_FOUND)
        {
        app_name->ChangeValue(app_to_open_book);
        if(app_to_open_book!="")
            {
            full_command->ChangeValue(app_to_open_book + " %f");
            }
        else
            {
            full_command->ChangeValue("");
            }
        custom_command_switch->SetValue(false);
        full_command->Disable();
        app_name->Enable();
        }
    else
        {
        app_name->ChangeValue("");
        full_command->ChangeValue(app_to_open_book);
        custom_command_switch->SetValue(true);
        app_name->Disable();
        full_command->Enable();
        }
    }
void CfgTabDefaultApps::on_sel_changed(wxCommandEvent& event)
    {
    wxArrayInt selections;
    list_of_extensions->GetSelections (selections);
    if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened(version2) in listbox of CfgTabDefaultApps.cpp. Aborted";
        abort();
        }
    int item_num = selections.Item(0);
    this->put_data_to_text_entries(item_num);
    }
void CfgTabDefaultApps::on_custom_command_switch_checkbox_changed(wxCommandEvent& event)
    {
    if(custom_command_switch->GetValue())
        {
        app_name->Disable();
        full_command->Enable();
        //wxArrayInt selections;
        //list_of_extensions->GetSelections (selections);
        //if(selections.GetCount()!=1)
        //    {
        //    std::cout << "\n some terrible happened(version3) in listbox of CfgTabDefaultApps.cpp. Aborted";
        //    abort();
        //    }
        //int item_num = selections.Item(0);
        //array_of_applications[item_num]=full_command->GetValue();
        //this->IsModified=true;
        //this->apply_button->Enable();
        }
    else
        {
        app_name->Enable();
        full_command->Disable();
        //wxArrayInt selections;
        //list_of_extensions->GetSelections (selections);
        //if(selections.GetCount()!=1)
        //    {
        //    std::cout << "\n some terrible happened(version3) in listbox of CfgTabDefaultApps.cpp. Aborted";
        //    abort();
        //    }
        //int item_num = selections.Item(0);
        //array_of_applications[item_num]=app_name->GetValue();
        //this->IsModified=true;
        //this->apply_button->Enable();
        }
    }

void CfgTabDefaultApps::on_text_changed_in_application_entry(wxCommandEvent& event)
    {
     wxArrayInt selections;
     list_of_extensions->GetSelections (selections);
     if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened(version4) in listbox of CfgTabDefaultApps.cpp. Aborted";
        abort();
        }
    int item_num = selections.Item(0);
    array_of_applications[item_num]=app_name->GetValue();
    full_command->ChangeValue(app_name->GetValue() + " %f");
    this->IsModified=true;
    this->apply_button->Enable();
    }

void CfgTabDefaultApps::on_text_changed_in_full_command_entry(wxCommandEvent& event)
    {
     wxArrayInt selections;
     list_of_extensions->GetSelections (selections);
     if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened(version4) in listbox of CfgTabDefaultApps.cpp. Aborted";
        abort();
        }
    int item_num = selections.Item(0);
    array_of_applications[item_num]=full_command->GetValue();
    this->IsModified=true;
    this->apply_button->Enable();
    }


}}// end of namespaces
