/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <ciso646>
#include <wx/window.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/fileconf.h>
#include <wx/dirdlg.h>
#include <wx/filedlg.h>
#include "Custom_IDs.h"
#include "Configure_tabs_parent.h"
#include "CfgTabPathsChooser.h"
#include "CfgTabFieldsChooser.h"
#include "string_functions.h"



namespace Khz{namespace CfgWin{



CfgTabFieldsChooser::CfgTabFieldsChooser(wxWindow* parent,  wxString path_to_config)
    : Configure_gui_tabs_parent(parent, path_to_config)
    {
    int left_right_border=5;
        //##############################caption

     wxStaticText *descr =
                new wxStaticText( this, wxID_ANY,
                                 wxT("Choose fields that are to be displayed on the right panel"));
     wxFont font = descr->GetFont();
        font.SetPointSize(10);
        font.SetWeight(wxFONTWEIGHT_BOLD);
        descr->SetFont(font);

    //#################################
    right_panel_outer=new wxPanel(this);
	wxColour temp = right_panel_outer->GetBackgroundColour();
    right_panel=new wxPanel(right_panel_outer);
    right_panel_outer->SetBackgroundColour (*wxWHITE);
	right_panel->SetBackgroundColour(temp);
    full_list_of_fields= new wxListBox (this, wxID_ANY);
    used_list_of_fields=new wxListBox(right_panel, wxID_ANY);

    wxBitmap bitmap_right;
    bitmap_right.LoadFile(wxString("icons")+wxFileName::GetPathSeparator()+"go-right.png", wxBITMAP_TYPE_PNG);

    wxBitmap bitmap_up;
    bitmap_up.LoadFile(wxString("icons")+wxFileName::GetPathSeparator()+"go-up.png", wxBITMAP_TYPE_PNG);

    wxBitmap bitmap_down;
    bitmap_down.LoadFile(wxString("icons")+wxFileName::GetPathSeparator()+"go-down.png", wxBITMAP_TYPE_PNG);


    button_move_right= 	new wxBitmapButton (this, wxID_ANY, bitmap_right);
    //button_move_left= 	new wxBitmapButton (this, wxID_ANY, bitmap_left);
    button_move_up= 	new wxBitmapButton (right_panel, wxID_ANY, bitmap_up);
    button_move_down= 	new wxBitmapButton (right_panel, wxID_ANY, bitmap_down);
    remove_button=new wxButton(right_panel, wxID_ANY, "Remove");

    visible_name_input=new wxTextCtrl(right_panel,wxID_ANY,wxEmptyString, wxDefaultPosition);

    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this, Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //################################################################################






    this->generate_full_arrays();
    this->generate_visible_arrays_from_config();
    wxArrayString temp_wx_array= wxArrayString();
    wxArrayString temp_wx_array2= wxArrayString();


    for(unsigned int i=0;i<full_fields_array.size();i++)
        {
        temp_wx_array.Add(full_fields_display_names[i]);
        }
    for(unsigned int i=0;i<choosen_fields_array.size();i++)
        {
        temp_wx_array2.Add(choosen_fields_array_display_names[i]);
        }


    full_list_of_fields->InsertItems(temp_wx_array, 0);

    used_list_of_fields->InsertItems(temp_wx_array2, 0);

    full_list_of_fields->SetSelection(0);
    used_list_of_fields->SetSelection(0);
    this->write_string_to_text_entry(0);
    button_move_up->Disable();

    only_sizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *horizontal_sizer = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *right_colored_panel=new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *right_panel_outer_sizer=new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *left_right_buttons= new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *up_down_buttons= new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *visible_name_sizer= new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *left_listbox_sizer= new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *right_listbox_sizer= new wxBoxSizer(wxVERTICAL);


            left_listbox_sizer->AddSpacer(10);
            left_listbox_sizer->Add(
                        new wxStaticText( this, wxID_ANY, wxT("Available fields"), wxDefaultPosition, wxSize(190,-1))
                                    ,0, wxALL);
            left_listbox_sizer->Add(full_list_of_fields,1, wxEXPAND|wxALL,0);
            left_right_buttons->Add(button_move_right,0, wxALL);
            //left_right_buttons->Add(button_move_left, 0, wxALL);

            right_listbox_sizer->Add(
                        new wxStaticText( right_panel, wxID_ANY, wxT("Visible fields"),wxDefaultPosition, wxSize(190,-1))
                                    ,0, wxALL);
            right_listbox_sizer->Add(used_list_of_fields,1, wxEXPAND|wxALL,0);
            right_listbox_sizer->AddSpacer(2);
            right_listbox_sizer->Add(remove_button,0,wxEXPAND|wxALL);
            int temp_w, temp_h;
            remove_button->GetSize (&temp_w,&temp_h);
            left_listbox_sizer->AddSpacer(temp_h+2+10);

        horizontal_sizer->Add( left_listbox_sizer,0, wxEXPAND|wxALL,5);
        horizontal_sizer->Add(left_right_buttons, 0, wxALIGN_CENTER);
            right_colored_panel->Add(right_listbox_sizer,0, wxEXPAND|wxALL,5);
                up_down_buttons->Add(button_move_up,0, wxALL);
                up_down_buttons->AddSpacer(5);
                up_down_buttons->Add(button_move_down, 0, wxALL);
            right_colored_panel->Add(up_down_buttons, 0, wxALIGN_CENTER);
                visible_name_sizer->Add(  new wxStaticText( right_panel, wxID_ANY, wxT("Visible caption of the field"))    ,0, wxALL);
                visible_name_sizer->Add(visible_name_input,0, wxEXPAND|wxALL);
            right_colored_panel->Add(visible_name_sizer,1,  wxEXPAND|wxALL,5);

            right_panel->SetSizerAndFit(right_colored_panel);
            right_panel_outer_sizer->Add(right_panel, 1, wxEXPAND|wxALL,5);
            right_panel_outer->SetSizerAndFit(right_panel_outer_sizer);
        horizontal_sizer->Add(right_panel_outer, 1, wxEXPAND|wxALL,5);
    only_sizer->Add(descr,0, wxEXPAND|wxALL,14);  //14 is border,
    only_sizer->Add(horizontal_sizer, 1, wxEXPAND|wxALL);



         //#################### buttons ##########################################################
        wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(left_right_border);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //########################################################################################
    only_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);
    this->SetSizerAndFit(only_sizer);


    //###########################BINDINGS###################################################################
    used_list_of_fields->Bind(wxEVT_LISTBOX, &CfgTabFieldsChooser::on_used_list_of_fields_sel_changed, this);
    button_move_right->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_button_move_right_pressed, this);
    remove_button->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_button_remove_pressed, this);
    button_move_up->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_button_up_pressed, this);
    button_move_down->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_button_down_pressed, this);
    visible_name_input->Bind(wxEVT_TEXT, &CfgTabFieldsChooser::on_text_changed_in_visible_fields_entry, this);
    apply_button->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabFieldsChooser::on_cancel_pressed, this);
    //######################################################################################################
    }





void CfgTabFieldsChooser::write_changes_to_config()
    {
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_RECONSTRUCT_RIGHT_PANEL;
    delete config;
    config =new wxFileConfig( "", "", config_file);
    config->DeleteGroup ("COLUMNS_TO_SHOW");
    config->DeleteGroup ("COLUMNS_VISIBLE_NAMES");
    config->Flush();
    delete config;
    config =new wxFileConfig( "", "", config_file);
    wxString delim;
    wxString fields_string="";
    for(unsigned int i=0;i<choosen_fields_array.size();i++)
        {
        if(   i==(choosen_fields_array.size()-1)  ) //last element
            {
            delim="";
            }
        else
            {
            delim=", ";
            }
        fields_string=fields_string+choosen_fields_array[i]+delim;
        wxString config_path_string="COLUMNS_VISIBLE_NAMES/"+choosen_fields_array[i];
        config->Write(config_path_string, visible_captions_array[i]);
        }
    config->Write( "COLUMNS_TO_SHOW/columns_order", fields_string );
    config->Flush();


    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }






void CfgTabFieldsChooser::on_apply_pressed(wxCommandEvent& event)
    {
    this->write_changes_to_config();
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_RECONSTRUCT_RIGHT_PANEL;
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }





void CfgTabFieldsChooser::on_ok_pressed(wxCommandEvent& event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_RECONSTRUCT_RIGHT_PANEL;
        this->send_reconfigure_panels_event();
        apply_button->Disable();
        IsModified=false;
        }
    this->send_close_event_manually();
    }





void CfgTabFieldsChooser::on_cancel_pressed(wxCommandEvent& event)
    {
    this->send_close_event_manually();
    }




void CfgTabFieldsChooser::restore_values()
    {
    this->generate_visible_arrays_from_config();

    wxArrayString temp_wx_array= wxArrayString();
    for(unsigned int i=0;i<choosen_fields_array.size();i++)
        {
        temp_wx_array.Add(choosen_fields_array_display_names[i]);
        }


    used_list_of_fields->Clear();
    used_list_of_fields->InsertItems(temp_wx_array, 0);

    used_list_of_fields->SetSelection(0);
    this->write_string_to_text_entry(0);
    button_move_up->Disable();
    this->IsModified=false;
    if(used_list_of_fields->GetCount()==0 or used_list_of_fields->GetCount()==1)
        {
        button_move_down->Disable();
        }
    else
        {
        button_move_down->Enable();
        }
    this->apply_button->Disable();
    if(used_list_of_fields->GetCount()!=0) remove_button->Enable();
    }





void CfgTabFieldsChooser::generate_full_arrays()
    {
    full_fields_array.push_back(wxString("Num")); full_fields_display_names.push_back(wxString("Num"));
    full_fields_array.push_back(wxString("Author")); full_fields_display_names.push_back(wxString("Author"));
    full_fields_array.push_back(wxString("Title")); full_fields_display_names.push_back(wxString("Title"));
    //full_fields_array.push_back(wxString("Volume")); full_fields_display_names.push_back(wxString("Volume"));
    full_fields_array.push_back(wxString("Year")); full_fields_display_names.push_back(wxString("Year"));
    full_fields_array.push_back(wxString("Publisher")); full_fields_display_names.push_back(wxString("Publisher"));
    full_fields_array.push_back(wxString("Pages")); full_fields_display_names.push_back(wxString("Number of pages"));
    full_fields_array.push_back(wxString("Extension")); full_fields_display_names.push_back(wxString("Extension"));
    full_fields_array.push_back(wxString("Topic")); full_fields_display_names.push_back(wxString("Topic"));
    full_fields_array.push_back(wxString("md5_of_file")); full_fields_display_names.push_back(wxString("md5"));
    }





void CfgTabFieldsChooser::generate_visible_arrays_from_config()
    {
            choosen_fields_array.clear();
            visible_captions_array.clear();
            choosen_fields_array_display_names.clear();
    wxString temp_string=config->Read(wxString("COLUMNS_TO_SHOW/columns_order"),"");
    std::vector<std::string> exploded_string=Khz::Libs::Mstr::explode(",", temp_string.ToStdString());

    std::vector<wxString>::iterator it;

    for(unsigned int i=0;i< exploded_string.size();i++)
        {
        std::string value=Khz::Libs::Mstr::trim(exploded_string[i]);
        //if full fields array have no such element than some error happened. We will skip such fields;
        it=std::find(full_fields_array.begin(), full_fields_array.end(), wxString(value));
        if(it == full_fields_array.end())  //there no such element in full_fields_array
            {
            continue;
            }
        else
            {
            int index=std::distance(full_fields_array.begin(), it);

            choosen_fields_array.push_back(wxString(value.c_str()));
            visible_captions_array.push_back(wxString (full_fields_display_names[index].c_str()));
            choosen_fields_array_display_names.push_back(wxString (full_fields_display_names[index].c_str()));
            }
        }

    for(unsigned int i=0;i<choosen_fields_array.size();i++)
        {
         wxString temp_string=config->Read(wxString("COLUMNS_VISIBLE_NAMES/")+wxString(choosen_fields_array[i]),"");
         if(temp_string!="")
            {
            visible_captions_array[i]=temp_string;
            }
        }
    }






void CfgTabFieldsChooser::write_string_to_text_entry(unsigned int index)
    {
    if(index >= visible_captions_array.size())
        {
        visible_name_input->ChangeValue("");
        }
    else
        {
        visible_name_input->ChangeValue(visible_captions_array[index]);
        visible_name_input->SetSelection(-1,-1);
        }
    }





void CfgTabFieldsChooser::on_used_list_of_fields_sel_changed(wxCommandEvent& event)
    {
    wxArrayInt selections;
    used_list_of_fields->GetSelections (selections);
    if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened in listbox of CfgTabFieldsChooser.cpp. Aborted";
        abort();
        }
    unsigned int item_num = selections.Item(0);
    this->write_string_to_text_entry(item_num);

    if(item_num==0)
        {
        button_move_up->Disable();
        button_move_down->Enable();
        }
    else if( (item_num+1)==used_list_of_fields->GetCount() )
        {
        button_move_down->Disable();
        button_move_up->Enable();
        }
    else
        {
        button_move_down->Enable();
        button_move_up->Enable();
        }
    }






void CfgTabFieldsChooser::on_button_move_right_pressed(wxCommandEvent& event)
    {
    wxArrayInt selections;
    full_list_of_fields->GetSelections (selections);
    if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened in listbox of CfgTabFieldsChooser.cpp. Aborted";
        abort();
        }
    int item_num = selections.Item(0);
    wxString new_visible_item=full_fields_display_names[item_num];
    wxString new_field=full_fields_array[item_num];

    choosen_fields_array.insert(choosen_fields_array.end(),new_field);
    visible_captions_array.insert(visible_captions_array.end(), new_visible_item);
    choosen_fields_array_display_names.insert(choosen_fields_array_display_names.end(),new_visible_item);

    std::cout << "\n visible items in fields_chosoer\n";
    for (unsigned int i=0;i<choosen_fields_array_display_names.size();i++)
        {
        std::cout << choosen_fields_array_display_names[i] << "\n";
        }

    //wxArrayString temp_wx_array= wxArrayString();
    //temp_wx_array.Add(choosen_fields_array_display_names[0]);
    used_list_of_fields->Append(choosen_fields_array_display_names.back());



    used_list_of_fields->SetSelection(used_list_of_fields->GetCount()-1);
    this->write_string_to_text_entry(used_list_of_fields->GetCount()-1);
    this->apply_button->Enable();
    this->button_move_down->Disable();
    if(used_list_of_fields->GetCount()>1)
        {
        this->button_move_up->Enable();
        }
    else
        {
        this->button_move_up->Disable();
        }
    this->remove_button->Enable();
    this->visible_name_input->Enable();
    this->IsModified=true;
    }







void CfgTabFieldsChooser::on_button_remove_pressed(wxCommandEvent& event)
    {
    if(used_list_of_fields->GetCount()==0) return;
    wxArrayInt selections;
    used_list_of_fields->GetSelections (selections);
    if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened in listbox of CfgTabFieldsChooser.cpp. Aborted";
        abort();
        }
    unsigned int item_num = selections.Item(0);


    choosen_fields_array.erase(choosen_fields_array.begin()+item_num);
    visible_captions_array.erase(visible_captions_array.begin()+ item_num);
    choosen_fields_array_display_names.erase(choosen_fields_array_display_names.begin()+item_num);
    used_list_of_fields->Delete(item_num);
    if(  (item_num==1 or item_num==0)  and used_list_of_fields->GetCount()!=0  )//select top item
        {
        used_list_of_fields->SetSelection(0);
        this->write_string_to_text_entry(0);
        this->apply_button->Enable();
        this->button_move_up->Disable();
        this->button_move_down->Enable();
        this->IsModified=true;
        }
    else if(item_num>1)
        {
        used_list_of_fields->SetSelection(item_num-1);
        this->write_string_to_text_entry(item_num-1);
        this->apply_button->Enable();
        this->button_move_up->Enable();
        if(item_num-1+1== used_list_of_fields->GetCount())
            {
            this->button_move_down->Disable();
            }
        else
            {
            this->button_move_down->Enable();
            }
        this->IsModified=true;
        }
    else
        {
        this->button_move_up->Disable();
        this->button_move_down->Disable();
        remove_button->Disable();
        this->apply_button->Enable();
        }
    if(used_list_of_fields->GetCount()==0 or  used_list_of_fields->GetCount()==1)
        {
        this->button_move_up->Disable();
        this->button_move_down->Disable();
        }
    if(used_list_of_fields->GetCount()==0)
        {
        visible_name_input->ChangeValue("");
        visible_name_input->Disable();
        }

    }






void CfgTabFieldsChooser::on_button_up_pressed(wxCommandEvent& event)
    {
    if (used_list_of_fields->GetCount()==0  or used_list_of_fields->GetCount()==1) return;
    wxArrayInt selections;
    used_list_of_fields->GetSelections (selections);
    if(selections.GetCount()!=1 )
        {
        return;
        }
    int item_num = selections.Item(0);
    if(item_num==0) return; //upper item;

    wxString temp_str_choosen_fields=choosen_fields_array[item_num-1];
    wxString temp_str_visible_captions_array= visible_captions_array[item_num-1];
    wxString temp_str_choosen_fields_array_display_name= choosen_fields_array_display_names[item_num-1];


    choosen_fields_array[item_num-1] = choosen_fields_array[item_num];
    visible_captions_array[item_num-1] = visible_captions_array[item_num];
    choosen_fields_array_display_names[item_num-1]=choosen_fields_array_display_names[item_num];


    choosen_fields_array[item_num] = temp_str_choosen_fields;
    visible_captions_array[item_num] = temp_str_visible_captions_array;
    choosen_fields_array_display_names[item_num]= temp_str_choosen_fields_array_display_name;


    used_list_of_fields->SetSelection(item_num-1);
    used_list_of_fields->SetString(item_num-1, choosen_fields_array_display_names[item_num-1]);
    used_list_of_fields->SetString(item_num, choosen_fields_array_display_names[item_num]);
    this->write_string_to_text_entry(item_num-1);
    this->apply_button->Enable();
    this->IsModified=true;
    if( (item_num-1)==0 ) this->button_move_up->Disable();
    if(choosen_fields_array.size()!=1) button_move_down->Enable();
    }





void CfgTabFieldsChooser::on_button_down_pressed(wxCommandEvent& event)
    {
    if (used_list_of_fields->GetCount()==0  or used_list_of_fields->GetCount()==1) return;
    wxArrayInt selections;
    used_list_of_fields->GetSelections (selections);
    if(selections.GetCount()!=1 )
        {
        return;
        }
    unsigned int item_num = selections.Item(0);
    if(   (item_num+1)>=used_list_of_fields->GetCount()  ) return; //lower item;

    wxString temp_str_choosen_fields=choosen_fields_array[item_num+1];
    wxString temp_str_visible_captions_array= visible_captions_array[item_num+1];
    wxString temp_str_choosen_fields_array_display_name= choosen_fields_array_display_names[item_num+1];


    choosen_fields_array[item_num+1] = choosen_fields_array[item_num];
    visible_captions_array[item_num+1] = visible_captions_array[item_num];
    choosen_fields_array_display_names[item_num+1]=choosen_fields_array_display_names[item_num];


    choosen_fields_array[item_num] = temp_str_choosen_fields;
    visible_captions_array[item_num] = temp_str_visible_captions_array;
    choosen_fields_array_display_names[item_num]= temp_str_choosen_fields_array_display_name;


    used_list_of_fields->SetSelection(item_num+1);
    used_list_of_fields->SetString(item_num+1, choosen_fields_array_display_names[item_num+1]);
    used_list_of_fields->SetString(item_num, choosen_fields_array_display_names[item_num]);
    this->write_string_to_text_entry(item_num+1);
    this->apply_button->Enable();
    this->IsModified=true;
    if( (item_num+1+1)==used_list_of_fields->GetCount() ) this->button_move_down->Disable();
    if(choosen_fields_array.size()!=1) button_move_up->Enable();
    }





void CfgTabFieldsChooser::on_text_changed_in_visible_fields_entry(wxCommandEvent& event)
    {
     wxArrayInt selections;
     used_list_of_fields->GetSelections (selections);
     if(selections.GetCount()!=1)
        {
        return;
        }
    int item_num = selections.Item(0);
    visible_captions_array[item_num] = visible_name_input->GetValue();
    this->IsModified=true;
    this->apply_button->Enable();
    }


}}//namespaces
