/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/window.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/fileconf.h>
#include <wx/dirdlg.h>
#include <wx/filedlg.h>
#include "Custom_IDs.h"
#include <wx/settings.h>
#include "Configure_tabs_parent.h"
#include "CfgTabFontsChooser.h"


namespace Khz{namespace CfgWin{



CfgTabFontsChooser::CfgTabFontsChooser(wxWindow* parent,  wxString path_to_config)
    : Configure_gui_tabs_parent(parent, path_to_config)
    {
     books_list_font_size_input = new wxSpinCtrl( this, wxID_ANY);
     books_list_font_size_input->SetRange(1,100);

	 //books_list_font_size_header_input = new wxSpinCtrl( this, wxID_ANY);
     //books_list_font_size_header_input->SetRange(1,100);

    library_tree_font_size_input = new wxSpinCtrl(this,wxID_ANY);
    library_tree_font_size_input->SetRange(1,100);


    wxString libr_tree_font_string =config->Read("FONTS/library_tree_font_size","");
    wxString books_list_font_string=config->Read("FONTS/books_list_font_size", "");
	//wxString books_list_font_header_string=config->Read("FONTS/books_list_font_size_header", "");

    default_font_size=wxSystemSettings::GetFont(wxSYS_SYSTEM_FONT).GetPointSize();

    library_tree_font_size=default_font_size;
    books_list_font_size=default_font_size;
	//books_list_font_size_header=default_font_size;

    if(libr_tree_font_string!="")
        {
        library_tree_font_size_input->SetValue(wxAtoi(libr_tree_font_string));
        }
    else
        {
        library_tree_font_size_input->SetValue(default_font_size);
        }

    if(books_list_font_string!="")
        {
        books_list_font_size_input->SetValue(wxAtoi(books_list_font_string));
        }
    else
        {
        books_list_font_size_input->SetValue(default_font_size);
        }

	/*if(books_list_font_header_string!="")
        {
        books_list_font_size_header_input->SetValue(wxAtoi(books_list_font_header_string));
        }
    else
        {
        books_list_font_size_header_input->SetValue(default_font_size);
        }
	*/





    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //#############################################################################################


    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   SIZING !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    int left_right_border=5;
    only_sizer = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer *inputs= new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *library_tree_sizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *books_list_sizer=new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);

            library_tree_sizer->Add(new wxStaticText( this, wxID_ANY, wxT("Library tree font size")), 0, wxEXPAND|wxALL);
            library_tree_sizer->Add(library_tree_font_size_input, 0, wxEXPAND|wxALL);


            books_list_sizer->Add(new wxStaticText( this, wxID_ANY, wxT("Books list font size")), 0, wxEXPAND|wxALL);
            books_list_sizer->Add(books_list_font_size_input, 0, wxEXPAND|wxALL);
			//books_list_sizer->AddSpacer(10);
			//books_list_sizer->Add(new wxStaticText( this, wxID_ANY, wxT("Books list header font size")), 0, wxEXPAND|wxALL);
            //books_list_sizer->Add(books_list_font_size_header_input, 0, wxEXPAND|wxALL);
        inputs->Add(library_tree_sizer, 0, wxALL|wxEXPAND);
        inputs->AddSpacer(20);
        inputs->Add(books_list_sizer,0, wxALL|wxEXPAND);

        //#################### buttons ##########################################################
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(left_right_border);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //###############################################################################################################


    only_sizer->Add(inputs, 1, wxALL|wxEXPAND, 5);
    only_sizer->AddStretchSpacer ();
    only_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);

    SetSizerAndFit(only_sizer);

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!! SIZING FINISHED !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    Bind(wxEVT_SPINCTRL, &CfgTabFontsChooser::on_something_changed, this);

    Bind(wxEVT_TEXT, &CfgTabFontsChooser::on_something_changed2, this);


    apply_button->Bind(wxEVT_BUTTON, &CfgTabFontsChooser::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabFontsChooser::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabFontsChooser::on_cancel_pressed, this);
    }







void CfgTabFontsChooser::write_changes_to_config()
    {
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_FONTS;
    delete config;
    config =new wxFileConfig( "", "", config_file);
    config->Write( wxString("FONTS/library_tree_font_size"), library_tree_font_size );
    config->Write( wxString("FONTS/books_list_font_size"), books_list_font_size );
	//config->Write(wxString("FONTS/books_list_font_size_header"), books_list_font_size_header);
    config->Flush();


    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }

void CfgTabFontsChooser::on_apply_pressed(wxCommandEvent& event)
    {
    this->write_changes_to_config();
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }

void CfgTabFontsChooser::on_ok_pressed(wxCommandEvent& event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    //std::cout << "\nclosing config\n";
    this->send_close_event_manually();
    }
void CfgTabFontsChooser::on_cancel_pressed(wxCommandEvent& event)
    {
    this->send_close_event_manually();
    }

void CfgTabFontsChooser::restore_values()
    {

    wxString temp_str_lib= config->Read( wxT("FONTS/library_tree_font_size"), "");
    wxString temp_str_book=config->Read( wxT("FONTS/books_list_font_size"), "" );
	//wxString books_list_font_header_string=config->Read("FONTS/books_list_font_size_header", "");
    if(temp_str_lib!="")
        {
        library_tree_font_size_input->SetValue(wxAtoi(temp_str_lib));
        }
    else
        {
        library_tree_font_size_input->SetValue(default_font_size);
        }

    if(temp_str_book!="")
        {
        books_list_font_size_input->SetValue(wxAtoi(temp_str_book));
        }
    else
        {
        books_list_font_size_input->SetValue(default_font_size);
        }
	/*if(books_list_font_header_string!="")
        {
        books_list_font_size_header_input->SetValue(wxAtoi(books_list_font_header_string));
        }
    else
        {
        books_list_font_size_header_input->SetValue(default_font_size);
        }
	*/
    apply_button->Disable();
    IsModified=false;
    }


void CfgTabFontsChooser::on_something_changed(wxSpinEvent& event)
    {
    library_tree_font_size=library_tree_font_size_input->GetValue();
    books_list_font_size=books_list_font_size_input->GetValue();
	//books_list_font_size_header=books_list_font_size_header_input->GetValue();
    IsModified=true;
    apply_button->Enable();
    }
void CfgTabFontsChooser::on_something_changed2(wxCommandEvent& event)
    {
    library_tree_font_size=library_tree_font_size_input->GetValue();
    books_list_font_size=books_list_font_size_input->GetValue();
	//books_list_font_size_header=books_list_font_size_header_input->GetValue();
    IsModified=true;
    apply_button->Enable();
    }


}}// namespaces
