/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/spinctrl.h>


namespace Khz{namespace CfgWin{


/**
@brief "Fonts" Tab of the "Main window"

"Fonts" Tab of the "Main window".

General idea: we have two variables:
@code
    int library_tree_font_size;
    int books_list_font_size;
@endcode
When user changes values in inputs (we have two inputs), we read new values from the inputs and change the variables accordingly.
When user presses "Apply" button we write current values of both of the variables to the config file, and send special event
that makes MyMainWindow to reload MyBooksList and TreeCtrlLibarary to apply changes (we use Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
**/
class CfgTabFontsChooser : public Configure_gui_tabs_parent
{
public:

    /**
    @param path_to_config full path the the main config file
    @param parent standard wxwidgets parent
    **/
    CfgTabFontsChooser(wxWindow* parent, wxString path_to_config);
private:
    /**
    Default font size for the library tree and for the books list. We use
    this size if there is no information about font sizes in the config file.
    Defined in the constructor.
    **/
    int default_font_size;

    /**
    Current value of the library tree font size
    **/
    int library_tree_font_size;


    /**
    Current value of the books list font size
    **/
    int books_list_font_size;


	/**
	Input for the books list font size
	**/
    wxSpinCtrl  *books_list_font_size_input;


    /**
    Input for the library tree font size
    **/
    wxSpinCtrl *library_tree_font_size_input;
	//wxSpinCtrl *books_list_font_size_header_input;


    /**
    Main window sizer
    **/
    wxBoxSizer    *only_sizer;


    /**
    Executes when user presses up/down arrow of any of the input.
    Gets values from  #books_list_font_size_input and #library_tree_font_size_input,
    updates #books_list_font_size and #library_tree_font_size
    accordingly.
    **/
    void on_something_changed(wxSpinEvent& event);




    /**
    Executes when user changes number inside of one of the inputs by using keyboard.
    Gets values from  #books_list_font_size_input and #library_tree_font_size_input,
    updates #books_list_font_size and #library_tree_font_size
    accordingly.
    **/
    void on_something_changed2(wxCommandEvent& event);

    /**
    "Apply" button"
    **/
    wxButton *apply_button;


    /**
    Writes current values of  #books_list_font_size and  #library_tree_font_size into the config file.
    **/
    void write_changes_to_config();

    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    void on_apply_pressed(wxCommandEvent& event);


    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent& event);

    /**
    Closes the configuration window
    **/
    void on_cancel_pressed(wxCommandEvent& event);


    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();
};


}}//namespaces
