/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CfgTabFullTextSearchArchiveExtractor.h"
#include "Custom_IDs.h"
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <iostream>
#include "DEBUGMESSAGE.h"
#include "FullTextSearchIndexerAddEditExtensionDialog.h"
#include <wx/msgdlg.h>
#include <wx/log.h>
#include "default_config_params.h"



namespace Khz{namespace CfgWin{



CfgTabFullTextSearchArchiveExtractor::CfgTabFullTextSearchArchiveExtractor(wxWindow* parent, wxString path_to_config)
        : Configure_gui_tabs_parent(parent, path_to_config)
    {
    extensions_listbox=new SyncListBox(this);
    commands_listbox=new SyncListBox(this);
    extensions_listbox->SetSyncWindow(commands_listbox);
    commands_listbox->SetSyncWindow(extensions_listbox);
    wxButton *add_button =new wxButton(this, wxID_ANY, "Add");
    wxButton *edit_button=new wxButton(this, wxID_ANY, "Edit");
    wxButton *delete_button=new wxButton(this, wxID_ANY, "Delete");

    set_wxlistboxes_content_from_config_file();



    wxHyperlinkCtrl *restore_defaults_hyperlink =
        new wxHyperlinkCtrl (this, wxID_ANY, "Restore defaults", "");
     // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //################################################################################



    int left_right_border=5;
    wxBoxSizer *main_sizer = new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *listboxes_sizer=new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *ext_listbox_sizer=new wxBoxSizer(wxVERTICAL);
            ext_listbox_sizer->Add(new wxStaticText(this, wxID_ANY, "Extension             "));
            ext_listbox_sizer->Add(extensions_listbox, 1, wxALL|wxEXPAND);
        listboxes_sizer->Add(ext_listbox_sizer,0, wxEXPAND|wxALL);
        listboxes_sizer->AddSpacer(5);
            wxBoxSizer *cmd_listbox_sizer = new wxBoxSizer(wxVERTICAL);
            cmd_listbox_sizer->Add(new wxStaticText(this, wxID_ANY, "Command"));
            cmd_listbox_sizer->Add(commands_listbox, 1, wxALL|wxEXPAND);
        listboxes_sizer->Add(cmd_listbox_sizer,1, wxEXPAND|wxALL);
    main_sizer->Add(listboxes_sizer, 1, wxALL|wxEXPAND, 5);
        wxBoxSizer *add_edit_delete_btns_sizer=new wxBoxSizer(wxHORIZONTAL);
        add_edit_delete_btns_sizer->AddSpacer(5);
        add_edit_delete_btns_sizer->Add(add_button, 0, wxALL);
        add_edit_delete_btns_sizer->AddSpacer(5);
        add_edit_delete_btns_sizer->Add(edit_button, 0, wxALL);
        add_edit_delete_btns_sizer->AddSpacer(5);
        add_edit_delete_btns_sizer->Add(delete_button, 0, wxALL);
        add_edit_delete_btns_sizer->AddStretchSpacer();
        add_edit_delete_btns_sizer->Add(restore_defaults_hyperlink, 0, wxALIGN_RIGHT);
    main_sizer->Add(add_edit_delete_btns_sizer, 0, wxALL|wxEXPAND);
         //#################### buttons ##########################################################
        wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(left_right_border);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
    main_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);
    SetSizerAndFit(main_sizer);
    this->GetSizer()->RecalcSizes();

    Bind(wxEVT_LISTBOX, &CfgTabFullTextSearchArchiveExtractor::OnListboxSelChanged, this);
    delete_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchArchiveExtractor::OnDeleteExtensionButtonPressed, this);
    add_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchArchiveExtractor::OnAddExtensionButtonPressed, this);
    edit_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchArchiveExtractor::OnEditExtensionButtonPressed, this);


    apply_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchArchiveExtractor::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchArchiveExtractor::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchArchiveExtractor::on_cancel_pressed, this);

    restore_defaults_hyperlink->Bind(wxEVT_HYPERLINK,
            &CfgTabFullTextSearchArchiveExtractor::OnRestoreDefaultsHyperlinkClicked, this);

    }


void CfgTabFullTextSearchArchiveExtractor::write_changes_to_config()
    {
    //set the reconfigure type for this->send_reconfigure_panels_event();
    this->ReconfigureType=-1;

    //delete info from the config file (we will write new values soon)
    config->DeleteGroup ("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS");
    config->Flush();
    delete config;
    config =new wxFileConfig( "", "", config_file);



    //write new values to the config file
    for(unsigned int i=0;i<extensions_listbox->GetCount();i++)
        {
        wxString ext="FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS/"+extensions_listbox->GetString(i);
        config->Write( ext, commands_listbox->GetString(i) );
        }
    config->Flush();




    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }

void CfgTabFullTextSearchArchiveExtractor::restore_values()
    {
    set_wxlistboxes_content_from_config_file();

    apply_button->Disable();
    IsModified=false;
    }

void CfgTabFullTextSearchArchiveExtractor::on_apply_pressed(wxCommandEvent &event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    }

void CfgTabFullTextSearchArchiveExtractor::on_ok_pressed(wxCommandEvent &event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    //std::cout << "\nclosing config\n";
    this->send_close_event_manually();
    }

void CfgTabFullTextSearchArchiveExtractor::on_cancel_pressed(wxCommandEvent &event)
    {
    this->send_close_event_manually();
    }

void CfgTabFullTextSearchArchiveExtractor::set_wxlistboxes_content_from_config_file()
    {
    extensions_listbox->Clear();
    commands_listbox->Clear();
    wxString temp_str;
    long dummy;
    std::vector<wxString> array_of_extensions;
     wxString strOldPath = config->GetPath();

    //#################################################################
    config->SetPath("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS");

    bool GCont = config->GetFirstEntry(temp_str, dummy);
    while ( GCont )
        {
        array_of_extensions.push_back(temp_str);
        GCont = config->GetNextEntry(temp_str, dummy);
        }
    config->SetPath(strOldPath);
    KHZNIK_DEBUGMESSAGE
        (
        std::cout << "CfgTabFullTextSearchArchiveExtractor::set_wxlistboxes_content_from_config_file";
        std::cout << " array of extensions is:\n";
        for(unsigned int i=0;i<array_of_extensions.size();i++)
            {
            std::cout << array_of_extensions[i] << "\n";
            }
        )
    for(unsigned int i=0;i<array_of_extensions.size();i++)
        {
        wxString command=config->Read("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS/"+array_of_extensions[i], "");
        extensions_listbox->Append(array_of_extensions[i]);
        commands_listbox->Append(command);
        }
    if(extensions_listbox->GetCount()>0)
        {
        extensions_listbox->SetSelection(0);
        commands_listbox->SetSelection(0);
        }
    }

void CfgTabFullTextSearchArchiveExtractor::OnListboxSelChanged(wxCommandEvent &event)
    {
    int id=event.GetInt();
    if(id>=0 and extensions_listbox->GetCount()>=0)
        {
        extensions_listbox->SetSelection(id);
        commands_listbox->SetSelection(id);
        }
    }

void CfgTabFullTextSearchArchiveExtractor::OnDeleteExtensionButtonPressed(wxCommandEvent &event)
    {
    wxMessageDialog msg(this, "Are you sure?", "Hi there", wxYES_NO|wxICON_QUESTION);
     switch ( msg.ShowModal() )
            {
            case wxID_YES:
                {
                break;
                };
            case wxID_NO:
                {
                return;
                };
            default:
                wxLogError(wxT("Unexpected wxMessageDialog return code in CfgTabFullTextSearchArchiveExtractor!"));
            }







    int sel_num_int= extensions_listbox->GetSelection();
    if(sel_num_int<0) return;
    unsigned int sel_num= (unsigned int) sel_num_int;
    if(sel_num>=0)
        {
        extensions_listbox->Delete(sel_num);
        commands_listbox->Delete(sel_num);
        if(sel_num==0 and extensions_listbox->GetCount()>0)
            {
            extensions_listbox->SetSelection(0);
            commands_listbox->SetSelection(0);
            }
        else if(sel_num>=extensions_listbox->GetCount() and extensions_listbox->GetCount()>0) // last item was deleted
            {
            int temp_num=extensions_listbox->GetCount();
            extensions_listbox->SetSelection(temp_num-1);
            commands_listbox->SetSelection(temp_num-1);
            }
        else if(sel_num>0 and sel_num< extensions_listbox->GetCount())
            {
            extensions_listbox->SetSelection(sel_num);
            commands_listbox->SetSelection(sel_num);
            }
        this->IsModified=true;
        this->apply_button->Enable();
        }
    }

void CfgTabFullTextSearchArchiveExtractor::OnAddExtensionButtonPressed(wxCommandEvent &event)
    {
    FullTextSearchIndexerAddEditExtensionDialog dlg
        (
        this,
        wxID_ANY,
        wxString("Add something"),
        FTS_ADD_EXTENSION, // we add extension
        FTS_ARCHIVEEXTRACTOR, //we add extension for text archive(not text extractor)
        wxRESIZE_BORDER
        );
     if (dlg.ShowModal() == wxID_OK)
        {
        wxString extension=dlg.GetExtension();
        wxString command=dlg.GetExtractor();
        if(extensions_listbox->GetCount()==0)
            {
            extensions_listbox->Append(extension);
            commands_listbox->Append(command);
            }
        else
            {
            extensions_listbox->Insert(extension, 0);
            commands_listbox->Insert(command, 0);
            }
        extensions_listbox->SetSelection(0);
        commands_listbox->SetSelection(0);

        this->IsModified=true;
        this->apply_button->Enable();
        }
    }
void CfgTabFullTextSearchArchiveExtractor::OnEditExtensionButtonPressed(wxCommandEvent &event)
    {
    int sel_num_int= extensions_listbox->GetSelection();
    if(sel_num_int<0) return;
    unsigned int sel_num= (unsigned int) sel_num_int;

    wxString extension=extensions_listbox->GetString(sel_num);
    wxString command=commands_listbox->GetString(sel_num);

    FullTextSearchIndexerAddEditExtensionDialog dlg
        (
        this,
        wxID_ANY,
        wxString("Add something"),
        FTS_EDIT_EXTENSION, // we add extension
        FTS_ARCHIVEEXTRACTOR, //we add extension for text extractor(not archive extractor)
        wxRESIZE_BORDER
        );
    dlg.SetExtension(extension);
    dlg.SetExtractor(command);
    dlg.fill_samples_fields_if_necessary(extension);
    if (dlg.ShowModal() == wxID_OK)
        {
        extensions_listbox->SetString(sel_num, dlg.GetExtension());
        commands_listbox->SetString(sel_num, dlg.GetExtractor());
        this->IsModified=true;
        this->apply_button->Enable();
        }
    }


void CfgTabFullTextSearchArchiveExtractor::OnRestoreDefaultsHyperlinkClicked(wxHyperlinkEvent& event)
    {
    extensions_listbox->Clear();
    commands_listbox->Clear();


    default_config_params defs = default_config_params();
    std::vector<std::pair<wxString, wxString>> extractors=defs.archive_extractors;

    for(unsigned int i=0;i<extractors.size();i++)
        {
        extensions_listbox->Append(extractors[i].first);
        commands_listbox->Append(extractors[i].second);
        }

    if(extensions_listbox->GetCount()>0)
        {
        extensions_listbox->SetSelection(0);
        commands_listbox->SetSelection(0);
        }

    this->IsModified=true;
    this->apply_button->Enable();
    }


}}//namespaces
