/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Configure_tabs_parent.h"
#include <wx/button.h>
#include "SyncListBox.h"
#include <iostream>
#include <wx/dc.h>
#include <wx/renderer.h>
#include <wx/hyperlink.h>


namespace Khz{namespace CfgWin{


/**
@brief "Archive extractor" tab of the "Full text search indexer params"

Allows to configure command that will be used to extract archive with particular extension.

General idea: user can add or edit something. when user presses "Apply" button
we write changes to the config file and call
Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
**/
class CfgTabFullTextSearchArchiveExtractor : public Configure_gui_tabs_parent
{
public:
    /**
    constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main configuration file
    **/
    CfgTabFullTextSearchArchiveExtractor(wxWindow* parent, wxString path_to_config);
private:
    /**
    Reads listboxes content and writes  current state of the control to the config file
    **/
    virtual void write_changes_to_config();

    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    virtual void restore_values();


    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    virtual void on_apply_pressed(wxCommandEvent &event);


    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    virtual void on_ok_pressed(wxCommandEvent &event);

    /**
    Closes the configure gui window
    **/
    virtual void on_cancel_pressed(wxCommandEvent &event);


    /**
    "Apply" button
    **/
    wxButton *apply_button;

    /**
    Lisbox containing list of extensions
    **/
    SyncListBox *extensions_listbox;

    /**
    Listbox containing commands to extract archive
    **/
    SyncListBox *commands_listbox;

    /**
    clears #extensions_listbox and #commands_listbox, than reads the
    main config file and fills these control based on values from
    the config file
    **/
    void set_wxlistboxes_content_from_config_file();

    /**
    Executes when user change selection inside of any of the listboxes
    (#extensions_listbox, #commands_listbox).
    Syncronizes selections of the listboxes.
    **/
    void OnListboxSelChanged(wxCommandEvent &event);

    /**
    Executes when user preses "Delete" extension button.
    Removes currently selected item from #extensions_listbox and #commands_listbox
    (selections of both of the listboxes are syncronized)
    **/
    void OnDeleteExtensionButtonPressed(wxCommandEvent &event);

    /**
    Executes when user presses the "Add" extension button.
    Opens #Khz::CfgWin::FullTextSearchIndexerAddEditExtensionDialog that allows user
    to specify extension, command and so on.
    **/
    void OnAddExtensionButtonPressed(wxCommandEvent &event);

    /**
    Executes when user presses the "Edit" extension button.
    Opens #Khz::CfgWin::FullTextSearchIndexerAddEditExtensionDialog that allows user
    to specify extension, command and so on.
    **/
    void OnEditExtensionButtonPressed(wxCommandEvent &event);

    /**
    clears #extensions_listbox and #commands_listbox, creates an
    instance of the #Khz::default_config_params class object, puts
    default values to #extensions_listbox and #commands_listbox
    by using #Khz::default_config_params.
    **/
    void OnRestoreDefaultsHyperlinkClicked(wxHyperlinkEvent& event);

};
}}// namespaces
