/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "CfgTabFullTextSearchFontsAndColors.h"
#include <wx/stattext.h>
#include <wx/sizer.h>
#include "Custom_IDs.h"
#include "MySizeFunctions.h"



namespace Khz{namespace CfgWin{


CfgTabFullTextSearchFontsAndColors::CfgTabFullTextSearchFontsAndColors(wxWindow* parent, wxString path_to_config)
        : Configure_gui_tabs_parent(parent, path_to_config)
    {
    ///////////////////////////////////////////////////////////////////////////////////////////////
    //
    //      CAPTION OF THE BOOK
    //
    ////////////////////////////////////////////////////////////////////////////////////////////////
    //############################## create bold text
     wxStaticText *caption_text =
                new wxStaticText( this, wxID_ANY,
                                 wxT("Caption of the book (link to the book)"));
     wxFont font = caption_text->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        caption_text->SetFont(font);
    //#################################

    //caption font size
     this->caption_font_size_input = new wxSpinCtrl( this, wxID_ANY);
     caption_font_size_input->SetRange(1,100);
     wxString caption_default_font_size=config->Read("FULL_TEXT_SEARCH/book_caption_font_size" , "12");
     caption_font_size_input->SetValue(wxAtoi(caption_default_font_size));
     this->caption_font_size=(wxAtoi(caption_default_font_size));

    //caption color
    this->caption_color_input=new wxColourPickerCtrl (this, wxID_ANY);
    wxString caption_color_in_config=config->Read("FULL_TEXT_SEARCH/book_caption_color" , "#4b49aa");
    caption_color_input->SetColour (caption_color_in_config);
    this->caption_color=caption_color_in_config;


    //caption bold checkbox
    this->caption_bold_checkbox=new wxCheckBox(this, wxID_ANY, "bold");
    wxString caption_bold_str=config->Read("FULL_TEXT_SEARCH/book_caption_is_bold", "no");
    if(caption_bold_str=="yes")
        {
        caption_bold=true;
        caption_bold_checkbox->SetValue(true);
        }
    else
        {
        caption_bold=false;
        caption_bold_checkbox->SetValue(false);
        }






    ///////////////////////////////////////////////////////////////////////////////////////////////
    //
    //      Additional info about the book
    //
    ////////////////////////////////////////////////////////////////////////////////////////////////
     wxStaticText *info_text =
                new wxStaticText( this, wxID_ANY,
                                 wxT("Additional info about the book"));
        info_text->SetFont(font);

    this->show_info_checkbox=new wxCheckBox(this, wxID_ANY, "show info");
    this->show_info_extension_checkbox=new wxCheckBox(this,wxID_ANY, "show extension");
    this->show_info_year_checkbox=new wxCheckBox(this, wxID_ANY, "show year");
    this->show_info_topic_checkbox=new wxCheckBox(this, wxID_ANY, "show topic");
    this->show_info_path_checkbox = new wxCheckBox(this, wxID_ANY, "show path");
    this->show_info_color_chooser= new  wxColourPickerCtrl (this, wxID_ANY);
    this->show_info_font_chooser=new wxSpinCtrl(this, wxID_ANY);
        show_info_font_chooser->SetRange(1,100);
    wxString show_info_str=config->Read("FULL_TEXT_SEARCH/show_info", "yes");
    wxString show_info_extension_str=config->Read("FULL_TEXT_SEARCH/show_extension", "yes");
    wxString show_info_year_str=config->Read("FULL_TEXT_SEARCH/show_year", "yes");
    wxString show_info_topic_str=config->Read("FULL_TEXT_SEARCH/show_topic", "yes");
    wxString show_info_path_str=config->Read("FULL_TEXT_SEARCH/show_relpath", "yes");
    this->show_info_color=config->Read("FULL_TEXT_SEARCH/info_color", "darkgrey");
        this->show_info_color_chooser->SetColour(show_info_color);
    this->show_info_font_size=wxAtoi(config->Read("FULL_TEXT_SEARCH/info_font_size", "11"));
        this->show_info_font_chooser->SetValue(show_info_font_size);
     if(show_info_str=="yes")
        {
        this->show_info=true;
        show_info_checkbox->SetValue(true);
        show_info_extension_checkbox->Enable();
        show_info_year_checkbox->Enable();
        show_info_topic_checkbox->Enable();
        show_info_path_checkbox->Enable();
        show_info_color_chooser->Enable();
        show_info_font_chooser->Enable();
        }
    else
        {
        this->show_info=false;
        show_info_checkbox->SetValue(false);
        show_info_extension_checkbox->Disable();
        show_info_year_checkbox->Disable();
        show_info_topic_checkbox->Disable();
        show_info_path_checkbox->Disable();
        show_info_color_chooser->Disable();
        show_info_font_chooser->Disable();
        };

    if(show_info_extension_str=="yes")
        {
        this->show_info_extension=true;
        this->show_info_extension_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_extension=false;
        this->show_info_extension_checkbox->SetValue(false);
        };

    if(show_info_year_str=="yes")
        {
        this->show_info_year=true;
        this->show_info_year_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_year=false;
        this->show_info_year_checkbox->SetValue(false);
        };

    if(show_info_topic_str=="yes")
        {
        this->show_info_topic=true;
        this->show_info_topic_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_topic=false;
        this->show_info_topic_checkbox->SetValue(false);
        };

    if(show_info_path_str=="yes")
        {
        this->show_info_path=true;
        this->show_info_path_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_path=false;
        this->show_info_path_checkbox->SetValue(false);
        }






     ///////////////////////////////////////////////////////////////////////////////////////////////
    //
    //      Snippet
    //
    ////////////////////////////////////////////////////////////////////////////////////////////////
     wxStaticText *snippet_text =
                new wxStaticText( this, wxID_ANY,
                                 wxT("Snippet"));
        snippet_text->SetFont(font);

    this->snippet_font_size_input=new wxSpinCtrl(this, wxID_ANY);
    snippet_font_size_input->SetRange(1,100);
    wxString snippet_font_size_str=config->Read("FULL_TEXT_SEARCH/snippet_font_size" , "10");
    snippet_font_size_input->SetValue(wxAtoi(snippet_font_size_str));
    this->snippet_font_size=(wxAtoi(snippet_font_size_str));

    this->snippet_highlight_color_input=new wxColourPickerCtrl (this, wxID_ANY);
    this->snippet_highlight_color=config->Read("FULL_TEXT_SEARCH/highlight_color", "blue");
    this->snippet_highlight_color_input->SetColour(snippet_highlight_color);



    this->snippet_highlighting_bold_checkbox = new wxCheckBox(this, wxID_ANY, "highlighting bold");
    wxString snippet_highlighting_bold_str= config->Read("FULL_TEXT_SEARCH/highlight_bold", "yes");
    if(snippet_highlighting_bold_str=="yes")
        {
        snippet_highlighting_bold_checkbox->SetValue(true);
        snippet_highlighting_is_bold=true;
        }
    else
        {
        snippet_highlighting_bold_checkbox->SetValue(false);
        snippet_highlighting_is_bold=false;
        }

    samples_delimiter= config->Read("FULL_TEXT_SEARCH/dots_symbol", "....");
    samples_delimiter_input=new wxTextCtrl(this, wxID_ANY);
    samples_delimiter_input->SetValue(samples_delimiter);


    this->samples_delim_color_input=new wxColourPickerCtrl (this, wxID_ANY);
    this->samples_delim_color=config->Read("FULL_TEXT_SEARCH/dots_color", "blue");
    this->samples_delim_color_input->SetColour(samples_delim_color);


    this->samples_delim_bold_checkbox=new wxCheckBox(this, wxID_ANY, "samples delimiter bold");
    wxString samples_delim_is_bold_str=config->Read("FULL_TEXT_SEARCH/dots_bold", "yes");
    this->samples_delim_bold= (samples_delim_is_bold_str=="yes") ? true : false;
    if(samples_delim_bold) samples_delim_bold_checkbox->SetValue(true); else samples_delim_bold_checkbox->SetValue(false);










    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this, Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //#############################################################################################


    int left_right_border=5;
    wxBoxSizer *main_sizer=new wxBoxSizer(wxVERTICAL);
    main_sizer->Add(caption_text, 0, wxALL|wxEXPAND, 5);
        wxBoxSizer *caption_opts_sizer=new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *caption_font_size_input_sizer=new wxBoxSizer(wxVERTICAL);
            caption_font_size_input_sizer->Add(new wxStaticText(this, wxID_ANY, "font size (pt)"));
            caption_font_size_input_sizer->Add(caption_font_size_input);
        caption_opts_sizer->Add(caption_font_size_input_sizer,0, wxALL, 5);
            wxBoxSizer *caption_color_sizer=new wxBoxSizer(wxVERTICAL);
            caption_color_sizer->Add(new wxStaticText(this, wxID_ANY, "color"));
            caption_color_sizer->Add(caption_color_input);
        caption_opts_sizer->Add(caption_color_sizer, 0, wxALL, 5);
            wxBoxSizer *caption_bold_sizer=new wxBoxSizer(wxVERTICAL);
            caption_bold_sizer->AddSpacer(Khz::Libs::MySizeFunctions::get_height_of_string("something", this));
            caption_bold_sizer->Add(caption_bold_checkbox);
        caption_opts_sizer->Add(caption_bold_sizer, 0, wxALL, 5);
    main_sizer->Add(caption_opts_sizer);
    main_sizer->AddStretchSpacer ();
    main_sizer->Add(info_text, 0, wxALL|wxEXPAND, 5);
        wxBoxSizer *info_checkboxes_sizer=new wxBoxSizer(wxHORIZONTAL);
        info_checkboxes_sizer->Add(show_info_checkbox, 0, wxALL, 5);
        info_checkboxes_sizer->Add(show_info_extension_checkbox, 0, wxALL, 5);
        info_checkboxes_sizer->Add(show_info_year_checkbox, 0, wxALL, 5);
        info_checkboxes_sizer->Add(show_info_topic_checkbox, 0, wxALL, 5);
        info_checkboxes_sizer->Add(show_info_path_checkbox, 0, wxALL, 5);
    main_sizer->Add(info_checkboxes_sizer, 0, wxALL|wxEXPAND, 5);
        wxBoxSizer *info_font_color_sizer=new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *info_font_sizer=new wxBoxSizer(wxVERTICAL);
            info_font_sizer->Add(new wxStaticText(this, wxID_ANY, "font size (pt)"));
            info_font_sizer->Add(show_info_font_chooser);
        info_font_color_sizer->Add(info_font_sizer, 0, wxALL, 5);
            wxBoxSizer *info_color_sizer=new wxBoxSizer(wxVERTICAL);
            info_color_sizer->Add(new wxStaticText(this, wxID_ANY, "color"));
            info_color_sizer->Add(show_info_color_chooser);
        info_font_color_sizer->Add(info_color_sizer, 0, wxALL, 5);
    main_sizer->Add(info_font_color_sizer);
    main_sizer->AddStretchSpacer ();
    main_sizer->Add(snippet_text, 0, wxALL|wxEXPAND, 5);
        wxBoxSizer *snippet_sizer=new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *snippet_font_sizer=new wxBoxSizer(wxVERTICAL);
            snippet_font_sizer->Add(new wxStaticText(this, wxID_ANY, "font size (pt)"));
            snippet_font_sizer->Add(snippet_font_size_input);
        snippet_sizer->Add(snippet_font_sizer, 0, wxALL, 5);
            wxBoxSizer *snippet_color_sizer=new wxBoxSizer(wxVERTICAL);
            snippet_color_sizer->Add(new wxStaticText(this, wxID_ANY, "highlighting color"));
            snippet_color_sizer->Add(snippet_highlight_color_input, 0, wxEXPAND);
        snippet_sizer->Add(snippet_color_sizer, 0, wxALL, 5);
            wxBoxSizer *snippet_hl_sizer = new wxBoxSizer(wxVERTICAL);
            snippet_hl_sizer->AddSpacer(Khz::Libs::MySizeFunctions::get_height_of_string("something", this));
            snippet_hl_sizer->Add(snippet_highlighting_bold_checkbox);
        snippet_sizer->Add(snippet_hl_sizer, 0, wxALL, 5);
    main_sizer->Add(snippet_sizer);
        wxBoxSizer *samples_delimiter_sizer= new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *samples_delim_symbol_sizer=new wxBoxSizer(wxVERTICAL);
            samples_delim_symbol_sizer->Add(new wxStaticText(this, wxID_ANY, "samples delimiter symbol"));
            samples_delim_symbol_sizer->Add(samples_delimiter_input, 0, wxEXPAND);
        samples_delimiter_sizer->Add(samples_delim_symbol_sizer, 0, wxALL, 5);
            wxBoxSizer *samples_delim_color_sizer=new wxBoxSizer(wxVERTICAL);
            samples_delim_color_sizer->Add(new wxStaticText(this, wxID_ANY, "samples delimiter color"));
            samples_delim_color_sizer->Add(samples_delim_color_input, 0, wxEXPAND);
        samples_delimiter_sizer->Add(samples_delim_color_sizer, 0, wxALL, 5);
            wxBoxSizer *snippet_delim_bold_sizer=new wxBoxSizer(wxVERTICAL);
            snippet_delim_bold_sizer->AddSpacer(Khz::Libs::MySizeFunctions::get_height_of_string("something", this));
            snippet_delim_bold_sizer->Add(samples_delim_bold_checkbox);
        samples_delimiter_sizer->Add(snippet_delim_bold_sizer, 0, wxALL, 5);
    main_sizer->Add(samples_delimiter_sizer);




     wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);

        //#################### buttons ##########################################################
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(left_right_border);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //###############################################################################################################
    //main_sizer->AddStretchSpacer ();
    main_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);
    SetSizerAndFit(main_sizer);



    apply_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchFontsAndColors::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchFontsAndColors::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchFontsAndColors::on_cancel_pressed, this);

    Bind(wxEVT_SPINCTRL, &CfgTabFullTextSearchFontsAndColors::on_some_spinctrl_or_textctrl_changed, this);

    Bind(wxEVT_TEXT, &CfgTabFullTextSearchFontsAndColors::on_some_spinctrl_or_textctrl_changed2, this);
    Bind(wxEVT_COLOURPICKER_CHANGED, &CfgTabFullTextSearchFontsAndColors::on_some_color_changed, this);
    Bind(wxEVT_CHECKBOX, &CfgTabFullTextSearchFontsAndColors::on_some_checkbox_changed, this);
    }


void CfgTabFullTextSearchFontsAndColors::write_changes_to_config()
    {
     this->ReconfigureType=Khz::my_wx_id__FULL_TEXT_SEARCH_REDRAW_HTML_EVENT;
    delete config;
    config =new wxFileConfig( "", "", config_file);
    config->Write( wxString("FULL_TEXT_SEARCH/book_caption_font_size"), caption_font_size );
    config->Write("FULL_TEXT_SEARCH/book_caption_color" , caption_color);

    wxString caption_bold_str= caption_bold ? "yes":"no";
    config->Write("FULL_TEXT_SEARCH/book_caption_is_bold", caption_bold_str);

    wxString show_info_str=show_info ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/show_info",  show_info_str);

    wxString show_info_ext_str=show_info_extension ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/show_extension", show_info_ext_str);


    wxString show_info_year_str=show_info_year ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/show_year", show_info_year_str);

    wxString show_info_topic_str=show_info_topic ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/show_topic", show_info_topic_str);

    wxString show_info_path_str = show_info_path ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/show_relpath", show_info_path_str);

    config->Write("FULL_TEXT_SEARCH/info_color", show_info_color);

    config->Write("FULL_TEXT_SEARCH/info_font_size", show_info_font_size);

    config->Write("FULL_TEXT_SEARCH/snippet_font_size", snippet_font_size);
    config->Write("FULL_TEXT_SEARCH/highlight_color", snippet_highlight_color);

    wxString shippet_highlighting_bold_str= snippet_highlighting_is_bold ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/highlight_bold", shippet_highlighting_bold_str);

    config->Write("FULL_TEXT_SEARCH/dots_symbol", samples_delimiter);

    config->Write("FULL_TEXT_SEARCH/dots_color", samples_delim_color);

    wxString samples_delim_is_bold_str= samples_delim_bold ? "yes" : "no";
    config->Write("FULL_TEXT_SEARCH/dots_bold", samples_delim_is_bold_str);

    config->Flush();


    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }

void CfgTabFullTextSearchFontsAndColors::restore_values()
    {
    wxString caption_default_font_size=config->Read("FULL_TEXT_SEARCH/book_caption_font_size" , "12");
    caption_font_size_input->SetValue(wxAtoi(caption_default_font_size));
    caption_font_size=(wxAtoi(caption_default_font_size));

    caption_color=config->Read("FULL_TEXT_SEARCH/book_caption_color" , "#4b49aa");
    caption_color_input->SetColour(caption_color);


    wxString caption_bold_str=config->Read("FULL_TEXT_SEARCH/book_caption_is_bold", "no");
    if(caption_bold_str=="yes")
        {
        caption_bold=true;
        caption_bold_checkbox->SetValue(true);
        }
    else
        {
        caption_bold=false;
        caption_bold_checkbox->SetValue(false);
        }

    wxString show_info_str=config->Read("FULL_TEXT_SEARCH/show_info", "yes");
    wxString show_info_extension_str=config->Read("FULL_TEXT_SEARCH/show_extension", "yes");
    wxString show_info_year_str=config->Read("FULL_TEXT_SEARCH/show_year", "yes");
    wxString show_info_topic_str=config->Read("FULL_TEXT_SEARCH/show_topic=", "yes");
    wxString show_info_path_str=config->Read("FULL_TEXT_SEARCH/show_relpath", "yes");
    this->show_info_color=config->Read("FULL_TEXT_SEARCH/info_color", "darkgrey");
    this->show_info_color_chooser->SetColour(show_info_color);
    this->show_info_font_size=wxAtoi(config->Read("FULL_TEXT_SEARCH/info_font_size", "10"));
    this->show_info_font_chooser->SetValue(show_info_font_size);
     if(show_info_str=="yes")
        {
        show_info=true;
        show_info_checkbox->SetValue(true);
        show_info_extension_checkbox->Enable();
        show_info_year_checkbox->Enable();
        show_info_topic_checkbox->Enable();
        show_info_path_checkbox->Enable();
        show_info_color_chooser->Enable();
        show_info_font_chooser->Enable();
        }
    else
        {
        show_info=false;
        show_info_checkbox->SetValue(false);
        show_info_extension_checkbox->Disable();
        show_info_year_checkbox->Disable();
        show_info_topic_checkbox->Disable();
        show_info_path_checkbox->Disable();
        show_info_color_chooser->Disable();
        show_info_font_chooser->Disable();
        };

    if(show_info_extension_str=="yes")
        {
        this->show_info_extension=true;
        this->show_info_extension_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_extension=false;
        this->show_info_extension_checkbox->SetValue(false);
        };

     if(show_info_year_str=="yes")
        {
        this->show_info_year=true;
        this->show_info_year_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_year=false;
        this->show_info_year_checkbox->SetValue(false);
        };

    if(show_info_topic_str=="yes")
        {
        this->show_info_topic=true;
        this->show_info_topic_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_topic=false;
        this->show_info_topic_checkbox->SetValue(false);
        };

     if(show_info_path_str=="yes")
        {
        this->show_info_path=true;
        this->show_info_path_checkbox->SetValue(true);
        }
    else
        {
        this->show_info_path=false;
        this->show_info_path_checkbox->SetValue(false);
        }

    snippet_font_size=wxAtoi(config->Read("FULL_TEXT_SEARCH/snippet_font_size", "10"));
    snippet_font_size_input->SetValue(snippet_font_size);
    snippet_highlight_color=config->Read("FULL_TEXT_SEARCH/highlight_color", "blue");
    snippet_highlight_color_input->SetColour(snippet_highlight_color);

    wxString snippet_highlighting_bold_str= config->Read("FULL_TEXT_SEARCH/highlight_bold", "yes");
    if(snippet_highlighting_bold_str=="yes")
        {
        snippet_highlighting_bold_checkbox->SetValue(true);
        snippet_highlighting_is_bold=true;
        }
    else
        {
        snippet_highlighting_bold_checkbox->SetValue(false);
        snippet_highlighting_is_bold=false;
        }

    this->samples_delimiter= config->Read("FULL_TEXT_SEARCH/dots_symbol", "....");
    this->samples_delimiter_input->SetValue(samples_delimiter);

    this->samples_delim_color=config->Read("FULL_TEXT_SEARCH/dots_color", "blue");
    this->samples_delim_color_input->SetColour(samples_delim_color);



    wxString samples_delim_is_bold_str=config->Read("FULL_TEXT_SEARCH/dots_bold", "yes");
    this->samples_delim_bold= (samples_delim_is_bold_str=="yes") ? true : false;
    if(samples_delim_bold) samples_delim_bold_checkbox->SetValue(true); else samples_delim_bold_checkbox->SetValue(false);

    apply_button->Disable();
    IsModified=false;
    }

void CfgTabFullTextSearchFontsAndColors::on_apply_pressed(wxCommandEvent &event)
    {
    this->write_changes_to_config();
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }

void CfgTabFullTextSearchFontsAndColors::on_ok_pressed(wxCommandEvent &event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    //std::cout << "\nclosing config\n";
    this->send_close_event_manually();
    }


void CfgTabFullTextSearchFontsAndColors::on_cancel_pressed(wxCommandEvent &event)
    {
    this->send_close_event_manually();
    }


void CfgTabFullTextSearchFontsAndColors::on_some_spinctrl_or_textctrl_changed(wxSpinEvent& event)
    {
    read_spinctrl_and_textctrl_values();

    IsModified=true;
    apply_button->Enable();
    }
void CfgTabFullTextSearchFontsAndColors::on_some_spinctrl_or_textctrl_changed2(wxCommandEvent& event)
    {
    read_spinctrl_and_textctrl_values();

    IsModified=true;
    apply_button->Enable();
    }


void CfgTabFullTextSearchFontsAndColors::read_spinctrl_and_textctrl_values()
    {
    caption_font_size=caption_font_size_input->GetValue();
    show_info_font_size=show_info_font_chooser->GetValue();
    snippet_font_size=snippet_font_size_input->GetValue();
    samples_delimiter= samples_delimiter_input->GetValue();
    }

void CfgTabFullTextSearchFontsAndColors::on_some_color_changed(wxColourPickerEvent& event)
    {
    caption_color=caption_color_input->GetColour().GetAsString();
    show_info_color=show_info_color_chooser->GetColour().GetAsString();
    snippet_highlight_color=snippet_highlight_color_input->GetColour().GetAsString();
    samples_delim_color=samples_delim_color_input->GetColour().GetAsString();

    IsModified=true;
    apply_button->Enable();
    }


void CfgTabFullTextSearchFontsAndColors::on_some_checkbox_changed(wxCommandEvent& event)
    {
    caption_bold = caption_bold_checkbox->GetValue() ? true:false;


    show_info= show_info_checkbox->GetValue() ? true : false;
    if(!show_info)
        {
        show_info_extension_checkbox->Disable();
        show_info_year_checkbox->Disable();
        show_info_topic_checkbox->Disable();
        show_info_path_checkbox->Disable();
        show_info_color_chooser->Disable();
        show_info_font_chooser->Disable();
        }
    else
        {
        show_info_extension_checkbox->Enable();
        show_info_year_checkbox->Enable();
        show_info_topic_checkbox->Enable();
        show_info_path_checkbox->Enable();
        show_info_color_chooser->Enable();
        show_info_font_chooser->Enable();
        }

    show_info_extension= show_info_extension_checkbox->GetValue() ? true : false;
    show_info_year=show_info_year_checkbox->GetValue() ? true : false;
    show_info_topic=show_info_topic_checkbox->GetValue() ? true : false;
    show_info_path=show_info_path_checkbox->GetValue() ? true : false;

    snippet_highlighting_is_bold= snippet_highlighting_bold_checkbox->GetValue() ? true : false;

    samples_delim_bold= samples_delim_bold_checkbox->GetValue() ? true : false;

    IsModified=true;
    apply_button->Enable();
    }


}}// namespaces
