/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Configure_tabs_parent.h"
#include <wx/spinctrl.h>
#include <wx/button.h>
#include <wx/clrpicker.h>
#include <wx/checkbox.h>
#include <wx/textctrl.h>


namespace Khz{namespace CfgWin{

/**
@brief "Fonts and colors" tab of the "Full text search window"

allows to configure appearance of the full text search window.

General idea: user can add or edit something. when user presses "Apply" button
we write changes to the config file and call
Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
**/
class CfgTabFullTextSearchFontsAndColors : public Configure_gui_tabs_parent
{
public:
    /**
    constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file
    **/
    CfgTabFullTextSearchFontsAndColors(wxWindow* parent, wxString path_to_config);
private:

    /**
    writes changes to the config file
    **/
    virtual void write_changes_to_config();

    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    virtual void restore_values();

    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    virtual void on_apply_pressed(wxCommandEvent &event);

    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    virtual void on_ok_pressed(wxCommandEvent &event);


    /**
    closes the window
    **/
    virtual void on_cancel_pressed(wxCommandEvent &event);

    /**
    input for the caption of the book font size
    **/
    wxSpinCtrl *caption_font_size_input;

    /**
    current value of the caption font size. This value is going to be
    written to the config file when user presses "Apply" button.
    **/
    int caption_font_size;

    /**
    input for the color of the book's caption
    **/
    wxColourPickerCtrl *caption_color_input;

    /**
    current value of the book's caption color. This value is
    gonna to be written to the config file by #write_changes_to_config ()
    **/
    wxString caption_color;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #caption_bold_checkbox is checked, false otherwise.
    **/
    bool caption_bold=true;

    /**
    "bold" checkbox. if checked, than caption of the book is going to be bold
    **/
    wxCheckBox *caption_bold_checkbox;

    /**
    "show info" checkbox
    **/
    wxCheckBox *show_info_checkbox;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #show_info_checkbox is checked, false otherwise.
    **/
    bool show_info;

    /**
    "show extension" checkbox"
    **/
    wxCheckBox *show_info_extension_checkbox;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #show_info_extension_checkbox is checked, false otherwise.
    **/
    bool show_info_extension;

    /**
    "show year" checkbox
    **/
    wxCheckBox *show_info_year_checkbox;



    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #show_info_year_checkbox is checked, false otherwise.
    **/
    bool show_info_year;



    /**
    "show topic" checkbox
    **/
    wxCheckBox *show_info_topic_checkbox;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #show_info_topic_checkbox is checked, false otherwise.
    **/
    bool show_info_topic;


    /**
    "show path" checkbox
    **/
    wxCheckBox *show_info_path_checkbox;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #show_info_path_checkbox is checked, false otherwise.
    **/
    bool show_info_path;


    /**
    "Additional info about the book" color chooser
    **/
    wxColourPickerCtrl *show_info_color_chooser;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #show_info_color_chooser
    **/
    wxString show_info_color;


    /**
    font size input of the "Additional info about the book" entry
    **/
    wxSpinCtrl *show_info_font_chooser;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #show_info_font_chooser
    **/
    int show_info_font_size;



    /**
    font size input of the "Snippet" entry
    **/
    wxSpinCtrl *snippet_font_size_input;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #snippet_font_size_input
    **/
    int snippet_font_size;

    /**
    color input of the "Snippet" entry
    **/
    wxColourPickerCtrl *snippet_highlight_color_input;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #snippet_highlight_color_input
    **/
    wxString snippet_highlight_color;



    /**
    "bold" checkbox of the "Snippet" entry
    **/
    wxCheckBox *snippet_highlighting_bold_checkbox;



    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #snippet_highlighting_bold_checkbox
    **/
    bool snippet_highlighting_is_bold;


    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #samples_delimiter_input
    **/
    wxString samples_delimiter;

    /**
    "samples delimiter symbol" input
    **/
    wxTextCtrl *samples_delimiter_input;


    /**
    "samples delimiter color" input
    **/
    wxColourPickerCtrl *samples_delim_color_input;



    /**
    #write_changes_to_config () uses this var when writing to the config file.
    We change this value when user change value of #samples_delim_color_input
    **/
    wxString samples_delim_color;


    /**
    "samples delimiter bold" checkbox
    **/
    wxCheckBox *samples_delim_bold_checkbox;

    /**
    #write_changes_to_config () uses this var when writing to the config file.
    This var is true if #samples_delim_bold_checkbox is checked, false otherwise.
    **/
    bool samples_delim_bold;

    /**
    Apply button
    **/
    wxButton *apply_button;

    /**
    executes when user changes state of any of the spinctrls by
    pressing up or down arrow.
    Reads values from all of the spinctrls and
    updates inner variables accordingly
    **/
    void on_some_spinctrl_or_textctrl_changed(wxSpinEvent& event);

    /**
    executes when user changes state of any of the spinctrls by a keyboard.
    Reads values from all of the spinctrls and
    updates all of the inner variables accordingly
    **/
    void on_some_spinctrl_or_textctrl_changed2(wxCommandEvent& event);

    /**
    executes when user changes state of any of the color pickers.
    Reads values from all of the color pickers and
    updates all of the inner variables accordingly
    **/
    void on_some_color_changed(wxColourPickerEvent& event);


    /**
    executes when user changes state of any of the checkboxes.
    Reads values from all of the checkboxes and
    updates all of the inner variables accordingly
    **/
    void on_some_checkbox_changed(wxCommandEvent& event);

    /**
    Sets #caption_font_size, #show_info_font_size,
    #snippet_font_size, #samples_delimiter to values
    that are reflecting states of the correspondign inputs
    **/
    void read_spinctrl_and_textctrl_values();
};

}}// namespaces
