/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CfgTabFullTextSearchStemmers.h"
#include "Custom_IDs.h"
#include "string_functions.h"
#include <xapian.h>
#include <wx/sizer.h>
#include <iostream>


namespace Khz{namespace CfgWin{


CfgTabFullTextSearchStemmers::CfgTabFullTextSearchStemmers(wxWindow* parent, wxString path_to_config)
        : Configure_gui_tabs_parent(parent, path_to_config)
    {
    ///////////////////////////some buttons//////////////////////////////
    wxBitmap bitmap_up;
    bitmap_up.LoadFile(wxString("icons")+wxFileName::GetPathSeparator()+"go-up.png", wxBITMAP_TYPE_PNG);

    wxBitmap bitmap_down;
    bitmap_down.LoadFile(wxString("icons")+wxFileName::GetPathSeparator()+"go-down.png", wxBITMAP_TYPE_PNG);
    button_move_up= 	new wxBitmapButton (this, wxID_ANY, bitmap_up);
    button_move_down= 	new wxBitmapButton (this, wxID_ANY, bitmap_down);

    set_to_default_button=new wxButton(this ,wxID_ANY, "Set to default");
    ///////////////////////////////////////////////////////////////////////////////////////



    /////////////////////stemmers list/////////////////////////////////
    stemmers_list=new wxListBox(this, wxID_ANY);
    std::vector<std::string> stemmers= GetStemmers();
    for(unsigned int i=0;i<stemmers.size();i++)
        {
        stemmers_list->Append(stemmers[i].c_str());
        }
    if (stemmers_list->GetCount()>0)
        {
        stemmers_list->SetSelection(0);
        button_move_up->Disable();
        }
    //////////////////////////////////////////////////////////////////////

    default_stemmer=GetDefaultStemmerFromConfig();

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    default_stemmer_text=new wxStaticText(this, wxID_ANY, wxString("Default stemmer: ")+default_stemmer.c_str());
     wxFont font = default_stemmer_text->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        default_stemmer_text->SetFont(font);
    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////




    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //################################################################################


    wxBoxSizer *only_sizer = new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *stemmers_sizer=new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *left_listbox_sizer= new wxBoxSizer(wxVERTICAL);
            left_listbox_sizer->Add(
                        new wxStaticText( this, wxID_ANY, wxT("stemmers                                ")));
            left_listbox_sizer->Add(stemmers_list,1, wxEXPAND|wxALL,0);
            left_listbox_sizer->AddSpacer(5);
            left_listbox_sizer->Add(set_to_default_button, 0, wxALL|wxALIGN_LEFT);
        stemmers_sizer->Add( left_listbox_sizer,0, wxEXPAND|wxALL,5);
            wxBoxSizer *up_down_buttons_sizer=new wxBoxSizer(wxVERTICAL);
            up_down_buttons_sizer->Add(button_move_up,0, wxALL);
            up_down_buttons_sizer->AddSpacer(5);
            up_down_buttons_sizer->Add(button_move_down, 0, wxALL);
        stemmers_sizer->Add(up_down_buttons_sizer, 0, wxALIGN_CENTER);
        stemmers_sizer->Add(default_stemmer_text, 1, wxALL|wxALIGN_TOP, 5);
    only_sizer->Add(stemmers_sizer, 1, wxALL|wxEXPAND, 5);
        //#################### buttons ##########################################################
        wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(5);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //########################################################################################
    only_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);
    SetSizerAndFit(only_sizer);



    button_move_up->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchStemmers::on_button_up_pressed, this);
    button_move_down->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchStemmers::on_button_down_pressed, this);
    stemmers_list->Bind(wxEVT_LISTBOX, &CfgTabFullTextSearchStemmers::on_stemmers_sel_changed, this);
    set_to_default_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchStemmers::OnButtonSetToDefaultPressed, this);

    apply_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchStemmers::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchStemmers::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabFullTextSearchStemmers::on_cancel_pressed, this);
    }

void CfgTabFullTextSearchStemmers::write_changes_to_config()
    {
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_STEMMERS_ORDER;

    delete config;
    config =new wxFileConfig( "", "", config_file);


    wxString stemmers_output_string="";
    unsigned int stemmers_list_size=stemmers_list->GetCount();
    for (unsigned int i=0;i<stemmers_list_size;i++)
        {
        if(i!=stemmers_list_size-1)
            {
            stemmers_output_string=stemmers_output_string+stemmers_list->GetString(i)+ " ";
            }
        else
            {
            stemmers_output_string=stemmers_output_string+stemmers_list->GetString(i);
            }
        }
    config->Write("FULL_TEXT_SEARCH/stemmers", stemmers_output_string);
    config->Write("FULL_TEXT_SEARCH/default_stemmer", default_stemmer.c_str());

    config->Flush();
    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }

void CfgTabFullTextSearchStemmers::restore_values()
    {
    std::vector<std::string> stemmers= GetStemmers();
    stemmers_list->Clear();
    for(unsigned int i=0;i<stemmers.size();i++)
        {
        stemmers_list->Append(stemmers[i].c_str());
        }
    if (stemmers_list->GetCount()>0)
        {
        stemmers_list->SetSelection(0);
        button_move_up->Disable();
        }

    default_stemmer=GetDefaultStemmerFromConfig();

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    default_stemmer_text->SetLabel(wxString("Default stemmer: ")+default_stemmer.c_str());
    }

void CfgTabFullTextSearchStemmers::on_apply_pressed(wxCommandEvent &event)
    {
    this->write_changes_to_config();
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_STEMMERS_ORDER;
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }

void CfgTabFullTextSearchStemmers::on_ok_pressed(wxCommandEvent &event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
         this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_STEMMERS_ORDER;
        this->send_reconfigure_panels_event();
        apply_button->Disable();
        IsModified=false;
        }
    this->send_close_event_manually();
    }

void CfgTabFullTextSearchStemmers::on_cancel_pressed(wxCommandEvent &event)
    {
    this->send_close_event_manually();
    }


std::string CfgTabFullTextSearchStemmers::GetDefaultStemmerFromConfig()
    {
    std::string stemmer = config->Read("FULL_TEXT_SEARCH/default_stemmer", "").ToStdString();
    if (stemmer=="") return "english"; else return stemmer;
    }


std::vector<std::string> CfgTabFullTextSearchStemmers::GetStemmers()
    {
    std::string stemmers=config->Read("FULL_TEXT_SEARCH/stemmers", "").ToStdString();


    //Let us make an array of the supported by xapian stemmers
    std::vector<std::string> stemmers_vector;

    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //Next, we explode string and create array of the supported languages
    stemmers_vector=Khz::Libs::Mstr::explode(" ", Khz::Libs::Mstr::trim(boost::regex_replace(stemmers, boost::regex("\\s+", boost::regex_constants::icase)," ")));

    if(AreStemmersGood(stemmers_vector)) return stemmers_vector; else return GetStemmersFromXapian();
    }


bool CfgTabFullTextSearchStemmers::AreStemmersGood(std::vector<std::string> stemmers)
    {
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //sort stemmers by insertion algorithm
    for(unsigned int i=1; i<stemmers.size() ;i++)
        for(int    j=i;    j>0   && stemmers[j-1]   >  stemmers[j];  j--) // пока j>0 и элемент j-1 > j, x-массив int
            std::swap(stemmers[j-1],stemmers[j]);        // меняем местами элементы j и j-1
    //sorting finished (successfully i hope)
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



    std::string supported_by_xapian_languages=Xapian::Stem::get_available_languages();

    //Let us make an array of the supported by xapian stemmers
    std::vector<std::string> xapian_stemmers;

    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //Next, we explode string and create array of the supported languages
    xapian_stemmers=Khz::Libs::Mstr::explode(" ",Khz::Libs::Mstr::trim(boost::regex_replace(supported_by_xapian_languages, boost::regex("\\s+", boost::regex_constants::icase)," ")));



    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //sort xapian_stemmers by insertion algorithm
    for(unsigned int i=1; i<xapian_stemmers.size() ;i++)
        for(int    j=i;    j>0   && xapian_stemmers[j-1]   >  xapian_stemmers[j];  j--) // пока j>0 и элемент j-1 > j, x-массив int
            std::swap(xapian_stemmers[j-1],xapian_stemmers[j]);        // меняем местами элементы j и j-1
    //sorting finished (successfully i hope)
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    if(stemmers==xapian_stemmers) return true; else return false; // compare SORTED vectors
    }


std::vector<std::string> CfgTabFullTextSearchStemmers::GetStemmersFromXapian()
    {
    std::string supported_by_xapian_languages=Xapian::Stem::get_available_languages();

    //Let us make an array of the supported by xapian stemmers
    std::vector<std::string> xapian_stemmers;

    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //Next, we explode string and create array of the supported languages
    xapian_stemmers=Khz::Libs::Mstr::explode(" ",Khz::Libs::Mstr::trim(boost::regex_replace(supported_by_xapian_languages, boost::regex("\\s+", boost::regex_constants::icase)," ")));
    return xapian_stemmers;
    }





void CfgTabFullTextSearchStemmers::on_button_up_pressed(wxCommandEvent& event)
    {
    if (stemmers_list->GetCount()==0  or stemmers_list->GetCount()==1) return;
    wxArrayInt selections;
    stemmers_list->GetSelections (selections);
    if(selections.GetCount()!=1 )
        {
        return;
        }
    int item_num = selections.Item(0);
    if(item_num==0) return; //upper item;


    wxString currently_selected_item= stemmers_list->GetString(item_num);
    wxString above_selected_string= stemmers_list->GetString(item_num-1);

    stemmers_list->SetString(item_num, above_selected_string);
    stemmers_list->SetString(item_num-1, currently_selected_item);
    stemmers_list->SetSelection(item_num-1);

    this->apply_button->Enable();
    this->IsModified=true;
    if( (item_num-1)==0 ) this->button_move_up->Disable();
    if(stemmers_list->GetCount()>1) button_move_down->Enable();
    }





void CfgTabFullTextSearchStemmers::on_button_down_pressed(wxCommandEvent& event)
    {
    if (stemmers_list->GetCount()==0  or stemmers_list->GetCount()==1) return;
    wxArrayInt selections;
    stemmers_list->GetSelections (selections);
    if(selections.GetCount()!=1 )
        {
        return;
        }
    unsigned int item_num = selections.Item(0);
    if(   (item_num+1)>=stemmers_list->GetCount()  ) return; //lower item;

    wxString currently_selected_item= stemmers_list->GetString(item_num);
    wxString below_selected_string= stemmers_list->GetString(item_num+1);

    stemmers_list->SetString(item_num, below_selected_string);
    stemmers_list->SetString(item_num+1, currently_selected_item);
    stemmers_list->SetSelection(item_num+1);

    this->apply_button->Enable();
    this->IsModified=true;
    if( (item_num+1+1)==stemmers_list->GetCount() ) this->button_move_down->Disable();
    if(stemmers_list->GetCount()!=1) button_move_up->Enable();
    }



void CfgTabFullTextSearchStemmers::on_stemmers_sel_changed(wxCommandEvent& event)
    {
    wxArrayInt selections;
    stemmers_list->GetSelections (selections);
    if(selections.GetCount()!=1)
        {
        std::cout << "\n some terrible happened in listbox of CfgTabFullTextSearchStemmers.cpp.";
        return;
        }
    unsigned int item_num = selections.Item(0);
    if(item_num==0)
        {
        button_move_up->Disable();
        button_move_down->Enable();
        }
    else if( (item_num+1)==stemmers_list->GetCount() )
        {
        button_move_down->Disable();
        button_move_up->Enable();
        }
    else
        {
        button_move_down->Enable();
        button_move_up->Enable();
        }
    }


void CfgTabFullTextSearchStemmers::OnButtonSetToDefaultPressed(wxCommandEvent & event)
    {
    wxArrayInt selections;
    stemmers_list->GetSelections (selections);
    if(selections.GetCount()!=1 )
        {
        return;
        }
    unsigned int item_num = selections.Item(0);
    default_stemmer= stemmers_list->GetString(item_num).ToStdString();
    default_stemmer_text->SetLabel(wxString("Default stemmer: ")+ default_stemmer.c_str());

    this->apply_button->Enable();
    this->IsModified=true;
    }


}}// namespaces
