/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/window.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/fileconf.h>
#include <wx/dirdlg.h>
#include <wx/filedlg.h>
#include "Custom_IDs.h"
#include <wx/settings.h>
#include "Configure_tabs_parent.h"
#include "CfgTabIconsChooser.h"
#include <wx/combobox.h>
#include <wx/msgdlg.h>


namespace Khz{namespace CfgWin{


CfgTabIconsChooser::CfgTabIconsChooser(wxWindow* parent,  wxString path_to_config)
    : Configure_gui_tabs_parent(parent, path_to_config)
    {
    change_icons_info_hyperlink=new wxHyperlinkCtrl (this, wxID_ANY, "How do i change icons theme?", "click");
    change_icons_info_hyperlink->SetVisitedColour (change_icons_info_hyperlink->GetNormalColour());

    books_list_icons_size_input = new wxChoice( this, wxID_ANY);
    library_tree_icons_size_input = new wxChoice(this,wxID_ANY);
    allow_tree_custom=new wxCheckBox(this, wxID_ANY, "Custom size");
    allow_books_custom=new wxCheckBox(this, wxID_ANY, "Custom size");;
    books_list_icons_size_input_custom= new wxSpinCtrl( this, wxID_ANY);
    books_list_icons_size_input_custom->Disable();
    books_list_icons_size_input_custom->SetRange(1,100);
    library_tree_icons_size_input_custom= new wxSpinCtrl( this, wxID_ANY);
    library_tree_icons_size_input_custom->Disable();
    library_tree_icons_size_input_custom->SetRange(1,100);



    wxString libr_tree_icons_string =config->Read("ICONS/library_tree_icons_size","");
    wxString books_list_icons_string=config->Read("ICONS/books_list_icons_size", "");


     if(libr_tree_icons_string!="")
        {
        library_tree_icons_size=wxAtoi(libr_tree_icons_string);
        }
    else
        {
        library_tree_icons_size=16;
        }

    if(books_list_icons_string!="")
        {
        books_list_icons_size=wxAtoi(books_list_icons_string);
        }
    else
        {
        books_list_icons_size=16;
        }


      books_list_icons_sizes_vector.push_back(std::make_pair(-1, wxString("No icons")));
    library_tree_icons_sizes_vector.push_back(std::make_pair(-1, wxString("No icons")));

      books_list_icons_sizes_vector.push_back(std::make_pair(16, wxString("16")));
    library_tree_icons_sizes_vector.push_back(std::make_pair(16, wxString("16")));

      books_list_icons_sizes_vector.push_back(std::make_pair(20, wxString("20")));
    library_tree_icons_sizes_vector.push_back(std::make_pair(20, wxString("20")));

      books_list_icons_sizes_vector.push_back(std::make_pair(22, wxString("22")));
    library_tree_icons_sizes_vector.push_back(std::make_pair(22, wxString("22")));

      books_list_icons_sizes_vector.push_back(std::make_pair(32, wxString("32")));
    library_tree_icons_sizes_vector.push_back(std::make_pair(32, wxString("32")));

      books_list_icons_sizes_vector.push_back(std::make_pair(48, wxString("48")));
    library_tree_icons_sizes_vector.push_back(std::make_pair(48, wxString("46")));

    bool is_books_icons_custom=true;
    bool is_tree_icons_custom=true;
    for(unsigned int i=0;i<books_list_icons_sizes_vector.size();i++)
        {
        books_list_icons_size_input->Append(books_list_icons_sizes_vector[i].second);
        if(books_list_icons_sizes_vector[i].first==books_list_icons_size)
            {
            books_list_icons_size_input->SetSelection (i);
            is_books_icons_custom=false;
            }
        }
    for(unsigned int i=0;i<library_tree_icons_sizes_vector.size();i++)
        {
        library_tree_icons_size_input->Append(library_tree_icons_sizes_vector[i].second);
        if(library_tree_icons_sizes_vector[i].first==library_tree_icons_size)
            {
            library_tree_icons_size_input->SetSelection (i);
            is_tree_icons_custom=false;
            }
        }
    if(library_tree_icons_size>0)
        {
        library_tree_icons_size_input_custom->SetValue(library_tree_icons_size);
        }
    if(books_list_icons_size>0)
        {
        books_list_icons_size_input_custom->SetValue(books_list_icons_size);
        }
    if(is_tree_icons_custom)
        {
        allow_tree_custom->SetValue(true);
        library_tree_icons_size_input_custom->Enable();
        library_tree_icons_size_input->Disable();
        }
    if(is_books_icons_custom)
        {
        allow_books_custom->SetValue(true);
        books_list_icons_size_input_custom->Enable();
        books_list_icons_size_input->Disable();
        }









    // ######################cancel, apply, ok buttons"###########################
    wxButton *cancel_button = new wxButton(this,wxID_ANY, "Cancel");
    apply_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_APPLY_CHANGES, "Apply");
    apply_button->Disable();
    wxButton *ok_button = new wxButton(this,Khz::my_wx_id__CONFIGURE_GUI_OK_CHANGES, "Ok");
    //#############################################################################################


    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   SIZING !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    int left_right_border=5;
    only_sizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *three_last_buttons = new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *fields=new wxBoxSizer(wxHORIZONTAL);
    wxBoxSizer *tree=new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *books=new wxBoxSizer(wxVERTICAL);


        wxStaticText *descr1 = new wxStaticText( this, wxID_ANY,  wxT("Library tree icons"));
        wxFont font = descr1->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        descr1->SetFont(font);
        wxStaticText *descr2 = new wxStaticText( this, wxID_ANY,  wxT("Books list icons"));
        font = descr2->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        descr2->SetFont(font);

        tree->Add(descr1, 0, wxEXPAND|wxALL);
        tree->AddSpacer(20);
        tree->Add(new wxStaticText( this, wxID_ANY,  wxT("icons size")) , 0, wxALL);
        tree->Add(library_tree_icons_size_input, 0, wxEXPAND|wxALL);
        tree->AddSpacer(10);
        tree->Add(allow_tree_custom, 0, wxEXPAND|wxALL);
        tree->Add(library_tree_icons_size_input_custom, 0, wxEXPAND|wxALL);

        books->Add(descr2, 0, wxEXPAND|wxALL);
        books->AddSpacer(20);
        books->Add(new wxStaticText( this, wxID_ANY,  wxT("icons size")) , 0, wxALL);
        books->Add(books_list_icons_size_input, 0, wxEXPAND|wxALL);
        books->AddSpacer(10);
        books->Add(allow_books_custom, 0, wxEXPAND|wxALL);
        books->Add(books_list_icons_size_input_custom, 0, wxEXPAND|wxALL);

        fields->Add(tree, 1, wxEXPAND|wxALL);
        fields->AddSpacer(80);
        fields->Add(books, 1, wxEXPAND|wxALL);

        //#################### buttons ##########################################################
        three_last_buttons->AddStretchSpacer ();
        three_last_buttons->Add(cancel_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(apply_button);
        three_last_buttons->AddSpacer(10);
        three_last_buttons->Add(ok_button);
        three_last_buttons->AddSpacer(left_right_border);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //###############################################################################################################

    only_sizer->Add(fields, 0, wxEXPAND|wxALL,5);
    only_sizer->AddSpacer(40);
    only_sizer->Add(change_icons_info_hyperlink, 0, wxALL|wxALIGN_CENTER, 5);
    only_sizer->AddStretchSpacer();
    only_sizer->Add(three_last_buttons, flag_to_fit_two_last_buttons);

    SetSizerAndFit(only_sizer);

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!! SIZING FINISHED !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    allow_tree_custom->Bind(wxEVT_CHECKBOX, &CfgTabIconsChooser::on_allow_tree_custom_icons_checkbox_changed, this);
    allow_books_custom->Bind(wxEVT_CHECKBOX, &CfgTabIconsChooser::on_allow_books_custom_checkbox_changed, this);
    library_tree_icons_size_input->Bind(wxEVT_CHOICE, &CfgTabIconsChooser::on_library_tree_icons_size_input_changed, this);
    books_list_icons_size_input->Bind(wxEVT_CHOICE, &CfgTabIconsChooser::on_books_list_icons_size_input_changed, this);

    library_tree_icons_size_input_custom->Bind(wxEVT_SPINCTRL, &CfgTabIconsChooser::on_library_tree_icons_size_input_custom_changed1,this);
    books_list_icons_size_input_custom->Bind(wxEVT_SPINCTRL, &CfgTabIconsChooser::on_books_list_icons_size_input_custom_changed1, this);
    library_tree_icons_size_input_custom->Bind(wxEVT_TEXT, &CfgTabIconsChooser::on_library_tree_icons_size_input_custom_changed2, this);
    books_list_icons_size_input_custom->Bind(wxEVT_TEXT, &CfgTabIconsChooser::on_books_list_icons_size_input_custom_changed2, this);
    change_icons_info_hyperlink->Bind(wxEVT_HYPERLINK, &CfgTabIconsChooser::on_change_icons_info_hyperlink_clicked, this);


    apply_button->Bind(wxEVT_BUTTON, &CfgTabIconsChooser::on_apply_pressed, this);
    ok_button->Bind(wxEVT_BUTTON, &CfgTabIconsChooser::on_ok_pressed, this);
    cancel_button->Bind(wxEVT_BUTTON, &CfgTabIconsChooser::on_cancel_pressed, this);
     //apply_button->Bind(wxEVT_BUTTON, &CfgTabFontsChooser::on_apply_pressed, this);
    //ok_button->Bind(wxEVT_BUTTON, &CfgTabFontsChooser::on_ok_pressed, this);
    //cancel_button->Bind(wxEVT_BUTTON, &CfgTabFontsChooser::on_cancel_pressed, this);
    }



void CfgTabIconsChooser::on_library_tree_icons_size_input_custom_changed1(wxSpinEvent& event)
    {
    on_library_tree_icons_size_input_custom_changed();
    }
void CfgTabIconsChooser::on_books_list_icons_size_input_custom_changed1(wxSpinEvent& event)
    {
    on_books_list_icons_size_input_custom_changed();
    }
void CfgTabIconsChooser::on_library_tree_icons_size_input_custom_changed2(wxCommandEvent& event)
    {
    on_library_tree_icons_size_input_custom_changed();
    }
void CfgTabIconsChooser::on_books_list_icons_size_input_custom_changed2(wxCommandEvent& event)
    {
    on_books_list_icons_size_input_custom_changed();
    }

void CfgTabIconsChooser::on_library_tree_icons_size_input_custom_changed()
    {
    int size=library_tree_icons_size_input_custom->GetValue();
    library_tree_icons_size=size;
    this->apply_button->Enable();
    this->IsModified=true;
    }


void CfgTabIconsChooser::on_books_list_icons_size_input_custom_changed()
    {
    int size=books_list_icons_size_input_custom->GetValue();
    books_list_icons_size=size;
    this->apply_button->Enable();
    this->IsModified=true;
    }




void CfgTabIconsChooser::on_allow_tree_custom_icons_checkbox_changed(wxCommandEvent& event)
    {
    if(allow_tree_custom->GetValue())
        {
        const wxChar* NEWLINE_CHAR = wxTextFile::GetEOL();
        wxString messagestr=wxString("The current kolkhoznik release contains only 16x16, 20x20, 22x22, 32x32, 48x48 icons. ")+
                         wxString(" If you want different size than you have two options:") + wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
                         wxT("1 (Recommended): Look into the \"icons\" folder within the kolkhoznik installation directory. ")+
                         wxT("Create new folder with name NEWSIZExNEWSIZE (for example 10x10). Put there your icons similar to the existing folders. ")+
                         wxT("The icons \"folder.png\" and \"book.png\" are necessary as well as the \"icons_for_files_by_extension\" directory. ")+
                         wxT("The \"icons_for_files_by_extension\" directory contains icons for different file types. For example, djvu.png will be used for djvu files. ")+
                         wxT("If icon for some file type is not presented, than the default one (book.png) will be used.")+ wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
                         wxT("2: Do not provide icons for the desired size. In this case kolkhoznik will rescale the existing icons, but")
                         wxT(" such icons will probably LOOK TERRIBLE.");

        wxMessageDialog *message =new wxMessageDialog (this,
                                   messagestr,
                                    wxString("Attention"),
                                     wxOK|wxCENTRE|wxSTAY_ON_TOP|wxCANCEL, wxDefaultPosition) ;

        switch ( message->ShowModal() )
            {
            case wxID_OK:
                {
                library_tree_icons_size_input_custom->Enable();
                library_tree_icons_size_input->Disable();
                break;
                }

            case wxID_CANCEL:
                {
                allow_tree_custom->SetValue(false);
                break;
                }
            }
        }
    else
        {
        library_tree_icons_size_input_custom->Disable();
        library_tree_icons_size_input->Enable();
        int selected_item=library_tree_icons_size_input->GetSelection();
        if(selected_item==wxNOT_FOUND) {selected_item=0;library_tree_icons_size_input->SetSelection(0);}
        if(selected_item!=wxNOT_FOUND)
            {
            library_tree_icons_size=library_tree_icons_sizes_vector[selected_item].first;
            this->apply_button->Enable();
            this->IsModified=true;
            if(library_tree_icons_size>0)
                {
                library_tree_icons_size_input_custom->SetValue(library_tree_icons_size);
                }
            }
        }
    }
void CfgTabIconsChooser::on_allow_books_custom_checkbox_changed(wxCommandEvent& event)
    {
    if(allow_books_custom->GetValue())
        {
        const wxChar* NEWLINE_CHAR = wxTextFile::GetEOL();
        wxString messagestr=wxString("The current kolkhoznik release contains only 16x16, 20x20, 22x22, 32x32, 48x48 icons. ")+
                         wxString(" If you want different size than you have two options:") + wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
                         wxT("1 (Recommended): Look into the \"icons\" folder within the kolkhoznik installation directory. ")+
                         wxT("Create new folder with name NEWSIZExNEWSIZE (for example 10x10). Put there your icons similar to the existing folders. ")+
                         wxT("The icons \"folder.png\" and \"book.png\" are necessary as well as the \"icons_for_files_by_extension\" directory. ")+
                         wxT("The \"icons_for_files_by_extension\" directory contains icons for different file types. For example, djvu.png will be used for djvu files. ")+
                         wxT("If icon for some file type is not presented, than the default one (book.png) will be used.")+ wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
                         wxT("2: Do not provide icons for desired size. In this case kolkhoznik will rescale existing icons, but")
                         wxT(" such icons will probably LOOK TERRIBLE.");

        wxMessageDialog *message =new wxMessageDialog (this,
                                   messagestr,
                                    wxString("Attention"),
                                     wxOK|wxCENTRE|wxSTAY_ON_TOP|wxCANCEL, wxDefaultPosition) ;

        switch ( message->ShowModal() )
            {
            case wxID_OK:
                {
                books_list_icons_size_input_custom->Enable();
                books_list_icons_size_input->Disable();
                break;
                }

            case wxID_CANCEL:
                {
                allow_books_custom->SetValue(false);
                break;
                }
            }
        }
    else
        {
        books_list_icons_size_input_custom->Disable();
        books_list_icons_size_input->Enable();
         int selected_item=books_list_icons_size_input->GetSelection();
        if(selected_item==wxNOT_FOUND) {selected_item=0;library_tree_icons_size_input->SetSelection(0);}
        if(selected_item!=wxNOT_FOUND)
            {
            books_list_icons_size=books_list_icons_sizes_vector[selected_item].first;
            this->apply_button->Enable();
            this->IsModified=true;
            if(books_list_icons_size>0)
                {
                books_list_icons_size_input_custom->SetValue(books_list_icons_size);
                }
            }
        }
    }

void CfgTabIconsChooser::on_library_tree_icons_size_input_changed(wxCommandEvent& event)
    {
    int selected_item=library_tree_icons_size_input->GetSelection();
    if(selected_item!=wxNOT_FOUND)
        {
        library_tree_icons_size=library_tree_icons_sizes_vector[selected_item].first;
        this->apply_button->Enable();
        this->IsModified=true;
        if(library_tree_icons_size>0)
            {
            library_tree_icons_size_input_custom->SetValue(library_tree_icons_size);
            }
        }
    else
        {
        std::cout << "\n something terrible happen in CfgTabIconsChooser.cpp, aborted\n";
        abort();
        }
    }



void CfgTabIconsChooser::on_books_list_icons_size_input_changed(wxCommandEvent& event)
    {
    int selected_item=books_list_icons_size_input->GetSelection();
    if(selected_item!=wxNOT_FOUND)
        {
        books_list_icons_size=books_list_icons_sizes_vector[selected_item].first;
        this->apply_button->Enable();
        this->IsModified=true;
        if(books_list_icons_size>0)
            {
            books_list_icons_size_input_custom->SetValue(books_list_icons_size);
            }
        }
    else
        {
        std::cout << "\n something terrible happen 2 in CfgTabIconsChooser.cpp, aborted\n";
        abort();
        }
    }

void  CfgTabIconsChooser::on_change_icons_info_hyperlink_clicked(wxHyperlinkEvent& event)
    {
     wxString sep=wxString (wxFileName::GetPathSeparator());
    wxString messagestr=wxString("Look into the \"icons\" folder within the kolkhoznik installation directory. ")+
                         wxT("Replace icons that you do not like by your own ones. For example, if you want to change the 16x16 ")+
                         wxT("djvu files icon, than you should replace  ")+"icons"+sep+"16x16"+sep+"icons_for_files_by_extension"+sep+"djvu.png.";

    wxMessageBox (
                                   messagestr,
                                    wxString("Hi there!"),
                                     wxOK|wxCENTRE|wxSTAY_ON_TOP) ;
    }


void CfgTabIconsChooser::write_changes_to_config()
    {
    this->ReconfigureType=Khz::my_wx_id__RECONFIGURE_ICONS;
    delete config;
    config =new wxFileConfig( "", "", config_file);
    config->Write( wxT("ICONS/library_tree_icons_size"), library_tree_icons_size );
    config->Write( wxT("ICONS/books_list_icons_size"), books_list_icons_size );
    config->Flush();


    delete config;
    config =new wxFileConfig( "", "", config_file);
    apply_button->Disable();
    this->IsModified=false;
    }

void CfgTabIconsChooser::on_apply_pressed(wxCommandEvent& event)
    {
    this->write_changes_to_config();
    this->send_reconfigure_panels_event();
    apply_button->Disable();
    IsModified=false;
    }

void CfgTabIconsChooser::on_ok_pressed(wxCommandEvent& event)
    {
    if(IsModified)
        {
        this->write_changes_to_config();
        this->send_reconfigure_panels_event();
        }
    //std::cout << "\nclosing config\n";
    this->send_close_event_manually();
    }
void CfgTabIconsChooser::on_cancel_pressed(wxCommandEvent& event)
    {
    this->send_close_event_manually();
    }

void CfgTabIconsChooser::restore_values()
    {
    books_list_icons_size_input_custom->Disable();
    library_tree_icons_size_input_custom->Disable();

    wxString libr_tree_icons_string=config->Read( wxT("ICONS/library_tree_icons_size"), "16" );
    wxString books_list_icons_string=config->Read( wxT("ICONS/books_list_icons_size"), "16" );
    if(libr_tree_icons_string!="")
        {
        library_tree_icons_size=wxAtoi(libr_tree_icons_string);
        }
    else
        {
        library_tree_icons_size=16;
        }

    if(books_list_icons_string!="")
        {
        books_list_icons_size=wxAtoi(books_list_icons_string);
        }
    else
        {
        books_list_icons_size=16;
        }
    bool is_books_icons_custom=true;
    bool is_tree_icons_custom=true;
    for(unsigned int i=0;i<books_list_icons_sizes_vector.size();i++)
        {
        if(books_list_icons_sizes_vector[i].first==books_list_icons_size)
            {
            books_list_icons_size_input->SetSelection (i);
            is_books_icons_custom=false;
            }
        }
    for(unsigned int i=0;i<library_tree_icons_sizes_vector.size();i++)
        {
        if(library_tree_icons_sizes_vector[i].first==library_tree_icons_size)
            {
            library_tree_icons_size_input->SetSelection (i);
            is_tree_icons_custom=false;
            }
        }
    if(library_tree_icons_size>0)
        {
        library_tree_icons_size_input_custom->SetValue(library_tree_icons_size);
        }
    if(books_list_icons_size>0)
        {
        books_list_icons_size_input_custom->SetValue(books_list_icons_size);
        }
    if(is_tree_icons_custom)
        {
        allow_tree_custom->SetValue(true);
        library_tree_icons_size_input_custom->Enable();
        library_tree_icons_size_input->Disable();
        }
    if(is_books_icons_custom)
        {
        allow_books_custom->SetValue(true);
        books_list_icons_size_input_custom->Enable();
        books_list_icons_size_input->Disable();
        }
	this->IsModified=false;
	this->apply_button->Disable();
    }



}}// namespaces
