/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/panel.h>

namespace Khz{namespace CfgWin{


/**
@brief "Paths" tab of the "General"

"Paths" tab of the "General".

General idea: we have three inputs:
@code
    wxTextCtrl *library_dir_input;
    wxTextCtrl *where_to_store_database_input;
    wxTextCtrl *xapian_dir_input;

@endcode
User can changes values in the inputs.
When user presses "Apply" button, we read values from the inputs and write them to the config file. Also, we send special event
that makes MyMainWindow to reload MyBooksList and TreeCtrlLibarary to apply changes (we use Configure_gui_tabs_parent::send_reconfigure_panels_event for it).
**/
class CfgTabPathsChooser : public Configure_gui_tabs_parent
{
public:
    /**
    @param parent standard wxwidgets parent
    @param path_to_config full path to the config file
    **/
    CfgTabPathsChooser(wxWindow* parent, wxString path_to_config);


    /**
     panel that is parent for all of the inputs. We need this panel
     in order to be able call this->GetSizer->Fit(   dummy_panel).

     This will Allow us to repaint the whole tab after the Configuration
     window is created; For some reason on gtk-3 there is some problem
     with sizer, and this tab stays not fitted (for some reason all other cCfgTab's are goood).
     In order to fix this we call this->GetSizer->Fit(   dummy_panel) after the Main configuration
     window is created, and this solves the problem.

    **/
    wxPanel *dummy_panel;



private:

    /**
    When user modifies text in the #library_dir_input, this variable becomes TRUE.
    @note
    We need #is_path_entry_modified, #is_database_entry_modified,
    is_xapian_entry_modified to set the #Khz::CfgWin::Configure_gui_tabs_parent::ReconfigureType variable.
    This variable is going to be used by the Configure_gui_tabs_parent::send_reconfigure_panels_event function.
    **/
    bool is_path_entry_modified;

    /**
    When user modifies text in the #where_to_store_database_input, this variable becomes TRUE.
    See #is_path_entry_modified for details.
    **/
    bool is_database_entry_modified;

    /**
    When user modifies text in #xapian_dir_input, this var becomes true.
    See #is_path_entry_modified for details.
    **/
    bool is_xapian_entry_modified;

    /**
    Input for the path to the database
    **/
    wxTextCtrl *where_to_store_database_input;

    /**
    Input for the full text search database.
    **/
    wxTextCtrl *xapian_dir_input;

    /**
    Main window sizer
    **/
    wxBoxSizer    *only_sizer;

    /**
    Apply button
    **/
    wxButton *apply_button;


     /**
    Input for the path to the library
    **/
    wxTextCtrl *library_dir_input;



    /**
    Reads current values of the #library_dir_input, #where_to_store_database_input inputs and writes them to the config file.
    **/
    void write_changes_to_config();





    /**
    Executes when user changes something in the path to the library entry.
    Enables apply button, sets #is_path_entry_modified to TRUE and so on.
    **/
    void OnKolkhozDirInputChanged(wxCommandEvent& event);


    /**
    Executes when user changes something in the path to the database entry.
    Enables apply button, sets #is_database_entry_modified and so on.
    **/
    void OnDatabasePathChanged(wxCommandEvent& event);

    /**
    Executes when user changes something in the path to the full text search database input.
    Enables apply button, sets #is_xapian_entry_modified and so on.
    **/
    void OnXapianPathChanged(wxCommandEvent& event);


    /**
    Executes when user presses "Choose" button near the #library_dir_input. Opens popup window
    that allows to select directory.
    **/
    void DoKolhozPathChoose(wxCommandEvent& event);


    /**
    Executes when user presses "Choose" button near the #where_to_store_database_input. Opens popup window
    that allows to select file.
    **/
    void DoKolhozDatabasePathChoose(wxCommandEvent &event);


    /**
    Executes when user presses "Choose" button near the #xapian_dir_input. Openss popup window
    that allows to select directory.
    **/
    void DoXapianDatabasePathChoose(wxCommandEvent &event);



    /**
    Executes when user presses "Apply" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event ()
    **/
    void on_apply_pressed(wxCommandEvent& event);

    /**
    Executes when user presses "Ok" button. Calls #write_changes_to_config (),
    Khz::CfgWin::Configure_gui_tabs_parent::send_reconfigure_panels_event (),
    closes the configure gui window.
    **/
    void on_ok_pressed(wxCommandEvent& event);


    /**
    Closes the configuration window
    **/
    void on_cancel_pressed(wxCommandEvent& event);

    /**
    See Configure_gui_tabs_parent::restore_values
    **/
    void restore_values();
};


}}// end of namespaces
