/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/frame.h>
#include <wx/imaglist.h>
#include <wx/listbook.h>

#include "ConfigureGuiSubwindow_main_window.h"
#include "ConfigureGuiSubwindow_general.h"
#include "ConfigureGuiSubwindow_full_text_search_window.h"
#include "ConfigureGuiSubwindow_full_text_search_indexer_params.h"

namespace Khz{namespace CfgWin{



/**
@brief The top level class of the configuration window
**/
class ConfigureGui: public wxFrame
{
public:

    /**
    constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file
    **/
    ConfigureGui(wxWindow *parent, wxString path_to_config);
    //~ConfigureGui()
    //   {
        //delete images_list;
        //we do not need this as we have in the constructor:
        //main_treebook->AssignImageList(images_list);
        // So, main_treebook will delete images_list;
    //    }


     ///////////// Creating subwindows////////////////////////////////////////////////
    ConfigureGuiSubwindow_full_text_search_window *fts_config_window;

    ConfigureGuiSubwindow_main_window *main_config_window;


    ConfigureGuiSubwindow_general *general_config_window;

    ConfigureGuiSubwindow_full_text_search_indexer_params *fts_indexer_params;

    /*void PrintSomeInfo()
        {

        std::cout << "\n########pesdec blyat###########\n";
        std::cout<< "\ncfgwin client size" << GetClientSize().GetWidth() << " " << GetClientSize().GetHeight() << "\n";
        //std::cout << "\ncfgwin sizer min size" << this->GetSizer()->GetMinSize().GetWidth() << " " << this->GetSizer()->GetMinSize().GetHeight() << "\n";
        //std::cout << "\ncfgwin sizer size" << this->GetSizer()->GetSize().GetWidth() << " " << this->GetSizer()->GetSize().GetHeight() << "\n";



        std::cout << "\nmain_treebok_size " << main_treebook->GetSize().GetWidth() << " "<< main_treebook->GetSize().GetHeight()  << "\n";
        std::cout << "\nmain_treebok_minsize " << main_treebook->GetMinSize().GetWidth() << " "<< main_treebook->GetMinSize().GetHeight()  << "\n";


        //std::cout << "\n general_config_window client size" << general_config_window->GetClientSize().GetWidth() << " " << general_config_window->GetClientSize().GetHeight() << "\n";
        //general_config_window->Layout();
        //std::cout << "\n general_config_windownotebook client size" << general_config_window->GetNotebookClientSize().GetWidth() << " " << general_config_window->GetNotebookClientSize().GetHeight() << "\n";

        //std::cout << "\nblahblah\n";

        main_config_window->PrintSizes();
        general_config_window->PrintSizes();
        fts_indexer_params->PrintSizes();
        fts_config_window->PrintSizes();

        std::cout << "\n #end of pisdect###############\n";
        }
    void RefitManually()
        {
        std::cout << "\n\n ##############################################\n";
        std::cout << "REFITMANUALLY BEGIN";
        std::cout << "\n\n ##############################################\n";
        int newx=0;
        int newy=0;
        Layout();
        Refresh();
        Update();


        wxSize temp_size=fts_config_window->GetClientSize() ;
        newx=std::max(newx, temp_size.GetWidth());
        newy=std::max(newy, temp_size.GetHeight());

        temp_size=main_config_window->GetClientSize();
        newx=std::max(newx, temp_size.GetWidth());
        newy=std::max(newy, temp_size.GetHeight());

        temp_size=general_config_window->GetClientSize();
        newx=std::max(newx,temp_size.GetWidth());
        newy=std::max(newy,temp_size.GetHeight());

        temp_size=fts_indexer_params->GetClientSize();
        newx=std::max(newx, temp_size.GetWidth());
        newy=std::max(newy, temp_size.GetHeight());

        std::cout << "\n newx" << newx << " newy " << newy << "\n";

        main_config_window->SetAllTabsSizersMinSizeAndFitAgain(newx,newy);
        general_config_window->SetAllTabsSizersMinSizeAndFitAgain(newx,newy);
        fts_indexer_params->SetAllTabsSizersMinSizeAndFitAgain(newx,newy);
        fts_config_window->SetAllTabsSizersMinSizeAndFitAgain(newx,newy);
        //Layout();
        //general_config_window->Layout();
        //general_config_window->LayoutAllTAbs();

        //Fit();

        main_config_window->PrintSizes();
        general_config_window->PrintSizes();
        fts_indexer_params->PrintSizes();
        fts_config_window->PrintSizes();
        std::cout << "\n\n ##############################################\n";
        std::cout << "REFITMANUALLY END";
        std::cout << "\n\n ##############################################\n";
        }
        */


private:
    /**
    wxWidgets standard images list associated with the control
    **/
    wxImageList *images_list;


     /**
    The wxListbook that allows user to switch between "subwindows" (see detailed description of #Khz::CfgWin ).
    This is the left vertical panel with icons where user can switch between subwindows.
    **/
    wxListbook *main_treebook;


    /**
    Catches events sended by #Khz::CfgWin::Configure_gui_tabs_parent::send_close_event_manually().
    Closes the configuration window.
    **/
    void on_close_event(wxCommandEvent &event);


    /**
    Executes when user switch "subwindow" (see detailed description of #Khz::CfgWin )
    Asks user to save changes if necessary
    **/
    void OnTabChanging(wxBookCtrlEvent& event);


};


}}// namespaces
