/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KOLKHOZNIK_CONFIGUREGUITABSPARENT_HEADER_INCLUDED
#define KOLKHOZNIK_CONFIGUREGUITABSPARENT_HEADER_INCLUDED


#include <wx/fileconf.h>
#include <wx/window.h>


namespace Khz{namespace CfgWin{


/**
@brief Parent class for all of the tabs of the configure gui window (childs are CfgTabAdvanced, CfgTabDefaultApps and so on).

Parent class for all of the tabs of the configure gui window (childs are CfgTabAdvanced, CfgTabDefaultApps and so on)
**/
class Configure_gui_tabs_parent : public wxWindow
{
public:
    /**
    Constructor
    @param parent standard wxwidgets parent
    @param path_to_config full path to the main config file.
    **/
    Configure_gui_tabs_parent(wxWindow* parent, wxString path_to_config);//constructor
    //this is the parent of all of the configure pages.
    virtual ~Configure_gui_tabs_parent()
        {
        delete config;
        }

    //#############################3##############################################################
    /**
    Returns #ReconfigureType
    **/
    inline int GetReconfigureType(){return ReconfigureType;};
    //###########################################################################################

    //###################################################################################################
    /**
    If #IsModified=true, than #Khz::CfgWin::ConfigureGui::OnTabChanging will show popup message
    asking user to save or discard changes when user tries to change tab.
    **/
    bool IsModified;
    //###################################################################################################

    /**
    Each child must define how to write changes to the config file.
    **/
    virtual void write_changes_to_config()=0;

    //#####################################################################################################################
    /**
    The event is going to be caught by MyMainWindow (MyMainWindow::ReconfigurePanels will be executed).
    MyMainWindow will reconfigure panels (reconnect to database, reload cache and so on).
    So, user does not need to restart the application when changing config parameters.
    See also #ReconfigureType
    **/
    void send_reconfigure_panels_event();
    //######################################################################################################################



    //##########################################################################################################
    /**
    When user changes tab without saving modifications, the message will appear asking user
        if he wants to save changes. If he does not, than we get the original values from the config file and
        replace the modified ones (restore original values) for the curren configuration tab.
        That's what this function is for.
    **/
    virtual void restore_values()=0;
    //##########################################################################################################
protected:

    /**
    Config associated with the main config file.
    **/
    wxFileConfig *config;

    /**
    Full path to the main config file
    **/
    wxString config_file;

    /**
    Pure virtual, must be implemented by a child.
    **/
    virtual void on_apply_pressed(wxCommandEvent& event)=0;

    /**
    Pure virtual, must be implemented by a child.
    **/
    virtual void on_ok_pressed(wxCommandEvent& event)=0;

    /**
    Pure virtual, must be implemented by a child.
    **/
    virtual void on_cancel_pressed(wxCommandEvent& event)=0;


    //#######################################################################################
    /**
    Closes the configuration window. The event is going to be cought by the #Khz::CfgWin::ConfigureGui class object.
    ConfigureGui will close the window.
    **/
    void send_close_event_manually();
    //#########################################################################################


    //##################################################################################################
    /**defines type of panels reconfiguration(complete, including database change; only appearance
    without database changing). Types are ints, see Custom_IDs.h

    send_reconfigure_panels_event() uses the #ReconfigureType variable when sending the reconfigure event.
    More precisely, Int (event->GetInt()) assoicated with the event is going to be equal to the #ReconfigureType
    **/
    int ReconfigureType;
    //##################################################################################################

};

}}// namespaces


#endif
