/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/






#include "FullTextSearchIndexerAddEditExtensionDialog.h"
#include <wx/button.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include "MySizeFunctions.h"
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/fileconf.h>
#include "ConfigManager.h"
#include "OpenTheBook.h"
#include <wx/filefn.h>
#include <wx/dir.h>
#include <fstream>
#include <wx/log.h>


namespace Khz{namespace CfgWin{


FullTextSearchIndexerAddEditExtensionDialog::FullTextSearchIndexerAddEditExtensionDialog
            (
            wxWindow *parent,
            wxWindowID id,
            wxString title,
            int edit_or_add,
            int text_or_archive_extractor,
            long style
            )
    :wxDialog(parent, id, title, wxDefaultPosition, wxDefaultSize, style)
    {
    this->action=edit_or_add;
    this->mode=text_or_archive_extractor;

    extension_input=new wxTextCtrl(this, wxID_ANY);
    extractor_input=new wxTextCtrl(this ,wxID_ANY);
    sample_file_input=new wxTextCtrl(this, wxID_ANY);

    choose_sample_button = new wxButton(this, wxID_ANY, "Choose");
    open_file_button=new wxButton(this, wxID_ANY, "Open for preview");




    if(mode==FTS_TEXTEXTRACTOR)
        {
        md5_or_substring_input = new wxTextCtrl (this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxDefaultSize,wxTE_MULTILINE);
        md5_or_substring_input->SetMinSize
            (
            wxSize(-1, Khz::Libs::MySizeFunctions::get_height_of_number_of_text_rows(10, md5_or_substring_input))
            );
        }
    else
        {
        md5_or_substring_input = new wxTextCtrl (this,wxID_ANY,wxEmptyString,wxDefaultPosition,wxDefaultSize);
        }
    if(edit_or_add==FTS_ADD_EXTENSION)
        {
        add_sample_checkbox=new wxCheckBox(this, wxID_ANY, "Add Sample file");
        add_sample_checkbox->SetValue(true);
        }
    else //edit
        {
        add_sample_checkbox=new wxCheckBox(this, wxID_ANY, "Add Sample file");
        add_sample_checkbox->SetValue(false);
        sample_file_input->Disable();
        choose_sample_button->Disable();
        open_file_button->Disable();
        md5_or_substring_input->Disable();
        }
    wxHyperlinkCtrl *what_is_sample_hyperlink =
            new wxHyperlinkCtrl (this, wxID_ANY, "What is sample?", "");


    wxButton *cancel_button = new wxButton(this, wxID_CANCEL, "Cancel");
    wxButton *ok_button = new wxButton (this, wxID_OK, "Ok");

    wxBoxSizer *only_sizer = new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *inputs_sizer=new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *extension_sizer = new wxBoxSizer(wxVERTICAL);
            extension_sizer->Add(new wxStaticText(this, wxID_ANY, "Extension             "));
            extension_sizer->Add(extension_input, 0, wxALL|wxEXPAND);
        inputs_sizer->Add(extension_sizer, 0, wxALL|wxALIGN_BOTTOM);
        inputs_sizer->AddSpacer(5);
            wxBoxSizer *extractor_sizer=new wxBoxSizer(wxVERTICAL);
            ////////////////////////////////////////////////////////////////////////////////////////
            wxString command_str_tmp= (mode==FTS_TEXTEXTRACTOR) ?
                "Command (%f will be replaced by a filename, executable starts with %e, like %edjvutxt)\n"
                "The command must write extracted text to the standard output (UTF8 encoded!!!!!!!)"
                : "Command (%f will be raplace by the archive filename; executable starts with %e, \n"
                  "like %e7z. %o represents the output directory)";
            /////////////////////////////////////////////////////////////////////////////////////////
            extractor_sizer->Add(new wxStaticText(this, wxID_ANY, command_str_tmp));
            extractor_sizer->Add(extractor_input, 1, wxALL|wxEXPAND);
        inputs_sizer->Add(extractor_sizer, 1, wxALL|wxEXPAND|wxALIGN_BOTTOM);
        //#################### buttons ##########################################################
        wxBoxSizer *two_last_buttons_sizer = new wxBoxSizer(wxHORIZONTAL);
        two_last_buttons_sizer->AddStretchSpacer ();
        two_last_buttons_sizer->Add(cancel_button);
        two_last_buttons_sizer->AddSpacer(10);
        two_last_buttons_sizer->Add(ok_button);
        two_last_buttons_sizer->AddSpacer(5);
        wxSizerFlags flag_to_fit_two_last_buttons(0);
        flag_to_fit_two_last_buttons.Expand();
        flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
        //########################################################################################
    only_sizer->Add(inputs_sizer, 0, wxALL|wxEXPAND, 5);
    only_sizer->AddSpacer(35);
        wxBoxSizer *sample_checkbox_sizer = new wxBoxSizer(wxHORIZONTAL);
        sample_checkbox_sizer->Add(add_sample_checkbox, 0, wxALIGN_BOTTOM);
        sample_checkbox_sizer->AddSpacer(10);
        sample_checkbox_sizer->Add(what_is_sample_hyperlink, 0, wxALIGN_BOTTOM);
    only_sizer->Add(sample_checkbox_sizer);
        wxBoxSizer *sample_sizer_vertical=new wxBoxSizer(wxVERTICAL);
        ///////////////////////////////////////////////////////////////////////////////////////
        sample_sizer_vertical->Add((new wxStaticText(this, wxID_ANY,
                    (mode==FTS_TEXTEXTRACTOR) ?
                    "Select a sample file with the specified extension.\n"
                    "The sample will be copied to path_to_dir_with_kolkhoznik_exe/full_text_search/test_extractors\n"
                    "One page document would be good enough."
                    :
                    "Select a sample archive with the specified extension.\n"
                    "The sample archive must contain only one file.\n"
                    "The archive will be copied to path_to_dir_with_kolkhoznik_exe/full_text_search/test_extractors"
                                )));
            wxBoxSizer *sample_sizer = new wxBoxSizer(wxHORIZONTAL);
            sample_sizer->Add(sample_file_input,1, wxALL|wxALIGN_CENTRE);
            sample_sizer->Add(choose_sample_button);
            sample_sizer->AddSpacer(5);
            sample_sizer->Add(open_file_button);
        sample_sizer_vertical->Add(sample_sizer,0, wxEXPAND|wxALL);
    only_sizer->Add(sample_sizer_vertical, 0, wxEXPAND|wxALL, 5);
    only_sizer->AddSpacer(10);
        wxBoxSizer *expected_output_sizer=new wxBoxSizer(wxVERTICAL);
        ///////////////////////////////////////////////////////////////////////////////////////////////
         wxString expected_res_str_tmp= (mode==FTS_TEXTEXTRACTOR) ?
                "Provide substring that is part of the text layer of the specified sample.\n"
                "P.S. we need only substing, not the whole text. About 5 lines of the text\n"
                "would be enough. Be sure that substring does not contain hyphen sumbols.\n"
                "Some extractors (for example pdftotext) remove them."
                :
                "md5 of the file in the sample archive.";
        //////////////////////////////////////////////////////////////////////////////////////////////
        expected_output_sizer->Add(new wxStaticText(this, wxID_ANY, expected_res_str_tmp));
        expected_output_sizer->Add(md5_or_substring_input, 1, wxALL|wxEXPAND);
    only_sizer->Add(expected_output_sizer,(mode==FTS_TEXTEXTRACTOR) ? 1 : 0  , wxALL|wxEXPAND,5);
    if (mode!=FTS_TEXTEXTRACTOR)only_sizer->AddStretchSpacer();
    only_sizer->Add(two_last_buttons_sizer, flag_to_fit_two_last_buttons);
    SetSizerAndFit(only_sizer);


    add_sample_checkbox->Bind(wxEVT_CHECKBOX, &FullTextSearchIndexerAddEditExtensionDialog::OnAddSampleCheckBoxClicked, this);
    what_is_sample_hyperlink->Bind(wxEVT_HYPERLINK, &FullTextSearchIndexerAddEditExtensionDialog::OnAWhatIsSampleHyperlinkClicked, this);
    choose_sample_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerAddEditExtensionDialog::OnChooseSampleButtonPressed, this);
    open_file_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerAddEditExtensionDialog::OnOpenSampleForPreviewButtonPressed, this);
    ok_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerAddEditExtensionDialog::OnOkButtonPressed, this);
    }

void FullTextSearchIndexerAddEditExtensionDialog::OnAddSampleCheckBoxClicked(wxCommandEvent &event)
    {
    if(this->action==FTS_ADD_EXTENSION)
        {
        add_sample_checkbox->SetValue(true);
        }
    else
        {
        if(add_sample_checkbox->GetValue())
            {
            wxString message="You are editing the existing extension. It means that the sample file"
                             " is already presented. If you are sure that you want to overwrite the"
                             " existing sample (if it exists, of course), than press \"Ok\". Otherwise press \"Cancel\"";
            wxMessageDialog dlg(this,message, "Hi there!", wxOK|wxCANCEL|wxCENTRE|wxICON_QUESTION|wxSTAY_ON_TOP);
            switch ( dlg.ShowModal() )
                {
                case wxID_OK:
                    {
                    break;
                    };
                case wxID_CANCEL:
                    {
                    add_sample_checkbox->SetValue(false);
                    return;
                    };
                default:
                wxLogError(wxT("Unexpected wxMessageDialog return code in path_to_database!"));
            }
            sample_file_input->Enable();
            choose_sample_button->Enable();
            open_file_button->Enable();
            md5_or_substring_input->Enable();
            }
        else
            {
            sample_file_input->Disable();
            choose_sample_button->Disable();
            open_file_button->Disable();
            md5_or_substring_input->Disable();
            }
        }
    }

void FullTextSearchIndexerAddEditExtensionDialog::OnAWhatIsSampleHyperlinkClicked(wxHyperlinkEvent& event)
    {
    wxString message;
    if(mode==FTS_TEXTEXTRACTOR)
        {
        message="Before kolkhoznik starts to index files, it checks if the specified command is working properly. "
                "To do this kolkhoznik performs the following procedure: "
                "first, it extracts the text layer from the specified sample. "
                "Next, it searches for the specified substring within the extracted text. "
                "If kolkhoznik finds the specified substring within the extracted text, than the "
                "extractor is supposed to be working. \n\n"
                "P.S. kolkhoznik starts indexing only if all of the extractors are working.";
        }
    else
        {
        message="Before kolkhoznik starts to index files, it checks if the specified command is working properly. "
                "To do this kolkhoznik performs the following procedure: "
                "first, it extracts the file from the specified sample archive. "
                "Next, it calculates the md5 of the extracted from the archive file. "
                "Finally, kolkhoznik compares the calculated md5 with the specified by the user value. "
                "If md5 sums are the same, than the "
                "extractor is supposed to be working. \n\n"
                "P.S. kolkhoznik starts indexing only if all of the extractors are working.";
        }
    wxMessageBox(message, "Hi there");
    }
void FullTextSearchIndexerAddEditExtensionDialog::OnChooseSampleButtonPressed(wxCommandEvent &event)
    {
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxFileDialog dialog(this, wxT("Choose sample file"), dirHome);

    if (dialog.ShowModal() == wxID_OK)
        {
        sample_file_input->SetValue (dialog.GetPath());
        }
    }


void FullTextSearchIndexerAddEditExtensionDialog::OnOpenSampleForPreviewButtonPressed(wxCommandEvent &event)
    {
    wxString path_to_sample= sample_file_input->GetValue();
    wxFileName sample_filename= wxFileName(path_to_sample);
    wxString extension=sample_filename.GetExt();



    if(  !wxFileExists(path_to_sample)  or   !wxFileName::IsFileReadable(path_to_sample)  )
        {
        wxString Message ="Cannot read file \"" + path_to_sample +"\". Check that there is such file and you have permission to read it."+
                            +wxTextFile::GetEOL()+wxTextFile::GetEOL();
         wxMessageBox(Message, wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }



    wxFileConfig *config = new wxFileConfig( "", "",Khz::ConfigManager::GetPathToConfigFile());
    Khz::Libs::OpentheBook(path_to_sample, extension, config );
    delete config;
    }
void FullTextSearchIndexerAddEditExtensionDialog::OnOkButtonPressed(wxCommandEvent &event)
    {
    wxString extension=extension_input->GetValue().Trim(true).Trim(false);
    wxString command= extractor_input->GetValue().Trim(true).Trim(false);
    wxString sample=sample_file_input->GetValue();

    ///////////////////////////////////////////////////////////////////////////////////////////
    //              checking that inputs are filled right.
    //////////////////////////////////////////////////////////////////////////////////////////
    if(extension=="" or command=="")
        {
        wxString message= "Error. Either extension or command is empty!!!";
        wxMessageBox(message, "Error", wxICON_ERROR);
        return;
        }
    if(!wxFileExists(sample) and add_sample_checkbox->GetValue() )
        {
        wxString message= "Error. The sample file \""+sample+"\" does not exist";
        wxMessageBox(message, "Error", wxICON_ERROR);
        return;
        }
    if(add_sample_checkbox->GetValue() and md5_or_substring_input->GetValue().Length()<20 and mode==FTS_TEXTEXTRACTOR)
        {
         wxString message= "Error. The sample substring is too short";
        wxMessageBox(message, "Error", wxICON_ERROR);
        return;
        }
    if(add_sample_checkbox->GetValue() and md5_or_substring_input->GetValue().Length()<3 and mode==FTS_ARCHIVEEXTRACTOR)
        {
        wxString message= "Error. Please provide valid md5";
        wxMessageBox(message, "Error", wxICON_ERROR);
        return;
        }
    //////////////////////////////////////////////////////////////////////////////////////////
    //              finished with checking inputs
    ///////////////////////////////////////////////////////////////////////////////////////////
    if(!add_sample_checkbox->GetValue())
        {
        event.Skip();
        return;
        }
    else if(!save_sample(sample, extension))
        {
        return;
        }
    else
        {
        event.Skip();
        }
    }
bool FullTextSearchIndexerAddEditExtensionDialog::save_sample(wxString path_to_sample, wxString extension)
    {
    wxDir current_dir(wxGetCwd());
    wxString destination_sample_dir=
        current_dir.GetNameWithSep()+"full_text_search"+wxFileName::GetPathSeparator()+"test_extractors"+
        wxFileName::GetPathSeparator() + extension;

    //////////////////////////////////////////////////////////////////////////////////////////////
    //try to create dir for sample if not exists
    if(!wxDirExists(destination_sample_dir) or !wxFileName::IsDirWritable(destination_sample_dir))
        {
        wxFileName::Mkdir(destination_sample_dir, 0777,  wxPATH_MKDIR_FULL);
        //steel no dir(((
        if(!wxDirExists(destination_sample_dir) or !wxFileName::IsDirWritable(destination_sample_dir))
            {
            wxString message= "Error. Cannot create/write into \""+destination_sample_dir+"\".";
            wxMessageBox(message, "Error", wxICON_ERROR);
            return false;
            }
        }
    //////////////////////////////////////////////////////////////////////////////////////////////



    /////////////////////////////////////////////////////////////////////////////////////////////
    /////////////  try to copy sample file to the appropriate dir
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    wxString destination_file=destination_sample_dir+wxFileName::GetPathSeparator()+ "test."+extension;
    if(wxFileExists(destination_file) and path_to_sample!=destination_file)
        {
        wxRemoveFile(destination_file);
        }
    if(!wxCopyFile
            (
            path_to_sample,
            destination_file
            )
      )
        {
        wxString message= "Error.  cannot copy \""+path_to_sample+"\" to \""+
             destination_file +"\".";
            wxMessageBox(message, "Error", wxICON_ERROR);
            return false;
        }



    ///////////////////////////////////////////////////////////////////////////////////////////////////
    ///////////    try to create file with expected output
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    std::string output_filename = (mode==FTS_TEXTEXTRACTOR) ? "substring.txt" : "md5.txt";
    std::string full_path_to_output_file=
                        destination_sample_dir.ToStdString() +
                        wxString(wxFileName::GetPathSeparator()).ToStdString() +
                        output_filename ;

    std::ofstream output_file;
    output_file.open(full_path_to_output_file.c_str(), std::ios::out | std::ios::trunc);
        if(!output_file)
            {
            wxString message= wxString("Cannot open \"")+full_path_to_output_file.c_str()+"\".";

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
            }
    output_file << md5_or_substring_input->GetValue().ToUTF8().data();
    output_file.close();
    return true;
    }
void FullTextSearchIndexerAddEditExtensionDialog::fill_samples_fields_if_necessary(wxString extension)
    {
    wxDir current_dir(wxGetCwd());
    wxString sample_dir=
        current_dir.GetNameWithSep()+"full_text_search"+wxFileName::GetPathSeparator()+"test_extractors"+
        wxFileName::GetPathSeparator() + extension;

    wxString destination_file=sample_dir+wxFileName::GetPathSeparator()+ "test."+extension;
    if(wxFileExists(destination_file))
        {
        sample_file_input->SetValue(destination_file);
        }



    ///////////////////////////////////////////////////////////////////////////////////////////////////
    ///////////
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    std::string temp_filename = (mode==FTS_TEXTEXTRACTOR) ? "substring.txt" : "md5.txt";
    std::string full_path_to_sample_content=
                        sample_dir.ToStdString() +
                        wxString(wxFileName::GetPathSeparator()).ToStdString() +
                        temp_filename ;

    std::ifstream data_file;
    data_file.open(full_path_to_sample_content.c_str());
        if(!data_file)
            {
            return;
            }
    std::string file_string;
    std::string output="";
    while(std::getline(data_file, file_string))
        {
        output=output+file_string;
        }
    data_file.close();
    md5_or_substring_input->SetValue(wxString::FromUTF8(output.c_str()));
    }


}}// namespaces
