/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <wx/dialog.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/hyperlink.h>

namespace Khz{namespace CfgWin{

enum some_enum
        {
        FTS_EDIT_EXTENSION,
        FTS_ADD_EXTENSION,
        FTS_TEXTEXTRACTOR,
        FTS_ARCHIVEEXTRACTOR
        };


/**
@brief Add/edit extension dialog for the "Full text search indexer params" subwindow.
Appear when user presses Add/edit extension button
**/
class FullTextSearchIndexerAddEditExtensionDialog : public wxDialog
{
public:
    /**
    constructor
    @param parent standard wxWidgets parent
    @param id standard wxWidgets id
    @param title the title of the window
    @param edit_or_add Could be Khz::CfgWin::FTS_EDIT_EXTENSION or
    Khz::CfgWin::FTS_ADD_EXTENSION for edit/add extension dialog respectively
    @param text_or_archive_extractor could be Khz::CfgWin::FTS_TEXTEXTRACTOR or
    Khz::CfgWin::FTS_ARCHIVEEXTRACTOR for text layer extractor or archive extractor
    respectively
    @param style see wxDialog possible styles
    **/
    FullTextSearchIndexerAddEditExtensionDialog
            (
            wxWindow *parent,
            wxWindowID id,
            wxString title,
            int edit_or_add,
            int text_or_archive_extractor,
            long style=wxDEFAULT_DIALOG_STYLE
            );

    /**
    Returns current value of the #extension_input.
    **/
    wxString GetExtension(){return extension_input->GetValue();};

    /**
    Returns current value of the #extractor_input
    **/
    wxString GetExtractor(){return extractor_input->GetValue();};

    /**
    Sets value of the #extension_input
    **/
    void SetExtension(wxString ext){extension_input->SetValue(ext);};

    /**
    Sets value of the #extractor_input
    **/
    void SetExtractor(wxString cmd){extractor_input->SetValue(cmd);};

    /**
    Fills #sample_file_input and #md5_or_substring_input.
    Reads  samples from /path/to/kolkhoznik_exe/full_text_search/test_extractors folder to do it.
    We use this function if we are in the "edit" (not in the "add extension") mode.
    **/
    void fill_samples_fields_if_necessary(wxString extension);
private:


    /**
    "Extension" input of the dialog
    **/
    wxTextCtrl *extension_input;


    /**
    "Command" input of the dialog
    **/
    wxTextCtrl *extractor_input;

    /**
    "Add sample file" checkbox
    **/
    wxCheckBox *add_sample_checkbox;


    /**
    Input for the sample file (archive
    or document example). The control is followed by the "Choose" button.
    **/
    wxTextCtrl *sample_file_input;


    /**
    Input where use can enter md5 of the file in the sample archive (archive extractor)
    or substring that is a part of the text layer of the sample file(text layer extractor)
    **/
    wxTextCtrl *md5_or_substring_input;

    /**
    "Choose" button. Invokes #OnChooseSampleButtonPressed(wxCommandEvent &event)
    **/
    wxButton *choose_sample_button;

    /**
    "Open for preview" button
    **/
    wxButton *open_file_button;

    /**
    Could be FTS_EDIT_EXTENSION, FTS_ADD_EXTENSION.
    This is just a copy of the "edit_or_add" constructor parameter.
    We use this value when user clicks on #add_sample_checkbox.
    If \p action is FTS_EDIT_EXTENSION than we show message
    warning user that he is about to override the existing files.
    If \p action is FTS_ADD_EXTENSION than we do not allow
    user to uncheck the checkbox.
    **/
    int action;




    /**
    Could be FTS_TEXTEXTRACTOR,
        FTS_ARCHIVEEXTRACTOR.
    This is just a copy of the "text_or_archive_extractor" constructor parameter.
    For example, based on this parameter we choose the name of the file to store
    the conntent of the
    #md5_or_substring_input. (the file name could be "md5.txt" , "substring.txt")
    **/
    int mode;

    /**
    Executes when user clicks #add_sample_checkbox.
    See #action for details
    **/
    void OnAddSampleCheckBoxClicked(wxCommandEvent &event);

    /**
    Shows messagebox explaining what the sample is
    **/
    void OnAWhatIsSampleHyperlinkClicked(wxHyperlinkEvent& event);

    /**
    Executes when user presses #choose_sample_button. Opens dialog
    that allow to select a sample file.
    **/
    void OnChooseSampleButtonPressed(wxCommandEvent &event);

    /**
    Opens sample for pveview
    **/
    void OnOpenSampleForPreviewButtonPressed(wxCommandEvent &event);


    /**
    Saves the specified sample file (#sample_file_input) and the content of the
    #md5_or_substring_input to the /path/to/kolkhoznik_exe/full_text_search/test_extractors folder
    **/
    bool save_sample(wxString path_to_sample, wxString extension);

    /**
    Checks that all of the inputs are filled and calls #save_sample
    **/
    void OnOkButtonPressed(wxCommandEvent &event);

};


}}// namespaces
