/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef KOLKHOZNIK_SYNCLISTBOX_HEADER_INCLUDED
#define KOLKHOZNIK_SYNCLISTBOX_HEADER_INCLUDED

#include <wx/htmllbox.h>
#include <wx/settings.h>

    #include "wx/settings.h"
    #include "wx/dcclient.h"
    #include "wx/listbox.h"


#include "wx/dcbuffer.h"
#include "wx/selstore.h"
#include "wx/renderer.h"

namespace Khz{namespace CfgWin{

/**
@brief listbox that allows to syncronize scrolling of two listboxes.
Used inside of Khz::CfgWin::CfgTabFullTextSearchArchiveExtractor, Khz::CfgWin::CfgTabFullTextSearchTextExtractor.

This class is inhereted from wxSimpleHtmlListBox.
Usage example:

@code
    extensions_listbox=new SyncListBox(this); // create the first listbox
    commands_listbox=new SyncListBox(this);   // create the second listbox

    extensions_listbox->SetSyncWindow(commands_listbox);  //commands_lisbox will be scrolled
    // autoamatically when user scrolls extensions_listbox

    commands_listbox->SetSyncWindow(extensions_listbox); //extensions_listbox will be scrolled
    // autoamatically when user scrolls commands_listbox
@endcode
**/
class SyncListBox : public wxSimpleHtmlListBox
    {
    public:

        /**
        constructor
        @param parent standard wxwidgets parent
        **/
        SyncListBox(wxWindow *parent)
            :wxSimpleHtmlListBox(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, NULL, 0)
            {
            SetMargins(2,2);
            is_sync_window_set=false;
            }

        /**
        REDEFINITION of the standard ScrollWindow function. As a result,
        this function scrolls not only the object itself, but #syncronized_window too.
        **/
        virtual void ScrollWindow(int dx, int dy, const wxRect *rect = NULL)
            {
            wxSimpleHtmlListBox::ScrollWindow(dx, dy, rect);
            if(is_sync_window_set)
                {
                syncronized_window->ScrollToRow(GetVisibleBegin());
                }
            }


        /**
        Sets syncronized listbox.
        @param win a listbox that should be scrolled automatically
        when user scrolls this listbox
        **/
        void SetSyncWindow(wxSimpleHtmlListBox *win)
            {
            syncronized_window=win;
            is_sync_window_set=true;
            };


    private:
        /**
        pointer to the syncronized listbox object. Copy of the pointer
        that was passed as a parameter to the #SetSyncWindow() function.
        **/
        wxSimpleHtmlListBox *syncronized_window;

        /**
        true if #SetSyncWindow() was ever called, false otherwise.
        We are using this var to prevent segfault onscroll if #SetSyncWindow() was never called.
        See code of #ScrollWindow()
        **/
        bool is_sync_window_set;



    protected:
        /**
        REDEFINITION of the GetSelectedTextColour function.
        Without such redefinition the selected item of the listbox would have a wrong text color.
        **/
        virtual wxColour 	GetSelectedTextColour (const wxColour &colFg) const
            {
            return wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHTTEXT);
            }
        virtual wxColour GetSelectedTextBgColour (const wxColour &colBg) const
            {
            return wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);
            }


       /**
       REDEFINITION to remove black border around item
       **/
       virtual void OnDrawBackground 	( 	wxDC &  	dc,
		                                    const wxRect &  	rect,
                                            size_t  	n
	                                    ) 		const
            {
                wxColour col=wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT);


                if ( !col.IsOk() )
                    throw("fuck");

                // we need to render selected and current items differently
                const bool isSelected = IsSelected(n),
                           isCurrent = IsCurrent(n);
                if ( isSelected || isCurrent )
                {
                    if ( isSelected )
                    {
                        dc.SetBrush(wxBrush(col, wxBRUSHSTYLE_SOLID));
                    }
                    else // !selected
                    {
                        dc.SetBrush(*wxTRANSPARENT_BRUSH);
                    }
                    // THIS WAS REMOVED BY ME (Black border) //dc.SetPen(*(isCurrent ? wxBLACK_PEN : wxTRANSPARENT_PEN));
                    dc.SetPen(*wxTRANSPARENT_PEN);
                    dc.DrawRectangle(rect);
                }
                //else: do nothing for the normal items

                return;
            }



    };





}}// namespaces
#endif
