/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/string.h>
#include <wx/fileconf.h>
#include <wx/stdpaths.h>
#include <wx/filename.h>
#include "ConfigManager.h"
#include <wx/msgdlg.h>
#include <iostream>
#include "default_config_params.h"


namespace Khz{



wxString ConfigManager::path_to_config="";
bool ConfigManager::is_path_to_config_set=false;
void ConfigManager::create_config_file_if_not_presented()
    {
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    if(!is_path_to_config_set)
        {
        wxString message="ConfigManager: before dealing with config you have to set path_to_config";
        wxMessageBox(message, "Error", wxOK|wxICON_ERROR);
        return;
        }
    ////////////////////////////////////////////////////////////////////////////////////////////////////
    wxFileName ini_file= wxFileName ( path_to_config);
    if(!ini_file.FileExists())
        {
        wxString dir_for_ini=ini_file.GetPath();

        if (!wxDirExists(dir_for_ini))
            {
            wxFileName::Mkdir(dir_for_ini, 0777,  wxPATH_MKDIR_FULL);
            }
        }
    if(!ini_file.FileExists())
        {
        bool everything_is_ok=true;
        wxFileConfig *testconfig = new wxFileConfig( "", "",  path_to_config);

         if(   !testconfig->Write("PATHS/path_to_dir_with_books"         , ""          )    ) everything_is_ok=false;
         if(   !testconfig->Write("PATHS/path_to_database"               , ""          )    ) everything_is_ok=false;
         if(   !testconfig->Write("DEFAULT_APPS/chm"                     , ""          )    ) everything_is_ok=false;
         if(   !testconfig->Write("DEFAULT_APPS/pdf"                     , ""          )    ) everything_is_ok=false;

         if(   !testconfig->Write("COLUMNS_TO_SHOW/columns_order"        ,
                                           "Num, Author, Title, Year, Publisher, Pages, Extension, Topic, md5_of_file")
                                                                                       ) everything_is_ok=false;


         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Num"            , "Num"      )    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Author"         , "Author"   )    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Title"          , "Title"    )    ) everything_is_ok=false;
         //testconfig->Write(wxT("COLUMNS_VISIBLE_NAMES/Volume"), "Vol");
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Year"           , "Year"     )    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Publisher"      , "Publisher")    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Pages"          , "Pages"    )    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Extension"      , "Ext"      )    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/Topic"          , "Topic"    )    ) everything_is_ok=false;
         if(   !testconfig->Write("COLUMNS_VISIBLE_NAMES/md5_of_file"    , "md5"      )    ) everything_is_ok=false;
		 if(   !testconfig->Write("FONTS/library_tree_font_size"         , ""         )    ) everything_is_ok=false;
		 if(   !testconfig->Write("FONTS/books_list_font_size"           , ""         )    ) everything_is_ok=false;
         if(   !testconfig->Flush()                                                    ) everything_is_ok=false;

        delete testconfig;

        if(!everything_is_ok)
            {
            wxString message= "Cannot create config file. It seems that you do not have enough disk space. The program will be terminated.";

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            abort();
            }
        }
    //////////////////////////////////////////////////////////////////////////////


    //wxFileConfig *config =new  wxFileConfig( "", "", this->path_to_config);
    //path_to_library= config->Read("PATHS/path_to_dir_with_books", "");
    //path_to_database=config->Read("PATHS/path_to_database","");
    //delete config;

    }

void ConfigManager::put_default_values_to_config_file_if_necessary()
    {

    ///////////////////////////////////////////////////////////////////////////////////////////////////
    if(!is_path_to_config_set)
        {
        wxString message="ConfigManager: before dealing with config you have to set path_to_config";
        wxMessageBox(message, "Error", wxOK|wxICON_ERROR);
        return;
        }
    ////////////////////////////////////////////////////////////////////////////////////////////////////


    default_config_params defs = default_config_params();

    /////////////////////////////////////////////////////////////////////////////////////
    //  the first value of the pair is extension, the second value is command
    ////////////////////////////////////////////////////////////////////////////////////////
    std::vector<std::pair<wxString, wxString>> archive_extractors=defs.archive_extractors;
    std::vector<std::pair<wxString, wxString>> text_extractors=defs.text_layer_extractors;
    /////////////////////////////////////////////////////////////////////////////////////////


    wxFileConfig *config = new wxFileConfig( "", "", path_to_config);
    if(!config->Exists("FULL_TEXT_SEARCH_TEXT_EXTRACTORS"))
        {
        for(unsigned int i=0;i<text_extractors.size();i++)
            {
            config->Write("FULL_TEXT_SEARCH_TEXT_EXTRACTORS/" + text_extractors[i].first, text_extractors[i].second);
            }
        }
    if(!config->Exists("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS"))
        {
        for(unsigned int i=0;i<archive_extractors.size();i++)
            {
            config->Write("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS/"+archive_extractors[i].first, archive_extractors[i].second);
            }
        }
    config->Flush();
    delete config;
    }

void ConfigManager::SetStaticPathToConfig()
    {
    wxSetlocale(LC_ALL, "");

    wxString home_dir=wxStandardPaths::Get().GetUserConfigDir() + wxFileName::GetPathSeparator() + ".kolkhoznik";
    path_to_config = home_dir + wxFileName::GetPathSeparator() + "config.cfg";

    is_path_to_config_set=true;
    std::cout << "Using config file " << path_to_config.ToStdString();
    std::cout.flush();

    }
void ConfigManager::SetStaticPathToConfig(wxString path)
    {
     wxSetlocale(LC_ALL, "");
    path_to_config=path;
    is_path_to_config_set=true;
    std::cout << "Using config file " << path_to_config.ToStdString();
    std::cout.flush();
    }



}// end of namespace
