/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <wx/listbox.h>
#include <wx/image.h>
#include "my_TaskBar.h"
#include <wx/bitmap.h>
#include <wx/settings.h>
#include <wx/app.h>
#include "TreeCtrlLibrary.h"
#include "MyleftPanel.h"
#include "MyMainWindow.h"
#include "SqliteDBGenerator.h"
#include "wx/splash.h"
#include "ConfigManager.h"



namespace Khz{

/**
@brief Each class of this namespaces represents a standalone wxwidgets executable.
**/
namespace Executables{



// Declare the application class
/**
@brief The main kolkhoznik executable.

Wxwidgets analog of the main function can be found in the kolkhoznik.cpp file.
**/
class kolkhoznik : public wxApp
    {
    public:
        // Called on application startup

         /**
         Initializes the  whole application.
         Creates the application taskbar #Khz::my_TaskBar and the main application window #frame.
         Checks if the config file exists. If it does not exist, than creates the default one and launches the database generator.
         Shows splash image.
         **/
         bool OnInit();

         /**
         The taskbar of the application (icon in the tray)
         **/
		 Khz::my_TaskBar *taskbar;

		 /**
		 Hides/shows the main application window
		 **/
         void OnMouseTaskBarIconLeftClick(wxTaskBarIconEvent&);

         /**
         Shots down the application completely
         **/
         void OnQuit(wxCommandEvent &event);

         /**
         The main application window
         **/
         Khz::MainWin::MyMainWindow *frame;
    };

void kolkhoznik::OnQuit(wxCommandEvent& event)
{
taskbar->Destroy();
frame->Destroy();
}
bool kolkhoznik::OnInit()
    {
    //load image handlers, without in png icons do not work.
	wxInitAllImageHandlers();


    //setting global for the application path to config
    if(argc>1)
        {
        Khz::ConfigManager::SetStaticPathToConfig(argv[1]);
        }
    else //default value
        {
        Khz::ConfigManager::SetStaticPathToConfig();
        };

    //asking user if he want to create config file (if it does not exist)
    if(  !wxFileExists( Khz::ConfigManager::GetPathToConfigFile())  )
        {
        wxString message="The specified config file \""+Khz::ConfigManager::GetPathToConfigFile()+
                            "\"does not exist. Should i create it?";
        wxMessageDialog   err_msg(NULL,
                                message,
                                "Hi from Kolkhoznik!!!",wxYES_NO|wxCENTRE|wxICON_QUESTION|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        switch ( err_msg.ShowModal() )
            {
            case wxID_YES:
                break;

            case wxID_NO:
                exit(0);
            default:
                {
                wxLogError(wxT("Unexpected wxMessageDialog return code!"));
                abort();
                }
            }
        }

    Khz::ConfigManager::create_config_file_if_not_presented();
    Khz::ConfigManager::put_default_values_to_config_file_if_necessary();
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
    // $$$$$$$$$    finished with config file   $$$$$$$$$$$$$$$$$$$$$$$$$
//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$



                                               //these could be already existing)
    taskbar= new Khz::my_TaskBar();
    //taskbar is icon in tray, user can right click on it, than popup menu will appear


    //we should set the icon for taskbar.
    #include "CoolIcon.xpm"
    if ( !taskbar->SetIcon(wxIcon(CoolIcon_xpm), "Kolkhoznik" )    )
    {
        wxLogError(wxT("Could not set icon."));
    }

    // Create the main application window
    frame = new Khz::MainWin::MyMainWindow(wxT("Kolkhoznik"));



    //when user choose File->Close we quit kolkhoznik
	frame->Bind(wxEVT_COMMAND_MENU_SELECTED, &kolkhoznik::OnQuit, this, wxID_EXIT);


    //right click on icon, that choosing "Exit" will close kolkhoznik
	taskbar->Bind(wxEVT_COMMAND_MENU_SELECTED, &kolkhoznik::OnQuit, this,  wxID_EXIT);



    //hide or show main window by left mouse click on taskbar
    Bind(wxEVT_TASKBAR_LEFT_DOWN, &kolkhoznik::OnMouseTaskBarIconLeftClick, this);


    taskbar->Bind(wxEVT_COMMAND_MENU_SELECTED, &Khz::MainWin::MyMainWindow::OnAbout, frame,  wxID_ABOUT);


    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!SPLASH!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    //this part just shows the splash image
    #include "splash.xpm"
      wxImage::AddHandler(new wxPNGHandler);
      bool     m_isPda = (wxSystemSettings::GetScreenType() <= wxSYS_SCREEN_PDA);
        wxBitmap bitmap= wxBitmap(splash);

        bool ok = bitmap.IsOk();

        if (ok && !m_isPda) //screen is big and image exist
            {

            //splash will be visible for 4 second, than it will disappear, so there no leaks here,
            // wxwidgets takes care of it.
            new wxSplashScreen(bitmap,
                wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT,
                4000, frame, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                wxBORDER_NONE|wxSTAY_ON_TOP);
            }
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!





    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //
    // get path to library and path to the database from config file.
    //if those are empty (global_config_parameters creates config file this way on first launch),
    //than it means that kolkhoznik probably launched for the first time.
    //
    wxFileConfig *config = new wxFileConfig( "", "",  Khz::ConfigManager::GetPathToConfigFile());
    wxString temp_path_to_libray = config->Read("PATHS/path_to_dir_with_books", "");
    wxString temp_path_to_database = config->Read("PATHS/path_to_database","");
    delete config;


    //show main frame
    frame->Show(true);

    //as mentioned before, if path to db and library are empty, show popup that asks user to
    // generate database.
    if(temp_path_to_database=="")
        {
                    wxMessageDialog *message =new wxMessageDialog (frame,
                                    "Hi. It seems that you have launched kolkhoznik for the first time. To make it working you have to"
                                    " generate the kolkhoz library database. Please press OK to begin the procedure.",
                                    "Hello there", wxOK|wxCENTRE|wxSTAY_ON_TOP|wxCANCEL, wxDefaultPosition) ;

        switch ( message->ShowModal() )
            {
            case wxID_OK:
                {
                Khz::GenDB::SqliteDBGenerator *gen_db_frame =
                            new Khz::GenDB::SqliteDBGenerator(frame, wxT("Create_database"));
                gen_db_frame->Show(true);
                break;
                }

            case wxID_CANCEL:
                //wxLogStatus(wxT("You pressed \"Cancel\""));
                break;

            default:
                wxLogError(wxT("Unexpected wxMessageDialog return code!"));
            }
        }
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@wxFileExists(current_file.GetFullPath())@@@@@@@

    return true;
    }


// what to do on left click on the tray icon
void kolkhoznik::OnMouseTaskBarIconLeftClick(wxTaskBarIconEvent&)
    {
    if(frame->IsShown())
        {
        frame->Hide();
        }
    else
        {
        frame->Show();
        }
    }



///////////////////////////////////////////////
}}// namespaces
/////////////////////////////////////////////////
IMPLEMENT_APP(Khz::Executables::kolkhoznik)


