/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include "ArchiveExtractorByCommand.h"
#include "string_functions.h"
#include <wx/dir.h>
#include "FileFunctions.h"
#include <fstream>
#include <wx/msgdlg.h>
#include <wx/filename.h>
#include <iostream>
#include "platform_specific_defs.h"
#include "DEBUGMESSAGE.h"

namespace Khz{ namespace FTS { namespace Engine{


// command is something like  %etar %f %o
ArchiveExtractorByCommand::ArchiveExtractorByCommand(std::string command)
    {
    using namespace boost;
    smatch res;
    regex rx("%e(\\w+)\\s+",regex_constants::icase);
    if (regex_search(command,res, rx))
        {
        external_applications.push_back(std::make_pair(res.str(1), res.str(1)));
        pattern=regex_replace(command, rx, "%e ", regex_constants::format_first_only);
        }
    else
        {
        external_applications.push_back(std::make_pair("unknown",""));
        pattern="";
        }
    }


std::string ArchiveExtractorByCommand::construct_full_command(
            std::string path_to_archive, std::string output_dir, std::string executable)
    {
    if (pattern=="") return "";

    std::string output=pattern;
    Khz::Libs::Mstr::replaceAll(output, "%e", "\""+executable+"\"");
    Khz::Libs::Mstr::replaceAll(output, "%f", "\""+path_to_archive+"\"");
    Khz::Libs::Mstr::replaceAll(output, "%o", "\""+output_dir+"\"");

	#ifdef _WIN32
	return "\"" +output +"\"";
	#endif

    return output;
    }


void ArchiveExtractorByCommand::try_to_extract(std::string path_to_archive, std::string output_dir)
    {
    if(external_applications.size()!=1) return;
    if (pattern=="") return;
    if (external_applications[0].second=="") return;
    std::string command=construct_full_command(path_to_archive, output_dir, external_applications[0].second);
    if(command=="") return;
    execute_command(command);
    }



bool ArchiveExtractorByCommand::execute_command(std::string cmd)
    {
	KHZNIK_DEBUGMESSAGE(std::cout << "\nexecuting command:"<< cmd << "\n\n";)
    FILE* pipe = popen(cmd.c_str(), "r");
    if (!pipe) return false;
    char buffer[128];
    //std::string result_of_execution = "";
    while(!feof(pipe)) {
    	if(fgets(buffer, 128, pipe) != NULL)
            {
            std::cout << buffer;
    		//result_of_execution += buffer;
    		}
    }
    pclose(pipe);

    return true;

    }



bool ArchiveExtractorByCommand::IfWorking(std::string test_archive, std::string output_dir, std::string path_to_file_with_md5)
    {
    if(!FileExist(test_archive)) return false;
    if(!FileExist(path_to_file_with_md5)) return false;
    /////////////////////////////////////////////////
    //unpacking archive
    //////////////////////////////////////////////////
    try_to_extract(test_archive, output_dir);


    ///////////////////////////////////////////////////////
    // get full path to the file that was in the archive (wxString full_path_to_test_file)
    ///////////////////////////////////////////////////////'
    wxString output_dir_wx=wxString(output_dir.c_str());
    wxDir dir(output_dir_wx);
      if ( !dir.IsOpened() )
        {
        // deal with the error here - wxDir would already log an error message
        // explaining the exact reason of the failure
		external_applications[0]=std::make_pair(external_applications[0].first, "");
        return false;
        }
    wxString filename;
    bool cont = dir.GetFirst(&filename);
    if(!cont) // output dir is empty
		{
		external_applications[0]=std::make_pair(external_applications[0].first, "");
		return false;
		}
    wxString full_path_to_test_file=dir.GetNameWithSep()+ filename;
    //////////////////////////////////////////////////////////////////////////


    //////////////////////////////////////////////////////////////////////
    //calculating md5 of the extracted file
    //////////////////////////////////////////////////////////////////////
    std::string md5_of_file_in_the_archive=Khz::Libs::Mfl::md5(full_path_to_test_file.ToStdString());



    /////////////////////////////////////////////////////////////////////////////////
    // open .txt file with known md5
    //////////////////////////////////////////////////////////////////////////////////
    std::ifstream tmp_file;
    tmp_file.open(path_to_file_with_md5);
    if(!tmp_file)
        {
          wxString message= "ArchiveExtractorBycommand: cannot read "+ path_to_file_with_md5;

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
        }

    std::string temp_str;
    std::getline(tmp_file, temp_str);

    Khz::Libs::Mstr::replaceAll(temp_str, " ", "");
    Khz::Libs::Mstr::replaceAll(temp_str, "\r", "");
    Khz::Libs::Mstr::replaceAll(temp_str, "\n", "");


    // now compare
    if(temp_str==md5_of_file_in_the_archive and temp_str!="")
		{
		return true;
		}
	else
		{
		external_applications[0]=std::make_pair(external_applications[0].first, "");
		return false;
		}
    }




bool ArchiveExtractorByCommand::FileExist(std::string file)
    {
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Check if test_file and file_with_expected_result are presented.
    //  It there is a problem, than we show an error message and return false.
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    if(!wxFileExists(file)  or   !wxFileName::IsFileReadable(file))
            {
            wxString message= "TextFromFileExtractor: cannot read test samples. File \""+file+"\" cannot be read";

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
            }
    return true;
    //////////////////////////////////////////////////////////////////////////////////
    //      Finished
    /////////////////////////////////////////////////////////////////////////////////////////////////
    }

void ArchiveExtractorByCommand::remove_dir_if_exists(std::string full_path_to_dir)
    {
    wxString dir_wx=wxString(full_path_to_dir.c_str());
    if(wxDirExists(dir_wx))
        {
        wxFileName::Rmdir( dir_wx,wxPATH_RMDIR_RECURSIVE);
        }
    }


}}}// namespaces
