/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "FullTextSearchDbQuery.h"
#include <iostream>
#include <math.h>
#include <wx/string.h>
#include "DEBUGMESSAGE.h"
#include "string_functions.h"
#include <boost/regex/icu.hpp> //for the regular expresssions unicode support

#ifdef KOLKHOZNIK_DEBUG_MODE_ENABLED_2
#include <ctime>
#endif

namespace Khz{ namespace FTS { namespace Engine{



//FullTextSearchDbQuery::FullTextSearchDbQuery()
//    {
//
//    }


FullTextSearchDbQuery::FullTextSearchDbQuery(std::string path_to_xapian_readable_db)
    {
    try
        {
        this->xapian_database=Xapian::Database(path_to_xapian_readable_db);
        everything_is_ok=true;
        error_message="";
        }
    catch(...)
        {
        everything_is_ok=false;
        error_message="cannot connect to the xapian database \""+ path_to_xapian_readable_db+"\""+
                        "\n\nP.S. Have you specified the right path to the database?";
        }
    this->matches_size=0;
    this->matches_estimated=-1;
    }




void FullTextSearchDbQuery::ReconnectToDB(std::string path_to_xapian_readable_db)
    {
    try
        {
        this->xapian_database=Xapian::Database(path_to_xapian_readable_db);
        everything_is_ok=true;
        error_message="";
        }
    catch(...)
        {
        everything_is_ok=false;
        error_message="cannot connect to the xapian database \""+ path_to_xapian_readable_db+"\""+
                        "\n\nP.S. Have you specified the right path to the database?";
        }
    }
//FullTextSearchDbQuery::FullTextSearchDbQuery(Xapian::Database _xapian_database)
//    {
//    this->xapian_database =_xapian_database;
//    error_message="";
//    }


bool FullTextSearchDbQuery::InitNewSearch(std::string language, std::string query_string,
                                          Xapian::doccount _num_of_books_per_page, Xapian::Query restriction_query,
                                           unsigned _samples_number, unsigned int _sample_size)
    {
    /////////////////////////
    // debug checks the function execution time
    //////////////////////////////////////
    KHZNIK_DEBUGMESSAGE_2
        (
        std::clock_t start;
        double duration;
        start = std::clock();
        )
    ///////////////////////////////////////////

    KHZNIK_DEBUGMESSAGE_3(std::cout << "\nFulltextSearchDbQuery starting\n";)
    if(!everything_is_ok) return false;
    KHZNIK_DEBUGMESSAGE_3(std::cout << "FulltextSearchDbQuery passed first check\n";)
    Xapian::Stem stemmer;
    samples_number=_samples_number;
    sample_size=_sample_size;
    if(language=="") using_stemmer=false; else using_stemmer=true;
    KHZNIK_DEBUGMESSAGE(std::cout << "FulltextSearchDbQuery::initNewSearch: restriction_query is " << restriction_query.get_description() << "\n";)
    Xapian::Query query_additional_conditions;
    try{
        std::string addition_conditions="";
            {
            using namespace boost;
            smatch res;
            std::string rx;


            rx= "(\\w+:\\w+)"; // like test:test
            while (  u32regex_search(query_string,res, make_u32regex(rx,regex_constants::icase)))
                {
                KHZNIK_DEBUGMESSAGE( std::cout << "FullTextSearchIndexer: constructing additional conditions\n";)
                query_string= u32regex_replace(query_string, make_u32regex(rx, regex_constants::icase),"",regex_constants::format_first_only);
                addition_conditions=addition_conditions +" "+res.str(1);
                }
            }
        KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSEarchDbQuery: additional conditions are " << addition_conditions << "\n";)
        if(addition_conditions!="")
            {
            Xapian::QueryParser qp_additional_conditions;
            qp_additional_conditions.set_database(this->xapian_database);
            //qp_additional_conditions.set_max_wildcard_expansion(1000000); //WAS WARKING, DEPRECATED, replaced by
            qp_additional_conditions.set_max_expansion(0);

            if(language=="") //no stemmer, "other" language (language is not supported by xapian)
                {
                stemmer=Xapian::Stem();
                qp_additional_conditions.set_stemmer(stemmer);
                qp_additional_conditions.set_stemming_strategy(Xapian::QueryParser::STEM_NONE);
                qp_additional_conditions.add_prefix("title", "XTNS"); // xapian title no stemmer
                qp_additional_conditions.add_prefix("author", "XANS");//xapian author no stemmer
                }
            else
                {
                //std::cout << "fulltextsearchdbquery: stemmer language set to " << language << "\n";
                stemmer = Xapian::Stem(language);
                qp_additional_conditions.set_stemmer(stemmer);
                qp_additional_conditions.set_stemming_strategy(Xapian::QueryParser::STEM_ALL);
                qp_additional_conditions.add_prefix("title", "XTS"); //xapian title stemmed
                qp_additional_conditions.add_prefix("author", "XAS"); //xapian author stemmed
                }
            qp_additional_conditions.set_default_op(Xapian::Query::OP_AND);
            query_additional_conditions=qp_additional_conditions.parse_query(addition_conditions
                                                                             //, Xapian::QueryParser::FLAG_WILDCARD
                                                                             );
            }
        if(!restriction_query.empty() and !query_additional_conditions.empty())
            {
            Xapian::Query temp_query=Xapian::Query(Xapian::Query::OP_AND, query_additional_conditions, restriction_query);
            query_additional_conditions=temp_query;
            }
        else if(!restriction_query.empty() and query_additional_conditions.empty())
            {
            query_additional_conditions=restriction_query;
            }
        KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchDbQuery::InitNewSearch: query_additional_conditions " << query_additional_conditions.get_description() << "\n";)


        this->num_of_books_per_page=_num_of_books_per_page;
        xapian_database.reopen();

        Xapian::QueryParser qp;
        qp.set_default_op(Xapian::Query::OP_NEAR);
        //std::cout << "fulltextsearchdbquery: language is "<< language<<"\n";
        if(language=="") //no stemmer, "other" language (language is not supported by xapian)
            {
            stemmer=Xapian::Stem();
            qp.set_stemmer(stemmer);
            }
        else
            {
            //std::cout << "fulltextsearchdbquery: stemmer language set to " << language << "\n";
            stemmer = Xapian::Stem(language);
            qp.set_stemmer(stemmer);
            }
        qp.set_database(xapian_database);
        //qp.set_max_wildcard_expansion(1000000);   //WAS BEFORE, DEPRECATED
        qp.set_max_expansion(0);
        if(language!="")
            {
            qp.set_stemming_strategy(Xapian::QueryParser::STEM_ALL_Z);
            using_stemmer=true;
            }
        else
            {
            qp.set_stemming_strategy(Xapian::QueryParser::STEM_NONE);
            using_stemmer=false;
            }
	    query = qp.parse_query(query_string //, Xapian::QueryParser::FLAG_WILDCARD
                            );



        if(!query_additional_conditions.empty() and !query.empty())
            {

            Xapian::Query temp_query=Xapian::Query(Xapian::Query::OP_AND, query, query_additional_conditions);
            query=temp_query;
            }
        else if(!query_additional_conditions.empty() and query.empty())
            {
            query=query_additional_conditions;
            }


	    // Find the top 10 results for the query.
	    Xapian::Enquire enquire(xapian_database);
	    enquire.set_query(query);



	    /////////////////////////
        // debug checks the function execution time
        //////////////////////////////////////
        KHZNIK_DEBUGMESSAGE_2
        (
         duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
        std::cout<< "\n" << "InitNewSearch: query preparing finish time is "<< duration <<" seconds\n";
        )
        ///////////////////////////////////////////




	    //std::cout << "Parsed query is: " << query.get_description() << "\n";
	    //Xapian::MSet matches_count = enquire.get_mset(0,0, xapian_database.get_doccount());



        /////////////////////////
        // debug checks the function execution time
        //////////////////////////////////////
        KHZNIK_DEBUGMESSAGE_2
        (
         duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
        std::cout<< "\n" << "InitNewSearch: mset forming time "<< duration <<" seconds\n";
        )
        ///////////////////////////////////////////





            //this->total_num_of_books=matches_count.get_matches_estimated();


            /////////////////////////
            // debug checks the function execution time
            //////////////////////////////////////
            KHZNIK_DEBUGMESSAGE_2
                (
                duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
                std::cout<< "\n" << "InitNewSearch: get match estimated time is "<< duration <<" seconds\n";
                )
            ///////////////////////////////////////////




            //this->total_pages= (int) ceil(1.0*total_num_of_books/num_of_books_per_page);
            //std::cout << "FulltextSearchDbQuery: total pages is " << total_pages<<"\n";
        this->setDbWideQTermsFreqs(xapian_database, query);
        everything_is_ok=true;



        /////////////////////////
        // debug checks the function execution time
        //////////////////////////////////////
        KHZNIK_DEBUGMESSAGE_2
            (
             duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
            std::cout<< "\n" << "InitNewSearch: total execution time is "<< duration <<" seconds\n";
            )
            ///////////////////////////////////////////
        return true;
        }
    catch (const Xapian::Error & error)
        {
        KHZNIK_DEBUGMESSAGE_3( std::cout << "Exception: " << error.get_msg() << "\n";)
        everything_is_ok=false;
        error_message="Error. Xapian says: "+error.get_msg();
        return false;
        }
    catch(...)
        {
        KHZNIK_DEBUGMESSAGE_3( std::cout << "initnewsearch " << "unknown error" << "\n";)
        error_message="unknown error";
        everything_is_ok=false;
        return false;
        }

    }
void FullTextSearchDbQuery::GiveMePageParallelThreadInternal
                    (   int page_num,
                        boost::function<void()> &finished_callback,
                        bool *output_status
                    )
    {
    // no simultaneous calling
    boost::mutex::scoped_lock total_lock(global_mutex);
    bool output = this->GiveMePage(page_num);
    *output_status=output;
    finished_callback();
    }


void FullTextSearchDbQuery::GiveMePageParallelThread
                    (   int page_num,
                        boost::function<void()> &finished_callback,
                        bool *output_status
                    )
    {
    boost::thread
            (
            &Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThreadInternal,
            this,
            page_num,
            finished_callback,
            output_status
            );
    }


bool FullTextSearchDbQuery::GiveMePage(int page_num)
    {
        /////////////////////////
    // debug checks the function execution time
    //////////////////////////////////////
    KHZNIK_DEBUGMESSAGE_2
        (
        std::clock_t start;
        double duration;
        start = std::clock();
        )
    ///////////////////////////////////////////
    if(!everything_is_ok) return false;
    //if(this->total_pages==0) return true; // nothing wrong, just empty search results
    try
        {
	    Xapian::Enquire enquire(xapian_database);
	    enquire.set_query(query);
	    KHZNIK_DEBUGMESSAGE (std::cout << "Parsed query is: " << query.get_description() << "\n";)

        Xapian::MSet matches = enquire.get_mset(this->num_of_books_per_page*(page_num-1), this->num_of_books_per_page);
        // debug checks the function execution time
        //////////////////////////////////////
        KHZNIK_DEBUGMESSAGE_2
            (
            duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
            std::cout<< "\n" << "GiveMePage: got mset time is "<< duration <<" seconds\n";
            )
            ///////////////////////////////////////////
        if(page_num==1 or matches_estimated==-1 or (this->num_of_books_per_page*(page_num))>=matches_estimated)
            {
            this->matches_estimated=matches.get_matches_estimated();
            KHZNIK_DEBUGMESSAGE_2
                (
                duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
                std::cout<< "\n" << "GiveMePage: got matches estimated "<< duration <<" seconds\n";
                )
            }
        KHZNIK_DEBUGMESSAGE
            (
            std::cout << matches.get_matches_estimated() << " results found.\n";
            std::cout << "Matches 1-" << matches.size() << ":\n" << "\n";
            )
	    this->matches_size=matches.size();

        authors.clear();
        titles.clear();
        weights_in_percents.clear();
        abstracts.clear();
        md5s.clear();
        hashs.clear();
        //ids.clear();
        topics.clear();
        extensions.clear();
        years.clear();
        relpaths.clear();
        filenames.clear();
        for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i)
            {
            Xapian::Document document=i.get_document();
            std::string data=i.get_document().get_data();

            KHZNIK_DEBUGMESSAGE( std::cout << "FullTextSearchDbQuery: data is \n\n\n" << data << "\n\n\n";)

            std::size_t first_newline =data.find_first_of("\n");
            std::string title =data.substr(0, first_newline);

            std::size_t second_newline = data.find_first_of("\n", first_newline+1 );
            std::string author= data.substr(first_newline+1, second_newline-first_newline-1);

            std::size_t third_newline = data.find_first_of("\n", second_newline+1 );
            std::string topic= data.substr(second_newline+1, third_newline-second_newline-1);

            std::size_t fourth_newline = data.find_first_of("\n", third_newline+1 );
            std::string extension= data.substr(third_newline+1, fourth_newline-third_newline-1);

            std::size_t fifth_newline = data.find_first_of("\n", fourth_newline+1 );
            std::string year= data.substr(fourth_newline+1, fifth_newline-fourth_newline-1);

            std::size_t sixth_newline = data.find_first_of("\n", fifth_newline+1 );
            std::string relpath= data.substr(fifth_newline+1, sixth_newline-fifth_newline-1);

            std::size_t seventh_newline = data.find_first_of("\n", sixth_newline+1 );
            std::string filename= data.substr(sixth_newline+1, seventh_newline-sixth_newline-1);




            this->authors.push_back(wxString::FromUTF8(author.c_str()).ToStdString());
            this->titles.push_back(wxString::FromUTF8(title.c_str()).ToStdString());
            this->topics.push_back(wxString::FromUTF8(topic.c_str()).ToStdString());
            this->extensions.push_back(wxString::FromUTF8(extension.c_str()).ToStdString());
            this->years.push_back(wxString::FromUTF8(year.c_str()).ToStdString());
            this->relpaths.push_back(wxString::FromUTF8(relpath.c_str()).ToStdString());
            this->filenames.push_back(wxString::FromUTF8(filename.c_str()).ToStdString());
            this->weights_in_percents.push_back(i.get_percent());
            this->md5s.push_back(document.get_value(0));
            this->hashs.push_back(document.get_value(1));
            //this->ids.push_back(Khz::Libs::Mstr::StringToLongLong(document.get_value(0)));
            std::vector<std::pair<std::string, int>> abstract;
            //long long int id;
            this->MakeAbstract(i.get_document().get_docid(), enquire, samples_number,sample_size, abstract);
            this->abstracts.push_back(abstract);

            //this->full_texts.push_back(wxString::FromUTF8(data.substr(second_newline+1, -1).c_str()).ToStdString());

            //std::string text=i.get_document().get_data().substr(second_newline+1, -1);
           // std::string snippet= snipper.generate_snippet(i.get_document().get_data().substr(second_newline+1, -1), 400, 50);
            KHZNIK_DEBUGMESSAGE
                (
                std::string result= wxString::FromUTF8(author.c_str()).ToStdString() + " " + wxString::FromUTF8(title.c_str()).ToStdString();
                //+ "<br><div>"+snippet+"</div>"
                ;

                std::cout << i.get_rank() + 1 << ": " << i.get_percent() << " docid=" << *i
                    << " [" << result<< "]\n\n";
                )
                        /////////////////////////
            // debug checks the function execution time
            //////////////////////////////////////
            KHZNIK_DEBUGMESSAGE_2
                (
                duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
                std::cout<< "\n" << "GiveMePage: total execution time is "<< duration <<" seconds\n";
                )
            ///////////////////////////////////////////
            }
        }
    catch(...)
        {
        return false;
        }
    return true;
    }


//this function is a complete mess. The reason is the following:
//it is based on Recoll's MakeAbstract function.
// The Recoll's Makeabstract is a complete mess, but it works.
//So, I just took this mess and modified it a little.
int FullTextSearchDbQuery::MakeAbstract(Xapian::docid docid,
                                        Xapian::Enquire &enquire, unsigned int imaxoccs, unsigned int ictxwords,
                                         std::vector<std::pair<std::string, int>> &output)
    {
    if(imaxoccs<=0) return 0;
    // we perform indexing like this:
    //        indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
    //        indexer.index_text(one_book["Title"], 1, "XTS"); //xapian title stemmed
    //        indexer.index_text(one_book["Author"],1, "XAS"); // xapian author stemmed
    //    indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_NONE);
    //        indexer.index_text(one_book["Title"], 1, "XTNS"); // xapian title not stemmed
    //        indexer.index_text(one_book["Author"],1, "XANS"); //xapian author not stemmed
    //        indexer.index_text("__begin_of_everything__", 1, "XDELIM"); // after this term the text of the book begin
    //    indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
    //        indexer.index_text(*(Text_Extractor.ptr_to_extracted_text), 1, "Z");
    //    indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_NONE);
    //        indexer.index_text("__end_of_stemmed_words__", 1, "XDELIM");
    //        indexer.index_text(*(Text_Extractor.ptr_to_extracted_text),1,"");
    //        indexer.index_text("__total_end__",1,"XDELIM");
    //So, first we index the author and the title of the book, next we index the text of the
    // book with stemmer, and, finally, we index the text of the book without stemmer
    //As a result we can search with or without stemmer.
    //
    //
    // The snippet generation is going like this:
    //1) first, we find positions of the search terms;
    //2) second, we get positions of the terms that are around of these search terms
    //3) third, we get terms that "sits" in positions found on the previous step.
    //This is the algorithm that is used when we perform a search without stemmer.
    //If we try to use this approach while using stemmer, we get snippet that
    // consists of stemmed word. This is not what we want. To get "good"
    // snippet while using stemmer we do the following:
    //1) first, we find positions of the search terms;
    //2) second, we get positions of the terms that are around of these search terms
    //3) by using position of the "XDELIM__begin_of_everything__",
    //"XDELIM__end_of_stemmed_words__", "XDELIM__end_of_stemmed_words__" we
    // calculate positions of the corresponding unstemmed terms (we just perform an appropriate shift).
    //4) we generate snippet as before.
    unsigned int m_begin=0;
    unsigned int delim=0;
    unsigned int m_end=0;
        {
        Xapian::PositionIterator pos;
        //if(using_stemmer)
            {
            for (pos = xapian_database.positionlist_begin(docid,"XDELIM__begin_of_everything__");
                pos != xapian_database.positionlist_end(docid,"XDELIM__begin_of_everything__"); pos++)
                    {
                    m_begin= *pos;
                    }
            for (pos = xapian_database.positionlist_begin(docid,"XDELIM__end_of_stemmed_words__");
                pos != xapian_database.positionlist_end(docid,"XDELIM__end_of_stemmed_words__"); pos++)
                    {
                    delim= *pos;
                    }
            for (pos = xapian_database.positionlist_begin(docid,"XDELIM__total_end__");
                pos != xapian_database.positionlist_end(docid,"XDELIM__total_end__"); pos++)
                    {
                    m_end= *pos;
                    }
            KHZNIK_DEBUGMESSAGE( std::cout << " begin: " << m_begin << " delim: " << delim << " end: " << m_end << "\n";)
            }
        }




    std::vector<std::string> MatchedTerms;
    this->GetMatchTerms(enquire, docid, MatchedTerms);
    KHZNIK_DEBUGMESSAGE(
        std::cout << "FullTextSearchDbQuery::MakeAbstract: matched terms are:\n";
        for(unsigned int i=0;i<MatchedTerms.size();i++)
            {
            std::cout << MatchedTerms[i] << "\n";
            }
    )
    std::multimap<double, std::string > byQ;
    double totalweight = QualityTerms(docid, MatchedTerms, byQ);
    if (totalweight == 0.0) return -1;

    // For each of the query terms, ask xapian for its positions list
    // in the document. For each position entry, insert it and its
    // neighbours in the set of 'interesting' positions

    // The terms 'array' that we partially populate with the document
    // terms, at their positions around the search terms positions:
    std::map<unsigned int, std::string> sparseDoc;
    std::map<unsigned int, std::string> sparseDocWithStemmer;
    int sparseDocMissingWords=0;// how many terms we need to get from db by position
    // Also remember apart the search term positions so that we can list
    // them with their snippets.
    std::set<unsigned int> searchTermPositions;
    std::set<unsigned int> searchTermPositionsWithStemmer;

    // Remember max position. Used to stop walking positions lists while
    // populating the adjacent slots.
    unsigned int maxpos = 0;

    // Total number of occurences for all terms. We stop when we have too much
    unsigned int totaloccs = 0;


    const unsigned int maxtotaloccs = imaxoccs;
    unsigned int ctxwords = ictxwords/2;

    KHZNIK_DEBUGMESSAGE
        (
         std::cout << "MakeAbstract::byQ is \n";
             for (std::multimap<double, std::string >::reverse_iterator mit = byQ.rbegin();
                mit != byQ.rend(); mit++)
                {
                std::cout << mit->first << " " << mit->second << "\n";
                }
        std::cout << "\n===============\n";
        )

    std::map<std::string, std::vector<unsigned int>> positions_list;
    std::map<unsigned int, bool> positions_list_plain;
    for (std::multimap<double, std::string >::reverse_iterator mit = byQ.rbegin();
    mit != byQ.rend(); mit++)
        {
        std::string qterm = mit->second;
        Xapian::PositionIterator pos;
        std::vector<unsigned int> positions;
        try
            {
            for (pos = xapian_database.positionlist_begin(docid, qterm);
            pos != xapian_database.positionlist_end(docid, qterm); pos++)
                {
                positions.push_back(*pos);
                positions_list_plain.insert(std::make_pair(*pos, true));
                }
            }
        catch(...){};
        positions_list.insert(std::make_pair(qterm, positions));
        KHZNIK_DEBUGMESSAGE
            (
            std::cout << "Makeabstract:: positions of term " << qterm << "are ";
            for (unsigned int i=0;i<positions.size();i++) std::cout << positions[i] << ",";
            std::cout << "\n";
            )
        }
      // Let's go populate
    for(unsigned int top_counter=0;top_counter< byQ.size();top_counter++)
        {
        unsigned int missed=0;
        unsigned int num;
        std::string qterm;
        std::multimap<double, std::string >::reverse_iterator mit;

        for ( mit = byQ.rbegin();
        mit != byQ.rend(); mit++)
            {
            unsigned int maxgrpoccs=0;
            unsigned int grpoccs = 0;
            qterm = mit->second;
            unsigned int maximum_missed_allowed=0; // by default we want to show ALL of the search terms in the snippet
            unsigned int maximum_distance_allowed=byQ.size()-1; // by default we want to show search terms WITHOUT
                                                                // GAPS between words.
            for (unsigned int i_search_place=0;i_search_place<positions_list[qterm].size();i_search_place++)
                {
                num=positions_list[qterm][i_search_place];
                unsigned int closest_pos=num;

                // search any place in the document where all of the terms are close enough to each other
                // if after this cycle "unsigned int missed" is small enough, than we show snippet
                // that consists of words around of the current position
                for (std::map<std::string, std::vector<unsigned int>>::reverse_iterator sit = positions_list.rbegin();
                    sit != positions_list.rend(); sit++)
                    {
                    if (qterm==sit->first) continue;

                    std::vector<unsigned int> terms_positions=positions_list[sit->first];
                    unsigned int closest=FindClosestInt(num, terms_positions);
                    closest_pos=terms_positions[closest];

                    if(
                        std::max(closest_pos,num)-std::min(closest_pos,num) >  maximum_distance_allowed  // we cannot show words without gaps
                                or  std::max(closest_pos,num)-std::min(closest_pos,num) > ctxwords // we cannot show all of the words
                      ) missed++;

                    if(missed>maximum_missed_allowed) break;
                    }


                // cycle above have returned too big "unsigned int missed"
                // we have not found good snippet candidate.
                if(missed>maximum_missed_allowed) // just search for other place in the document
                    {
                    if (  i_search_place   <   (positions_list[qterm].size()-1)    )
                        {
                        missed=0;
                        continue;
                        }
                    else if(maximum_distance_allowed < ctxwords)
                            // there is no way to show all of the search terms
                            // together, so we allow gaps between search terms
                            // not that "i_search_place" is set to zero,
                            // so we start search from beginning after we have allowed gaps
                        {
                        missed=0;
                        maximum_distance_allowed++;
                        i_search_place=0;
                        continue;
                        }
                    else
                        // there is no way to show all of the search terms,
                          // so we allow to omit some terms (first we allow to omit
                          // only one term. If this does not work too, we allow
                          // to omit two terms. And so on.
                        {
                        missed=0;
                        maximum_missed_allowed++;
                        i_search_place=0;
                        continue;
                        }
                    }

                KHZNIK_DEBUGMESSAGE(std::cout << "genAbstract: match is" << num << " " << closest_pos<< "\n";)

                float q;
                if (byQ.size() == 1)
                    {
                    maxgrpoccs = maxtotaloccs;
                    q = 1.0;
                    }
                else
                    {
                    // We give more slots to the better term groups
                    q = mit->first / totalweight;
                    maxgrpoccs = int(ceil(maxtotaloccs * q));
                    }

                // The match term may span several words
                int qtrmwrdcnt =
                Khz::Libs::Mstr::count_words(qterm);

                Xapian::PositionIterator pos;
                // There may be query terms not in this doc. This raises an
                // exception when requesting the position list, we catch it ??
                // Not clear how this can happen because we are walking the
                // match list returned by Xapian. Maybe something with the
                // fields?
                std::string emptys;
                try
                    {
                    //for (pos = xapian_database.positionlist_begin(docid, qterm);
                    //pos != xapian_database.positionlist_end(docid, qterm); pos++)
                    //   {
                    unsigned int delta=abs((long)num-(long)closest_pos);
                    if(delta>ctxwords) delta=0;
                    unsigned int ipos = (num<closest_pos)? (ceil(num+delta/2)) : (ceil(closest_pos+delta/2));
                    if(delta>ctxwords) ipos=num;
                    //if (ipos < int(baseTextPosition)) // Not in text body
                    //  continue;
                    if (sparseDoc.find(num)!= sparseDoc.end() ) continue;
                    totaloccs++;
                    grpoccs++;
                    // Add adjacent slots to the set to populate at next
                    // step by inserting empty strings. Special provisions
                    // for adding ellipsis and for positions overlapped by
                    // the match term.
                    unsigned int sta = ( (ipos - ctxwords) > 0 ? (ipos - ctxwords) : 0);
                    unsigned int sto = (  (ipos + qtrmwrdcnt-1 + ctxwords) < xapian_database.get_doclength (docid) ?
                                            (ipos + qtrmwrdcnt-1 + ctxwords) : xapian_database.get_doclength (docid) );

                    KHZNIK_DEBUGMESSAGE ( std::cout << "\n\n\noldnum is " << num << ", old start is " << sta << "old end is " << sto << "\n";)
                    unsigned int old_sta=sta;
                    unsigned int old_sto=sto;

                   if(using_stemmer)
                        {
                        if(sta<=m_begin)
                            {
                            sta=m_begin+1;
                            sto=sta+(old_sto-old_sta);
                            }
                        if(sto>=delim)
                            {
                            sto=delim-1;
                            sta=sto-(old_sto-old_sta);
                            }
                        if(sta<=m_begin) sta=m_begin+1;
                        if(sto>=delim) sto=delim-1;
                        if(sto<sta) sto=sta;
                        }
                    else if(!using_stemmer)
                        {
                        KHZNIK_DEBUGMESSAGE ( std::cout << "\n\n\ndelim is " << delim << ", m_end is " << m_end << "\n";)
                        if(sta<=delim)
                            {
                            sta=delim+1;
                            sto=sta+(old_sto-old_sta);
                            }
                        if(sto>=m_end)
                            {
                            sto=m_end-1;
                            sta=sto-(old_sto-old_sta);
                            }
                        if(sta<=delim) sta=delim+1;
                        if(sto>=m_end) sto=m_end-1;
                        if(sto<sta) sto=sta;
                        }
                    KHZNIK_DEBUGMESSAGE ( std::cout << "\n\n\nnum is " << num << ", start is " << sta << " end is " << sto << "\n";)
                    for (unsigned int ii = sta; ii <= sto; ii++)
                        {
                        if (ii == num)
                            {
                            sparseDoc[ii] = emptys;
                            sparseDocMissingWords++;
                           // searchTermPositions.insert(num);
                            if (ii > maxpos) maxpos = ii;
                            }
                        else if (ii > ipos && ii < ipos + qtrmwrdcnt)
                            {
                            sparseDoc[ii] = "?";
                            sparseDocMissingWords++;
                            }
                        else if (sparseDoc[ii].compare("...")!=0)
                            {
                            // For an empty slot, the test has a side
                            // effect of inserting an empty string which
                            // is what we want.
                            sparseDoc[ii] = emptys;
                            sparseDocMissingWords++;
                            }
                        }
                    // Add ellipsis at the end. This may be replaced later by
                    // an overlapping extract. Take care not to replace an
                    // empty string here, we really want an empty slot,
                    // use find()
                    if (sparseDoc.find(sto+1) == sparseDoc.end())
                        {
                        sparseDoc[sto+1] = "...";
                        sparseDocMissingWords++;//just in case...
                        }

                    // Group done ?
                    if (grpoccs >= maxgrpoccs) break;

                    // Global done ?
                    if (totaloccs >= maxtotaloccs)
                        {
                        break;
                        }
                    //}
                    }
                catch (...)
                    {
                    // Term does not occur. No problem.
                    }

                if (totaloccs >= maxtotaloccs)
                    {
                    break;
                    }
                }
            // Global done ?
            if (totaloccs >= maxtotaloccs)
                    {
                    break;
                    }
            //group done
            if (grpoccs >= maxgrpoccs) break;
            }
         // total done ?
        if (totaloccs >= maxtotaloccs) break;
        }


    maxpos += ctxwords + 1;
    if (totaloccs == 0) return -1;
    // Walk all document's terms position lists and populate slots
    // around the query terms. We arbitrarily truncate the list to
    // avoid taking forever. If we do cutoff, the abstract may be
    // inconsistant (missing words, potentially altering meaning),
    // which is bad.
    {
        Xapian::PositionIterator pos;
            for(std::map<unsigned int, std::string>::iterator temp_it=sparseDoc.begin();temp_it!=sparseDoc.end();temp_it++)
                {
                sparseDocWithStemmer[delim+temp_it->first-m_begin]= temp_it->second;
                }
            //for( std::set<unsigned int>::iterator temp_it=
            //    searchTermPositions.begin();temp_it!=searchTermPositions.end();temp_it++)
            //    {
            //    searchTermPositionsWithStemmer.insert(delim + (*temp_it) - begin);
            //    }

        Xapian::TermIterator term;
        int cutoff = 1000000;
        for (term = xapian_database.termlist_begin(docid);
        term != xapian_database.termlist_end(docid); term++)
            {
            if (sparseDocMissingWords==0) break;// we have filled everything we need.
            // Ignore prefixed terms
            if( (*term).find("Z")!=std::string::npos) continue;
            //continue;
            if ( cutoff-- < 0)  break;

            std::map<unsigned int, std::string>::iterator vit;
            for (pos = xapian_database.positionlist_begin(docid, *term);
            pos != xapian_database.positionlist_end(docid, *term); pos++)
                {
                if (cutoff-- < 0)  break;
                // If we are beyond the max possible position, stop
                // for this term
                if (*pos > maxpos+delim) break;

                if (!using_stemmer and (vit = sparseDoc.find(*pos)) != sparseDoc.end())
                    {
                    // Don't replace a term: the terms list is in
                    // alphabetic order, and we may have several terms
                    // at the same position, we want to keep only the
                    // first one (ie: dockes and dockes@wanadoo.fr)
                    if (vit->second.empty())
                        {
                        sparseDoc[*pos] = *term;
                        sparseDocMissingWords--;
                        }
                    }
                else if(using_stemmer and (vit = sparseDocWithStemmer.find(*pos)) != sparseDocWithStemmer.end())
                    {
                     if (vit->second.empty())
                        {
                        sparseDocWithStemmer[*pos] = *term;
                        sparseDocMissingWords--;
                        }
                    }

                }
            }
    }



    KHZNIK_DEBUGMESSAGE
        (
        // Debug only: output the full term[position] vector
        int ipos = 0;
        for (std::map<unsigned int, std::string>::iterator it = sparseDoc.begin();
        it != sparseDoc.end();
        it++, ipos++) {
        std::string temp=it->second;
        int temp_num= it->first;
        std::cout << "Makeabstract["<<temp_num<<"] = " << temp << "\n";
        }


        ipos = 0;
        for (std::map<unsigned int, std::string>::iterator it = sparseDocWithStemmer.begin();
        it != sparseDocWithStemmer.end();
        it++, ipos++) {
        std::string temp=it->second;
        int temp_num= it->first;
        std::cout << "MakeabstractWithStemmer["<<temp_num<<"] = " << temp << "\n";
        }
        )

    if(!using_stemmer)
        {
         for(std::map<unsigned int, std::string>::iterator temp_it=sparseDoc.begin();temp_it!=sparseDoc.end();temp_it++)
                {
                if(temp_it->second=="...")
                    {
                    output.push_back(std::make_pair(temp_it->second,-1));
                    }
                else if(positions_list.find(temp_it->second) != positions_list.end())
                    {
                    output.push_back(std::make_pair(temp_it->second, 1));
                    }
                else
                    {
                    output.push_back(std::make_pair(temp_it->second,0));
                    }
                }
        }
    else
        {
        for(std::map<unsigned int, std::string>::iterator temp_it=
            sparseDocWithStemmer.begin();temp_it!=sparseDocWithStemmer.end();temp_it++)
                {
                if(temp_it->second=="...")
                    {
                    output.push_back(std::make_pair(temp_it->second,-1));
                    }
                else if(positions_list_plain.find(temp_it->first-delim+m_begin) != positions_list_plain.end())
                    {
                    output.push_back(std::make_pair(temp_it->second, 1));
                    }
                else
                    {
                    output.push_back(std::make_pair(temp_it->second,0));
                    }
                }
        }

    return 0;
    }


bool FullTextSearchDbQuery::GetMatchTerms(Xapian::Enquire &enquire,unsigned long xdocid, std::vector<std::string>& terms)
    {
    terms.clear();
    Xapian::TermIterator it;
    Xapian::docid id = Xapian::docid(xdocid);
    std::vector<std::string> iterms;
    iterms.insert(iterms.begin(),
    enquire.get_matching_terms_begin(id),
    enquire.get_matching_terms_end(id));
    NoPrefixList(iterms, terms);
    KHZNIK_DEBUGMESSAGE(
        std::cout << "FullTextSearchDbQuery::GetMatchTerms: terms are:\n";
        for(unsigned int i=0;i<terms.size();i++)
            {
            std::cout << terms[i] << "\n";
            }
    )
  //noPrefixList(iterms, terms);
  return true;
 }

void FullTextSearchDbQuery::setDbWideQTermsFreqs(Xapian::Database &xapian_database, Xapian::Query &query)
    {
    /////////////////////////
    // debug checks the function execution time
    //////////////////////////////////////
    KHZNIK_DEBUGMESSAGE_2
        (
        std::clock_t start;
        double duration;
        start = std::clock();
        )
    ///////////////////////////////////////////

    termfreqs.clear();
    std::vector<std::string> qterms;
    std::vector<std::string> iqterms;
    Xapian::TermIterator it;
    for (it = query.get_terms_begin();
    it != query.get_terms_end(); it++)
        {
        iqterms.push_back(*it);
        }
    NoPrefixList(iqterms, qterms);
  // listList("Query terms: ", qterms);
    xapian_database.reopen();
    double doccnt = xapian_database.get_doccount();
    if (doccnt == 0)
    doccnt = 1;

    KHZNIK_DEBUGMESSAGE( std::cout << "FullTextSearchDbQuery::setDbWideQTermsFreqs :\n";)

    for (std::vector<std::string>::const_iterator qit = qterms.begin();
        qit != qterms.end(); qit++)
        {
        termfreqs[*qit] = xapian_database.get_termfreq(*qit) / doccnt;
        KHZNIK_DEBUGMESSAGE( std::cout << "termfrequence in doc" << *qit << " " << xapian_database.get_termfreq(*qit) / doccnt << "\n";)
        }


    KHZNIK_DEBUGMESSAGE_2
        (
         duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;

        std::cout<< "\n" << "setDbWideQTermsFreqs: time execution is "<< duration <<" seconds\n";
        )
    }




double FullTextSearchDbQuery::QualityTerms(Xapian::docid docid,
const std::vector<std::string>& terms,
std::multimap<double, std::string >& byQ)
    {

    std::map<std::string, double> termQcoefs;
    double totalweight = 0;

    double doclen = this->xapian_database.get_doclength(docid);
    if (doclen == 0)
    doclen = 1;



    std::map<std::string, double> grpwdfs;
    std::map<std::string, double> grptfreqs;
    for (unsigned int i=0;i<terms.size();i++)
        {
        Xapian::TermIterator term = this->xapian_database.termlist_begin(docid);
        term.skip_to(terms[i]);
        if (term != this->xapian_database.termlist_end(docid) && *term == terms[i])
            {
            if (grpwdfs.find(terms[i]) != grpwdfs.end())
                {
                grpwdfs[terms[i]] += term.get_wdf() / doclen;
                grptfreqs[terms[i]] += termfreqs[terms[i]];
                }
            else
                {
                grpwdfs[terms[i]] = term.get_wdf() / doclen;
                grptfreqs[terms[i]] = termfreqs[terms[i]];
                }
            KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchDbQuery::QualityTerms WDF of " << terms[i] << " " << grpwdfs[terms[i]] << "\n";)
            }
        }

    // Build a sorted by quality container for the groups

    for (unsigned int i=0;i<terms.size();i++)
        {
        double q = (grpwdfs[terms[i]]) * grptfreqs[terms[i]];
        q = -log10(q);
        if (q < 3)
            {
            q = 0.05;
            }
        else if (q < 4)
            {
            q = 0.3;
            }
        else if (q < 5)
            {
            q = 0.7;
            }
        else if (q < 6)
            {
            q = 0.8;
            }
        else
            {
            q = 1;
            }
        totalweight += q;
        byQ.insert(std::pair<double, std::string>(q, terms[i]));
        }

    return totalweight;
    }


unsigned int FullTextSearchDbQuery::FindClosestInt(unsigned int int_num, std::vector<unsigned int> &some_vector)
    {
    if(some_vector.size()==0) return 0;

    bool started = false;
    unsigned int distance=0;
    unsigned int return_value=0;
    for (unsigned int i =0;i<some_vector.size();i++)
        {
        unsigned int new_distance = abs( (long)some_vector[i]-(long)int_num);
        if(!started)
            {
            return_value=i;
            distance=new_distance;
            started=true;
            }
        else
            {
            if(new_distance<distance)
                {
                return_value=i;
                distance=new_distance;
                }
            }
        }
    return return_value;
    }

void FullTextSearchDbQuery::NoPrefixList(const std::vector<std::string> &in_qterms, std::vector<std::string> &out_qterms)
    {
    for(unsigned int i=0;i<in_qterms.size();i++)
        {
        if (in_qterms[i].find("XTNS")!=std::string::npos or
            in_qterms[i].find("XANS")!=std::string::npos or
            in_qterms[i].find("XTS")!=std::string::npos or
            in_qterms[i].find("XAS")!=std::string::npos or
            in_qterms[i].find("XTOPIC")!=std::string::npos
            )
            {
            // this is a term with one of the prefixes above, skipping it
            continue;
            }
        else
            {
            out_qterms.push_back(in_qterms[i]);
            }
        }
    }


std::string FullTextSearchDbQuery::SuggestSpellingCorrection(std::string str)
    {
    // explode str by spaces into array of words
    std::vector<std::string> words=
        Khz::Libs::Mstr::explode(
                        " ",
                        Khz::Libs::Mstr::trim(boost::regex_replace(str, boost::regex("\\s+", boost::regex_constants::icase)," ")));
    bool corrected_something=false;
    for(unsigned int i=0;i<words.size();i++)
        {
        //Xapian::doccount 	t_num= xapian_database.get_termfreq (words[i]);
        //if(t_num<=3)
            {
            std::string suggestion= xapian_database.get_spelling_suggestion(words[i]);
            KHZNIK_DEBUGMESSAGE(std::cout << "\nSuggestSpellingCorrection: word is " << words[i] << " correction is " << suggestion << "\n";)
            if(suggestion!="" and suggestion!=words[i])
                {

                corrected_something=true;
                words[i]=suggestion;
                }
            }
        }
    if(corrected_something)
        {
        return Khz::Libs::Mstr::implode(" ", words);
        }
    else
        {
        return "";
        }
    }

}}}// namespaces
