/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <string>
#include <vector>
#include <wx/progdlg.h>
#include "sqlite3.h"
#include <queue>
#include <map>
#include <boost/thread/mutex.hpp>
#include <boost/thread/condition.hpp>
#include <boost/thread/shared_mutex.hpp>
#include "Mdb_fuctions.h"
#include "get_books_from_db_by_query.h"
#include "MainTextExtractor.h"
#include "FullTextSearchIndexer.h"
#include <wx/msgdlg.h>
#include <wx/textfile.h>
#include "string_functions.h"
#include <unicode/unistr.h>
#include "DEBUGMESSAGE.h"
#include "FileFunctions.h"



namespace Khz{ namespace FTS { namespace Engine{



FullTextSearchIndexer::FullTextSearchIndexer
                        (
                        std::string xapian_path,
                        std::string path_to_log,
                        std::string path_to_main_database,
                        std::string _path_to_library,
                        std::vector<std::pair<std::string, std::string> > text_extractors_commands,
                        std::vector<std::pair<std::string, std::string> > archive_extractors_commands,
                        std::string temp_dir,
                        boost::function<void(int, std::string, bool*)> &_update_progress_barcallback,
                        boost::function<void()> &_indexing_finished_callback,
                        unsigned int _cache_size
                        )
: text_extractor(temp_dir, text_extractors_commands, archive_extractors_commands)
    {
    KHZNIK_DEBUGMESSAGE(debug_temp_num=0;)
    this->cache_size=_cache_size;
    this->force_stop=false;
    this->update_progress_bar_callback =_update_progress_barcallback;
    this->indexing_finished_callback=_indexing_finished_callback;
    GenXapianStemLanguagesMap();
    this->path_to_xapian_database = xapian_path;
    this->xapian_ok=true;
    this->unknown_error=false;
    this->total_length_of_extracted_docs=0;
    try
        {
        xapian_writable_db=new Xapian::WritableDatabase (this->path_to_xapian_database, Xapian::DB_CREATE_OR_OPEN);
        }
    catch(...)
        {
        xapian_ok=false;
        }
    this->path_to_library = _path_to_library;
    this->total_num_of_books=0;
    this->processed_books=0;



    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //              connecting to the main kolkhoznik database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    sqlite3_open( path_to_main_database.c_str(), &db);
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //connected to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



    std::string table="main";
    std::string where_statement=""; // "SELECT * FROM main"
    std::vector<std::string> placeholder_parameters;  //no placeholders (empty vector)

    database_asker = new Khz::Libs::DB::get_books_from_db_by_query(db, "main", where_statement, 100, placeholder_parameters); //100 books per query

    num_of_currently_working_threads=0;



    // we already checked if the log file exits(FullTextSearchIndexerWindow did the checking), can truncate now
    log_file.open(path_to_log.c_str(), std::ios::out | std::ios::trunc);
    if(!log_file)
        {
        //wxString message= "Cannot create the specified log file. The database generation will be terminated";

        //wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        log_ok=false;
        return;
        }
    else
        {
        log_ok=true;
        wxDateTime now = wxDateTime::Now();
        std::string newline_char = wxString(wxTextFile::GetEOL()).ToStdString(); // \n on linux, \r\n on windows;
        log_file << "#this is the log file of the full text search db generation on " << now.Format().ToStdString() << newline_char;
        log_file << "#" << newline_char;
        log_file << "#" << newline_char;
        }
    }

void FullTextSearchIndexer::FillQueueOfBooksToIndex()
    {
    RemoveObsoleteBooksFromDb();
    //wxGenericProgressDialog progress_bar("Index generation progress", "The full text search index is being generated",
    //                                                 100);
    SayHi("entering FillQueueOfBooksToIndex");
    bool there_no_more_books_to_index= false;
    for(;;)
        {
        SayHi("entering FillQueueOfBooksToIndex cycle");
       // why we need this at all??? boost::mutex::scoped_lock lock_database(the_database_access_mutex);
       boost::mutex::scoped_lock unknown_error_lock(unknow_error_mutex);
        if(!there_no_more_books_to_index and database_asker->Step() and !unknown_error )
            {
            unknown_error_lock.unlock();
            boost::mutex::scoped_lock total_books_lock(total_num_of_books_mutex);
            this->total_num_of_books=database_asker->GetTotalLines();
            total_books_lock.unlock();
          //  lock_database.unlock();
            for(unsigned long i=0;i<database_asker->current_result.size();i++)
                {
                boost::mutex::scoped_lock lock_queue(the_queue_access_mutex);
                 while(status_queue.size()>=max_queue_size)
                    {
                    SayHi("FillQueueOfBooksToIndex: waiting fo book remove from queue");
                    the_condition_variable_book_removed_from_queue.wait(lock_queue);
                    }
                SayHi("FillQueueOfBooksToIndex: pushing book");




                // user pressed "cancel" button on progress bar, we should stop indexing
                boost::mutex::scoped_lock force_stop_lock(force_stop_mutex);
                if(force_stop)
                    {
                    SayHi("\nFillQueueOfBooksToIndex: stopping indexing\n");
                    //clear queues
                    while(!status_queue.empty()) status_queue.pop();
                    while(!the_queue.empty()) the_queue.pop();


                    //the_queue.push(database_asker->current_result[i]);
                    status_queue.push(false);
                    there_no_more_books_to_index=true;
                    lock_queue.unlock();
                    the_condition_variable_book_added_to_queue.notify_one();
                    break;
                    }


                the_queue.push(database_asker->current_result[i]);
                status_queue.push(true);
                //UpdateProgressBar(progress_bar);
                lock_queue.unlock();
                the_condition_variable_book_added_to_queue.notify_one();

                }
            }
        else
            {
            //UpdateProgressBar(progress_bar);
            unknown_error_lock.unlock();
            boost::mutex::scoped_lock num_of_threads_lock(num_of_currently_working_threads_mutex);
             if(num_of_currently_working_threads==0)
                {
                SayHi("FillQueueOfBooksToIndex:quit");
                boost::mutex::scoped_lock commit_lock(CommitChangesToDbIfNecessary_mutex);
                CommitChangesToDbIfNecessary(true);
                SayHi("FillQueueOfBooksToIndex: closing the database1");
                xapian_writable_db->close();
                SayHi("FillQueueOfBooksToIndex: calling callback1");
                indexing_finished_callback();
                SayHi("FillQueueOfBooksToIndex: calling return1");
                return;
                }
            num_of_threads_lock.unlock();
            SayHi("FillQueueOfBooksToIndex: locking queue to push false");
            boost::mutex::scoped_lock lock_queue(the_queue_access_mutex);
            SayHi("FillQueueOfBooksToIndex: queue locked");
            while(status_queue.size()>=max_queue_size)
                {
                SayHi("FillQueueOfBooksToIndex: waiting for book to be removed");
                the_condition_variable_book_removed_from_queue.wait(lock_queue);
                }
            SayHi("pushing false to queue");

            status_queue.push(false);
            there_no_more_books_to_index = true;
            lock_queue.unlock();
            the_condition_variable_book_added_to_queue.notify_one();
            boost::mutex::scoped_lock num_of_threads_lock2(num_of_currently_working_threads_mutex);
            if(num_of_currently_working_threads==0)
                {
                SayHi("FillQueueOfBooksToIndex:quit");
                boost::mutex::scoped_lock commit_lock(CommitChangesToDbIfNecessary_mutex);
                CommitChangesToDbIfNecessary(true);
                SayHi("FillQueueOfBooksToIndex: closing the database1");
                xapian_writable_db->close();
                SayHi("FillQueueOfBooksToIndex: calling callback1");
                indexing_finished_callback();
                SayHi("FillQueueOfBooksToIndex: calling return1");
                return;
                }
            //the_condition_variable_book_added_to_queue.notify_all();
            //break;
            }
        }
    }
void FullTextSearchIndexer::GetBookFromQueueAndIndexIt()
    {
    int this_thread_num;
    KHZNIK_DEBUGMESSAGE
        (
        boost::mutex::scoped_lock temp_lock(num_of_currently_working_threads_mutex);
        debug_temp_num++;
        this_thread_num= debug_temp_num;
        temp_lock.unlock();
        )
    SayHi("entering GetBookFromQueueAndIndexIt");
    // each thread creates it's own copy of the text_extractor.
    boost::mutex::scoped_lock lock_extractor(text_extractor_mutex);
    boost::mutex::scoped_lock lock_database_path(path_to_xapian_database_mutex);
    // protects readable db creation and readable_databases_pointers manipulations
    boost::unique_lock< boost::shared_mutex > uniqueLock(commit_mutex);

    MainTextExtractor Text_Extractor =text_extractor;
    readable_databases_pointers.push_back(new Xapian::Database(path_to_xapian_database));// protected by
    int ThisThreadReadableDbNum=this->readable_databases_pointers.size()-1;
    lock_extractor.unlock();
    lock_database_path.unlock();
    uniqueLock.unlock();

    //every thread have it's own xapian_readable_database
    for(;;)
        {
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: locking queue_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )
        boost::mutex::scoped_lock lock_queue(the_queue_access_mutex);
        while(status_queue.size()==0)
            {
            KHZNIK_DEBUGMESSAGE
                (
                SayHi("GetBookFromQueueAndIndexIt: queue is empty, waiting_"+Khz::Libs::Mstr::IntToString(this_thread_num));
                )
            the_condition_variable_book_added_to_queue.wait(lock_queue);
            }

        if(status_queue.front() == false) // no more books to index, closing the thread
            {
            KHZNIK_DEBUGMESSAGE
                (
                SayHi("GetBookFromQueueAndIndexIt: closing the thread_"+Khz::Libs::Mstr::IntToString(this_thread_num));
                )
            status_queue.pop();
            boost::mutex::scoped_lock temp_lock(num_of_currently_working_threads_mutex);
            num_of_currently_working_threads--;
            //std::cout << "\n num of working threads is " << num_of_currently_working_threads << "\n";
            KHZNIK_DEBUGMESSAGE(std::cout << "\n num of working threads is "+ Khz::Libs::Mstr::IntToString(num_of_currently_working_threads) + "\n";)
            temp_lock.unlock();
            lock_queue.unlock();
            the_condition_variable_book_removed_from_queue.notify_one();
            KHZNIK_DEBUGMESSAGE
                (
                SayHi("\nGetBookFromQueueAndIndexIt: return called_"+Khz::Libs::Mstr::IntToString(this_thread_num));
                )
            return;
            }
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: ready to index_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )
        std::map<std::string, std::string> current_book = the_queue.front();
        the_queue.pop();
        status_queue.pop();
        lock_queue.unlock();
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: UNlocking queue_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )
        the_condition_variable_book_removed_from_queue.notify_one();
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: Starting IndexBook_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )
        // all of the parameters go by reference
        for(unsigned int i=0;i<=5;i++)
            {
            try
                {
                IndexBook(current_book, Text_Extractor, ThisThreadReadableDbNum);
                break;
                }
            catch(...)
                {
                if(i<5)
                    {
                    std::cout << "\exception while indexing, attempt"+ Khz::Libs::Mstr::IntToString(i)+"\n";
                    boost::mutex::scoped_lock commit_lock(CommitChangesToDbIfNecessary_mutex);
                    CommitChangesToDbIfNecessary(true);
                    }
                else
                    {
                    //stop indexing and show error message
                     boost::mutex::scoped_lock lock_error(unknow_error_mutex);
                    this->unknown_error=true;
                    boost::mutex::scoped_lock force_stop_lock(force_stop_mutex);
                    force_stop=true;
                    lock_error.unlock();
                    }
                }
            }
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: IndexBook finished_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )

        //////// commit to hard disk ////////////////////////////////////////////////////
        boost::mutex::scoped_lock commit_lock(CommitChangesToDbIfNecessary_mutex);
        CommitChangesToDbIfNecessary();
        commit_lock.unlock();
        ////////////////////////////////////////////////////////////////////////////////
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: xapian database unlocked_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )




        //////////////////////////////////////////////////////////////////////////////
        //
        //  UPDATIND PROGRESS BAR
        //
        // protedcted by processed_books_mutex
        ////////////////////////////////////////////////////////////////////////////
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: locking processed_books_mutex_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )
        boost::mutex::scoped_lock processed_bookslock(processed_books_mutex);
        boost::mutex::scoped_lock total_books_lock(total_num_of_books_mutex);
        KHZNIK_DEBUGMESSAGE
            (
            SayHi("GetBookFromQueueAndIndexIt: processed_books_mutex locked_"+Khz::Libs::Mstr::IntToString(this_thread_num));
            )
        processed_books++;
        if(total_num_of_books!=0)
            {
            int temp_num=100.0*processed_books/total_num_of_books;
            std::string message=Khz::Libs::Mstr::IntToString(processed_books)+"/"+Khz::Libs::Mstr::IntToString(total_num_of_books)+" completed";
            boost::mutex::scoped_lock force_stop_lock(force_stop_mutex);
            update_progress_bar_callback(temp_num, message, &force_stop);
            if(force_stop)
                {
                force_stop_lock.unlock();
                boost::mutex::scoped_lock lock_queue(the_queue_access_mutex);
                while(!status_queue.empty()) status_queue.pop();
                while(!the_queue.empty()) the_queue.pop();
                lock_queue.unlock();
                the_condition_variable_book_removed_from_queue.notify_one();
                }


            KHZNIK_DEBUGMESSAGE
                (
                if(force_stop)
                    {
                    std::cout << "\n########################################\n";
                    std::cout << "force_stop is true";
                    std::cout << "\n########################################\n";
                    }
                )
            }
        }
        ////////////////////////////////////////////////////////////////////////////
    }



bool FullTextSearchIndexer::IndexBook
                    (
                     std::map<std::string, std::string> one_book,
                     MainTextExtractor &Text_Extractor,
                     int xapian_readable_database_num
                    )
    {
    // DO NOT USE CommitChangesToDBIfNecessary inside of IndexBook, or you get complete freeze.
    boost::shared_lock<boost::shared_mutex> lock_commits(commit_mutex); // no commit during book's indexing
                                                                        // due to the fact that we write data
                                                                        // to db
        ///////////////////////////////////////////////////////////////////////////////
        // prepare terms that we search for to check if book is already indexed
        ////////////////////////////////////////////////////////////////////////////////
        std::string md5=one_book["md5_of_file"];

        std::string Hash=one_book["m_hash"];


        std::string md5_with_prefix="XMD5"+ md5;
        std::string hash_with_prefix="XHASH"+Hash;
        //std::cout << "author with prefix is " << author_with_prefix <<"\n";


        ///////////////////////////////////////////////////////////////////////
        // search terms. Book is supposed to be indexed if all
        // of the search terms are presented
        //////////////////////////////////////////////////////////////////
        std::vector<std::string> search_terms;
        search_terms.push_back(md5_with_prefix);
        search_terms.push_back(hash_with_prefix);
        Xapian::doccount num_matched_books;
        //Xapian::Enquire enquire;
        Xapian::MSet matches;

        ////////////////////////////////////////////////////////////////
        // check if the book is already in the database
        ///////////////////////////////////////////////////////////////

        KHZNIK_DEBUGMESSAGE(std::cout << "\nIndexBook: entering cycle\n";)
            KHZNIK_DEBUGMESSAGE(std::cout << "\nIndexBook: cycling\n";)
            {
                Xapian::Document doc;
                    // we get access only if "commit" is NOT being executed
                    KHZNIK_DEBUGMESSAGE(std::cout << "\nIndexBook: got in try methond\n";)
                    Xapian::Query if_book_already_there_query(Xapian::Query::OP_AND, search_terms.begin(), search_terms.end());
                    KHZNIK_DEBUGMESSAGE( std::cout << "\n\n\nFulltextSearchIndexer: find book query is " << if_book_already_there_query.get_description() << "\n\n\n";)
                    Xapian::Enquire enquire = Xapian::Enquire(   (*readable_databases_pointers[xapian_readable_database_num])   );
                    enquire.set_query(if_book_already_there_query);
                    matches = enquire.get_mset(0, 1);
                    num_matched_books = matches.size ();
                    KHZNIK_DEBUGMESSAGE(std::cout << "num_matched_books " << num_matched_books << "\n";)
                    if(num_matched_books>=1) // book is already in the database, nothing to index
                        {
                        KHZNIK_DEBUGMESSAGE("dealing with document with id"+one_book["ID"] +"\n";)
                        doc=matches.begin().get_document();
                        std::string file_md5_in_xapian_db=doc.get_value(0);
                        std::string hash_in_xapian_db=doc.get_value(1);
                        if(file_md5_in_xapian_db==one_book["md5_of_file"] and hash_in_xapian_db==one_book["m_hash"])
                            {
                            KHZNIK_DEBUGMESSAGE(std::cout << "document is good, id is"+one_book["ID"] +"\n";)
                            return true;
                            }
                        else
                            {
                            std::cout << "\n PISDEC KOTAM in FullTextSearchIndexer.cpp\n";
                            std::cout << "ABORTED";
                            abort();
                            }
                        }
                    else
                        {
                        goto out_of_cycle;
                        }
            }
        ////////////////////////////////////////////////////////////////
        //     check finished
        //////////////////////////////////////////////////////////////////


        // index the book
        out_of_cycle:
        if(  ExtractTextFromFile(one_book,Text_Extractor ) !=1)
            {
            return false;
            };
        Xapian::TermGenerator indexer;
        Xapian::Stem stemmer;

        std::string language =  one_book["Language"];
        std::transform(language.begin(),language.end(), language.begin(), ::tolower); //convert to lower case
        boost::mutex::scoped_lock steam_languages_lock(StemLanguagesMap_mutex);
        if (StemLanguagesMap.count(language) != 0)
            {
            stemmer=Xapian::Stem(language);
            }
        else if(language=="") // default is english
            {
            stemmer= Xapian::Stem("english");
            }
        else
            {
            stemmer=Xapian::Stem(); // no stemming
            }
        steam_languages_lock.unlock();
        indexer.set_stemmer(stemmer);
        Xapian::Document doc;


        doc.set_data( one_book["Title"]+"\n" + one_book["Author"]+"\n"+
                    one_book["Topic"]+"\n" +one_book["Extension"]+"\n" +one_book["Year"]+"\n" +
                    one_book["RelPathToFile"]+"\n"+one_book["Filename"]+"\n" );

        // we use this to search if book is already in the database
        doc.add_term(md5_with_prefix);
        doc.add_term(hash_with_prefix);

        /////////////////////////////////////////////////////////////
        //these two are used to check if book is already indexed
        doc.add_value (0, one_book["md5_of_file"] );
        doc.add_value(1, one_book["m_hash"]);
        /////////////////////////////////////////////////////////////

        indexer.set_document(doc);

        indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
            indexer.index_text(one_book["Title"], 1, "XTS"); //xapian title stemmed
            indexer.index_text(one_book["Author"],1, "XAS"); // xapian author stemmed
        indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_NONE);
            indexer.index_text(one_book["Title"], 1, "XTNS"); // xapian title not stemmed
            indexer.index_text(one_book["Author"],1, "XANS"); //xapian author not stemmed
            indexer.index_text("__begin_of_everything__", 1, "XDELIM"); // after this term the text of the book begin
        indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_ALL);
            indexer.index_text(*(Text_Extractor.ptr_to_extracted_text), 1, "Z");
        indexer.set_stemming_strategy(Xapian::TermGenerator::STEM_NONE);
            indexer.index_text("__end_of_stemmed_words__", 1, "XDELIM");
            indexer.index_text(*(Text_Extractor.ptr_to_extracted_text),1,"");
            indexer.index_text("__total_end__",1,"XDELIM");


        // add term with topic to enable narrowing search by topic
        std::string topic=one_book["Topic"];
        Khz::Libs::Mstr::replaceAll(topic, " ", "_");
             doc.add_term("XTOPIC"+topic);

        boost::mutex::scoped_lock lock_xapian(xapian_database_access_mutex);
        xapian_writable_db->add_document(doc);
        lock_xapian.unlock();
        IncrCommitCounters(doc.termlist_count());
        return true;

    return false;
    }


void FullTextSearchIndexer::GenXapianStemLanguagesMap()
    {
    //languages separeted by spaces;
    std::string supported_by_xapian_languages=Xapian::Stem::get_available_languages();

    //Let us make an array of these languages
    std::vector<std::string> keywords;
    std::map<std::string, std::string> answer;


    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //so, we create keywords=std::vector<std::string> of the supported languages
    keywords=Khz::Libs::Mstr::explode(" ",Khz::Libs::Mstr::trim(boost::regex_replace(supported_by_xapian_languages, boost::regex("\\s+", boost::regex_constants::icase)," ")));
    for(unsigned int i=0;i<keywords.size();i++)
        {
        answer.insert(std::make_pair(keywords[i], ""));
        }


    this->StemLanguagesMap=answer;
    }


//this function always gets called from the main thread, so we do not need any mutexes.
void FullTextSearchIndexer::RemoveObsoleteBooksFromDb()
    {
    std::vector<Xapian::doccount> ids;
    std::vector<std::string> md5s;
    std::vector<std::string> hashs;

    unsigned int buffer_size=100;
    unsigned int temp_counter=0;
    boost::mutex::scoped_lock lock_xapian(xapian_database_access_mutex);
    Xapian::doccount total_docs=xapian_writable_db->get_doccount();
    if(total_docs==0) return;



    std::vector<Xapian::docid> documents_to_remove;
    Xapian::PostingIterator iter;
    Xapian::doccount temp_num=0;
    for (iter=xapian_writable_db->postlist_begin("");
    // iterate through all of the documents in the xapian database.
    iter != xapian_writable_db->postlist_end(""); iter++)
        {
        boost::mutex::scoped_lock force_stop_lock(force_stop_mutex);
        if(force_stop) return;
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            temp_num++;
            update_progress_bar_callback(double(temp_num)/double(total_docs)*100.0, "checking the index", &force_stop);
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        force_stop_lock.unlock();

        Xapian::Document doc = xapian_writable_db->get_document(*iter);
        std::string md5_in_xapian_db=doc.get_value(0);
        std::string hash_in_xapian_db=doc.get_value(1);
        Xapian::doccount id_in_xapian_db=(*iter);

        ids.push_back(id_in_xapian_db);
        md5s.push_back(md5_in_xapian_db);
        hashs.push_back(hash_in_xapian_db);
        temp_counter++;
        //from time to time we query the main kolkhoznik sqlite database
        // and check if books from buffer are presented in the sqlite database.
        // if some books are not presented, than they should be removed from the
        // xapian database. The id's of the documents that should be removed
        // are stored in documents_to_remove.

        // we could just check every book (without buffer) but it takes too long time.
        if(temp_counter>=buffer_size)
            {
            PrepareListOfDocsToRemove
                (
                &ids,
                &md5s,
                &hashs,
                &documents_to_remove
                );
            ids.clear();
            md5s.clear();
            hashs.clear();
            temp_counter=0;
            }
        }
    PrepareListOfDocsToRemove
        (
        &ids,
        &md5s,
        &hashs,
        &documents_to_remove
        );
    lock_xapian.unlock();
    std::cout << "\nRemoveObsoleteBooksFromDd about to remove " << documents_to_remove.size() << " documents\n";

    //////////////////////////////////////////////////////////////////////////////////////
    // update progress bar
            boost::mutex::scoped_lock force_stop_lock(force_stop_mutex);
            if(force_stop) return;
            temp_num++;
            update_progress_bar_callback(100.0, "removing obsolete books from index", &force_stop);
            force_stop_lock.unlock();
    // update finished
    //////////////////////////////////////////////////////////////////////////////////////////
    for (Xapian::docid ii=0;ii<documents_to_remove.size(); ii++)
        {
        boost::mutex::scoped_lock lock_xapian2(xapian_database_access_mutex);
        xapian_writable_db->delete_document(documents_to_remove[ii]);
        lock_xapian2.unlock();
        IncrCommitCounters(0);
        std::cout << "\nRemoveObsoleteBooksFromDd: removing doc with id " << documents_to_remove[ii] << "\n";
        CommitChangesToDbIfNecessary();
        }
    CommitChangesToDbIfNecessary(true);
    }
void FullTextSearchIndexer::PrepareListOfDocsToRemove
                (
                std::vector<Xapian::docid> *ids,
                std::vector<std::string> *md5s,
                std::vector<std::string> *hashs,
                std::vector<Xapian::docid> *vector_to_add_id_for_removal
                )
    {
    if( ids->size()!=md5s->size() or md5s->size()!=hashs->size() )
        {
        std::cout << "FUCK YOU ALL"; // should never happen
        abort();
        }
    std::string where_statement="";
    for(unsigned int ii=0;ii<md5s->size();ii++)
        {
        if(where_statement=="")
            {
            where_statement=" (md5_of_file=? and m_hash=?) ";
            }
        else
            {
            where_statement=where_statement+ " OR (md5_of_file=? and m_hash=?) ";
            }
        }
      std::string query= "select count(*) from main where ("+where_statement+");";
        sqlite3_stmt* stmt = NULL;
        int retval = sqlite3_prepare_v2(db, query.c_str(), -1, &stmt, NULL);
        if(retval) throw(std::string("cannot connect to db in FullTextSearchIndexer.cpp"));


    unsigned int temp_counter=1;
    for(unsigned int ii=0;ii<md5s->size();ii++)
        {
        sqlite3_bind_text(stmt,  temp_counter, (*(md5s))[ii].c_str() , -1, SQLITE_TRANSIENT);
        temp_counter++;
        sqlite3_bind_text(stmt,  temp_counter, (*hashs)[ii].c_str() , -1, SQLITE_TRANSIENT);
        temp_counter++;
        }



        retval = sqlite3_step(stmt);
        unsigned int answer = sqlite3_column_int(stmt, 0);
        //std::cout << "RemoveObsoleteBooksFromDb answer is " << answer << "\n";
        sqlite3_finalize(stmt);
        if(answer!=md5s->size()) // our sqlite db contains not all of the docs that are
                                 // presented in xapian db. We should remove extra docs from the xapian db.
            {
            for(unsigned int ii=0;ii<md5s->size();ii++)
                {
                std::string query= "select count(*) from main where (md5_of_file=? and m_hash=?);";
                sqlite3_stmt* stmt = NULL;
                int retval = sqlite3_prepare_v2(db, query.c_str(), -1, &stmt, NULL);
                if(retval) throw(std::string("cannot connect to db in FullTextSearchIndexer.cpp"));
                sqlite3_bind_text(stmt,  1, (*md5s)[ii].c_str() , -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  2, (*hashs)[ii].c_str() , -1, SQLITE_TRANSIENT);
                retval = sqlite3_step(stmt);
                int answer = sqlite3_column_int(stmt, 0);
                sqlite3_finalize(stmt);
                if(answer!=1)
                    {
                    vector_to_add_id_for_removal->push_back((*ids)[ii]);
                    }
                }
            }
    }



int FullTextSearchIndexer::ExtractTextFromFile(std::map<std::string, std::string> one_book, MainTextExtractor &Text_Extractor )
    {
    //boost::mutex::scoped_lock lock(extract_text_mutex);

    /////////////////////////////////////////////////////////////////////////////////
    // construct full path to book
    ////////////////////////////////////////////////////////////////////////////////
    std::string extension=one_book["Extension"];
    std::string rel_path_to_book=wxString::FromUTF8(one_book["RelPathToFile"].c_str()).ToStdString();
    std::string filesep_std= wxString( wxFileName::GetPathSeparator()).ToStdString();
    Khz::Libs::Mstr::replaceAll(rel_path_to_book, "/",filesep_std);
    std::string filename=wxString::FromUTF8(one_book["Filename"].c_str()).ToStdString();

    boost::mutex::scoped_lock library_path_lock(path_to_library_mutex);
    std::string full_path_to_book=this->path_to_library + rel_path_to_book +
    filesep_std+filename;
    library_path_lock.unlock();
    /////////////////////////////////////////////////////////////////////////////////
    // full path to book has been constructed
    ////////////////////////////////////////////////////////////////////////////////////


    if(!wxFileExists(wxString(full_path_to_book)) )
        {
        boost::mutex::scoped_lock log_lock(log_file_mutex);
        log_file << "warning: file does not exist: " << full_path_to_book << "\n";
        std::cout << "\nFullTextSearchIndexer::IndexBook: full path_to_book not exists " << full_path_to_book.c_str() << "\n";
        return -1;
        }

    int return_value = Text_Extractor.extract_text(one_book["ID"], extension, full_path_to_book);
    if(return_value==1)
        {
        return return_value;
        }
    else if(return_value==0)
        {
        boost::mutex::scoped_lock log_lock(log_file_mutex);
        log_file << "error: cannot extract text layer from " << full_path_to_book << ". Reason: error executing external command\n";
        }
    else if(return_value==-1)
        {
        boost::mutex::scoped_lock log_lock(log_file_mutex);
        log_file << "warning: text extractor is not implemented for this type of file " << full_path_to_book << "\n";
        }
    else
        {
        std::cout << "\n unexpected return value in FullTextSearchIndexer::ExtractTextFromFile. Aborted.";
        abort();
        }
    return return_value;
    }

void FullTextSearchIndexer::CommitChangesToDbIfNecessary(bool force_commit)
    {
    boost::mutex::scoped_lock commit_lock(commit_counter_mutex);
    KHZNIK_DEBUGMESSAGE(std::cout << "\n COMMIT_COUNTER is " << commit_counter <<"\n";)
    if(commit_counter>=this->cache_size or
            total_length_of_extracted_docs>20000*cache_size or //20 000 terms per doc
                force_commit
      )
        {
        commit_lock.unlock();
        unsigned int commit_attempts;
        ///////////////////////////////////////////////////////////////////////
        // We must be sure that all of the indexers wait until we commit
        // and "reload" all of the databases. Reloading databases while indexing
        //used to create an exception (without 3 mutexes below).
        // The below 3 lines have solved the problem.
        boost::upgrade_lock<boost::shared_mutex> lock(commit_mutex);
        boost::upgrade_to_unique_lock<boost::shared_mutex> uniqueLock(lock);
        boost::mutex::scoped_lock lock_xapian(xapian_database_access_mutex);
        /////////////////////////////////////////////////////////////////////////


  // now we have exclusive access
        for(commit_attempts=0;commit_attempts<=5;commit_attempts++)
            {
            try
                {
                SayHi("\n#######################################\n");
                SayHi("\nCommiting, attempt"+Khz::Libs::Mstr::IntToString(commit_attempts));
                SayHi("\n#######################################\n");
                std::cout << "\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n";
                std::cout << "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n";
                std::cout << "WRITING DATA TO HARD DISK\n";
                xapian_writable_db->commit();
                //xapian_writable_db->close();
                //delete xapian_writable_db;
                //xapian_writable_db= new Xapian::WritableDatabase (this->path_to_xapian_database, Xapian::DB_CREATE_OR_OPEN);
                std::cout << "FINISHED\n";
                std::cout << "total length of the extracted docs " << total_length_of_extracted_docs << " words \n";
                std::cout << "number of indexed documents " << commit_counter << "\n";
                std::cout << "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n";
                std::cout << "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n";
                for(unsigned int i=0;i<readable_databases_pointers.size();i++)
                    {
                    //readable_databases_pointers[i]->close();
                    //delete readable_databases_pointers[i];
                    readable_databases_pointers[i]->reopen();
                    //readable_databases_pointers[i]=new Xapian::Database(this->path_to_xapian_database);
                    }
                //xapian_writable_db.close();
                //boost::mutex::scoped_lock path_lock(path_to_xapian_database_mutex);
                //xapian_writable_db= Xapian::WritableDatabase (this->path_to_xapian_database, Xapian::DB_CREATE_OR_OPEN);
                SayHi("\n#######################################\n");
                SayHi("\nCommiting completed\n");
                SayHi("\n#######################################\n");
                break;
                }
            catch (const Xapian::Error & error)
                {
                std::cout << "Exception: " << error.get_msg() << "\n";
                std::cout << "@@@@@   commiting error  @@@@@\n";
                boost::mutex::scoped_lock log_lock(log_file_mutex);
                log_file << "warning: error writing to disk while commiting, trying again\n";
                boost::this_thread::sleep( boost::posix_time::seconds(4) );
                }
            }
        if(commit_attempts>=4)
            {
            boost::mutex::scoped_lock lock_error(unknow_error_mutex);
            this->unknown_error=true;
            lock_error.unlock();
            boost::mutex::scoped_lock force_stop_lock(force_stop_mutex);
            force_stop=true;
            }
        boost::mutex::scoped_lock commit_lock2(commit_counter_mutex);

        commit_counter=0;
        total_length_of_extracted_docs=0;
        }

    }

}}}// namespaces
