/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <string>
#include <vector>
#include <wx/string.h>
#include <wx/filename.h>

#include <wx/gdicmn.h>
#include <wx/dir.h>
#include <map>
#include "MainTextExtractor.h"
#include "TextFromFileExtractorByCommand.h"
#include "string_functions.h"
#include "DEBUGMESSAGE.h"






namespace Khz{ namespace FTS { namespace Engine{




///////////////////////////////////////////////////////////
// Ordinary constructor and copy constructor
////////////////////////////////////////////////////////////


MainTextExtractor::MainTextExtractor
                (
                std::string _temp_dir,
                std::vector<std::pair<std::string, std::string> > &text_extractors_commands,
                std::vector<std::pair<std::string, std::string> > &archive_extractors_commands
                )
    {/////////////////////////////////////////////////////////////
    //djvu_extractor_obj = DjvuTextExtractor_dvjutxt();
    wxString temp_dir_wx=wxString(_temp_dir.c_str());
    wxDir dir(temp_dir_wx);
    this->temp_dir_with_last_separator=dir.GetNameWithSep().ToStdString();
    this->additional_extracted_text="";


    text_extractors_list.clear();
    archive_extractors_list.clear();
    for(unsigned int i=0;i<text_extractors_commands.size();i++)
        {
        text_extractors_list.push_back(std::make_pair(
            text_extractors_commands[i].first, new TextFromFileExtractorByCommand(text_extractors_commands[i].second)));
        }

    for(unsigned int i=0;i<archive_extractors_commands.size();i++)
        {
        archive_extractors_list.push_back(std::make_pair(
           archive_extractors_commands[i].first,  new ArchiveExtractorByCommand(archive_extractors_commands[i].second)));
        }

    //extractors_list.push_back(std::make_pair("djvu", &djvu_extractor_obj));
    //extractors_list.push_back(std::make_pair("djv", &djvu_extractor_obj));


    //std::cout << "\n normal constructor says:\n";
    //for (unsigned int i=0;i<extractors_list.size();i++)
    //    {
    //    text_extractors_list[i].second->SayHi();
    //    }
    }/////////////////////////////////////////////////////////////

MainTextExtractor::~MainTextExtractor()
    {
    for(unsigned int i=0;i<text_extractors_list.size();i++)
        {
        delete text_extractors_list[i].second;
        }

    for(unsigned int i=0;i<archive_extractors_list.size();i++)
        {
        delete archive_extractors_list[i].second;
        }
    }


MainTextExtractor::MainTextExtractor(const MainTextExtractor &obj)
    {/////////////////////////////////////////////////////////////
    this->temp_dir_with_last_separator=obj.temp_dir_with_last_separator;
    this->additional_extracted_text="";

    this->text_extractors_list.clear();//just in case
    this->archive_extractors_list.clear();
    for(unsigned int i=0;i<obj.text_extractors_list.size();i++)
        {
        this->text_extractors_list.push_back(std::make_pair
                //making pair
                (
                obj.text_extractors_list[i].first,
                new TextFromFileExtractorByCommand(*((TextFromFileExtractorByCommand*)obj.text_extractors_list[i].second))
                //prev line: we converted pointer to TextFromFileExtractor_parent
                //to  pointer to TextFromFileExtractorByCommand (derived from virtual class TextFromFileExtractor_parent).
                // Finally, we took value (like *ptr) to pass to the copy constryctor

                )
        );
        }
    for(unsigned int i=0;i<obj.archive_extractors_list.size();i++)
        {
        this->archive_extractors_list.push_back(std::make_pair(
                                                      // copying archive extractor
            obj.archive_extractors_list[i].first, new ArchiveExtractorByCommand(*(obj.archive_extractors_list[i].second))));
        }
    }/////////////////////////////////////////////////////////////





/////////////////////////////////////////////////////////////
// Finished with constructors
////////////////////////////////////////////////////////////////

int MainTextExtractor::extract_text(std::string unique_id, std::string extension, std::string full_path_to_file)
    {
    KHZNIK_DEBUGMESSAGE(std::cout << "MainTextExtractor::extract_text: extracting " << full_path_to_file << "\n";)
    std::transform(extension.begin(),extension.end(), extension.begin(), ::tolower); //convert to lower case
    std::vector<std::pair<std::string, std::string>> unarchived_files;
    std::vector<std::string> temporarily_created_dirs;


    bool unpacked_something= exctact_archive_recursivelely_if_necessary(
        this->temp_dir_with_last_separator,
        unique_id,
        std::make_pair(full_path_to_file, extension),
        unarchived_files, //output, goes by reference
        temporarily_created_dirs //output, goes_by_reference
        );
    KHZNIK_DEBUGMESSAGE(std::cout << "MainTextExtractor::extract_text: got out of exctact_archive_recursivelely_if_necessary\n";)
    if(!unpacked_something) //file is not archive, just extract text
        {
        for(unsigned int i=0;i<text_extractors_list.size();i++)
            {
            if (text_extractors_list[i].first == extension)  //extractors_list cointains extractor for the extension
                {
                auto extractor =text_extractors_list[i].second;  // Now it->second is corresponding text extractor.
                if(extractor->extract_text(full_path_to_file))
                    {
                    this->ptr_to_extracted_text =extractor->GetPtrToExtractedText();  //JUST POINTER
                    //std::cout << "\n MainTextExtractor res is \n\n\n" << *extracted_text<< "\n\n\n";
                    return 1;
                    }
                else
                    {
                    return 0;
                    }
                }
            }
        return -1;
        }
    else // we have an archive, something was unpacked
        {
        int return_code=-1;
        this->additional_extracted_text="";
        for (unsigned i=0;i<unarchived_files.size();i++)
            {
            std::string path_to_file=unarchived_files[i].first;
            std::string just_ext=unarchived_files[i].second;
             for(unsigned int j=0;j<text_extractors_list.size();j++)
                {
                if (text_extractors_list[j].first == just_ext)  //extractors_list cointains extractor for the extension
                    {
                    auto extractor =text_extractors_list[j].second;  // Now it->second is corresponding text extractor.
                    if(extractor->extract_text(path_to_file))
                        {
                        this->additional_extracted_text += *(extractor->GetPtrToExtractedText());  //JUST POINTER
                        //std::cout << "\n MainTextExtractor res is \n\n\n" << *extracted_text<< "\n\n\n";
                        return_code=1;
                        }
                    else
                        {
                        if(return_code!=1) return_code=0;
                        }
                    break;
                    }
                }
            }
        this->ptr_to_extracted_text=&(this->additional_extracted_text);
        //removing created dirs
        for (unsigned int i=0;i<temporarily_created_dirs.size();i++)
            {
            wxString dir_to_remove=wxString(temporarily_created_dirs[i].c_str());
            if(dir_to_remove=="")
                {
                std::cout << "\nMainTextExtractor: pizdec blyat, aborted";
                abort();
                }
            if (wxDirExists(dir_to_remove)) wxFileName::Rmdir( dir_to_remove,wxPATH_RMDIR_RECURSIVE);
            }
        return return_code;
        }
    }





bool MainTextExtractor::extractors_working()
    {
    empty_apps.clear();
    empty_apps_helper.clear();
    std::string FileSepStd=wxString(wxFileName::GetPathSeparator()).ToStdString();
    std::string path_to_test_samples="full_text_search" +  FileSepStd+ "test_extractors"+ FileSepStd;


    TextFromFileExtractor_parent *cur_extractor;
    bool all_of_the_extractors_are_working=true;
    bool we_need_install_something=false;

    for (unsigned int i=0;i<text_extractors_list.size();i++)
        {
        std::string extension=text_extractors_list[i].first;
        cur_extractor= text_extractors_list[i].second;
        std::string path_to_test_sample=path_to_test_samples+extension+FileSepStd+"test."+ extension;
        std::string path_to_output=     path_to_test_samples+extension+FileSepStd+"substring.txt";



        if(!cur_extractor->IfWorking(path_to_test_sample,path_to_output))
           {
            all_of_the_extractors_are_working=false;
            for (unsigned int j=0;j<cur_extractor->external_applications.size();j++)
                {
                if(cur_extractor->external_applications[j].second=="")
                    {
                    std::string missing_app=cur_extractor->external_applications[j].first;
                    we_need_install_something=true;
                    if (empty_apps_helper.count(missing_app) == 0)
                        {
                        empty_apps.push_back(std::make_pair(missing_app, ""));
                        empty_apps_helper.insert(std::make_pair(missing_app, empty_apps.size()-1));
                        }
                    }
                }
            }
        }



    ArchiveExtractorByCommand *archive_extractor;
    for (unsigned int i=0;i<archive_extractors_list.size();i++)
        {
        std::string extension=archive_extractors_list[i].first;
        archive_extractor= archive_extractors_list[i].second;
        std::string path_to_test_sample=path_to_test_samples+extension+FileSepStd+"test."+ extension;
        std::string path_to_output=     path_to_test_samples+extension+FileSepStd+"md5.txt";


        bool archive_extractor_working=
            archive_extractor->IfWorking(path_to_test_sample,temp_dir_with_last_separator+"testing", path_to_output);
            ArchiveExtractorByCommand::remove_dir_if_exists(temp_dir_with_last_separator+"testing");


        if(!archive_extractor_working)
           {
            all_of_the_extractors_are_working=false;
            for (unsigned int j=0;j<archive_extractor->external_applications.size();j++)
                {
                if(archive_extractor->external_applications[j].second=="")
                    {
                    std::string missing_app=archive_extractor->external_applications[j].first;
                    we_need_install_something=true;
                    if (empty_apps_helper.count(missing_app) == 0)
                        {
                        empty_apps.push_back(std::make_pair(missing_app, ""));
                        empty_apps_helper.insert(std::make_pair(missing_app, empty_apps.size()-1));
                        }
                    }
                }
            }
        }
    if(we_need_install_something)
        {
        if(!this->ask_user_to_install_something())
            {
            return false;
            }
        return this->extractors_working();
        }
    else if(!all_of_the_extractors_are_working)
        {
        return false;
        }
    else
        {
        return true;
        }
    }
bool MainTextExtractor::ask_user_to_install_something()
    {
    Khz::FTS::Gui::SelectAppsToExtractTextWindow *entries_dialog = new Khz::FTS::Gui::SelectAppsToExtractTextWindow(NULL, wxID_ANY, "Select apps", empty_apps);
    if(entries_dialog->ShowModal()==wxID_CANCEL)
        {
        delete entries_dialog;
        return false;
        }
    else
        {
        int temp_size = empty_apps.size();
        std::vector<std::pair<std::string, std::string> > temp_v=empty_apps;
        empty_apps.clear();
        for(int ksi=0;ksi<temp_size;ksi++)
            {
             empty_apps.push_back(std::make_pair(temp_v[ksi].first, entries_dialog->pointers_to_entries[ksi]->GetValue().ToStdString()));
            }
        TextFromFileExtractor_parent *cur_extractor;
        for (unsigned int i=0;i<text_extractors_list.size();i++)
            {
            cur_extractor= text_extractors_list[i].second;
            for (unsigned int j=0;j<cur_extractor->external_applications.size();j++)
                {
                if(cur_extractor->external_applications[j].second=="")
                    {
                    std::string missing_app=cur_extractor->external_applications[j].first;
                    int num =empty_apps_helper[missing_app];
                    cur_extractor->external_applications[j].second = empty_apps[num].second;
                    }
                }
            }
        cur_extractor=NULL;
        ArchiveExtractorByCommand *archive_extractor;
        for (unsigned int i=0;i<archive_extractors_list.size();i++)
            {
            archive_extractor= archive_extractors_list[i].second;
            for (unsigned int j=0;j<archive_extractor->external_applications.size();j++)
                {
                if(archive_extractor->external_applications[j].second=="")
                    {
                    std::string missing_app=archive_extractor->external_applications[j].first;
                    int num =empty_apps_helper[missing_app];
                    archive_extractor->external_applications[j].second = empty_apps[num].second;
                    }
                }
            }
        delete entries_dialog;
        return true;
        }
    }


void MainTextExtractor::gen_list_of_files_in_dir
    (std::string full_path_to_dir, std::vector<std::pair<std::string, std::string>> &output)
    {
    //if(!wxDirExists(wxString(full_path_to_dir))) return;
    wxArrayString files;
    wxDir::GetAllFiles 	( wxString(full_path_to_dir),&files);
    for(unsigned int i=0; i<files.GetCount(); i++)
        {
        wxFileName temp(files.Item(i));
        std::string extension= temp.GetExt().ToStdString();
        std::transform(extension.begin(),extension.end(), extension.begin(), ::tolower); //convert to lower case
        output.push_back(std::make_pair(files.Item(i).ToStdString(), extension));
        }
    }

bool MainTextExtractor::exctact_archive_recursivelely_if_necessary
    (std::string output_dir_with_last_separator,
     std::string unique_id,
     std::pair<std::string,std::string> input_file,
     std::vector<std::pair<std::string, std::string>> &produced_files,
     std::vector<std::string> &produced_dirs
    )
    {
    bool extracted_something=false;
    std::string extension=get_extension_from_doulbe_extension(input_file.second); // input_file.second is always lowercase
    for(unsigned i=0;i<archive_extractors_list.size();i++)
        {
        if (extension!=archive_extractors_list[i].first) continue;
        std::string output_dir=output_dir_with_last_separator+unique_id;
        (archive_extractors_list[i].second)->try_to_extract(input_file.first, output_dir);
        std::vector<std::pair<std::string, std::string>> files_list;
        if(wxDirExists(wxString(output_dir)))
            {
            gen_list_of_files_in_dir(output_dir,files_list); //extension part of files_list is lowercase
            produced_dirs.push_back(output_dir);
            }
        else
            {
            return false;
            }
        for(unsigned j=0;j<files_list.size();j++)
            {
            extracted_something=true;  // we get here only if files_list is not empty
            if( !exctact_archive_recursivelely_if_necessary
                (output_dir_with_last_separator,
                unique_id+"_"+Khz::Libs::Mstr::IntToString(j),
                files_list[j],
                produced_files,
                produced_dirs )
                )
                {
                produced_files.push_back(files_list[j]);
                }
            }
        break;
        }
    return extracted_something;
    }

std::string MainTextExtractor::get_extension_from_doulbe_extension(const std::string &ext)
    {
    std::vector<std::string> exploded = Khz::Libs::Mstr::explode(".", ext);
    if(exploded.size()==0)
        {
        std::cout << "MainTextEctractor::get_extension_from_doulbe_extension: something terrible happened, aborted\n";
        abort();
        }
    else if(exploded.size()==1)
        {
        return ext;
        }
    else
        {
        return exploded[exploded.size()-1]; //last
        }
    }



}}}// namespaces
