/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include "TextFromFileExtractorByCommand.h"
#include <boost/regex.hpp>
#include "string_functions.h"
#include <wx/filefn.h>
#include <wx/filename.h>
#include <wx/msgdlg.h>
#include <fstream>
#include "DEBUGMESSAGE.h"
#include "platform_specific_defs.h"
namespace Khz{ namespace FTS { namespace Engine{


TextFromFileExtractorByCommand::TextFromFileExtractorByCommand(std::string command)
// command is something like "%edjvutxt %f", %e is executable, %f is filename(full path)
:TextFromFileExtractor_parent()
    {
    using namespace boost;
    smatch res;
    regex rx("%e(\\w+)\\s+",regex_constants::icase);
    if (regex_search(command,res, rx))
        {
        external_applications.push_back(std::make_pair(res.str(1), res.str(1)));
        pattern=regex_replace(command, rx, "%e ", regex_constants::format_first_only);
        }
    else
        {
        external_applications.push_back(std::make_pair("unknown",""));
        pattern="";
        }
    }


std::string TextFromFileExtractorByCommand::construct_full_command(
            std::string path_to_file, std::string executable)
    {
    if (pattern=="") return "";

    std::string output=pattern;
    Khz::Libs::Mstr::replaceAll(output, "%e", "\""+executable+"\"");
    Khz::Libs::Mstr::replaceAll(output, "%f", "\""+path_to_file+"\"");

	#ifdef _WIN32
	return "\"" +output +"\"";
	#endif
    return output;
    }


bool TextFromFileExtractorByCommand::extract_text (std::string filename)
    {
    try
        {
        ////////////////////    Check if file is readable. If not than show error mesaage and return false
        if(  !wxFileExists(filename)  or   !wxFileName::IsFileReadable(filename)  )
            {
            wxString message="Cannot read  \"" + filename + "\". Check that you have permission to read this file.";
            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
            }
        //////////////////    Ok, check completed


        ////////////////////////////////////////////////////////////////////////////////
        //                 NOW extracting text from file
        ////////////////////////////////////////////////////////////////////////////////
        std::string cmd =construct_full_command(filename, external_applications[0].second);

        // DO NOT FORGET TO CLEAR THE PREVIOUS RESULT
        this->result_of_text_extraction="";

        if(!this->ExecCommandAndSaveResults(cmd))
            {
            return false;
            }
        else
            {
            return true;
            }
        }
    catch(...)
        {
        std::cout << "\nUnknown error occured when processing  \"" + filename + "\". Contact the developers please.\n";
        return false;
        }
    }



bool TextFromFileExtractorByCommand::IfWorking (std::string test_file, std::string file_with_expected_result)
    {
    if(!FileExist(test_file)) return false;
    if(!FileExist(file_with_expected_result)) return false;

    ////////////////////////////////////////////////////////////////////////////
    //            Extract data from djvu file and save results
    /////////////////////////////////////////////////////////////////////////////
    extract_text(test_file);

    //////////////////////////////////////////////////////////////////////////
    // Reading file_with_expected_result, put data to the text_in_txt_file string
    ////////////////////////////////////////////////////////////////////////////

    std::ifstream tmp_file;
    tmp_file.open(file_with_expected_result);
    if(!tmp_file)
        {
          wxString message= "DjvuTextEctractor_djvutxt: cannot read test samples. One or both of the files \""+test_file+"\", \""+
            file_with_expected_result +"\" cannot be read";

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
        }

    std::string text_in_txt_file="";
    std::string temp_str;
    while (std::getline(tmp_file, temp_str))
    {
    text_in_txt_file=text_in_txt_file+temp_str;
    }
    Khz::Libs::Mstr::replaceAll(text_in_txt_file, " ", "");
    Khz::Libs::Mstr::replaceAll(text_in_txt_file, "\r", "");
    Khz::Libs::Mstr::replaceAll(text_in_txt_file, "\n", "");
    ////////////////////////////////////////////////////////////////////////////
    //    Ok, finished with file_with_expected_result
    /////////////////////////////////////////////////////////////////////////////



    /////////////////////////////////////////////////////////////////////////////////
    //   Compare the data
    /////////////////////////////////////////////////////////////////////////////////
    std::string temp_string = this->result_of_text_extraction;
    Khz::Libs::Mstr::replaceAll(temp_string, " ", "");
    Khz::Libs::Mstr::replaceAll(temp_string, "\r", "");
    Khz::Libs::Mstr::replaceAll(temp_string, "\n", "");

        std::cout << "\n#########################################################\n";
        std::cout << "#########################################################\n";
        std::cout << "CHECKIGN IF TEXT EXTRACTORS ARE WORKING\n";
        std::cout << "From " << test_file << " i have extracted the following string:\n";
        std::cout << wxString::FromUTF8(temp_string.c_str()) << "\n\n\n";
        std::cout << "The sample string is:\n\n";
        std::cout <<  wxString::FromUTF8(text_in_txt_file.c_str());
        std::cout << "\n#########################################################\n";
        std::cout << "#########################################################\n";
    //std::cout << "res1 is \n"<< result_of_text_extraction << "\n";
    //std::cout << "original is \n" << text_in_txt_file << "\n";
    if(text_in_txt_file=="")
        {
        return false;
        }
    else if(temp_string.find(text_in_txt_file)==std::string::npos) //extractor not working
        {
         external_applications[0]=std::make_pair(external_applications[0].first, "");
         return false;
        }
    else //extractor working
        {
        return true;
        }
    }

}}}// namespaces
