/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef KOLKHOZNIK_TEXTFROMFILEXTRACTOR_PARENT_HEADER_INCLUDED
#define KOLKHOZNIK_TEXTFROMFILEXTRACTOR_PARENT_HEADER_INCLUDED

#include <iostream>
#include <vector>
#include <cstdlib>


namespace Khz{ namespace FTS { namespace Engine{


/**
 @brief The every file text extractor must be inherited from this class.
**/
class TextFromFileExtractor_parent
{
public:
    /**
    constructor
    **/
    TextFromFileExtractor_parent()
        {
        this->result_of_text_extraction="";
        }
    virtual ~TextFromFileExtractor_parent(){};


    /**
    Returns pointer to the extracted text.
    This function code is:
    @code
    std::string * GetPtrToExtractedText()
        {
        return &result_of_text_extraction;
        }
    @endcode
    **/
    std::string * GetPtrToExtractedText()
        {
        return &result_of_text_extraction;
        }


    /**
    Checks if the extractor is working.
    This function extracts a text layer from \a test_file and compares it with
    the content of the \a  file_with_expected_result.
    If content of the \a file_with_expected_result is substring
    of the extracted text, than this function returns true, false otherwise.
    @param test_file full path to the sample file.
    @param file_with_expected_result full path to the .txt file with expected result.

    @note If you are sure that your extractor is going to work always, than you should define this function like this:
    @code
    virtual bool IfWorking(test_file, expected_result)
        {
        return true;
        }
    @endcode
    **/
    virtual bool IfWorking(std::string test_file, std::string file_with_expected_result)
        {
        std::cout << "\npisdec calling base IfWorking\n";
        abort();
        return false;
        };

    /**
    Extracts text from \p filename.

    The result should be saved to #result_of_text_extraction.
    To get extracted text we should use #GetPtrToExtractedText ().
    @note each time when this function gets called, it overwrites #result_of_text_extraction
    @param filename full path to the file
    @return true if there is no errors, false otherwise
    **/
    virtual bool extract_text(std::string filename)=0;



    /**
    The same as #Khz::FTS::Engine::ArchiveExtractorByCommand::external_applications
    **/
    std::vector<std::pair<std::string, std::string>> external_applications;
protected:

    /**
    The result of the text extraction
    **/
    std::string result_of_text_extraction;


    /**
    executes external command and saves it's output to #result_of_text_extraction
    **/
    bool ExecCommandAndSaveResults(std::string cmd);

    /**
    checks if the file exists.
    If file does not exists, than shows messagebox explaining the problem.
    **/
    bool FileExist(std::string file);


};



}}}// namespaces

#endif
