/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include <wx/frame.h>
#include <wx/spinctrl.h>
#include <wx/textctrl.h>
#include <wx/button.h>
#include <wx/stdpaths.h>
#include <wx/filename.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/progdlg.h>
#include <wx/fileconf.h>
#include <wx/msgdlg.h>
#include <wx/dirdlg.h>
#include <map>
#include <vector>
#include "sqlite3.h"
#include "FullTextSearchIndexerWindow.h"
#include <boost/thread.hpp>
#include <boost/function.hpp>
#include "MySizeFunctions.h"
#include <wx/filectrl.h>
#include <wx/filedlg.h>
//#include "MyFileDialog.h"
#include <wx/log.h>
#include <wx/dir.h>
#include "ConfigManager.h"
#include "Custom_IDs.h"
#include <wx/panel.h>
namespace Khz{ namespace FTS { namespace Gui{


FullTextSearchIndexerWindow::FullTextSearchIndexerWindow(wxWindow *parent, const wxString& title)
    : wxFrame(parent, wxID_ANY, title)
    {
    we_would_like_to_stop_indexing=false;
    callback_update_progress=boost::bind(&Khz::FTS::Gui::FullTextSearchIndexerWindow::UpdateProgressDialogCallBack, this, _1, _2, _3);
    callback_finish_indexing=boost::bind(&Khz::FTS::Gui::FullTextSearchIndexerWindow::FinishIndexingCallBAck, this);
    wxSetlocale(LC_ALL, ""); //this is needed to non-latin characters support in paths;
    ////////////////////////////////////////////////////////////////////////////////////////////////
    // Let us look into config file end get path to the main kolkhoznik database
    // and path to the directory with books
    ////////////////////////////////////////////////////////////////////////////////////////////////
    wxString ini_filename = Khz::ConfigManager::GetPathToConfigFile();
    config =new  wxFileConfig( "", "", ini_filename);
    wxString path_to_library = config->Read( wxT("PATHS/path_to_dir_with_books"), ""); // the second parameter is a default value
    wxString path_to_main_db = config->Read( wxT("PATHS/path_to_database"),"");



	wxPanel *main_panel = new wxPanel(this, wxID_ANY);
    // ##### Threads number input
    threads_number_input = new wxSpinCtrl(main_panel, wxID_ANY);
    threads_number_input->SetRange(1,10000000);
    wxHyperlinkCtrl *what_is_threads_num_hyperlink=new wxHyperlinkCtrl(main_panel, wxID_ANY, "what is this?", "what is this?");


    cache_size_input= new wxSpinCtrl(main_panel, wxID_ANY);
    cache_size_input->SetRange(1,10000000);
    cache_size_input->SetValue(1000);
    wxHyperlinkCtrl *what_is_cache_size_hyperlink=new wxHyperlinkCtrl(main_panel, wxID_ANY, "what is this?", "what is this?");




   // #########PATH TO LIBRARY########################################
    library_dir_input=new wxTextCtrl(main_panel,wxID_ANY,path_to_library, wxDefaultPosition);

     wxButton *choose_path_to_library_button = new wxButton(main_panel,wxID_ANY, "Choose");
    // ######################################################################################



       // #########PATH TO xapian database (empty dir)########################################
    xapian_database_path_input=new wxTextCtrl(main_panel,wxID_ANY,"");

    wxButton *choose_path_to_xapian_button = new wxButton(main_panel,wxID_ANY, "Choose");
    // ######################################################################################


        // ####### PATH TO THE log otput #################################################
    where_to_store_log_input = new wxTextCtrl(main_panel,wxID_ANY,
                                             wxStandardPaths::Get().GetUserConfigDir() + wxFileName::GetPathSeparator() + "."
                                              + "kolkhoznik" + wxFileName::GetPathSeparator() + "xapian_database_creation_log.txt" , wxDefaultPosition);
    choose_path_to_log_button = new wxButton(main_panel,wxID_ANY, "Choose");
    //######################################################################




    // ####### PATH TO DATABASE input #############################################
   the_main_database_path_input = new wxTextCtrl(main_panel, wxID_ANY, path_to_main_db, wxDefaultPosition);
    wxButton *choose_path_to_database_button = new wxButton(main_panel,wxID_ANY, "Choose");
    //######################################################################



    temp_dir_input=new wxTextCtrl(main_panel, wxID_ANY, "");
    wxButton *choose_temp_dir_button=new wxButton(main_panel, wxID_ANY, "Choose");


      //ok cancel buttonschoose_path_to_database_button
    ok_button=new wxButton(main_panel, wxID_OK, "Generate");
    cancel_button=new wxButton(main_panel, wxID_CANCEL);




    wxBoxSizer *only_sizer = new wxBoxSizer (wxVERTICAL);
    //topic radiobutton
        wxBoxSizer *library_input_vertical_sizer= new wxBoxSizer(wxVERTICAL);
        library_input_vertical_sizer->Add( new wxStaticText( main_panel, wxID_ANY, wxT("Specify the kolkhoz library directory")),0, wxALL);
            wxBoxSizer *input_for_library_path_sizer= new wxBoxSizer( wxHORIZONTAL );
                input_for_library_path_sizer->Add(library_dir_input,1, wxCENTRE | wxALL);
                input_for_library_path_sizer->Add(choose_path_to_library_button);
        library_input_vertical_sizer->Add(input_for_library_path_sizer,0, wxALL|wxEXPAND);
    only_sizer->Add(library_input_vertical_sizer, 0, wxALL|wxEXPAND, 5);
    only_sizer->AddSpacer(20);
        wxBoxSizer *database_input_vertical_sizer= new wxBoxSizer(wxVERTICAL);
        database_input_vertical_sizer->Add( new wxStaticText( main_panel, wxID_ANY, wxT("Specify the main kolkhoznik database")),0, wxALL);
            wxBoxSizer *input_for_database_path_sizer= new wxBoxSizer( wxHORIZONTAL );
                input_for_database_path_sizer->Add(the_main_database_path_input,1, wxCENTRE | wxALL);
                input_for_database_path_sizer->Add(choose_path_to_database_button);
        database_input_vertical_sizer->Add(input_for_database_path_sizer,1, wxALL|wxEXPAND);
    only_sizer->Add(database_input_vertical_sizer, 0, wxALL|wxEXPAND, 5);
    only_sizer->AddSpacer(20);
        wxBoxSizer *log_input_vertical_sizer= new wxBoxSizer(wxVERTICAL);
        log_input_vertical_sizer->Add( new wxStaticText( main_panel, wxID_ANY, wxT("Specify where to save the log file")),0, wxALL);
            wxBoxSizer *input_for_log_path_sizer= new wxBoxSizer( wxHORIZONTAL );
                input_for_log_path_sizer->Add(where_to_store_log_input,1, wxCENTRE | wxALL);
                input_for_log_path_sizer->Add(choose_path_to_log_button);
        log_input_vertical_sizer->Add(input_for_log_path_sizer,1, wxALL|wxEXPAND);
    only_sizer->Add(log_input_vertical_sizer, 0, wxALL|wxEXPAND, 5);
    only_sizer->AddSpacer(20);
        wxBoxSizer *temp_dir_sizer=new wxBoxSizer(wxVERTICAL);
        temp_dir_sizer->Add( new wxStaticText( main_panel, wxID_ANY, wxT("Specify a EMPTY!!!! directory for temporary files")),0, wxALL);
                wxBoxSizer *temp_dir_input_sizer=new wxBoxSizer(wxHORIZONTAL);
                temp_dir_input_sizer->Add(temp_dir_input, 1, wxCENTRE|wxALL);
                temp_dir_input_sizer->Add(choose_temp_dir_button);
        temp_dir_sizer->Add(temp_dir_input_sizer,1, wxALL|wxEXPAND);
    only_sizer->Add(temp_dir_sizer, 0, wxALL|wxEXPAND, 5);
    only_sizer->AddSpacer(20);
        wxBoxSizer *xapian_database_vertical_sizer= new wxBoxSizer(wxVERTICAL);
        xapian_database_vertical_sizer->Add(new wxStaticText(main_panel, wxID_ANY, wxT("Specify a SEPARATE!!! directory for the full text search database")));
            wxBoxSizer *input_for_xapian_sizer = new wxBoxSizer(wxHORIZONTAL);
            input_for_xapian_sizer->Add(xapian_database_path_input, 1, wxCENTRE, wxALL);
            input_for_xapian_sizer->Add(choose_path_to_xapian_button);
        xapian_database_vertical_sizer->Add(input_for_xapian_sizer, 1, wxEXPAND|wxALL);
    only_sizer->Add(xapian_database_vertical_sizer, 0, wxALL|wxEXPAND, 5);
    only_sizer->AddSpacer(20);



    //    wxBoxSizer *threads_number_sizer = new wxBoxSizer(wxHORIZONTAL);
    //        threads_number_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "Indexing threads Number:"), 0, wxALL|wxEXPAND, 5);
    //        threads_number_sizer->Add(threads_number_input, 0, wxALL, 5);
    //only_sizer->Add(threads_number_sizer, 0, wxALL, 5);
    //    wxBoxSizer *cache_size_sizer=new wxBoxSizer(wxHORIZONTAL);
    //        cache_size_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "Cache size (book number):"), 0, wxALL|wxEXPAND, 5);
    //        cache_size_sizer->Add(cache_size_input, 0, wxALL, 5);
    //only_sizer->Add(cache_size_sizer, 0, wxALL, 5);

        wxGridSizer *spin_sizer=new wxGridSizer(2, 3, 5, 5);
            spin_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "Indexing threads Number:"), 0, wxALL|wxALIGN_CENTER );
            spin_sizer->Add(threads_number_input, 0, wxALL|wxALIGN_CENTER );
            spin_sizer->Add(what_is_threads_num_hyperlink,0, wxALL);
            spin_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "Cache size (book's number):"), 0, wxALL|wxALIGN_CENTER);
            spin_sizer->Add(cache_size_input, 0, wxALL|wxALIGN_CENTER);
            spin_sizer->Add(what_is_cache_size_hyperlink,0, wxALL);
    only_sizer->Add(spin_sizer, 0, wxALL, 5);


    //only_sizer->AddSpacer(20);
    only_sizer->AddStretchSpacer();
    //ok cancel buttons
        wxBoxSizer *buttons_sizer=new wxBoxSizer(wxHORIZONTAL);
        buttons_sizer->Add(cancel_button,0,wxALL);
        buttons_sizer->AddSpacer(3);
        buttons_sizer->Add(ok_button,0, wxALL);
    only_sizer->Add(buttons_sizer,0,wxCENTER|wxALL,5);
    only_sizer->AddSpacer(5);

      // this->SetInitialSize();
    main_panel->SetSizerAndFit(only_sizer);


	wxBoxSizer *top_sizer=new wxBoxSizer(wxVERTICAL);
	top_sizer->Add(main_panel, 1, wxALL|wxEXPAND);
	SetSizerAndFit(top_sizer);
	top_sizer->RecalcSizes();



    ok_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::Index, this);
    cancel_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::OnCancelPressed, this);

    choose_path_to_library_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::DoKolhozPathChoose, this);
    choose_path_to_database_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::DoKolhozDatabasePathChoose, this);
    choose_path_to_log_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::DoKolhozLogPathChoose, this);
    choose_path_to_xapian_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::DoXapianDatabasePathChoose, this);
    choose_temp_dir_button->Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::DoTempDirPathChoose, this);
    Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::UpdateProgressBar, this, my_wx_id__BOOK_GOT_FULL_TEXT_SEARCH_INDEXED_EVENT);
    Bind(wxEVT_BUTTON, &FullTextSearchIndexerWindow::FinishIndexing, this, my_wx_id__FULL_TEXT_SEARCH_INDEXING_FINISHED);
    what_is_threads_num_hyperlink->Bind
        (wxEVT_HYPERLINK, &FullTextSearchIndexerWindow::OnWhatIsThreadsNumberHyperlinkClicked, this);
    what_is_cache_size_hyperlink->Bind
        (wxEVT_HYPERLINK, &FullTextSearchIndexerWindow::OnWhatIsCacheSizeHyperlinkClicked, this);



    this->Show();
    this->SetSize(wxSize(Khz::Libs::MySizeFunctions::get_width_of_number_of_text_symbols(130, this), -1));
    show_info_message_showing_which_filetypes_we_can_index();
    }




void FullTextSearchIndexerWindow::Index(wxCommandEvent &event)
    {
    we_would_like_to_stop_indexing=false;
    threads_number = threads_number_input->GetValue();


    ////////////////////////////////////////////////////////////////////////////
    // path to the main kolkhoznik database and path to the dir with books, path
    // to the xapian database
    //
    //we check that everything is good
    /////////////////////////////////////////////////////////////////////////////
    wxString path_to_library=library_dir_input->GetValue();
    wxString path_to_database=the_main_database_path_input->GetValue();
    wxString temp_dir=temp_dir_input->GetValue();
    std::string path_to_log=where_to_store_log_input->GetValue().ToStdString();
    wxString xapian_path=xapian_database_path_input->GetValue();
    if( !wxDirExists(path_to_library)  or   !wxFileName::IsDirReadable(path_to_library) )
        {
        wxMessageDialog   err_msg (this,
                                    "Cannot read the Library directory \"" + path_to_library + "\". Check that this directory exists and you have permission to read it.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg.ShowModal ();
        return;
        }
    if (  !wxFileExists(path_to_database)    )
        {
                wxMessageDialog   err_msg(this,
                path_to_database+ ": file not found",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg.ShowModal ();
        return;
        }
     if(wxFileExists(wxString(path_to_log)) )
        {
        wxMessageDialog err_msg(this,
                                    "The specified log file already exists. Overwrite?",
                                    "Error", wxYES_NO|wxCENTRE|wxICON_INFORMATION|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition);


        switch ( err_msg.ShowModal() )
            {
            case wxID_YES: // we will overwrite it later, just go on
                break;

            case wxID_NO:
                return;
                break;
            default:
                wxLogError(wxT("Unexpected wxMessageDialog return code!"));
                return;
            }
        }
     if(!wxDirExists(xapian_path) or !wxFileName::IsDirWritable(xapian_path))
        {
        wxMessageDialog   err_msg (this,
                                   "Error: "+ xapian_path + " is not writable",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg.ShowModal ();
        return;
        }
    if(!wxDirExists(temp_dir) or !wxFileName::IsDirWritable(temp_dir))
        {
        wxMessageDialog   err_msg (this,
                                   "Error: "+ temp_dir + " is not writable",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg.ShowModal ();
        return;
        }
    if(wxDirExists(temp_dir))
        {
        wxDir dir(temp_dir);
        if(dir.HasFiles() or dir.HasSubDirs() )
            {
            wxMessageDialog   err_msg (this,
                                   "Error: "+ temp_dir + " is not empty",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
            err_msg.ShowModal ();
            return;
            }
        }
    if(wxDirExists(xapian_path))
        {
        wxDir dir(xapian_path);
        if( dir.HasFiles() or dir.HasSubDirs() )
            {
            wxString message= xapian_path + " is not empty. It probably means that you have some xapian "+
                                   "database there. You can remove it and generate a completely new database or you can "+
                                   "make an incremental index. If making the incremental index than the only files that "
                                   "are not indexed are going to be indexed. We will skip files that are already indexed."+
                                   "\n\nP.S. if this directory contains some random files "+
                                   "(I.E. this is not a full text search database) than it is STRICTLY recommended to "
                                   "remove all of the files from this directory. "+
                                   "\n\n\nWhat do you want to do?";

            wxMessageDialog   err_msg (this,"",
                                    "?", wxYES_NO|wxCANCEL|wxCENTRE|wxICON_QUESTION|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
            err_msg.SetYesNoLabels("Remove everything from the directory", "Incremental index");
            err_msg.SetMessage(message);
            //err_msg.GetSizer()->RecalcSizes();
            //err_msg.GetSizer()->Layout();


            switch ( err_msg.ShowModal() )
                {
                case wxID_YES: //remove content
                    {
                    dir.Close();
                    if(!wxFileName::Rmdir( xapian_path,wxPATH_RMDIR_RECURSIVE))
                        {
                        ///////////////////////////////////////////////
                        // try to remove files
                        /////////////////////////////////////////////
                        wxDir dir2(xapian_path);
                        if(dir2.IsOpened())
                            {
                            wxString filename;
                            bool cont=dir2.GetFirst(&filename);
                            std::vector<wxString> files;
                            while (cont)
                                {
                                files.push_back(filename);
                                cont=dir.GetNext(&filename);
                                }
                            dir2.Close();
                            for (unsigned int i=0;i<files.size();i++)
                                {
                                wxRemoveFile(files[i]);
                                }
                            }
                        wxDir dir3(xapian_path);
                        if( dir3.HasFiles() or dir3.HasSubDirs() )
                            {
                            wxString message="Cannot remove content of \""+xapian_path+"\". Please remove it manually and try again" +
                            "\n\nP.S. If the kolkhoznik full text search window is open, than close it first";
                            wxMessageBox(message, wxT("Error"), wxOK );
                            return;
                            }
                        else
                            {
                            break;
                            }
                        }
                    else
                        {
                        wxFileName::Mkdir(xapian_path);
                        }
                    if(!wxDirExists(xapian_path) or !wxFileName::IsDirWritable(xapian_path))
                        {
                        wxMessageDialog   err_msg (this,
                                   "Error: "+ xapian_path + " is not writable",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
                        err_msg.ShowModal ();
                        return;
                        }
                    break;
                    }
                case wxID_NO: //nothing to do
                    {
                    break;
                    }
                case wxID_CANCEL:
                    {
                    return;
                    }
                default:
                    {
                    wxLogError(wxT("Unexpected wxMessageDialog return code!"));
                    return;
                    }
                }
            }
        }
    ///////////////////////////////////////////////////////////////////////////////////
    // finished with paths
    //////////////////////////////////////////////////////////////////////////////////////////




    ////////////////////////////////////////////////////////////////////////////////
    // Now the indexing itself
    ///////////////////////////////////////////////////////////////////////////////
    text_indexer= new Khz::FTS::Engine::FullTextSearchIndexer
                                    (
                                    xapian_path.ToStdString(),
                                    path_to_log,
                                    path_to_database.ToStdString(),
                                    path_to_library.ToStdString(),
                                    gen_text_extractors_commands(),
                                    gen_archive_extractors_commands(),
                                    temp_dir.ToStdString(),
                                    callback_update_progress, // goes by reference
                                    callback_finish_indexing,  //goes by reference
                                    cache_size_input->GetValue()
                                    );

    //std::cout << "\n text indexer created\n";
    if(!text_indexer->AreTextExtractorsWorking())
        {
        delete text_indexer;
        return;
        }
    if(!text_indexer->log_ok)
        {
        delete text_indexer;
        wxMessageBox("Cannot open the specified log file", wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }
    if(!text_indexer->xapian_ok)
        {
        delete text_indexer;
        wxMessageBox("Cannot open the specified full text search index database", wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }
    if(text_indexer->unknown_error)
        {
        delete text_indexer;
        wxString message= "Unknown error occured. Check that just one instance of the indexer is running and that you have enough free space on your hard drive";
         wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }

    progress_bar=new MyProgressDialog(this, wxID_ANY, "Index generation progress", "The full text search index is being generated");
    this->Disable();









    for (int i=0; i< threads_number; i++)
        {
        indexing_threads.push_back
            (
            new boost::thread
                (
                &Khz::FTS::Engine::FullTextSearchIndexer::GetBookFromQueueAndIndexIt, text_indexer
                )
            );

        text_indexer->Incr_num_of_currently_working_threads();
        }

    queue_prepare_thread= new boost::thread(&Khz::FTS::Engine::FullTextSearchIndexer::FillQueueOfBooksToIndex, text_indexer);
    }


void FullTextSearchIndexerWindow::OnCancelPressed(wxCommandEvent& event)
    {
    this->Destroy();
    event.Skip();
    }

void FullTextSearchIndexerWindow::DoKolhozPathChoose(wxCommandEvent& event)
{
    // pass some initial dir to wxDirDialog
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxDirDialog dialog(this, wxT("Select Kolkhoz library directory"), dirHome, wxDD_DEFAULT_STYLE && wxDD_DIR_MUST_EXIST);

    if (dialog.ShowModal() == wxID_OK)
    {
        library_dir_input->SetValue (dialog.GetPath());
    }
}

void FullTextSearchIndexerWindow::DoKolhozDatabasePathChoose(wxCommandEvent &event)
{
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxFileDialog dlg(this, "Select the main kolkhoznik database",dirHome, "database.bib",
                      "", wxFD_OPEN, wxDefaultPosition);
    if(dlg.ShowModal()==wxID_CANCEL) return;

    wxString path= dlg.GetPath();
    the_main_database_path_input->SetValue(path);
}


void FullTextSearchIndexerWindow::DoKolhozLogPathChoose(wxCommandEvent &event)
{
    wxString home_dir;
    wxGetHomeDir(&home_dir);
    wxFileDialog dlg(this, "Choose where to store the log file",
                     home_dir, "xapian_database_creation_log.txt",
                     "log files (*.log)|*.log", wxFD_SAVE, wxDefaultPosition);
    if(dlg.ShowModal()==wxID_CANCEL) return;

    wxString path= dlg.GetPath();
    //std::cout << "\nfull path "<<  path.ToStdString().c_str();
    //std::cout << "\n filename " << dlg.file_ctrl->GetFilename();
    where_to_store_log_input->SetValue(path);
}




void FullTextSearchIndexerWindow::DoXapianDatabasePathChoose(wxCommandEvent &event)
    {
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxDirDialog dialog(this, wxT("Select a directory for the full text search database"), dirHome, wxDD_DEFAULT_STYLE);

    if (dialog.ShowModal() == wxID_OK)
        {
        xapian_database_path_input->SetValue (dialog.GetPath());
        }
    }


void FullTextSearchIndexerWindow::DoTempDirPathChoose(wxCommandEvent &event)
    {
     wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxDirDialog dialog(this, wxT("Select a empty dir for temporary files"), dirHome, wxDD_DEFAULT_STYLE);

    if (dialog.ShowModal() == wxID_OK)
        {
        temp_dir_input->SetValue (dialog.GetPath());
        }
    }


std::vector<std::pair<std::string, std::string>> FullTextSearchIndexerWindow::gen_text_extractors_commands()
    {
    ///////////////////////////////////////////////////////////////////////////////////////
    // just in case if user changes config params while the indexing window is open.
    wxString ini_filename = Khz::ConfigManager::GetPathToConfigFile();
    delete config;
    config =new wxFileConfig( "", "", ini_filename);
    //////////////////////////////////////////////////////////////////////////////////////

    long dummy;
    wxString temp_str;
    std::vector<wxString> array_of_extensions;

    wxString strOldPath = config->GetPath();
    //#################################################################
    config->SetPath("FULL_TEXT_SEARCH_TEXT_EXTRACTORS");

    bool GCont = config->GetFirstEntry(temp_str, dummy);
    while ( GCont )
        {
        array_of_extensions.push_back(temp_str);
        GCont = config->GetNextEntry(temp_str, dummy);
        }

    config->SetPath(strOldPath);



    std::vector<std::pair<std::string, std::string>> output;
    for(unsigned int i=0;i<array_of_extensions.size();i++)
        {
        std::string command=config->Read("FULL_TEXT_SEARCH_TEXT_EXTRACTORS/"+array_of_extensions[i], "").ToStdString();
        output.push_back(std::make_pair(array_of_extensions[i].ToStdString(), command));
        }

    return output;
    }





std::vector<std::pair<std::string, std::string>> FullTextSearchIndexerWindow::gen_archive_extractors_commands()
    {
    ///////////////////////////////////////////////////////////////////////////////////////
    // just in case if user changes config params while the indexing window is open.
    wxString ini_filename = Khz::ConfigManager::GetPathToConfigFile();
    delete config;
    config =new wxFileConfig( "", "", ini_filename);
    //////////////////////////////////////////////////////////////////////////////////////

    long dummy;
    wxString temp_str;
    std::vector<wxString> array_of_extensions;

    wxString strOldPath = config->GetPath();
    //#################################################################
    config->SetPath("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS");

    bool GCont = config->GetFirstEntry(temp_str, dummy);
    while ( GCont )
        {
        array_of_extensions.push_back(temp_str);
        GCont = config->GetNextEntry(temp_str, dummy);
        }

    config->SetPath(strOldPath);




    std::vector<std::pair<std::string, std::string>> output;
    for(unsigned int i=0;i<array_of_extensions.size();i++)
        {
        std::string command=config->Read("FULL_TEXT_SEARCH_ARCHIVE_EXTRACTORS/"+array_of_extensions[i], "").ToStdString();
        output.push_back(std::make_pair(array_of_extensions[i].ToStdString(), command));
        }

    return output;
    }

void FullTextSearchIndexerWindow::show_info_message_showing_which_filetypes_we_can_index()
    {
    std::vector<std::pair<std::string, std::string>> files_to_index= gen_text_extractors_commands();
    std::vector<std::pair<std::string, std::string>> archives_to_index= gen_archive_extractors_commands();

    wxString msg="Kolkhoznik is configured to index the following filetypes: ";
    for (unsigned int i=0;i<files_to_index.size();i++)
        {
        if(i!=files_to_index.size()-1)
            {
            msg=msg+files_to_index[i].first+", ";
            }
        else
            {
            msg=msg+files_to_index[i].first+".";
            }
        }
    msg=msg+" Also we are going to unpack the following archive types: ";
    for (unsigned int i=0;i<archives_to_index.size();i++)
        {
        if(i!=archives_to_index.size()-1)
            {
            msg=msg+archives_to_index[i].first+", ";
            }
        else
            {
            msg=msg+archives_to_index[i].first;
            }
        }
    msg = msg + " (double archives, like tar.gz are supported if each of the the subtypes (tar and gz) are supported).";
    msg=msg+"\n\nIf this is not good enough, than close the indexing window, configure kolkhoznik accordingly and try again.";



    wxMessageBox(msg, wxT("Hi there!"), wxOK);
    }
void FullTextSearchIndexerWindow::UpdateProgressDialogCallBack(int percents, std::string message, bool *stop)
    {
    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__BOOK_GOT_FULL_TEXT_SEARCH_INDEXED_EVENT);
    event_to_propagate.SetInt(percents);
    event_to_propagate.SetString(message.c_str());
    this->GetEventHandler()->QueueEvent 	( 	event_to_propagate.Clone()	) ;
    if (we_would_like_to_stop_indexing) (*stop)=true;

    }

void FullTextSearchIndexerWindow::FinishIndexingCallBAck()
    {
    KHZNIK_DEBUGMESSAGE(std::cout << "\nFullTextSearchIndexerWindow::FinishIndexingCallBAck called\n";)
    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__FULL_TEXT_SEARCH_INDEXING_FINISHED);
    this->GetEventHandler()->QueueEvent 	( 	event_to_propagate.Clone()	) ;
    }


void FullTextSearchIndexerWindow::UpdateProgressBar(wxCommandEvent &event)
    {
    wxString message = event.GetString();
    int percents=event.GetInt();
    bool return_val=progress_bar->Update(percents, message);
    if(!return_val) we_would_like_to_stop_indexing=true;
    }

void FullTextSearchIndexerWindow::UpdateProgressBar2(int percents, std::string message)
    {
    progress_bar->Update(percents, message);
    }


void FullTextSearchIndexerWindow::FinishIndexing(wxCommandEvent &event)
    {
    ////////////////////////////////////////////////////////
    //wait for all indexing threads to finish
    ///////////////////////////////////////////////////////
    for (int i=0; i< threads_number; i++)
        {
        indexing_threads[i]->join();
        delete indexing_threads[i];
        }
    queue_prepare_thread->join();
    delete queue_prepare_thread;
    indexing_threads.clear();
    /////////////////////////////////////////////////////////
    // Ok, indexing threads have finished the execution
    ///////////////////////////////////////////////////////

    progress_bar->Destroy();
    if(text_indexer->unknown_error)
        {
        wxString message= "Unknown error occured. Check that just one instance of the indexer is running and that you have enough free space on your hard drive";
        wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        delete text_indexer;
        this->Enable();
        return;
        }
    else
        {
        wxString ini_filename = Khz::ConfigManager::GetPathToConfigFile();
        delete config;
        config =new wxFileConfig( "", "", ini_filename);
        config->Write( "FULL_TEXT_SEARCH/database_path",xapian_database_path_input->GetValue());
        config->Flush();
        delete config;
        config =new wxFileConfig( "", "", ini_filename);
        delete text_indexer;
        wxString message;
        if(we_would_like_to_stop_indexing)
            {
            message= "The indexing process has been interrupted. Please use the \"incremental "
                     "index\" option next time you start the indexing process.";
            }
        else
            {
            message= "The full text search index created successfully";
            }
         wxMessageBox(message, wxT("Everything is alright"), wxOK |  wxICON_INFORMATION);
        this->Enable();
        }
    }



void FullTextSearchIndexerWindow::OnWhatIsThreadsNumberHyperlinkClicked(wxHyperlinkEvent &event)
    {
    wxString message= "The number of the threads is the number of the parallel running indexers. "
             "The parameter is useful if your processor have more than one core. "
             "Set this parameter to the cores number to get maximum performance.";

    wxMessageBox(message, wxT("Hi there!"), wxOK |  wxICON_INFORMATION);
    }

void FullTextSearchIndexerWindow::OnWhatIsCacheSizeHyperlinkClicked(wxHyperlinkEvent &event)
    {
    wxString message= "The indexing process is running like this:\n"
             "First, we index books without saving data to the hard disk (data are stored in RAM)\n"
             "Second, after enough data has been accumulated, we write data to the hard disk.\n"
             "The reason for this is the following: Xapian writes data to the hard disk much faster "
             "if data are written by big pieces (for example, it writes 1000 books per query much  faster than "
             "100 books per 10 queries (i have no idea why).\n\n"
             "So, the cache size is the number of the books that Xapian writes to the database per query. "
             "The larger the cache size is, the faster indexing goes.\n\n"
             "BUT!!! As the cached books are saved in RAM, the cache size is restricted by the RAM size.\n\n"
             "The rule of the thumb is the following: 1000 books require about 2.5 GB of RAM, depending on the book's size.\n\n\n"
             "DO NOT MAKE THE CACHE SIZE TOO BIG, BE SURE THAT YOU HAVE ENOUGH RAM!!!"
             ;

    wxMessageBox(message, wxT("Hi there!"), wxOK |  wxICON_INFORMATION);
    }


}}}// namespaces
