/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include "FullTextSearchResults.h"
#include <wx/stattext.h>
#include <xapian.h>
#include "string_functions.h"
#include <wx/msgdlg.h>
#include "Custom_IDs.h"
#include <wx/stdpaths.h>
#include "clear_database_path.h"
#include <sqlite3.h>
#include "get_books_from_db_by_query.h"
#include <wx/dcclient.h>

#include "OpenTheBook.h"
#include <wx/settings.h>

#include <wx/image.h>
#include <wx/menu.h>
#include <wx/artprov.h>
#include "InfoAboutThisBook.h"
#include "BibtexTranslator.h"
#include <wx/frame.h>
#include "BibTexEntryWindow.h"


namespace Khz{ namespace FTS { namespace Gui{


FullTextSearchResults::FullTextSearchResults(wxWindow* parent, std::string path_to_xapian_db,wxString path_to_main_config_file, int _tab_index)
:wxPanel(parent, wxID_ANY), database_asker(path_to_xapian_db), database_asker_2(path_to_xapian_db)
{
this->current_page=1;
this->database_asker_2_is_searching_right_now=false;
this->next_page_shold_be_generated_by_database_asker_2=false;
this->topics_list_height_have_changed=false;
this->tab_index=_tab_index;
this->this_is_the_last_page=false;
this->display_search_results_callback=boost::bind(&Khz::FTS::Gui::FullTextSearchResults::display_search_results_callback_void, this);
this->display_search_results_callback_2=boost::bind(&Khz::FTS::Gui::FullTextSearchResults::display_search_results_callback_void_2, this);
wxInitAllImageHandlers();
just_resized=false;
config =new wxFileConfig( "", "", path_to_main_config_file);
m_HtmlConfParams.ReadConfigFile(&config, path_to_main_config_file);
path_to_config=path_to_main_config_file;



scrolled=new wxScrolledWindow(this, wxID_ANY);
scrolled->SetBackgroundColour(*wxWHITE);
this->BooksNumberInTopicsListText=new wxStaticText(scrolled, wxID_ANY, "0 topics in list");
//scrolled=this;
//SetBackgroundColour(*wxRED);
 Search_input=new wxSearchCtrl(scrolled,wxID_ANY,wxEmptyString, wxDefaultPosition,
                                wxSize(-1,30), wxTE_PROCESS_ENTER);
status_text=new wxStaticText(scrolled, wxID_ANY, "");
language_choise= new LanguageSelectorWxChoice(scrolled);
full_text_search_use_stemmer_checkbox = new wxCheckBox(scrolled, wxID_ANY, "use stemmer");
full_text_search_use_stemmer_checkbox->SetValue(true);
wxHyperlinkCtrl *what_is_stemmer_hyperlink=new wxHyperlinkCtrl (scrolled, wxID_ANY, "what is this", "click");
topics_list=new FullTextSearchTopicsList(scrolled, wxID_ANY, "horizontal");
topics_list->Disable();
search_button=new wxButton(scrolled, wxID_ANY, "Search!");
results_list= new MywxHtmlWindow (scrolled);
//paginator= new MyPaginationClass(scrolled, 0);
first_button = new wxButton(scrolled, wxID_ANY, "First");
next_button = new wxButton(scrolled, wxID_ANY, "Next");
prev_button = new wxButton(scrolled, wxID_ANY, "Previous");

  // restrict search by topics checkbox (full text search) together with topics list
restrict_topics_checkbox=new wxCheckBox(scrolled, wxID_ANY, "restrict search by topics");
restrict_topics_checkbox->SetValue(false);


topics_list_height_changer = new wxSpinCtrl(scrolled,wxID_ANY);
topics_list_height_changer->SetRange(0,1000000);

/*-1*/only_sizer = new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *input_sizer=new wxBoxSizer(wxHORIZONTAL);
            input_sizer->Add(Search_input, 1, wxEXPAND|wxALL);
            input_sizer->AddSpacer(2);
            input_sizer->Add(search_button, 0, wxEXPAND|wxALL);

/*0*/only_sizer->Add(input_sizer, 0, wxEXPAND|wxALL,10);
        wxBoxSizer *under_search_input_hor_ajuster_sizer = new wxBoxSizer(wxHORIZONTAL);
            wxBoxSizer *under_search_input_left_sizer=new wxBoxSizer(wxVERTICAL);
                wxBoxSizer *stemmer_sizer=new wxBoxSizer(wxHORIZONTAL);
                stemmer_sizer->AddSpacer(5);
                stemmer_sizer->Add(full_text_search_use_stemmer_checkbox, 0, wxALL|wxEXPAND);
                stemmer_sizer->AddSpacer(5);
                stemmer_sizer->Add(language_choise, 0, wxEXPAND|wxALL);
                stemmer_sizer->AddSpacer(5);
                stemmer_sizer->Add(what_is_stemmer_hyperlink, 0, wxEXPAND|wxALL);
            under_search_input_left_sizer->Add(stemmer_sizer, 0, wxALL);
            under_search_input_left_sizer->AddStretchSpacer();
            under_search_input_left_sizer->Add(status_text, 1, wxALL|wxEXPAND,5);
        under_search_input_hor_ajuster_sizer->Add(under_search_input_left_sizer, 1, wxEXPAND|wxALL);
            wxBoxSizer *under_search_input_right_sizer=new wxBoxSizer(wxVERTICAL);
                wxBoxSizer *checkbox_plus_spinctrl_sizer=new wxBoxSizer(wxHORIZONTAL);
                checkbox_plus_spinctrl_sizer->Add(restrict_topics_checkbox, 0, wxALL);
                checkbox_plus_spinctrl_sizer->AddSpacer(15);
                checkbox_plus_spinctrl_sizer->Add(new wxStaticText(scrolled, wxID_ANY, "height:"), 0, wxALL|wxALIGN_CENTER);
                checkbox_plus_spinctrl_sizer->Add(topics_list_height_changer,0, wxALL);
                checkbox_plus_spinctrl_sizer->AddSpacer(15);
                checkbox_plus_spinctrl_sizer->Add(BooksNumberInTopicsListText, 0, wxALL|wxALIGN_CENTER);
            under_search_input_right_sizer->Add(checkbox_plus_spinctrl_sizer, 0, wxALL);
			under_search_input_right_sizer->AddSpacer(1);
            under_search_input_right_sizer->Add(topics_list, 1, wxEXPAND|wxALL);
        under_search_input_hor_ajuster_sizer->Add(under_search_input_right_sizer, 1, wxALL);
        under_search_input_hor_ajuster_sizer->AddSpacer(5);
/*1*/only_sizer->Add(under_search_input_hor_ajuster_sizer, 0, wxEXPAND|wxALL);
/*2*/only_sizer->AddSpacer(10);
/*3*/only_sizer->Add(results_list, 1, wxEXPAND|wxALL);
        wxBoxSizer *nav_btns_sizer=new wxBoxSizer(wxHORIZONTAL);
            nav_btns_sizer->Add(first_button, 0, wxALL,5);
            nav_btns_sizer->Add(prev_button, 0, wxALL, 5);
            nav_btns_sizer->Add(next_button, 0, wxALL, 5);
     only_sizer->Add(nav_btns_sizer, 0, wxALL|wxALIGN_CENTER);

// /*4*/only_sizer->Add(paginator, 0, wxALL|wxCENTRE, 10);  //paginator have index 6
            //scrl_sizer->FitInside(this); // ask the sizer about the needed size
scrolled->SetSizerAndFit(only_sizer);
wxBoxSizer *main_sizer=new wxBoxSizer(wxVERTICAL);
main_sizer->Add(scrolled, 1, wxEXPAND|wxALL);
scrolled->SetScrollRate(5, 5);
SetSizerAndFit(main_sizer);
only_sizer->FitInside(this);



// user wants to search
search_button->Bind(wxEVT_BUTTON, &FullTextSearchResults::OnSearchButtonPressed, this);
Search_input->Bind(wxEVT_SEARCH, &FullTextSearchResults::OnSearchButtonPressed, this);
//Bind(wxEVT_COMMAND_ENTER, &FullTextSearchResults::ChangePage, this, Khz::my_wx_id__FULL_TEXT_SEARCH_CHANGE_PAGE);

Bind (wxEVT_MOUSEWHEEL, &FullTextSearchResults::OnScrollEvent, this);

full_text_search_use_stemmer_checkbox->
    Bind(wxEVT_CHECKBOX, &FullTextSearchResults::OnUseStemmerCheckboxClicked, this);

what_is_stemmer_hyperlink->Bind(wxEVT_HYPERLINK, &FullTextSearchResults::OnWhatIsStemmerLinkClicked, this);

restrict_topics_checkbox->
    Bind(wxEVT_CHECKBOX, &FullTextSearchResults::OnRestrictTopicsCheckboxClicked, this);

scrolled->Bind(wxEVT_CHILD_FOCUS, &FullTextSearchResults::OnChildFocus, this);
Bind(wxEVT_SIZE, &FullTextSearchResults::OnSize, this);
results_list->Bind(wxEVT_IDLE, &FullTextSearchResults::OnIdle, this);
results_list->Bind(wxEVT_HTML_LINK_CLICKED, &FullTextSearchResults::OnLinkLeftOrRightMouseButtonClicked, this);
Bind(wxEVT_COMMAND_MENU_SELECTED, &FullTextSearchResults::OnInfoAboutBookWanted, this, Khz::my_wx_id__SHOW_INFO_ABOUT_BOOK);

Bind(wxEVT_COMMAND_MENU_SELECTED, &FullTextSearchResults::OnGenBibtexMenuSelected, this,Khz::my_wx_id__GEN_BIBTEX_FOR_ONE_BOOK);

first_button->Bind(wxEVT_BUTTON, &FullTextSearchResults::OnFirstPageButtonPressed, this);
next_button->Bind(wxEVT_BUTTON, &FullTextSearchResults::OnNextPageButtonPressed, this);
prev_button->Bind(wxEVT_BUTTON, &FullTextSearchResults::OnPrevPageButtonPressed, this);

Bind(wxEVT_BUTTON, &FullTextSearchResults::FinishPageGeneration, this,my_wx_id__FULL_TEXT_SEARCH_DISPLAY_RESULTS_EVENT);
Bind(wxEVT_BUTTON, &FullTextSearchResults::UpdateInfoAboutBooksInTopicsList, this, my_wx_id__FULL_TEXT_TOPICS_LIST_SOMETHING_CHANGED);

topics_list_height_changer->Bind(wxEVT_SPINCTRL, &FullTextSearchResults::ChangeTopicsListHeight, this);


SetTopicsListHeightFromConfigFile();
scrolled->SetFocus();
SetFocusToSearchInput();
}


void FullTextSearchResults::ReconnectToDB(std::string path_to_xapian_readable_db)
    {
    database_asker.ReconnectToDB(path_to_xapian_readable_db);
    database_asker_2.ReconnectToDB(path_to_xapian_readable_db);
    this->InitNewSearch
        (
        this->language,
        this->query_string,
        this->books_per_page,
        false
        );
    if(!database_asker.IsOk() or !database_asker_2.IsOk())
        {
        }
    else
        {
        this->InitPageGeneration(1);
        }
    }
bool FullTextSearchResults::InitNewSearch(std::string language, std::string query_string, Xapian::doccount _num_of_books_per_page, bool show_error_messages)
    {
    this->this_is_the_last_page=false;


    //prevent using old search results
    this->page_num_that_database_asker_2_was_searching_last_or_is_searching_right_now=-1;
    ///////////////////////////////////////////////////////////////////////////
    if(!search_button->IsEnabled())
    // Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThreadInternal
    // is executing right now. We MUST NOT initiate a new search.
        {
       wxString message=wxString("You tried to initiate a new search while the search results generation is in progress.")+
                        " It means that there is a bug in the program. Please contact the developer, i will be "+
                         "very grateful if you help me to eliminate this bug.";
         wxMessageBox(message, "Error", wxOK|wxCENTRE|wxICON_ERROR);
         return false;
        }
    ////////////////////////////////////////////////////////////////////////////



    if(    !database_asker.InitNewSearch(
                                    language,
                                    query_string,
                                    _num_of_books_per_page,
                                    topics_list->GetXapianQuery(),
                                    m_HtmlConfParams.samples_number,
                                    m_HtmlConfParams.sample_size
                                    )
        or !database_asker_2.InitNewSearch(
                                    language,
                                    query_string,
                                    _num_of_books_per_page,
                                    topics_list->GetXapianQuery(),
                                    m_HtmlConfParams.samples_number,
                                    m_HtmlConfParams.sample_size
                                    )
      )
        {
        if (show_error_messages)
            {
            std::string message;
            if(database_asker.get_error_message()!="")
                {
                message = database_asker.get_error_message();
                }
            else
                {
                message = "error accessing xapian database";
                }
            wxMessageDialog msg(this, message, "Error", wxOK|wxCENTRE|wxICON_ERROR);
            msg.ShowModal();
            }


        ////////////////////////////////////////////////////////////////////////////////
        // Something is wrong! We just clear search results and return
        ////////////////////////////////////////////////////////////////////////////////
        //this->total_num_of_books=0;
        results_list->SetPage(wxString(""));
        //if(!only_sizer->Detach(paginator)) return false; // return here should never gets executed
        //paginator->Destroy();
        //paginator=new MyPaginationClass(scrolled, 0);
        //only_sizer->Add(paginator, 0, wxALL|wxCENTRE, 10);
        //only_sizer->RecalcSizes();
        first_button->Disable();
        next_button->Disable();
        prev_button->Disable();
        this->current_page=1;
        search_button->Enable();
        //only_sizer->Layout();


        results_list->SetMinSize(wxSize(-1,
                results_list->GetInternalRepresentation()->GetHeight()));
        results_list->SetSize(wxSize(-1,
                results_list->GetInternalRepresentation()->GetHeight()));
        scrolled->GetSizer()->RecalcSizes();
        scrolled->GetSizer()->Layout();
        this->GetSizer()->RecalcSizes();
        this->Layout();
        this->Refresh();
        return false;
        /////////////////////////////////////////////////////////////////////////////////////
        //            Search results are cleared
        /////////////////////////////////////////////////////////////////////////////////////
        }
    else
        {
        //this->total_num_of_books=database_asker.GetTotalMatches();
        //this->total_pages= database_asker.GetTotalPages();
        //if(!only_sizer->Detach(paginator)) return false; // return here should never gets executed
        //paginator->Destroy();
        //paginator=new MyPaginationClass(scrolled, database_asker.GetTotalPages() );
        //only_sizer->Add(paginator, 0, wxALL|wxCENTRE, 10);
        //only_sizer->RecalcSizes();
        only_sizer->Layout();
        return true;
        }
    }

void FullTextSearchResults::Search(wxString _search_string, wxString _language, std::vector<wxString> &topics_array)
    {
    ///////////////////////////////////////////////////////////////////////////
    if(!search_button->IsEnabled()) return ;
        // Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThreadInternal
    // is executing right now. We MUST NOT initiate a new search.
    ////////////////////////////////////////////////////////////////////////////

    Search_input->SetValue(_search_string);
    if(_language=="")
        {
        language_choise->Disable();
        full_text_search_use_stemmer_checkbox->SetValue(false);
        }
    else
        {
        language_choise->SetSelection (language_choise->FindString (_language));
        }


    if(topics_array.size()==0)
        {
        topics_list->Clear();
        restrict_topics_checkbox->SetValue(false);//uncheck
        topics_list->Disable();
        }
    else
        {
        restrict_topics_checkbox->SetValue(true);//uncheck
        wxArrayString temp_array;
        for(unsigned int i=0;i<topics_array.size();i++) temp_array.Add(topics_array[i]);

        topics_list->Clear();
        topics_list->InsertItems(temp_array, 0);
        topics_list->Enable();
        }
    this->query_string= Search_input->GetValue().ToUTF8().data(); //xapian works only with utf8
    this->language=_language.ToStdString();


    if( this->InitNewSearch(this->language,this->query_string, this->books_per_page) )
        {
        this->InitPageGeneration(1);
        }
    }


void FullTextSearchResults::InitPageGeneration(int page_num)
    {
    KHZNIK_DEBUGMESSAGE_3( std::cout << "entering InitPageGeneration: " << "\n";)
    this->current_page=page_num;
    search_button->Disable();
    prev_button->Disable();
    next_button->Disable();
    first_button->Disable();
        wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
        event_to_propagate.SetId(my_wx_id__EVENT_TO_CHANGE_TEXT_OF_THE_TAB);
        event_to_propagate.SetInt(tab_index);
        event_to_propagate.SetString("Searching...");
        ProcessWindowEvent 	( 	event_to_propagate	);
    database_asker.GiveMePageParallelThread(page_num, display_search_results_callback, &InitPageGenerationStatus);
    }


void FullTextSearchResults::FinishPageGeneration(wxCommandEvent &event)
    {

    ///////////////////////////////////////////////////////////////////////////////////////////
    // check that everything is ok
    ////////////////////////////////////////////////////////////////////////////////////////////
     KHZNIK_DEBUGMESSAGE_3( std::cout << "entering FinishPageGeneration: " << "\n";)
    if(event.GetInt()==1 and !InitPageGenerationStatus)
        {
        KHZNIK_DEBUGMESSAGE_3( std::cout << "FinishPageGeneration: " << "error blyat" << "\n";)
        wxMessageBox("error accessing xapian database", wxT("Error"), wxOK | wxICON_ERROR);
        event.StopPropagation();
        return;
        }
    else if(event.GetInt()==2 and !InitPageGenerationStatus_2)
        {
        KHZNIK_DEBUGMESSAGE_3( std::cout << "FinishPageGeneration: " << "error blyat" << "\n";)
        wxMessageBox("error accessing xapian database", wxT("Error"), wxOK | wxICON_ERROR);
        event.StopPropagation();
        return;
        }
    ///////////////////////////////////////////////////////////////////////////////////////////////


    // database_asker_2 have finished page generation
    if(event.GetInt()==2){database_asker_2_is_searching_right_now=false;};



    //////////////////////////////////////////////////////////////////////////
    // form current_page_data
    ///////////////////////////////////////////////////////////////////////////
    if (event.GetInt()==2 and next_page_shold_be_generated_by_database_asker_2)
        {
        next_page_shold_be_generated_by_database_asker_2=false;
        current_page_data=database_asker_2.GetCurrentResults();
        }
    else if(event.GetInt()==2 and !next_page_shold_be_generated_by_database_asker_2)
        {
        return;
        }
    else // database_asker (not database_asker_2) have finished page generation
        {
        current_page_data=database_asker.GetCurrentResults();
        }
    /////////////////////////////////////////////////////////////////////////////



    results_list->SetPage(wxString(""));
    wxString left_righ_border=config->Read("FULL_TEXT_SEARCH/left_right_border", "10");
    results_list->AppendToPage("<table WIDTH='100%' BORDER=0  CELLSPACING=0 CELLPADDING=0><tr><td COLSPAN=0 width="+left_righ_border+" style='margin:0;padding:0'></td><td>");

    bool just_started=true;
    for (int i=0;i<current_page_data.GetCurPageSize();i++)
        {
        if(!just_started)
            {
            results_list->AppendToPage("<br>");
            }
        else
            {
            just_started=false;
            }
        //KHZNIK_DEBUGMESSAGE_3 (std::cout << "\n trying to display abstract size\n";)
        //KHZNIK_DEBUGMESSAGE_3(std::cout << "fulltextsearchresults: befor appendbooktopage abstract size is" << (current_page_data.GetAbstract(i)).size() << "\n";)
        this->AppendBookToPage
                (
                i,
                current_page_data.GetAuthor(i),
                current_page_data.GetTitle(i),
                current_page_data.GetYear(i),
                current_page_data.GetTopic(i),
                current_page_data.GetExtension(i),
                current_page_data.GetRelPath(i),
                current_page_data.GetFileName(i),
                current_page_data.GetAbstract(i)
                //current_page_data.GetId(i),
                //current_page_data.GetWeight(i)
                );

        }
    results_list->AppendToPage("</td><td width="+left_righ_border+"></td></tr></table>");
        this->UpdateStatusText(this_is_the_last_page);
                results_list->SetMinSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));
        results_list->SetSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));


    //results_list->SetFocus();
    scrolled->GetSizer()->RecalcSizes();
    scrolled->GetSizer()->Layout();
    this->GetSizer()->RecalcSizes();
    this->GetSizer()->Layout();
    this->Layout();
    this->Refresh();
    scrolled->Scroll(0,0);
     //KHZNIK_DEBUGMESSAGE(std::cout << "FulltextSearchResults: html page is \n" << results_list->ToText().ToStdString() << "\n";)
    //this->GetParent()->Refresh();
    if(this->current_page!=1)
        {
        first_button->Enable();
        prev_button->Enable();
        }
    else
        {
        first_button->Disable();
        prev_button->Disable();
        }


    if(current_page_data.GetCurPageSize()!=0)
        {
        if(!this_is_the_last_page) next_button->Enable();
        this->this_is_the_last_page=false;
        }
    else
        {
        next_button->Disable();
        if(this->current_page > 1)// we hit the last page
            {
            this->this_is_the_last_page=true;
            this->InitPageGeneration(this->current_page-1);
            return;
            }
        }



    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__EVENT_TO_CHANGE_TEXT_OF_THE_TAB);
    event_to_propagate.SetInt(tab_index);
    event_to_propagate.SetString(wxString::FromUTF8(this->query_string.c_str()));
    ProcessWindowEvent 	( 	event_to_propagate	);

    ////////////////////////////////////////////////////////////////////////////////
    // i have no idea why, but without this second resizing some problems appear.
    results_list->SetMinSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));
    results_list->SetSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));


    //results_list->SetFocus();
    scrolled->GetSizer()->RecalcSizes();
    scrolled->GetSizer()->Layout();
    this->GetSizer()->RecalcSizes();
    this->GetSizer()->Layout();
    this->Layout();
    this->Refresh();
    scrolled->Scroll(0,0);
    //////////////////////////////////////////////////////////////////////////////////

    // starting generating next page in background
    if(!database_asker_2_is_searching_right_now and !this->this_is_the_last_page )
        {
        database_asker_2_is_searching_right_now=true;
        page_num_that_database_asker_2_was_searching_last_or_is_searching_right_now=this->current_page+1;
        database_asker_2.GiveMePageParallelThread(this->current_page+1, display_search_results_callback_2, &InitPageGenerationStatus_2);
        }
    search_button->Enable();
    }


/*
void FullTextSearchResults::ShowMePage(int page_num)
    {
    if(!database_asker.GiveMePage(page_num))
        {
        wxMessageBox("error accessing xapian database", wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }
    results_list->SetPage(wxString(""));
    wxString left_righ_border=config->Read("FULL_TEXT_SEARCH/left_right_border", "10");
    results_list->AppendToPage("<table WIDTH='100%' BORDER=0  CELLSPACING=0 CELLPADDING=0><tr><td COLSPAN=0 width="+left_righ_border+" style='margin:0;padding:0'></td><td>");

    bool just_started=true;
    for (int i=0;i<database_asker.GetCurPageSize();i++)
        {
        if(!just_started)
            {
            results_list->AppendToPage("<br>");
            }
        else
            {
            just_started=false;
            }
        this->AppendBookToPage
                (
                i,
                database_asker.GetAuthor(i),
                database_asker.GetTitle(i),
                database_asker.GetYear(i),
                database_asker.GetTopic(i),
                database_asker.GetExtension(i),
                database_asker.GetRelPath(i),
                database_asker.GetFileName(i),
                database_asker.GetAbstract(i)
                //database_asker.GetId(i),
                //database_asker.GetWeight(i)
                );

        }
    results_list->AppendToPage("</td><td width="+left_righ_border+"></td></tr></table>");
     //KHZNIK_DEBUGMESSAGE(std::cout << "FulltextSearchResults: html page is \n" << results_list->ToText().ToStdString() << "\n";)
        results_list->SetMinSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));
        results_list->SetSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));


    results_list->SetFocus();
    scrolled->GetSizer()->RecalcSizes();
    scrolled->GetSizer()->Layout();
    this->GetSizer()->RecalcSizes();
    this->Layout();
    this->Refresh();
    scrolled->Scroll(0,0);
    //this->GetParent()->Refresh();
    }
*/

void FullTextSearchResults::OnSearchButtonPressed(wxCommandEvent &event)
    {
    // this function gets executed not only if user presses the search button,
    // but also if user hits enter.


    ///////////////////////////////////////////////////////////////////////////
    // Khz::FTS::Engine::FullTextSearchDbQuery::GiveMePageParallelThreadInternal
    // is executing right now. We MUST NOT initiate a new search.
    if(!search_button->IsEnabled()) return; // this function(OnSearchButtonPressed) gets executed
    // if user hits enter too, even if search_button is disabled.
    //////////////////////////////////////////////////////////////////////////


    this->query_string= Search_input->GetValue().ToUTF8().data(); //xapian works only with utf8
    if(full_text_search_use_stemmer_checkbox->GetValue())
        {
        int  language_num= language_choise->GetSelection();
        this->language=language_choise->GetString(language_num).ToStdString();
        }
    else
        {
        this->language="";
        }
    if ( this->InitNewSearch(this->language,this->query_string, this->books_per_page) )
        {
        this->InitPageGeneration(1);
        }
    }

void FullTextSearchResults::OnNextPageButtonPressed(wxCommandEvent &event)
    {
    if  (
        database_asker_2_is_searching_right_now
        and  page_num_that_database_asker_2_was_searching_last_or_is_searching_right_now==(this->current_page+1)
        )
        {
        this->current_page++;
        search_button->Disable();
        prev_button->Disable();
        next_button->Disable();
        first_button->Disable();
        next_page_shold_be_generated_by_database_asker_2=true;
                wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
                event_to_propagate.SetId(my_wx_id__EVENT_TO_CHANGE_TEXT_OF_THE_TAB);
                event_to_propagate.SetInt(tab_index);
                event_to_propagate.SetString("Searching...");
                ProcessWindowEvent 	( 	event_to_propagate	);
        KHZNIK_DEBUGMESSAGE_3( std::cout << "OnNextPageButtonPressed: database_asker_2 is searching" << "\n";)
        }
    else if(page_num_that_database_asker_2_was_searching_last_or_is_searching_right_now==(this->current_page+1)
            and !database_asker_2_is_searching_right_now
           )
        {
            this->current_page++;
            search_button->Disable();
            prev_button->Disable();
            next_button->Disable();
            first_button->Disable();
            next_page_shold_be_generated_by_database_asker_2=true;
                wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
                event_to_propagate.SetId(my_wx_id__FULL_TEXT_SEARCH_DISPLAY_RESULTS_EVENT);
                event_to_propagate.SetInt(2); // database_asker, not database_asker_2;
                this->GetEventHandler()->QueueEvent 	( 	event_to_propagate.Clone()	) ;
        KHZNIK_DEBUGMESSAGE_3( std::cout << "OnNextPageButtonPressed: using database_asker_2" << "\n";)
        }
    else
        {
        KHZNIK_DEBUGMESSAGE_3( std::cout << "OnNextPageButtonPressed: calling InitPageGeneration" << "\n";)
        this->InitPageGeneration(this->current_page+1);
        }
    }

void FullTextSearchResults::OnPrevPageButtonPressed(wxCommandEvent &event)
    {
    this->InitPageGeneration(this->current_page-1);
    }

void FullTextSearchResults::OnFirstPageButtonPressed(wxCommandEvent &event)
    {
        first_button->Disable();
    prev_button->Disable();
    next_button->Disable();
    this->InitPageGeneration(1);
    }
//void FullTextSearchResults::ChangePage(wxCommandEvent &event)
//    {
//    int page_num= event.GetInt();
//    this->ShowMePage(page_num);
//    this->current_page=page_num;
//
//    this->UpdateStatusText();
//    //only_sizer->RecalcSizes();
//    }

void FullTextSearchResults::UpdateStatusText(bool this_is_last_page)
    {
    if(current_page_data.GetMatchesEstimated() ==0)
        {
        status_text->SetLabel("nothing found");
        status_text->Wrap(only_sizer->GetSize().GetWidth()-20);
        }
    else
        {
        int first_book=books_per_page*(current_page-1)+1;
        long long int last_book;
        if(current_page==1)
            {
            last_book=current_page_data.GetCurPageSize();
            }
        else if(current_page_data.GetCurPageSize()< books_per_page)
            {
            last_book=(current_page-1)*books_per_page + current_page_data.GetCurPageSize();
            }
        else
            {
            last_book=current_page*books_per_page;
            }


        wxString prefix;
        wxString books_found;
        if(!this_is_last_page)
            {
            prefix="approx.";
            books_found= wxString::Format(wxT("%i"),current_page_data.GetMatchesEstimated());
            }
        else
            {
            prefix = "exactly";
            books_found=books_found= wxString::Format(wxT("%i"), last_book);
            }

        wxString status=prefix + wxString (" ")+books_found +" books found";
            //if(total_num_of_books!=0)
            {
            status=status+", "+
            //status=status+
                    wxString::Format(wxT("%i"),first_book)+"-"+
                    wxString::Format(wxT("%i"),last_book) + " shown";
            }
            status_text->SetLabel(status);
            status_text->Wrap(only_sizer->GetSize().GetWidth()-20);

        }//endif

    }


void FullTextSearchResults::OnUseStemmerCheckboxClicked(wxCommandEvent &event)
    {
    if(full_text_search_use_stemmer_checkbox->GetValue())
        {
        language_choise->Enable();
        int  language_num= language_choise->GetSelection();
        this->language=language_choise->GetString(language_num).ToStdString();
        }
    else
        {
        language_choise->Disable();
        this->language="";
        };
    }

void FullTextSearchResults::OnRestrictTopicsCheckboxClicked(wxCommandEvent &event)
    {
    if(restrict_topics_checkbox->GetValue())
        {
        topics_list->Enable();
        }
    else
        {
        topics_list->Disable();
        }
    }

void FullTextSearchResults::OnWhatIsStemmerLinkClicked(wxHyperlinkEvent& event)
    {
    wxString message=
    "Stemming is a  process of mapping related words to a common stem. "
    "This enables matching on variants of a word. For example, while "
    "using stemmer, a search for \"horse\" could return matches for "
    "\"horses\", \"horseback\", and \"horsing\", as well as \"horse\". "
    "Without stemmer only matches  for \"horse\" are gonna to be returned";
    wxMessageBox(  message,wxString("Hi there!"), wxOK|wxCENTRE|wxSTAY_ON_TOP) ;
    }


void FullTextSearchResults::AppendBookToPage

        (
        int num,
        std::string author,
        std::string title,
        std::string year,
        std::string topic,
        std::string extension,
        std::string relpath,
        std::string filename,
        std::vector<std::pair<std::string, int>> abstract
        //long long int *id,
        //int* weight_in_percents
        )
    {
    wxString icons_size_str=wxString::Format(wxT("%i"),m_HtmlConfParams.icons_size+7);
    wxString output;
    if(m_HtmlConfParams.icons_size>0)
        {
        wxString path_to_icon="\""+ChooseIcon(extension)+"\"";
        KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchResults::AppendBookToPage: path to icon is " << path_to_icon.ToStdString() << "\n";)
        output=output + "<table valign=top BORDER=0  CELLSPACING=0 CELLPADDING=0><tr><td CELLSPACING=3  align=left valign=top width="+icons_size_str+"><img src="+path_to_icon+" ALIGN=TEXTTOP></img></td><td valign=top >";
        }
    else
        {
        output="<div>";
        }



    wxString caption_style="font-family: arial,sans-serif; font-size:"+m_HtmlConfParams.caption_font_size+"pt;color:"+m_HtmlConfParams.caption_color+";";
    //wxString caption_style="font-family: arial,sans-serif; font-size:16px;color:"+m_HtmlConfParams.caption_color+";";

    if (m_HtmlConfParams.is_caption_bold)
        {
        caption_style=caption_style+"font-weight:bold;";
        }
    wxString author_wx=wxString((author).c_str());
    wxString title_wx=wxString((title).c_str());
    output=output+"<span style='"+caption_style +"'>"+
    "<a href='"+wxString::Format(wxT("%i"),num) + "' style='color:"+m_HtmlConfParams.caption_color+";text-decoration:none;' >"+
    //wxString::Format(wxT("%i"),*weight_in_percents)+"% "+
    author_wx + " " + title_wx +
    "</a></span>";
    if(m_HtmlConfParams.icons_size>0)
        {
        output=output+"<br>";
        }
    else
        {
        output =output+"<br>";
        }

    if(m_HtmlConfParams.show_info)
        {
        output=output+"<span style='color:"+m_HtmlConfParams.info_color+";font-size:"+m_HtmlConfParams.info_font_size+"pt;'>";
        output =output + "<table CELLSPACING=8><tr></tr></table>"; // add little space
        wxString delim="";
        if(m_HtmlConfParams.show_extension)
            {
            output=output+ delim+ "Ext: "+ extension;
            delim="; ";
            }
        if(m_HtmlConfParams.show_year)
            {
            output=output+ delim +"Year: " + wxString((year).c_str());
            delim="; ";
            }
        if(m_HtmlConfParams.show_topic)
            {
            output=output+ delim+ "Topic: " + wxString((topic).c_str());
            delim="; ";
            }
        if(m_HtmlConfParams.show_relpath)
            {
            output = output + delim + "Relpath: " + wxString((relpath).c_str()) + "/" + wxString((filename).c_str());
            delim="; ";
            }
        output=output + "</span><br>";

        }

    output =output + "<table CELLSPACING=8><tr></tr></table>"; //little vertical gap
    output=output+"<span style='font-size:"+m_HtmlConfParams.snippet_font_size+"pt;'>";

    wxString highlighted_span="<span style='color:"+m_HtmlConfParams.highlight_color +";";
    wxString dots_span="<span style='color:"+m_HtmlConfParams.dots_color+";";
    if(m_HtmlConfParams.dots_bold)
        {
        dots_span=dots_span+"font-weight:bold;";
        }
    if(m_HtmlConfParams.highlight_bold)
        {
        highlighted_span=highlighted_span+"font-weight:bold;";
        }
    highlighted_span=highlighted_span+"'>";
    dots_span=dots_span+"'>";
     //KHZNIK_DEBUGMESSAGE_3(std::cout << "\n line 692 entering for\n";)
    // KHZNIK_DEBUGMESSAGE_3(std::cout << "line 693 size is " << (abstract).size();)
    for (unsigned int i=0;i< (abstract).size();i++)
        {
        if( (abstract)[i].second==0)
            {
            output=output+ wxString::FromUTF8(abstract[i].first.c_str()) + " ";
            }
        else if( abstract[i].second==1 )
            {
            output=output+ highlighted_span + wxString::FromUTF8(abstract[i].first.c_str()) +"</span> ";
            }
        else if( abstract[i].second==-1 )
            {
            if(abstract.size()>1 and i!=0 and i==abstract.size()-1)
                {
                // do nothing (remove dots in the end of the snippet
                }
            else
                {
                output=output+ dots_span + m_HtmlConfParams.dots_symbol +"</span> ";
                }
            }
        }
    output=output+"\n</span>";
    if(m_HtmlConfParams.icons_size>0)
        {
        output=output + "</td></tr></table><br><br>";
        }
    else
        {
        output=output+"</div><br>";
        }
    results_list->AppendToPage(output);
    }












   void FullTextSearchResults::OnScrollEvent(wxMouseEvent& event)
        {
        int rotation=event.GetWheelRotation();
        if(rotation>0)
            {
            scrolled->ScrollLines(-m_HtmlConfParams.scrollrate);
            }
        else if (rotation<0)
            {
            scrolled->ScrollLines(+m_HtmlConfParams.scrollrate);
            }
        KHZNIK_DEBUGMESSAGE(std::cout << "scrolling in parent\n";)
        event.Skip();
        this->scrolled->GetEventHandler()->ProcessEvent(event);
        event.ResumePropagation (2);
        }

void FullTextSearchResults::OnLinkLeftOrRightMouseButtonClicked(wxHtmlLinkEvent& event)
    {
    if(event.GetLinkInfo().GetEvent()->RightUp()) //user clicked link by right mouse button
        {
        last_right_clicked_link=wxAtoi(event.GetLinkInfo().GetHref()); // href is just number
        ShowLinkRightClickMenu();
        }
    else
        {
        wxString book_num=event.GetLinkInfo().GetHref();
        int num=wxAtoi(book_num);
        Khz::Libs::OpentheBook
                (
                current_page_data.GetRelPath(num),
                current_page_data.GetExtension(num),
                current_page_data.GetFileName(num),
                config
                );

        event.StopPropagation();
        }
    }


void FullTextSearchResults::OnSize(wxSizeEvent& event)
    {
    KHZNIK_DEBUGMESSAGE(std::cout << "FulltextSEarchResults::onsize called\n";)
    just_resized=true;
    }



void FullTextSearchResults::OnIdle(wxIdleEvent &event)
    {
    if(just_resized)
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchResults::OnIdle: resizing something\n";)

        scrolled->GetSizer()->RecalcSizes();
        scrolled->GetSizer()->Layout();
        this->GetSizer()->RecalcSizes();
        this->Layout();
        this->Refresh();

        results_list->SetMinSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));
        results_list->SetSize(wxSize(-1,
                  results_list->GetInternalRepresentation()->GetHeight()));

        //results_list->SetFocus();
        scrolled->GetSizer()->RecalcSizes();
        scrolled->GetSizer()->Layout();
        this->GetSizer()->RecalcSizes();
        this->Layout();
        this->Refresh();
        just_resized=false;
        }
    //else
    //    {
    //    KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchResults::OnIdle: doing nothing\n";)
    //    }
    }

void FullTextSearchResults::OnChildFocus(wxChildFocusEvent& event)
    {
    KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchResults:: child focus caught\n";)
    event.StopPropagation();
    }




wxString FullTextSearchResults::ChooseIcon(std::string Extension)
    {
    if(m_HtmlConfParams.icons_size==-1) return ""; //no icons
    if (Extension=="") return "";  //default icon

    if (File_icons_map.count(Extension) != 0)  //icon is already in the images list, return index.
        {
        auto it =File_icons_map.find(Extension);
        return  it->second;
        }
    else  //there no icon for such Extension in the m_pImageList, so we should insert icon there and return index.
        {
        wxString Ext = wxString(Extension.c_str());
        wxString temp_string;


        //folder where icon with current icons_size should be (like 16x16)
        wxString size_folder= wxString::Format(wxT("%i"),m_HtmlConfParams.icons_size)+
        "x"  +   wxString::Format(wxT("%i"),m_HtmlConfParams.icons_size);


        wxString search_folder=wxString("icons")+wxFileName::GetPathSeparator()+size_folder;

        if(wxDirExists(search_folder))
            {
            temp_string=wxString("icons")+wxFileName::GetPathSeparator()+size_folder+wxFileName::GetPathSeparator()+wxString("icons_for_files_by_extension")+wxFileName::GetPathSeparator();
            }
        else
            {
            temp_string=wxString("icons")+wxFileName::GetPathSeparator()+"48x48"+wxFileName::GetPathSeparator()+wxString("icons_for_files_by_extension")+wxFileName::GetPathSeparator();
            }
        wxString Search_path=temp_string+Ext+".png";
        if (wxFileExists(Search_path))
            {
            File_icons_map[Extension]=Search_path;
            return Search_path;
            }
        else if(wxFileExists(wxString("icons")+wxFileName::GetPathSeparator()+size_folder+wxFileName::GetPathSeparator()+"book.png"))
            {
            File_icons_map[Extension]=wxString("icons")+wxFileName::GetPathSeparator()+size_folder+wxFileName::GetPathSeparator()+"book.png";
            return File_icons_map[Extension];
            }
        else
            {
            File_icons_map[Extension]="";
            return "";
            }
        }
    }

void FullTextSearchResults::ReDraw()
    {
    KHZNIK_DEBUGMESSAGE_3(std::cout << "\nFullTextSearchResults::ReDraw: called\n";)


    m_HtmlConfParams.ReadConfigFile(&config, path_to_config);
    File_icons_map.clear();

        // InitNewSearch sets current_page to 1, so we have to remember it.
        unsigned int old_current_page=this->current_page;

        this->InitNewSearch(this->language,this->query_string, this->books_per_page);
        KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchResults::ReDraw: page is " << old_current_page << "\n";)
        this->current_page=old_current_page;
        this->InitPageGeneration(this->current_page);
        return;

    }



void FullTextSearchResults::HtmlDisplayParams::ReadConfigFile(wxFileConfig **config, wxString path_to_config )
    {
    KHZNIK_DEBUGMESSAGE(std::cout << "FullTextSearchResults::HtmlDisplayParams::ReadConfigFile: path to conf is " << path_to_config.ToStdString()<< "\n";)
    delete *config;
    *config =new wxFileConfig( "", "", path_to_config);

    int temp=wxSystemSettings::GetFont(wxSYS_SYSTEM_FONT).GetPointSize();
    wxString default_font_size=wxString::Format(wxT("%i"),temp);


    caption_font_size=(*config)->Read("FULL_TEXT_SEARCH/book_caption_font_size", default_font_size);
    wxString is_caption_bold_string=(*config)->Read("FULL_TEXT_SEARCH/book_caption_is_bold", "false");
    if(is_caption_bold_string=="yes")
        {
        is_caption_bold=true;
        }
    else
        {
        is_caption_bold=false;
        }

    caption_color=(*config)->Read("FULL_TEXT_SEARCH/book_caption_color", "black");


    wxString show_info_str=(*config)->Read("FULL_TEXT_SEARCH/show_info", "yes");
    if(show_info_str=="yes") show_info=true; else show_info=false;

    info_color=(*config)->Read("FULL_TEXT_SEARCH/info_color", "grey");
    info_font_size=(*config)->Read("FULL_TEXT_SEARCH/info_font_size", "12");


    wxString show_topic_str=(*config)->Read("FULL_TEXT_SEARCH/show_topic", "yes");
    if(show_topic_str=="yes") show_topic=true; else show_topic=false;

    wxString show_year_str=(*config)->Read("FULL_TEXT_SEARCH/show_year", "yes");
    if(show_year_str=="yes") show_year=true; else show_year=false;

    wxString show_extension_str=(*config)->Read("FULL_TEXT_SEARCH/show_extension", "yes");
    if(show_extension_str=="yes") show_extension=true; else show_extension=false;

    wxString show_relpath_str=(*config)->Read("FULL_TEXT_SEARCH/show_relpath", "yes");
    if(show_relpath_str=="yes") show_relpath=true; else show_relpath=false;

    snippet_font_size=(*config)->Read("FULL_TEXT_SEARCH/snippet_font_size", "12");

    highlight_color=(*config)->Read("FULL_TEXT_SEARCH/highlight_color", "blue");
    dots_color=(*config)->Read("FULL_TEXT_SEARCH/dots_color", "blue");


    wxString dots_bold_str=(*config)->Read("FULL_TEXT_SEARCH/dots_bold", "yes");
    if(dots_bold_str=="yes")dots_bold=true; else dots_bold=false;

    dots_symbol=(*config)->Read("FULL_TEXT_SEARCH/dots_symbol", "...");


    wxString highlight_is_bold_str=(*config)->Read("FULL_TEXT_SEARCH/highlight_bold", "yes");
    if(highlight_is_bold_str=="yes") highlight_bold=true; else highlight_bold=false;



    samples_number = wxAtoi((*config)->Read("FULL_TEXT_SEARCH/samples_number", "3"));
    sample_size=wxAtoi((*config)->Read("FULL_TEXT_SEARCH/sample_size", "12"));
    scrollrate=wxAtoi((*config)->Read("FULL_TEXT_SEARCH/scrollrate", "5"));
    icons_size=wxAtoi((*config)->Read("FULL_TEXT_SEARCH/icons_size", "16"));
    }

void FullTextSearchResults::ShowLinkRightClickMenu()
    {
    wxMenu *menu = new wxMenu;
    wxMenuItem *item1 = new wxMenuItem(menu, Khz::my_wx_id__SHOW_INFO_ABOUT_BOOK,  wxT("Info"));
    wxMenuItem *item2 = new wxMenuItem(menu, Khz::my_wx_id__GEN_BIBTEX_FOR_ONE_BOOK,  wxT("Generate Bibtex"));
    //#include "CoolIcon.xpm"
    item1->SetBitmap(wxArtProvider::GetBitmap(wxART_INFORMATION, wxART_MENU));
    menu->Append(item1);
    menu->Append(item2);

    PopupMenu(menu);
    delete menu;
    }


void FullTextSearchResults::OnInfoAboutBookWanted(wxCommandEvent &event)
    {
     wxString main_kolkhoznik_database=config->Read("PATHS/path_to_database","");
    sqlite3 *db;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //              connecting to the main kolkhoznik database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    sqlite3_open( main_kolkhoznik_database.c_str(), &db);
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //connected to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


    std::string where_statement= "md5_of_file='"+current_page_data.GetMd5(last_right_clicked_link)+"' AND m_hash='"+
                                                current_page_data.GetHash(last_right_clicked_link)+"'";

    Khz::Libs::DB::get_books_from_db_by_query temp_db_asker(db, "main",
                                                       where_statement,1, {});
    temp_db_asker.Step();
    if(temp_db_asker.GetTotalLines()!=1)
        {
         wxString message= wxString("Something terrible just happened. ")+
         "\""+ main_kolkhoznik_database+"\" does not have this book.";

        wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }

    wxString RelPathToFile = wxString::FromUTF8(temp_db_asker.current_result[0]["RelPathToFile"].c_str());  //data are saved in utf8 encoding in the database.
    wxString filename=temp_db_asker.current_result[0]["Filename"];
    RelPathToFile.Replace 	("/", wxFileName::GetPathSeparator() , true );
    wxString path_to_library= config->Read(wxString("PATHS/path_to_dir_with_books"),wxString(""));
    //std::cout << "path blin is" <<  path_to_library.mb_str(wxConvLocal).data() << "\n";







    if (path_to_library.Right(1)== wxFileName::GetPathSeparator() )
        {
        path_to_library=path_to_library.RemoveLast();
        }
    wxString full_path_to_book = path_to_library + RelPathToFile +  wxFileName::GetPathSeparator() +filename;


    Khz::PopupWins::InfoAboutThisBook *popup_info= new Khz::PopupWins::InfoAboutThisBook(this, temp_db_asker.current_result, full_path_to_book);
    //popup_info->SetSize (800, -1);
    popup_info->Show();
    popup_info->Fit();
     sqlite3_close(db);
    //delete popup_info;
    }


void FullTextSearchResults::OnGenBibtexMenuSelected(wxCommandEvent &event)
    {
    wxString main_kolkhoznik_database=config->Read("PATHS/path_to_database","");
    sqlite3 *db;
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //              connecting to the main kolkhoznik database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    sqlite3_open( main_kolkhoznik_database.c_str(), &db);
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //connected to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


    std::string where_statement= "md5_of_file='"+current_page_data.GetMd5(last_right_clicked_link)+"' AND m_hash='"+
                                                current_page_data.GetHash(last_right_clicked_link)+"'";
    //std::cout << "where is " << where_statement << "\n";
    Khz::Libs::DB::get_books_from_db_by_query temp_db_asker(db, "main",
                                                       where_statement,1, {});
    temp_db_asker.Step();
    //std::cout << "total lines is " << temp_db_asker.GetTotalLines() << "\n";
    if(temp_db_asker.GetTotalLines()!=1)
        {
         wxString message= wxString("Something terrible just happened. ")+
         "\""+ main_kolkhoznik_database+"\" does not have this book.";

        wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        return;
        }
    //string with bibtex entry
    std::string bibtex_entry=Khz::BibTex::GenBib::GenBibtexEntryFromFields
                            (temp_db_asker.current_result[0]["Author"],
                             temp_db_asker.current_result[0]["Title"],
                             temp_db_asker.current_result[0]["Publisher"],
                             temp_db_asker.current_result[0]["Year"],
                             temp_db_asker.current_result[0]["Edition"],
                             temp_db_asker.current_result[0]["ID"]
                             );

    //prepare wxstring
    wxString Bibtex_wxString = wxString::FromUTF8(bibtex_entry.c_str());
    Khz::BibTex::BibtexEntryWindow *BibWindow= new Khz::BibTex::BibtexEntryWindow(this, Bibtex_wxString);
    BibWindow->Show();
    sqlite3_close(db);
    }

void FullTextSearchResults::display_search_results_callback_void()
    {
    KHZNIK_DEBUGMESSAGE_3(std::cout << "\nFullTextSearchResults::display_search_results_callback_void called\n";)
    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__FULL_TEXT_SEARCH_DISPLAY_RESULTS_EVENT);
    event_to_propagate.SetInt(1); // database_asker, not database_asker_2;
    this->GetEventHandler()->QueueEvent 	( 	event_to_propagate.Clone()	) ;
    }
void FullTextSearchResults::display_search_results_callback_void_2()
    {
    KHZNIK_DEBUGMESSAGE_3(std::cout << "\nFullTextSearchResults::display_search_results_callback_void_2 called\n";)
    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__FULL_TEXT_SEARCH_DISPLAY_RESULTS_EVENT);
    event_to_propagate.SetInt(2); // database_asker_2, not database_asker;
    this->GetEventHandler()->QueueEvent 	( 	event_to_propagate.Clone()	) ;
    }

void FullTextSearchResults::ChangeTopicsListHeight(wxSpinEvent& event)
    {
    this->topics_list_height_have_changed=true;
    int height_increment=topics_list->GetSizer()->ComputeFittingWindowSize(topics_list).GetHeight()/3; // we have 3 buttons there,
                                                                                                       // incrementing by button size
    int m_height= height_increment*topics_list_height_changer->GetValue();
//    int current_topics_list_height=topics_list->GetSize().GetHeight();


    topics_list->topics_list_itself->SetMinSize (wxSize(-1, 0));
    topics_list->topics_list_itself->SetSize (wxSize(-1, 0));

    if(m_height!=0)
        {
        if(!topics_list->IsShown())
            {
            topics_list->Show();
            }
//         int min_good_height=topics_list->GetSizer()->ComputeFittingWindowSize(topics_list).GetHeight();
//         if(m_height<min_good_height)
//            {
//            std::cout << "\nHERE\n";
//            if (current_topics_list_height< m_height)  //user just INCREASED the height
//                {
//                std::cout << "Here again";
//                m_height=min_good_height;
//                int topics_list_height_changer_new_value=m_height/height_increment;
//                topics_list_height_changer->SetValue(m_height/topics_list_height_changer_new_value);
//                }
//            else   //user just DECREASEd the height
//                {
//                std::cout << "\nhere now in decresed";
//                m_height=0;
//                if(topics_list->IsShown())
//                    {
//                    topics_list->Hide();
//                    }
//                topics_list_height_changer->SetValue(0);
//                }
//            };
         std::cout << "\n m_height before set " << m_height << "\n";
         topics_list->SetMinSize (wxSize(-1, m_height));
         topics_list->SetSize (wxSize(-1, m_height));
         //topics_list->GetSizer()->RecalcSizes();
         //topics_list->GetSizer()->Fit(topics_list); // this Fit will make buttons visible
         //topics_list->SetMinSize(topics_list->GetSize());
        }
    else
        {
        if(topics_list->IsShown())
            {
            topics_list->Hide();
            }
        topics_list->SetMinSize(wxSize(-1, 0));
        topics_list->SetSize(wxSize(-1, 0));
        }
    scrolled->GetSizer()->RecalcSizes();
    scrolled->GetSizer()->Layout();
    this->GetSizer()->RecalcSizes();
    this->Layout();
    this->Refresh();
    }
void FullTextSearchResults::SetTopicsListHeightFromConfigFile()
    {
    int height_increment=topics_list->GetSizer()->ComputeFittingWindowSize(topics_list).GetHeight()/3; // we have 3 buttons there,
                                                                                                       // incrementing by button size
    //std::cout << "\nheight_increment in SET " << height_increment << "\n";
    int m_height= config->Read("FULL_TEXT_SEARCH/topic_list_height", 3);
    if(m_height<0)
        {
        m_height=0;
        }
    topics_list_height_changer->SetValue(m_height);
    m_height=m_height*height_increment;

    topics_list->SetMinSize (wxSize(-1, m_height));
    topics_list->SetSize (wxSize(-1, m_height));
    if(m_height==0)
        {
        topics_list->Hide();
        }


    scrolled->GetSizer()->RecalcSizes();
    scrolled->GetSizer()->Layout();
    this->GetSizer()->RecalcSizes();
    this->Layout();
    this->Refresh();
    }



void FullTextSearchResults::UpdateInfoAboutBooksInTopicsList(wxCommandEvent &event)
    {
    wxString number_of_books_in_list=wxString::Format(wxT("%i"),event.GetInt());
    BooksNumberInTopicsListText->SetLabel(number_of_books_in_list+ " topics in list");


    //results_list->SetFocus();
    scrolled->GetSizer()->RecalcSizes();
    scrolled->GetSizer()->Layout();
    this->GetSizer()->RecalcSizes();
    this->Layout();
    this->Refresh();
    }

void FullTextSearchResults::SetFocusToSearchInput()
    {
    int left=Search_input->GetRect().GetLeft();
    int right=Search_input->GetRect().GetRight() ;
    int top=Search_input->GetRect().GetTop();
    int bottom=Search_input->GetRect().GetBottom();

    int x_center= left+(right-left)/2;
    int y_center=top+(bottom-top)/2;
    if(!Search_input->GetRect().Contains(x_center, y_center))
        {
        std::cout << "\n FullTextSearchResults::SetFocusToSearchInput() WTF???\n";
        return;
        }
    wxMouseEvent event(wxEVT_LEFT_DOWN);
    event.SetX(x_center);
    event.SetY(y_center);
    Search_input->GetEventHandler()->ProcessEvent(event);
    }
}}}// namespace
