/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include "FullTextSearchTopicsList.h"
#include <wx/button.h>
#include <wx/sizer.h>
#include "MyTopicSelectorDialog.h"
#include "string_functions.h"
#include <unicode/unistr.h>
#include <wx/settings.h>
#include "MySizeFunctions.h"
#include "Custom_IDs.h"
namespace Khz{ namespace FTS { namespace Gui{


FullTextSearchTopicsList::FullTextSearchTopicsList(wxWindow *parent, wxWindowID id, std::string orientation)
:wxPanel(parent, id)
    {
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW ));

    wxButton *add_button=new wxButton(this, wxID_ANY, "Add", wxDefaultPosition, wxDefaultSize,wxBU_EXACTFIT);
    wxButton *remove_button =new wxButton(this, wxID_ANY, "Remove", wxDefaultPosition, wxDefaultSize,wxBU_EXACTFIT);
    wxButton *clear_button=new wxButton (this, wxID_ANY, "Clear", wxDefaultPosition, wxDefaultSize,wxBU_EXACTFIT);
    topics_list_itself=new wxListBox(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, NULL,wxLB_EXTENDED);
    topics_list_itself->SetMinSize (wxSize(30,-1));
    wxBoxSizer *only_sizer;
    if(orientation=="vertical")
        {
        only_sizer = new wxBoxSizer(wxVERTICAL);
        only_sizer->Add(topics_list_itself, 1, wxEXPAND|wxALL);

            wxBoxSizer *buttons_sizer=new wxBoxSizer(wxHORIZONTAL);
            buttons_sizer->Add(add_button, 1, wxALL|wxEXPAND);
            //buttons_sizer->AddSpacer(0);
            buttons_sizer->Add(remove_button, 1, wxALL|wxEXPAND);
            //buttons_sizer->AddSpacer(0);
            buttons_sizer->Add(clear_button, 1, wxALL|wxEXPAND);
        only_sizer->Add(buttons_sizer, 0, wxEXPAND|wxALL);
        }
    else
        {
        only_sizer = new wxBoxSizer(wxHORIZONTAL);
        only_sizer->Add(topics_list_itself, 1, wxEXPAND|wxALL);

            wxBoxSizer *buttons_sizer=new wxBoxSizer(wxVERTICAL);
            buttons_sizer->Add(add_button, 0, wxALL|wxEXPAND);
            //buttons_sizer->AddSpacer(3);
            buttons_sizer->Add(remove_button, 0, wxALL|wxEXPAND);
            //buttons_sizer->AddSpacer(3);
            buttons_sizer->Add(clear_button, 0, wxALL|wxEXPAND);
        only_sizer->Add(buttons_sizer, 0, wxEXPAND|wxALL);
        }
    SetSizerAndFit(only_sizer);

    add_button->Bind(wxEVT_BUTTON, &FullTextSearchTopicsList::OnAddPressed, this);
    remove_button->Bind(wxEVT_BUTTON, &FullTextSearchTopicsList::OnRemovePressed, this);
    clear_button->Bind(wxEVT_BUTTON, &FullTextSearchTopicsList::OnClearPressed, this);
    topics_list_itself->Bind(wxEVT_RIGHT_UP, &FullTextSearchTopicsList::OnRighMouseButtonUp, this);
    topics_list_itself->Bind(wxEVT_RIGHT_DOWN, &FullTextSearchTopicsList::OnRighMouseButtonDown, this);
    }


void FullTextSearchTopicsList::OnAddPressed(wxCommandEvent &event)
    {
    wxArrayString temp_wx_array= wxArrayString();

        wxSize temp_size =wxSize
            (
            Khz::Libs::MySizeFunctions::get_width_of_number_of_text_symbols(100, this),
            Khz::Libs::MySizeFunctions::get_height_of_number_of_text_rows(25, this)
            );



    Khz::PopupWins::MyTopicSelectorDialog dlg(this, wxID_ANY, "Select topic", wxDefaultPosition, temp_size, true);
    if(dlg.ShowModal()==wxID_CANCEL) return;
    wxString topic = dlg.library_tree->GetTopicString(dlg.library_tree->GetSelection());
    if(!IsTopicInList(topic)) temp_wx_array.Add(topic);
    if(dlg.include_subtopics_checkbox->GetValue())
        {
        std::vector<wxString> subtopics;
        dlg.GetArrayOfSubtopics(topic, subtopics);
        for(unsigned int i=0;i<subtopics.size();i++)
            {
            if(!IsTopicInList(subtopics[i])) temp_wx_array.Add(subtopics[i]);
            }
        }
    topics_list_itself->InsertItems(temp_wx_array, 0);
    topics_list_itself->SetSelection(0);




        wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
        event_to_propagate.SetId(my_wx_id__FULL_TEXT_TOPICS_LIST_SOMETHING_CHANGED);
        event_to_propagate.SetInt(topics_list_itself->GetCount());
        ProcessWindowEvent 	( 	event_to_propagate	);
    }

void FullTextSearchTopicsList::OnRemovePressed(wxCommandEvent &event)
    {
    if(topics_list_itself->GetCount()==0) return;

    wxArrayInt selections;
    topics_list_itself->GetSelections(selections);
    std::vector<int> selections_std;
     for(unsigned int i=0;i<selections.GetCount();i++)
        {
        selections_std.push_back(selections[i]);
        }

     //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //reverse sort by insertion algorithm
    for(unsigned int i=1; i<selections_std.size() ;i++)
        for(int    j=i;    j>0   && selections_std[j-1]    <   selections_std[j];  j--) // пока j>0 и элемент j-1 > j, x-массив int
            std::swap(selections_std[j-1],selections_std[j]);        // меняем местами элементы j и j-1
    //sorting finished (successfully i hope)
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

    int temp_index=0;
    for(unsigned int i=0;i<selections_std.size();i++)
        {
        temp_index=selections_std[i];
        topics_list_itself->Delete(temp_index);
        // from last to first, otherwise would not work
        }
    if( (temp_index-1) >=0 )
        {
        topics_list_itself->SetSelection(temp_index-1);
        }
    else if(topics_list_itself->GetCount()>0 )
        {
        topics_list_itself->SetSelection(0);
        }

    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__FULL_TEXT_TOPICS_LIST_SOMETHING_CHANGED);
    event_to_propagate.SetInt(topics_list_itself->GetCount());
    ProcessWindowEvent 	( 	event_to_propagate	);
    }

void FullTextSearchTopicsList::OnClearPressed(wxCommandEvent &event)
    {
    topics_list_itself->Clear();


        wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
        event_to_propagate.SetId(my_wx_id__FULL_TEXT_TOPICS_LIST_SOMETHING_CHANGED);
        event_to_propagate.SetInt(0);
        ProcessWindowEvent 	( 	event_to_propagate	);
    }

Xapian::Query FullTextSearchTopicsList::GetXapianQuery()
    {
    Xapian::Query query;
    std::vector<std::string> search_terms;
    for(unsigned int i=0;i<topics_list_itself->GetCount();i++)
        {
        if(!this->IsEnabled()) break;
        std::string topic= topics_list_itself->GetString(i).ToUTF8().data();

        Khz::Libs::Mstr::replaceAll(topic, " ", "_");

        search_terms.push_back("XTOPIC"+topic);
        }
    if(search_terms.size()!=0)
        {
        query = Xapian::Query(Xapian::Query::OP_OR, search_terms.begin(), search_terms.end());
        }
    return query;
    }

void FullTextSearchTopicsList::GetArrayOfTopics(std::vector<wxString> &output)
    {
    for(unsigned int i=0;i<topics_list_itself->GetCount();i++)
        {
        output.push_back( topics_list_itself->GetString(i));
        }
    }
bool FullTextSearchTopicsList::IsTopicInList(wxString topic)
    {
    for(unsigned int i=0;i<topics_list_itself->GetCount();i++)
        {
        if (topics_list_itself->GetString(i)==topic)
            {
            return true;
            }
        }
    return false;
    }

void FullTextSearchTopicsList::OnRighMouseButtonUp(wxMouseEvent &event)
    {
    if (event.HasAnyModifiers ())
        {
        event.Skip();
        return;
        }
    wxMenu mnu;
 	mnu.Append(1, 	"Add topic");
 	mnu.Append(2, 	"Remove");
 	mnu.Append(3, 	"Clear");
 	mnu.Connect(wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FullTextSearchTopicsList::OnPopupClick), NULL, this);
 	PopupMenu(&mnu);
    }

void FullTextSearchTopicsList::OnPopupClick(wxCommandEvent &event)
    {
    if(event.GetId()==1)// add topic menu item selected
        {
        wxArrayInt selections;
        topics_list_itself->GetSelections(selections);
        for (unsigned int i=0;i<selections.GetCount();i++)
            {
            topics_list_itself->Deselect(selections[i]);
            }
        this->OnAddPressed(event);
        }
    else if(event.GetId()==2) //remove
        {
        OnRemovePressed(event);
        }
    else if(event.GetId()==3) //clear
        {
        OnClearPressed(event);
        }
    }

void FullTextSearchTopicsList::OnRighMouseButtonDown(wxMouseEvent &event)
    {
    wxArrayInt selections;
    topics_list_itself->GetSelections(selections);
    int m_count=selections.GetCount();
    if(event.HasAnyModifiers () or m_count <=1)
        {
        event.Skip();
        return;
        }
    //by default we block the event(do nothing) to prevent selections from disappearing (when multiple topics are selected)
    }
}}}// namespaces
