/*
Copyright (C) Ivan Ivanov, 2015-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include "FullTextSearchWindow.h"
#include <wx/stdpaths.h>
#include <iostream>
#include "Custom_IDs.h"
#include <wx/sizer.h>
#include "ConfigManager.h"
#include "HtmlMessageBox.h"
#include "DEBUGMESSAGE.h"
#include <wx/msgdlg.h>
namespace Khz{ namespace FTS { namespace Gui{


FullTextSearchWindow::FullTextSearchWindow(wxWindow *parent)
    : wxFrame(parent, wxID_ANY, "Full text search")
    {
    //tab_to_delete=NULL;
        #include "CoolIcon.xpm"
        SetIcon(wxIcon(CoolIcon_xpm));
    //let us look into config file
    ini_filename = Khz::ConfigManager::GetPathToConfigFile();
    config =new wxFileConfig( "", "", ini_filename);
	// config file is opened now.

	//now we read main window and splitter sizes. (they were remembered from last session)
    int window_width= wxAtoi(config->Read("FULL_TEXT_SEARCH/window_width", "800"));
    int window_height=wxAtoi(config->Read("FULL_TEXT_SEARCH/window_height", "600"));
    wxString is_maximized=config->Read("FULL_TEXT_SEARCH/window_is_expanded", "no" );
    path_to_xapian_db=config->Read("FULL_TEXT_SEARCH/database_path", "").ToStdString();
    wxString show_menubar=config->Read("FULL_TEXT_SEARCH/show_menubar","true");
	// reading of sizes finished.

	//let us set widh, height of the main window
    this->SetSize(window_width, window_height);
    if(is_maximized=="yes") this->Maximize();

    //win = new FullTextSearchResults(this, path_to_xapian_db, ini_filename);

    CreateStatusBar(2);
    SetStatusText(wxT("Kolkhoznik full text search window"));



    ////////////////////////////////////////////////////////////////////////////
    // menubar
    ///////////////////////////////////////////////////////////////////////////
    menuBar = new wxMenuBar();

        wxMenu *fileMenu = new wxMenu;
            fileMenu->Append(wxID_CLOSE, wxT("Close"),
                wxT("Close"));
    menuBar->Append(fileMenu, wxT("&File"));

        wxMenu *helpMenu = new wxMenu;
        helpMenu->Append(wxID_HELP_CONTENTS , wxT("&How to search"),
            wxT("Show about dialog"));
    menuBar->Append(helpMenu, wxT("&Help"));

    if(show_menubar=="true")
        {
        SetMenuBar(menuBar);
        this->is_menubar_shown=true;
        }
    else
        {
        is_menubar_shown=false;
        }
    ///////////////////////////////////////////////////////////////////////////
    // finishes with menubar.
    ////////////////////////////////////////////////////////////////////////////////////


    //////////////////////////////////////////////////////////////
    // notebook with tabs
    //////////////////////////////////////////////////////////////
    tabs= new wxAuiNotebook(this, wxID_ANY);
    new_tab_tab=new wxPanel(tabs);
    tabs->AddPage(new_tab_tab, "");

        // the "plus" icon for creating a new tab
        int temp_height=Khz::Libs::MySizeFunctions::get_height_of_string("QGATRESXC", tabs);
        m_pImageList = new wxImageList( temp_height,  temp_height);
        //std::cout << "tabs height is " << tabs->GetTabCtrlHeight() << "\n";
        wxIcon temp_icon;
        temp_icon.LoadFile(wxString("icons")+wxFileName::GetPathSeparator()+"new_tab.png", wxBITMAP_TYPE_PNG);
        m_pImageList->Add(wxBitmap(wxBitmap(temp_icon).ConvertToImage().Rescale(temp_height, temp_height)));
        tabs->AssignImageList(m_pImageList);
        tabs->SetPageImage 	(0,0);
    /////////////////////////////////////////////////////////////////////////////////////////


    wxBoxSizer *sizer =new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(tabs, 1, wxEXPAND|wxALL);
    SetSizerAndFit(sizer);
    this->SetSize(window_width, window_height);
    //temp_panel->GetSizer()->FitInside(this);
    //search_results_panel->GetSizer()->FitInside(this);
     //something is being resized
    Bind(wxEVT_SIZE, &FullTextSearchWindow::OnResize, this);
    Bind(wxEVT_MAXIMIZE, &FullTextSearchWindow::OnMaximize, this);
    //Bind( wxEVT_SPLITTER_SASH_POS_CHANGED, &FullTextSearchWindow::OnSashPositionChanged, this);
    Bind(wxEVT_CLOSE_WINDOW , &FullTextSearchWindow::OnQuit, this);
    //Bind(wxEVT_IDLE, &FullTextSearchWindow::OnIdle, this);
    Bind(wxEVT_COMMAND_MENU_SELECTED, &FullTextSearchWindow::OnHowToSearchPressed, this, wxID_HELP_CONTENTS );
    Bind(wxEVT_COMMAND_MENU_SELECTED, &FullTextSearchWindow::OnCloseMenuSelected, this, wxID_CLOSE);
    Bind(wxEVT_AUINOTEBOOK_PAGE_CHANGING, &FullTextSearchWindow::OnTabChanging, this);
    Bind(wxEVT_BUTTON, &FullTextSearchWindow::SetTabText, this, my_wx_id__EVENT_TO_CHANGE_TEXT_OF_THE_TAB);
    Bind(wxEVT_AUINOTEBOOK_PAGE_CLOSE, &FullTextSearchWindow::OnTabClosing, this);
    Bind(wxEVT_AUINOTEBOOK_PAGE_CLOSED, &FullTextSearchWindow::OnTabClosed, this);
    }





//remembers window size (puts data to the config file
void FullTextSearchWindow::OnResize(wxSizeEvent &event)
    {
        delete config; // without this changes made by other classes that use the same config file will not be saved.
        config =new wxFileConfig( "", "", ini_filename);

      //  std::cout << "sizing";
        int width;
        int height;
        this->GetSize (&width, &height);
        if (!this->IsMaximized())
            {
            config->Write("FULL_TEXT_SEARCH/window_width",wxString::Format(wxT("%i"),width) );
            config->Write("FULL_TEXT_SEARCH/window_height", wxString::Format(wxT("%i"),height) );
            config->Write("FULL_TEXT_SEARCH/window_is_expanded", "no" );
            }
        config->Flush();
        event.Skip();
    }


//remembers that window should be maximized
void FullTextSearchWindow::OnMaximize(wxMaximizeEvent& event)
    {

    delete config; // without this changes made by other classes that use the same config file will not be saved.
    config =new wxFileConfig( "", "", ini_filename);

    if (!this->IsMaximized())
        {
        config->Write("FULL_TEXT_SEARCH/window_is_expanded", "no" );
        }
    else
        {
        config->Write( "FULL_TEXT_SEARCH/window_is_expanded", "yes" );
        }
     config->Flush();
    event.Skip();
    }



void FullTextSearchWindow::OnQuit(wxCloseEvent& event)
    {
    OnQuit();
    event.Veto();
    }



void FullTextSearchWindow::OnQuit()
    {
    int temp_num = (int)(tabs->GetPageCount()-1);
    for (int i=0;i<temp_num;i++)
        {
        FullTextSearchResults *temp_ptr=(FullTextSearchResults *)(  tabs->GetPage(i)  );
        if(!temp_ptr->IsIdle())
            {
            wxMessageBox("A search process is going on right now, the window cannot be closed. Try later.","Hi there");
            return;
            }
        }


    KHZNIK_DEBUGMESSAGE( std::cout << "i am in OnQuit() "<< "\n"; )
     wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
        event_to_propagate.SetId(Khz::my_wx_id__CLOSE_FULL_TEXT_SEARCH_WINDOW);
        ProcessWindowEvent 	( 	event_to_propagate	) ;
    }



void FullTextSearchWindow::OnCloseMenuSelected(wxCommandEvent &event)
    {
    OnQuit();
    }




void FullTextSearchWindow::OnSearchWanted(wxString search_string, wxString language, std::vector<wxString> &topics_array)
    {
    wxString tab_text;
    if( search_string.Trim().Trim(true)=="")
        {
        tab_text="hello friend!";
        }
    else if(search_string.Len()>15)
        {
        tab_text= search_string.Left(15)+"...";
        }
    else
        {
        tab_text=search_string;
        }
    int temp_num=tabs->GetPageCount()-1;
    tabs->InsertPage(
                tabs->GetPageCount()-1, // we always have a "New" tab( adds another tab ), so we always >0
                new FullTextSearchResults(tabs, path_to_xapian_db, ini_filename , temp_num),
                tab_text,
                true //select
                    );

    (  (FullTextSearchResults *)  (  tabs->GetPage(tabs->GetPageCount()-2)  )  ) ->Search(search_string, language, topics_array);
    }




void FullTextSearchWindow::OnHowToSearchPressed(wxCommandEvent &event)
    {
    wxString html_path=wxString("html") + wxFileName::GetPathSeparator()+ "HowToFullTextSearch.html";
    Khz::PopupWins::HtmlMessageBox *msg = new Khz::PopupWins::HtmlMessageBox
        (
        this,
        wxID_ANY,
        wxString("Hi there!"),
        html_path,
        wxDefaultPosition,
        wxDefaultSize,
        wxRESIZE_BORDER,
        wxDialogNameStr
        );
    msg->ShowModal();
    msg->Destroy();
    }





void FullTextSearchWindow::OnTabChanging(wxAuiNotebookEvent& event)
    {
    int current_tabs_count=(int)(tabs->GetPageCount());
    if(          event.GetSelection()   ==   (   current_tabs_count -1 )          ) // user pressed the "New" tab, the last one
        {
        tabs->InsertPage(
                tabs->GetPageCount()-1, // we always have a "New" tab( adds another tab ), so we always >0
                new FullTextSearchResults(tabs, path_to_xapian_db, ini_filename,current_tabs_count-1),
                "hello friend!",
                true //select
                    );
        event.Veto();
        }
    else
        {
        event.Skip();
        }
    }





void FullTextSearchWindow::SetTabText(wxCommandEvent &event)
    {
    wxString tab_text;
    wxString search_string=event.GetString();
    int tab_index=event.GetInt();

    if( search_string.Trim().Trim(true)=="")
        {
        search_string ="hello friend!";
        }
    tab_text=ellipsize_str(search_string, 15);

    KHZNIK_DEBUGMESSAGE_3(std::cout << "setting text for tab " << tab_index << "\n";)
    tabs->SetPageText(tab_index, tab_text);
    }






void FullTextSearchWindow::OnTabClosing(wxAuiNotebookEvent& event)
    {
    int id=event.GetOldSelection();
    FullTextSearchResults *temp_ptr=(FullTextSearchResults *)(  tabs->GetPage(id)  );
    if (!temp_ptr->IsIdle())
        {
        event.Veto();
        wxMessageBox( "A search process is going on right now, the tab cannot be closed. Try later.", "Hi there");
        return;
        }
    //tab_to_delete=tabs->GetCurrentPage();
    //std::cout << "\n" << tabs->GetSelection()  << " is set for deleting\n";
    if(tabs->GetPageCount()==2) // we closed the last tab (excluding "New").
        {
        //tab_to_delete=NULL;
        OnQuit();
        }
    else if (  tabs->GetSelection()  ==  (  ((int)(tabs->GetPageCount()))      -2)        )
        {
        tabs->SetSelection(tabs->GetPageCount()-3);
        }
    }



void FullTextSearchWindow::ReDraw()
    {
    /////////////////////////////////////////////////////////////////////
    // show/hide    menubar
    /////////////////////////////////////////////////////////////////////
    delete config; // without this changes made by other classes that use the same config file will not be saved.
    config =new wxFileConfig( "", "", ini_filename);
    wxString show_menubar=config->Read("FULL_TEXT_SEARCH/show_menubar","true");
    if(show_menubar=="true")
        {
        if(!is_menubar_shown)
            {
            SetMenuBar(menuBar);
            is_menubar_shown=true;
            }
        }
    else
        {
        if(is_menubar_shown)
            {
            SetMenuBar(nullptr);
            is_menubar_shown=false;
            }
        }
    /////////////////////////////////////////////////////////////////////
    // finished with menubar
    /////////////////////////////////////////////////////////////////

    for(unsigned int i=0;i<(tabs->GetPageCount()-1);i++)
        {
        ((FullTextSearchResults *)(  tabs->GetPage(i)  ))  ->ReDraw();
        }
        //win->ReDraw();
    };







 void FullTextSearchWindow::OnTabClosed(wxAuiNotebookEvent& event)
    {
    int temp_num = (int)(tabs->GetPageCount()-1);
    for (int i=0;i<temp_num;i++)
        {
        ((FullTextSearchResults *)(  tabs->GetPage(i)  ))  ->SetTabIndex(i);
        }
    }





wxString FullTextSearchWindow::ellipsize_str(wxString &str, unsigned int length)
    {
    if(str.Length()==length)
        {
        return str;
        }
    else if (str.Length()> length)
        {
        return str.Left(length-3)+"...";
        }
    else if (str.Length() < length)
        {
        wxString output=str;
        while(output.Length()< length)
            {
            output=output+" ";
            }
        return output;
        }
    else
        {
        return "PIZDEC KOTAM!!!";
        }
    }
}}}// namespaces
