/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include "LanguageSelectorWxChoice.h"
#include <xapian.h>
#include "string_functions.h"
#include <wx/stdpaths.h>
#include "ConfigManager.h"


namespace Khz{ namespace FTS { namespace Gui{



LanguageSelectorWxChoice::LanguageSelectorWxChoice(wxWindow *parent)
:wxChoice(parent, wxID_ANY)
{
ini_filename = Khz::ConfigManager::GetPathToConfigFile();
config =new wxFileConfig( "", "", ini_filename);

std::vector<std::string> languages=GetStemmers();
std::string default_stemmer=GetDefaultStemmerFromConfig();

for(unsigned int i=0;i<languages.size();i++)
    {
    this->Append(  wxString(languages[i])  );
    if(languages[i]==default_stemmer)
        {
        this->SetSelection (i);
        }
    }

}

std::vector<std::string> LanguageSelectorWxChoice::GetStemmers()
    {
    std::string stemmers=config->Read("FULL_TEXT_SEARCH/stemmers", "").ToStdString();


    //Let us make an array of the supported by xapian stemmers
    std::vector<std::string> stemmers_vector;

    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //Next, we explode string and create array of the supported languages
    stemmers_vector=Khz::Libs::Mstr::explode(" ",Khz::Libs::Mstr::trim(boost::regex_replace(stemmers, boost::regex("\\s+", boost::regex_constants::icase)," ")));

    if(AreStemmersGood(stemmers_vector)) return stemmers_vector; else return GetStemmersFromXapian();
    }


std::string LanguageSelectorWxChoice::GetDefaultStemmerFromConfig()
    {
    std::string stemmer = config->Read("FULL_TEXT_SEARCH/default_stemmer", "").ToStdString();
    if (stemmer=="") return "english"; else return stemmer;
    }


bool LanguageSelectorWxChoice::AreStemmersGood(std::vector<std::string> stemmers)
    {
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //sort stemmers by insertion algorithm
    for(unsigned int i=1; i<stemmers.size() ;i++)
        for(int    j=i;    j>0   && stemmers[j-1]   >  stemmers[j];  j--) // пока j>0 и элемент j-1 > j, x-массив int
            std::swap(stemmers[j-1],stemmers[j]);        // меняем местами элементы j и j-1
    //sorting finished (successfully i hope)
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



    std::string supported_by_xapian_languages=Xapian::Stem::get_available_languages();

    //Let us make an array of the supported by xapian stemmers
    std::vector<std::string> xapian_stemmers;

    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //Next, we explode string and create array of the supported languages
    xapian_stemmers=Khz::Libs::Mstr::explode
        (" ",
         Khz::Libs::Mstr::trim
                    (
                    boost::regex_replace(supported_by_xapian_languages, boost::regex("\\s+", boost::regex_constants::icase)," ")
                    )
        );



    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //sort xapian_stemmers by insertion algorithm
    for(unsigned int i=1; i<xapian_stemmers.size() ;i++)
        for(int    j=i;    j>0   && xapian_stemmers[j-1]   >  xapian_stemmers[j];  j--) // пока j>0 и элемент j-1 > j, x-массив int
            std::swap(xapian_stemmers[j-1],xapian_stemmers[j]);        // меняем местами элементы j и j-1
    //sorting finished (successfully i hope)
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    if(stemmers==xapian_stemmers) return true; else return false; // compare SORTED vectors
    }


std::vector<std::string> LanguageSelectorWxChoice::GetStemmersFromXapian()
    {
    std::string supported_by_xapian_languages=Xapian::Stem::get_available_languages();

    //Let us make an array of the supported by xapian stemmers
    std::vector<std::string> xapian_stemmers;

    //regex removes spaces in beginning and end of the string (just in case) and
    //replaces myltiply spaces between words by a single space.
    //Next, we explode string and create array of the supported languages
    xapian_stemmers=Khz::Libs::Mstr::explode(" ",Khz::Libs::Mstr::trim(boost::regex_replace(supported_by_xapian_languages, boost::regex("\\s+", boost::regex_constants::icase)," ")));
    return xapian_stemmers;
    }

void LanguageSelectorWxChoice::ReDraw()
    {
    delete config;
    config =new wxFileConfig( "", "", ini_filename);
    std::vector<std::string> languages=GetStemmers();
    std::string default_stemmer=GetDefaultStemmerFromConfig();

    this->Clear();
    for(unsigned int i=0;i<languages.size();i++)
        {
        this->Append(  wxString(languages[i])  );
        if(languages[i]==default_stemmer)
            {
            this->SetSelection (i);
            }
        }
    }


}}}// namespaces
