/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include "MyPaginationClass.h"
#include <iostream>
#include "Custom_IDs.h"
#include "DEBUGMESSAGE.h"

namespace Khz{ namespace FTS { namespace Gui{


MyPaginationClass::MyPaginationClass(wxWindow *parent, int _total_pages)
    :wxPanel(parent, wxID_ANY)
    {
    SetBackgroundColour(*wxWHITE);
    this->total_pages=_total_pages;
    this->current_page=1;
    not_number_links.insert(std::make_pair("First", new PageLink(this, 1,  "First")));
    not_number_links.insert(std::make_pair("Prev",  new PageLink(this, 1,  "Prev")));
    not_number_links.insert(std::make_pair("Next",  new PageLink(this, 2,  "Next")));
    not_number_links.insert(std::make_pair("Last",  new PageLink(this, _total_pages, "Last")));

    not_number_links["First"]->SetBackgroundColour(*wxWHITE);
    not_number_links["Prev"]->SetBackgroundColour(*wxWHITE);
    not_number_links["Next"]->SetBackgroundColour(*wxWHITE);
    not_number_links["Last"]->SetBackgroundColour(*wxWHITE);

    if(_total_pages>1)
        {
        not_number_links["Next"]->Enable();
        not_number_links["Last"]->Enable();
        }

    if(total_pages>0)
        {
        for(int i=1;i<= _total_pages;i++)
            {
            number_links.insert(std::make_pair(i, new PageLink(this,i)));
            number_links[i]->Enable();
            number_links[i]->UnHighlight();
            if(i==max_pages_to_display) break;
            }
        number_links[1]->Highlight();
        }

    only_sizer=new wxBoxSizer(wxHORIZONTAL);

    only_sizer->Add(not_number_links["First"], 0, wxALL, 2);
    only_sizer->Add(not_number_links["Prev"], 0, wxALL, 2);

    if(total_pages>0)
        {
        for (int i=1; i<= _total_pages; i++)
            {
            only_sizer->Add(number_links[i], 1, wxALL, 2);
            //number_links[i]->GetSizer()->RecalcSizes();
             if(i==max_pages_to_display) break;
            }
        }

    only_sizer->Add(not_number_links["Next"], 0, wxALL, 2);
    only_sizer->Add(not_number_links["Last"], 0, wxALL, 2);
    SetSizerAndFit(only_sizer);

    Bind(wxEVT_BUTTON, &MyPaginationClass::OnLinkClicked, this);
    }
void MyPaginationClass::ChangePage(int page)
    {
    if(page==1)
        {
        not_number_links["First"]->Disable();
        not_number_links["Prev"]->Disable();
        }
    else
        {
        not_number_links["First"]->Enable();
        not_number_links["Prev"]->Enable();
        not_number_links["Prev"]->SetPageNum(page-1);
        }
    if(total_pages>1 and page!=total_pages)
        {
        not_number_links["Last"]->Enable();
        not_number_links["Next"]->Enable();
        not_number_links["Next"]->SetPageNum(page+1);
        }
    else
        {
        not_number_links["Last"]->Disable();
        not_number_links["Next"]->Disable();
        }

    // we display all available pages
    if(total_pages<=max_pages_to_display)
        {
        number_links[page]->Highlight();
        number_links[current_page]->UnHighlight();
        this->current_page=page;
        }
    // our page in in the beginning of the pages list,
    // we have to cut pages from the right.
    else if(    page <=    ( (max_pages_to_display-1)/2 )       )
        {
        for(int i=1; i<=max_pages_to_display; i++)
            {
            number_links[i]->SetPageNumAndLabel(i);
            if(page==i)
                {
                number_links[i]->Highlight();
                }
            else
                {
                number_links[i]->UnHighlight();
                }
            }
        this->current_page=page;
        }
    // our page is in the end of the pages list,
    // we have to cut from the left
    else if(    page > (total_pages - (max_pages_to_display-1)/2 )       )
        {
        KHZNIK_DEBUGMESSAGE (std::cout << "\n i am in more\n";)
        int counter=(total_pages + 1 -max_pages_to_display );
        for(int i=1; i<=max_pages_to_display; i++)
            {
            number_links[i]->SetPageNumAndLabel(counter);
            KHZNIK_DEBUGMESSAGE (std::cout << "\n setting " << i<< " th link to " << counter << "\n";)
            if(page==counter)
                {
                number_links[i]->Highlight();
                }
            else
                {
                number_links[i]->UnHighlight();
                }
            counter++;
            }
        this->current_page=page;
        }
    // our page is somewhere in the middle, we cut from both sides.
    else
        {
        int counter= (page)  -   (  (max_pages_to_display-1)/2  );
        for(int i=1; i<=max_pages_to_display; i++)
            {
            number_links[i]->SetPageNumAndLabel(counter);
            KHZNIK_DEBUGMESSAGE (std::cout << "\n setting " << i<< " th link to " << counter << "\n";)
            if(page==counter)
                {
                number_links[i]->Highlight();
                KHZNIK_DEBUGMESSAGE (std::cout << "disabled " << counter << " page\n";)
                }
            else
                {
                number_links[i]->UnHighlight();
                }
            counter++;
            }
        this->current_page=page;
        }
    only_sizer->RecalcSizes();
    }
void MyPaginationClass::OnLinkClicked(wxCommandEvent& event)
    {
    int page_num= event.GetId();
    KHZNIK_DEBUGMESSAGE(std::cout << "MyPaginationClass::OnLinkClicked; page num is " << page_num << "\n";)
    if(current_page==page_num) return;
    ChangePage(page_num);

    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_COMMAND_ENTER);
    event_to_propagate.SetId(Khz::my_wx_id__FULL_TEXT_SEARCH_CHANGE_PAGE);
    event_to_propagate.SetInt(page_num);
    ProcessWindowEvent 	( 	event_to_propagate	) ;
    //pointer_to_callback->ChangePage(page_num);
    }

void MyPaginationClass::SetNewTotalPagesNumber(int _total_pages)
    {
    only_sizer->Clear();
    this->only_sizer->Clear(true);// true means delete all windows
    this->not_number_links.clear(); //contains only pointers, actual windows were deleleted by only_sizer.Clear(true);
    number_links.clear(); //contains only pointers, actual windows were deleleted by only_sizer.Clear(true);
    this->total_pages=_total_pages;
    this->current_page=1;
    not_number_links.insert(std::make_pair("First", new PageLink(this, 1,  "First")));
    not_number_links.insert(std::make_pair("Prev",  new PageLink(this, 1,  "Prev")));
    not_number_links.insert(std::make_pair("Next",  new PageLink(this, 2,  "Next")));
    not_number_links.insert(std::make_pair("Last",  new PageLink(this, _total_pages, "Last")));

    if(_total_pages>1)
        {
        not_number_links["Next"]->Enable();
        not_number_links["Last"]->Enable();
        }

    if(total_pages>0)
        {
        KHZNIK_DEBUGMESSAGE (std::cout << "mypaginationclass: total pages i " << _total_pages << "\n";)
        for(int i=1;i<= _total_pages;i++)
            {
            KHZNIK_DEBUGMESSAGE (std::cout << "mypaginationclass: inserting link\n";)
            number_links.insert(std::make_pair(i, new PageLink(this,i)));
            number_links[i]->Enable();
            number_links[i]->UnHighlight();
            if(i==max_pages_to_display) break;
            }
        number_links[1]->Highlight();
        }
    only_sizer->Add(not_number_links["First"], 0, wxALL, 2);
    only_sizer->Add(not_number_links["Prev"], 0, wxALL, 2);

    if(total_pages>0)
        {
        for (int i=1; i<= _total_pages; i++)
            {
            KHZNIK_DEBUGMESSAGE (std::cout << "mypaginationclass: adding links to sizer\n";)
            only_sizer->Add(number_links[i], 1, wxALL, 2);
            number_links[i]->GetSizer()->RecalcSizes();
             if(i==max_pages_to_display) break;
            }
        }

    only_sizer->Add(not_number_links["Next"], 0, wxALL, 2);
    only_sizer->Add(not_number_links["Last"], 0, wxALL, 2);
    this->GetSizer()->Layout();
        this->GetSizer()->RecalcSizes();
    this->Fit();
    this->Refresh();
    }

}}}//namespaces
