/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <wx/panel.h>
#include <wx/hyperlink.h>
#include <wx/string.h>
#include <map>
#include <iostream>
#include <wx/sizer.h>
#include <wx/panel.h>
#include "MySizeFunctions.h"
#include <wx/button.h>

namespace Khz{ namespace FTS { namespace Gui{



/**
@brief shows buttons with page numbers. User can switch page here.

When changing page, this class emits a wxEVT_COMMAND_ENTER event.
The id of the event is Khz::my_wx_id__FULL_TEXT_SEARCH_CHANGE_PAGE.
So, to catch this event you have to do something like this:
@code
Bind(wxEVT_COMMAND_ENTER, &FullTextSearchResults::ChangePage, this, Khz::my_wx_id__FULL_TEXT_SEARCH_CHANGE_PAGE);
@endcode
To get the page number use event.GetInt().
**/
class MyPaginationClass : public wxPanel
{
public:


    /**
    constructor
    @param parent standard wxwidgets parent
    @param _total_pages Total number of pages
    **/
    MyPaginationClass(wxWindow *parent, int _total_pages);


    /**
    Activates the page without emitting any events.
    Be sure that \a page is smaller than the total
    pages number
    **/
    void ChangePage(int page);


    /**
    Sets new value for the total number of pages.
    **/
    void SetNewTotalPagesNumber(int _total_pages);
private:

    /**
    @brief One button with the page number
    **/
    class PageLink : public wxButton
    {
    public:

        /**
        Constructor
        @param parent standard wxwidgets parent
        @param num page number associated with this button
        @param label visible label of the button.
        **/
        PageLink(wxWindow *parent, int num, wxString label)
        : wxButton(parent, num, label)
            {
            this->Disable();
            this->SetMinSize(wxSize(Khz::Libs::MySizeFunctions::get_width_of_number_of_text_symbols(12, parent), -1));
            };




        /**
        Additional constructor. Use this constructor if the button's label is a number
        that is equal to the page number associated with  the button.

        @param parent standard wxwidgets parent
        @param num page number associated with this button
        **/
        PageLink(wxWindow *parent, int num)
         : wxButton(parent, num, wxString::Format(wxT("%i"),num))
            {
            this->Disable();
            this->SetMinSize(wxSize(Khz::Libs::MySizeFunctions::get_width_of_number_of_text_symbols(12, parent), -1));
            }



        /**
        Sets the page number associated with the button
        **/
        void SetPageNum(int num)
            {
            this->SetId(num);
            };




        /**
        Highlights (actually disables ) the button.
        We use this to mark the current page.
        **/
        void Highlight()
            {
            this->Disable();
            }


        /**
        Unhighlights (actually enable) the button.
        See also #Highlight.
        **/
        void UnHighlight()
            {
            this->Enable();
            }


        /**
        Sets new label and new page number
        associated with the button.
        The label is just the number.
        **/
        void SetPageNumAndLabel(int num)
            {
            this->SetLabel(wxString::Format(wxT("%i"),num));
            this->SetId(num);
            };
    };

    /**
    By using this map we can get the button pointer
    by it's int label. See also #not_number_links.
    **/
    std::map<int, PageLink*> number_links;



    /**
    By using this map we can get the pointer to the button by using the label of the button.
    The label could be "Next", "Prev", "Last", "First".
    See also #number_links.
    **/
    std::map<wxString, PageLink*> not_number_links;



    /**
    Total number of pages
    **/
    int total_pages;

    /**
    Currently active page
    **/
    int current_page;


    /**
    ODD number indicating how many page buttons we can show at the same time.
    **/
    int max_pages_to_display=9;


    /**
    Executes when user presses a page button.
    Emits wxEVT_COMMAND_ENTER event (see MyPagination detailed description)
    and calls #ChangePage
    **/
    void OnLinkClicked(wxCommandEvent& event);



    /**
    The main sizer
    **/
    wxBoxSizer *only_sizer;

};



}}}// namespaces
