/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/dialog.h>
#include <wx/gauge.h>
#include <wx/stattext.h>
#include <wx/button.h>
namespace Khz{namespace FTS {namespace Gui{


/**
@brief The full text search index progress dialog
**/
class MyProgressDialog : public wxDialog
{
public:

/**
constructor
@param parent standard wxwidgets parent
@param id standard wxwidgets id
@param title The window title
@param top_message some message
**/
MyProgressDialog(wxWindow *parent, wxWindowID id, const wxString title, wxString top_message);

/**
Button that interrupts indexing;
**/
wxButton* cancel_button;

/**
The progress bar itself
**/
wxGauge* progress_bar;


/**
Updates the state of the progress bar
@param percents Sets the progress bar position, 0-100 percent
@param message additional message to show to the user


@returns false if user has pressed #cancel_button, true otherwise.
**/
bool Update(int percents, wxString message);

private:


/**
If user presses #cancel_button than this value gets set to "true"
**/
bool cancel_was_pressed;

/**
Some message to show to the user
**/
wxStaticText *message;

/**
Executes when user presses #cancel_button. Sets #cancel_was_pressed to true.
**/
void OnCancelPressed(wxCommandEvent &event);


/**
Executes when user presses any button on the keyboard.
Does nothing, just blocks the keyboard event.
Without this function user could stop the indexing process
by pressing the "space" button. We definitely do not want this.
**/
void OnKeyboardButtonPressed(wxKeyEvent& event)
    {
    event.StopPropagation();
    }
};


}}}// namespaces
