/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/event.h>
#include <iostream>

namespace Khz{ namespace FTS { namespace Gui{



/**
@brief wxHtmlWindow without scrollbars that sends mouse wheel events to the parent (standard wxwidgets parent) window

wxHtmlWindow with \a wxHW_SCROLLBAR_NEVER style blocks all of the mouse wheel events, so scrolling
by the mouse wheel from the parent window is impossible.
This class is a standard wxHtmlWindow class except for the fact that it sends all of the mouse wheel events to the parent.
**/
class MywxHtmlWindow : public wxHtmlWindow
{
public:

    /**
    Constructor
    @param parent standard wxwidgets parent
    **/
    MywxHtmlWindow(wxWindow *parent):wxHtmlWindow(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHW_SCROLLBAR_NEVER)
    {
    Bind (wxEVT_MOUSEWHEEL, &MywxHtmlWindow::OnScrollEvent, this);
    };




private:

    /**
    Sends event to the parent window.
    **/
    void OnScrollEvent(wxMouseEvent & event)
        {
        //std::cout << "scrolling";
        event.Skip();
        //this->GetParent()->GetEventHandler()->ProcessEvent(event);
        event.ResumePropagation (2);
        //event.StopPropagation();
        }
};


}}}//namespaces
