/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include <vector>
#include <string>
#include <wx/dialog.h>
#include <wx/scrolwin.h>
#include <wx/button.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include "SelectAppsToExtractTextWindow.h"
#include "MySizeFunctions.h"
#include <wx/statline.h>
#include "platform_specific_defs.h"


/////////////////////////////////////////////////////////////////
namespace Khz{ namespace FTS { namespace Gui{
///////////////////////////////////////////////////////////////



SelectAppsToExtractTextWindow::SelectAppsToExtractTextWindow(wxWindow *parent, const wxWindowID id, wxString Title, std::vector<std::pair<std::string, std::string>> &empty_apps)
:wxDialog(parent, id, Title, wxDefaultPosition,  wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
    {
    Bind(wxEVT_SIZE, &SelectAppsToExtractTextWindow::OnSize, this);
    //std::cout << "\n in SelectAppsToExtractTextWindow size is " << empty_apps.size() << "\n";


     wxPanel *panel = new wxPanel(this);
    scrlWindow= new wxScrolledWindow(panel, wxID_ANY);
     cancel_button = new wxButton(panel, wxID_CANCEL);
    ok_button = new wxButton(panel, wxID_OK);



    wxBoxSizer *scrl_sizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *main_sizer = new wxBoxSizer(wxVERTICAL);

    wxString header_str=
			"Kolkhoznik uses external applications to extract the text layer from\n"
            "the books. We cannot find some of the required applications (or they are\n"
			"not working properly). If you have them installed, than please provide the\n"
			"full path to the executable for each of the missing application. If some of the\n"
			"applications are not installed, than press \"Cancel\",  install all of the\n"
			"missing applications and try again.\n\n"
			"You could modify %PATH% to make kolkhoznik"
			#ifdef _WIN32
			" (google for \"windows PATH\")"
			#endif // _WIN32
			"\nto find all of the applications automatically if they are already installed";
    wxStaticText* header = new wxStaticText(panel, wxID_ANY,header_str);
    this->SetMinSize(header->GetSize());
    //header->Wrap(MySizeFunctions::get_width_of_number_of_text_symbols(170, scrlWindow));

    main_sizer->Add(header, 0, wxALL|wxEXPAND, 15);
    main_sizer->Add(new wxStaticLine (panel),0, wxEXPAND|wxLEFT|wxRIGHT);
    int entry_height=0;
    for (unsigned int i=0;i<empty_apps.size();i++)
        {
        scrl_sizer->AddSpacer(15);
        PathToAppEntry *temp_pointer = new PathToAppEntry(scrlWindow, wxString(empty_apps[i].first));
        entry_height=temp_pointer->GetMinHeight();
        scrl_sizer->Add(temp_pointer, 0, wxALL|wxEXPAND, 5);

        //push_back copies pointer
        this->pointers_to_entries.push_back(temp_pointer);
        }
    scrlWindow->SetMinSize(wxSize(-1,  Khz::Libs::MySizeFunctions::get_height_of_number_of_text_rows(1,this)+empty_apps.size()*(15+entry_height)));
    scrlWindow->SetSizer(scrl_sizer);
    main_sizer->Add(scrlWindow,1, wxALL|wxEXPAND);
    //main_sizer->Add(new wxStaticText(panel, wxID_ANY, "pisdec"),0);
    //main_sizer->AddStretchSpacer();
    main_sizer->Add(new wxStaticLine (panel),0, wxEXPAND|wxLEFT|wxRIGHT);
        wxBoxSizer *buttons_sizer=new wxBoxSizer(wxHORIZONTAL);
            buttons_sizer->AddStretchSpacer ();
            buttons_sizer->Add(cancel_button,0);
            buttons_sizer->AddSpacer(3);
            buttons_sizer->Add(ok_button,0);
    main_sizer->Add(buttons_sizer,0,wxALL|wxEXPAND,5);

    scrl_sizer->FitInside(this);
        // this part makes the scrollbars show up
        scrl_sizer->FitInside(this); // ask the sizer about the needed size
        scrlWindow->SetScrollRate(5, 5);
        this->SetInitialSize();
         panel->SetSizerAndFit(main_sizer);
        this->Fit();
        panel->SetMinSize(header->GetSize());
    }


/*wxSize SelectAppsToExtractTextWindow::DoGetBestClientSize( ) const
    {
    int good_width=MySizeFunctions::get_width_of_number_of_text_symbols(170, this);
    return wxSize(good_width, -1);
    }
*/

/*void SelectAppsToExtractTextWindow::OnSize(wxSizeEvent &event)
    {
    static bool just_launched=true;
    if (just_launched) {just_launched=false;event.Skip(); return;};
    header->SetLabel(header_str);
    header->Wrap(main_sizer->GetSize().GetWidth()-50);
    main_sizer->RecalcSizes();
    //std::cout << "\nsize isss" << only_sizer->GetSize().GetWidth() <<"\n";
    event.Skip();
    }
*/


////////////////////////////////////////////////////////////////////
}}}// namespaces
/////////////////////////////////////////////////////////////////
