/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/dialog.h>
#include <wx/textctrl.h>
#include <wx/stattext.h>
#include "PathToAppEntry.h"


////////////////////////////////////////////////////////////
namespace Khz{ namespace FTS { namespace Gui{
////////////////////////////////////////////////////////////



/**
@brief Window to ask user to provide the full path to the missing executables.
Used by #Khz::FTS::Engine::MainTextExtractor::extractors_working().

Assume that we want user to provide the full path to the "djvutxt" and to "pdftotext" executables.
We can do the following:
@code
std::vector<std::pair<std::string, std::string>> empty_apps;
empty_apps.push_back(std::make_pair("djvutxt", "")); // we want full path to the djvutxt executable;
empty_apps.push_back(std::make_pair("pdftotext", "")); // we want full path to the pdftotext executable;

// we want to fill the second element of the every pair by the full paths to the executable.
// for example, we want to make empty_apps[0].second to be equal to "/full/path/to/djvutxt", the same for "pdftotext"

Khz::FTS::Gui::SelectAppsToExtractTextWindow *entries_dialog =
 new Khz::FTS::Gui::SelectAppsToExtractTextWindow(NULL, wxID_ANY, "Select apps", empty_apps);
// entries dialog would show two inputs asking user to provide
//the full path to the executable

    if(entries_dialog->ShowModal()==wxID_CANCEL)
        {
        delete entries_dialog;
        return false;
        }
    else
        {
        int temp_size = empty_apps.size();
        std::vector<std::pair<std::string, std::string> > temp_v=empty_apps;
        empty_apps.clear();
        for(int ksi=0;ksi<temp_size;ksi++)
            {
             empty_apps.push_back(std::make_pair(temp_v[ksi].first, entries_dialog->pointers_to_entries[ksi]->GetValue().ToStdString()));
            }
        }
// now empty_apps[0].second="/full/path/to/djvutxt"
//     empty_apps[1].second="/full/path/to/pdftotext"
@endcode
**/
class SelectAppsToExtractTextWindow : public wxDialog
{
public:
    /**
    Constructor
    @param parent standard wxwidgets parent
    @param id standard wxwidgets id
    @param Title the window title
    @param empty_apps see the usage example
    **/
    SelectAppsToExtractTextWindow(wxWindow *parent, const wxWindowID id,
                                      wxString Title, std::vector<std::pair<std::string, std::string>> &empty_apps);


    /**
    Ok button
    **/
    wxButton *cancel_button;


    /**
    Cancel button
    **/
    wxButton *ok_button;

    /**
    Each PathToAppEntry consists of a wxTextCtrl input and "Choose" button.
    So, user can choose the full path to the executable here.
    Size of #pointers_to_entries is equal to the size of the \a empty_apps constructor
    parameter. See #SelectAppsToExtractTextWindow usage example for more info.
    **/
    std::vector<PathToAppEntry*> pointers_to_entries;
    //wxSize DoGetBestClientSize( ) const;

private:
    /**
    scolled window that holds all of the inputs (#pointers_to_entries).
    If size of #pointers_to_entries is big, than scrollbar would appear.
    **/
    wxScrolledWindow *scrlWindow;
    //void OnSize(wxSizeEvent &event);
};


//////////////////////////////////////////////////////////////
}}}// namespaces
////////////////////////////////////////////////////////////////
