/*
Copyright (C) Ivan Ivanov, 2014-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/log.h>
#include <wx/sizer.h>
#include <wx/wrapsizer.h>
#include <wx/filedlg.h>
#include "sqlite3.h"
#include "parse_string.h"
#include <iostream>
#include <fstream>
#include <wx/dir.h>
#include "create_database.h"
//#include "Custom_IDs.h"
#include <wx/progdlg.h>
#include <wx/panel.h>
#include <wx/filename.h>
#include <wx/dirdlg.h>
#include <wx/stattext.h>
#include <wx/stdpaths.h>
#include <wx/frame.h>
#include <wx/textctrl.h>
#include "SqliteDBGenerator.h"
#include <wx/button.h>
#include <wx/msgdlg.h>
#include <openssl/md5.h>
#include <wx/fileconf.h>
#include "clear_database_path.h"
#include "platform_specific_defs.h"
#include <iso646.h>
#include <cstdlib>
#include "Mdb_fuctions.h"
#include "ConfigManager.h"
#include "DEBUGMESSAGE.h"
#include "Custom_IDs.h"



namespace Khz{namespace GenDB{




SqliteDBGenerator::SqliteDBGenerator(wxWindow* parent, const wxString& title)
    : wxFrame(parent, wxID_ANY, title)
    {
    check_for_missing_files=false;
    virify_files_md5=false;
	wxPanel *main_panel =new wxPanel(this, wxID_ANY);

   // #########PATH TO LIBRARY########################################
     choose_path_to_library_text=
                new wxStaticText( main_panel, wxID_ANY, wxT("Specify kolkhoz library directory"),wxDefaultPosition, wxSize(24, wxDefaultCoord));
     choose_path_to_library_text->Disable();

    library_dir_input=new wxTextCtrl(main_panel,wxID_ANY,wxEmptyString, wxDefaultPosition);
    library_dir_input->Disable();
    choose_path_to_library_button = new wxButton(main_panel,wxID_ANY, "Choose");
    choose_path_to_library_button->Disable();
    // ######################################################################################




    // ####### PATH TO DATABASE#################################################
    choose_path_to_database_text=
                new wxStaticText( main_panel, wxID_ANY, wxT("Specify path for new database"),wxDefaultPosition, wxSize(24, wxDefaultCoord));
    where_to_store_database_input = new wxTextCtrl(main_panel,wxID_ANY,
                                             wxStandardPaths::Get().GetUserConfigDir() + wxFileName::GetPathSeparator() + "."
                                              + "kolkhoznik" + wxFileName::GetPathSeparator() + "database.sqlite" , wxDefaultPosition);
    wxButton *choose_path_to_database_button = new wxButton(main_panel,wxID_ANY, "Choose");
    //######################################################################





    // ###################path to the "full_md5_len.txt" file####################################

    full_md5_len_input=new wxTextCtrl(main_panel,wxID_ANY,wxEmptyString, wxDefaultPosition);
    wxButton *choose_path_to_to_full_md5_len_button = new wxButton(main_panel,wxID_ANY, "Choose");


    // ############################################################################################



    //
    check_files_checkbox=new wxCheckBox(main_panel, wxID_ANY, "Check my library for missing files");
    check_files_checkbox->SetValue(false);

    check_for_missing_files_checkbox = new wxCheckBox(main_panel, wxID_ANY, "Check files existence");
    check_for_missing_files_checkbox->SetValue(true);
    check_for_missing_files_checkbox->Disable();
    check_md5_checkbox = new wxCheckBox(main_panel, wxID_ANY, "Verify files md5 (slow)");
    check_md5_checkbox->SetValue(false);
    check_md5_checkbox->Disable();



    // ####### PATH TO THE log otput #################################################
    choose_path_to_log_text=
                new wxStaticText( main_panel, wxID_ANY, wxT("Specify where to save the log file"),wxDefaultPosition, wxSize(24, wxDefaultCoord));
    choose_path_to_log_text->Disable();
    where_to_store_log_input = new wxTextCtrl(main_panel,wxID_ANY,
                                             wxStandardPaths::Get().GetUserConfigDir() + wxFileName::GetPathSeparator() + "."
                                              + "kolkhoznik" + wxFileName::GetPathSeparator() + "kolkhoz_database_creation_log.txt" , wxDefaultPosition);
    where_to_store_log_input->Disable();
    choose_path_to_log_button = new wxButton(main_panel,wxID_ANY, "Choose");
    choose_path_to_log_button->Disable();
    //######################################################################





    // ######################two buttons, "create database" and "cancel"###########################
    create_database_button = new wxButton(main_panel,wxID_ANY, "Generate database");
    cancel_creating_database_button = new wxButton(main_panel,wxID_ANY, "Cancel");
    //#############################################################################################


    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   SIZING !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    int left_right_border=10;
     wxBoxSizer    *only_sizer = new wxBoxSizer( wxVERTICAL );
    only_sizer->SetMinSize (600,-1);

    //##############choose path to library sizers##################################################################
     wxBoxSizer *input_dir_text= new wxBoxSizer( wxHORIZONTAL );                                                               //
    input_dir_text->AddSpacer(left_right_border);                                                                 //
    input_dir_text->Add(choose_path_to_library_text, 1, wxALL|wxEXPAND);                                              //
    input_dir_text->AddSpacer(left_right_border);                                                                 //
    wxSizerFlags flags_choose_path_to_library_sizer_text(0);                                                      //
    flags_choose_path_to_library_sizer_text.Expand();                                                             //
    flags_choose_path_to_library_sizer_text.Border(wxTOP, 5);                                                    //
                                                                                                                  //
                                                                                                                  //
    wxBoxSizer *input_dir_with_library_sizer = new wxBoxSizer( wxHORIZONTAL );                                    //
    input_dir_with_library_sizer->AddSpacer(left_right_border);                                                   //
    input_dir_with_library_sizer->Add(library_dir_input,1, wxCENTRE|wxALL);  //input for library path                 //
    input_dir_with_library_sizer->Add(choose_path_to_library_button);  //button to choose dir with library        //
    input_dir_with_library_sizer->AddSpacer(left_right_border);                                                            //
    //##############finished choose path for library  #############################################################




    //##############choose path to log sizers##################################################################
    wxBoxSizer *input_log_text_sizer= new wxBoxSizer( wxHORIZONTAL );                                                               //
    input_log_text_sizer->AddSpacer(left_right_border);                                                                 //
    input_log_text_sizer->Add(choose_path_to_log_text, 1, wxALL|wxEXPAND);                                              //
    input_log_text_sizer->AddSpacer(left_right_border);                                                                 //
    wxSizerFlags flags_choose_path_to_log_sizer_text(0);                                                      //
    flags_choose_path_to_log_sizer_text.Expand();                                                             //
    flags_choose_path_to_log_sizer_text.Border(wxTOP, 20);                                                    //
                                                                                                                  //
                                                                                                                  //
    wxBoxSizer *input_log_sizer = new wxBoxSizer( wxHORIZONTAL );                                    //
    input_log_sizer->AddSpacer(left_right_border);                                                   //
    input_log_sizer->Add(where_to_store_log_input,1, wxCENTRE|wxALL);       //
    input_log_sizer->Add(choose_path_to_log_button);         //
    input_log_sizer->AddSpacer(left_right_border);
    //##############finished choose path for library  #############################################################




    //##############choose path to full_md5_len sizers##################################################################
    wxWrapSizer *input_full_md5_len_text_sizer= new wxWrapSizer( wxHORIZONTAL );                                                               //
                                                                 //
    input_full_md5_len_text_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "Select the file "), 0, wxALL);                                              //
    input_full_md5_len_text_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "\"full_md5_len.txt\" "), 0, wxALL);
    input_full_md5_len_text_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "(part of the kolkhoz "), 0, wxALL);
    input_full_md5_len_text_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "library distribution, "), 0, wxALL);
    input_full_md5_len_text_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "unpack \"full_md5_len.txt.gz\" "), 0, wxALL);
    input_full_md5_len_text_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "to get it)"), 0, wxALL);
                                                              //
    wxSizerFlags flags_choose_input_full_md5_len_text_sizer_text(0);                                                      //
    flags_choose_input_full_md5_len_text_sizer_text.Expand();                                                             //
    flags_choose_input_full_md5_len_text_sizer_text.Border(wxTOP, 20);                                                    //
                                                                                                                  //
                                                                                                                  //
    wxBoxSizer *input_md5_len_sizer = new wxBoxSizer( wxHORIZONTAL );                                    //
    input_md5_len_sizer->AddSpacer(left_right_border);                                                   //
    input_md5_len_sizer->Add(full_md5_len_input,1, wxCENTRE|wxALL);       //
    input_md5_len_sizer->Add(choose_path_to_to_full_md5_len_button);         //
    input_md5_len_sizer->AddSpacer(left_right_border);                                                               //
    //##############finished choose path for library  #############################################################








        //##############choose path to database sizers#############################################################
    wxBoxSizer *input_database_text= new wxBoxSizer( wxHORIZONTAL );                                               //
    input_database_text->AddSpacer(left_right_border);                                                             //
    input_database_text->Add(choose_path_to_database_text, 1, wxALL|wxEXPAND);                                         //
    input_database_text->AddSpacer(left_right_border);                                                             //
    wxSizerFlags flags_choose_path_to_database_sizer_text(0);                                                      //
    flags_choose_path_to_database_sizer_text.Expand();                                                             //
    flags_choose_path_to_database_sizer_text.Border(wxTOP, 20);                                                    //
                                                                                                                   //
                                                                                                                   //
    wxBoxSizer *input_database_path_sizer = new wxBoxSizer( wxHORIZONTAL );                                        //
    input_database_path_sizer->AddSpacer(left_right_border);                                                       //
    input_database_path_sizer->Add(where_to_store_database_input,1,wxCENTRE|wxALL);  //input for library path         //
    input_database_path_sizer->Add(choose_path_to_database_button);  //button to choose dir with library           //
    input_database_path_sizer->AddSpacer(left_right_border);
                                                            //
    //##############finished choose path for library  #############################################################



    //#################### two buttons -- create and cancel##########################################################
    wxBoxSizer *two_last_buttons = new wxBoxSizer(wxHORIZONTAL);
    two_last_buttons->AddSpacer(left_right_border);
    two_last_buttons->Add(create_database_button);
    two_last_buttons->AddSpacer(5);
    two_last_buttons->Add(cancel_creating_database_button);
    two_last_buttons->AddSpacer(left_right_border);
    wxSizerFlags flag_to_fit_two_last_buttons(1);
    flag_to_fit_two_last_buttons.Center();
    flag_to_fit_two_last_buttons.Border(wxTOP, 50);
    flag_to_fit_two_last_buttons.Border(wxBOTTOM, 10);
    //flag_to_fit_two_last_buttons.Expand();
    //###############################################################################################################





    only_sizer->Add(input_full_md5_len_text_sizer,0, wxLEFT|wxRIGHT|wxEXPAND, left_right_border); //text choose md5_full
    only_sizer->Add(input_md5_len_sizer,0, wxEXPAND|wxALL); // input for md5_len_txt
    //
    //
    only_sizer->Add(input_database_text,flags_choose_path_to_database_sizer_text);  // text "choose where to store database"
    only_sizer->Add(input_database_path_sizer,0, wxEXPAND|wxALL); //input for database path(where to store) and button


    only_sizer->AddSpacer(55);
    only_sizer->Add(check_files_checkbox,0,wxEXPAND|wxALL,2);
        wxBoxSizer *two_checkboxes_sizer= new wxBoxSizer(wxVERTICAL);
            two_checkboxes_sizer->Add(check_for_missing_files_checkbox, 0, wxALL);
            two_checkboxes_sizer->Add(check_md5_checkbox, 0, wxALL);
    only_sizer->Add(two_checkboxes_sizer, 0, wxALL, 15);
    //
    only_sizer->Add(input_dir_text,flags_choose_path_to_library_sizer_text);   //text "choose library path"
    only_sizer->Add(input_dir_with_library_sizer,0, wxEXPAND|wxALL); //button and input for path to library
    //
    only_sizer->Add(input_log_text_sizer,flags_choose_path_to_log_sizer_text); //text choose md5_full
    only_sizer->Add(input_log_sizer,0, wxEXPAND|wxALL); // input for md5_len_txt
    //
    only_sizer->AddSpacer(20);
    only_sizer->AddStretchSpacer();
    only_sizer->Add(two_last_buttons,0, wxCENTER|wxALL, 5);


        main_panel->Fit();

        main_sizer = new wxBoxSizer( wxVERTICAL );
        main_sizer->Add(main_panel,1, wxALL|wxEXPAND);
        main_panel->SetSizerAndFit(only_sizer);



    choose_path_to_library_text->SetLabel(wxT("Specify the kolkhoz library directory"));
    choose_path_to_library_text->Wrap(input_dir_text->GetSize().GetWidth() - 2*left_right_border);



    SetSizerAndFit(main_sizer);


    only_sizer->RecalcSizes();
    only_sizer->SetSizeHints (main_panel) ;
    main_sizer->RecalcSizes ();
    main_sizer->SetSizeHints (this) ;
    Refresh();
    Update();

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!! SIZING FINISHED !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   // Bind(wxEVT_SIZE, &SqliteDBGenerator::OnSize, this);

    choose_path_to_library_button->Bind(wxEVT_BUTTON, &SqliteDBGenerator::DoKolhozPathChoose, this);
    choose_path_to_database_button->Bind(wxEVT_BUTTON, &SqliteDBGenerator::DoKolhozDatabasePathChoose, this);
    choose_path_to_to_full_md5_len_button->Bind(wxEVT_BUTTON, &SqliteDBGenerator::DoKolhoz_full_md5_len_Choose, this);
    choose_path_to_log_button->Bind(wxEVT_BUTTON, &SqliteDBGenerator::DoKolhozLogPathChoose, this);
    check_files_checkbox->Bind(wxEVT_CHECKBOX, &SqliteDBGenerator::OnCheckFilesCheckboxClicked, this);


    cancel_creating_database_button->Bind(wxEVT_BUTTON, &SqliteDBGenerator::OnCancel, this);
    create_database_button->Bind(wxEVT_BUTTON, &SqliteDBGenerator::OnCreateDatabaseButtonPressed, this);
    check_for_missing_files_checkbox->Bind(
        wxEVT_CHECKBOX, &SqliteDBGenerator::On_check_for_missing_files_checkbox_clicked, this);
    }

//void SqliteDBGenerator::OnSize(wxSizeEvent &event)
//    {
/*
    choose_path_to_library_text->SetLabel(wxT("Specify kolhoz library directory (optionally, needed for generating appropriate log)"));
    choose_path_to_library_text->Wrap(input_dir_text->GetSize().GetWidth() - 2*left_right_border);



    choose_path_to_full_md5_len_text->SetLabel(wxT("Select the \"full_md5_len.txt\" file (part of the kolkhoz library distribution, unpack \"full_md5_len.txt.gz\" to get it) "));
    choose_path_to_full_md5_len_text->Wrap(input_dir_text->GetSize().GetWidth() - 2*left_right_border);
    //std::cout << "wide is set to " << input_dir_text->GetSize().GetWidth() - 2*left_right_border;
    main_sizer->RecalcSizes ();
    main_sizer->Layout();
    */
 //   event.Skip();
 //   }

void SqliteDBGenerator::DoKolhozPathChoose(wxCommandEvent& event)
{
    // pass some initial dir to wxDirDialog
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxDirDialog dialog(this, wxT("Select Kolkhoz library directory"), dirHome, wxDD_DEFAULT_STYLE && wxDD_DIR_MUST_EXIST);

    if (dialog.ShowModal() == wxID_OK)
    {
        library_dir_input->SetValue (dialog.GetPath());
    }
}

void SqliteDBGenerator::DoKolhozDatabasePathChoose(wxCommandEvent &event)
{
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    //wxDirDialog dialog(this, wxT("Choose where to store database"), dirHome, wxDD_DEFAULT_STYLE);

    wxFileDialog
        dialog(this, _("Choose where to save database"), "", "",
                       "sqlite files (*.sqlite)|*.sqlite", wxFD_SAVE);


    if (dialog.ShowModal() == wxID_OK)
        {
        if(dialog.GetPath().EndsWith(".sqlite") )
            {
            where_to_store_database_input->SetValue (dialog.GetPath());
            }
        else
            {
            where_to_store_database_input->SetValue (dialog.GetPath()+".sqlite");
            }
        }
}



void SqliteDBGenerator::DoKolhoz_full_md5_len_Choose(wxCommandEvent &event)
{
    wxString dirHome;
    wxGetHomeDir(&dirHome);

    wxFileDialog dialog(this, wxT("Choose the full_md5_len.txt file"), dirHome);

    if (dialog.ShowModal() == wxID_OK)
    {
        full_md5_len_input->SetValue (dialog.GetPath());
    }
}


void SqliteDBGenerator::DoKolhozLogPathChoose(wxCommandEvent &event)
    {
     wxFileDialog
        dialog(this, _("Choose where to store the log file"), "", "kolkhoz_database_creation_log.txt",
                       "text files (*.txt)|*.txt", wxFD_SAVE);


    if (dialog.ShowModal() == wxID_OK)
        {
        if(dialog.GetPath().EndsWith(".txt") )
            {
            where_to_store_log_input->SetValue (dialog.GetPath());
            }
        else
            {
            where_to_store_log_input->SetValue (dialog.GetPath()+".txt");
            }
        }
    }

void SqliteDBGenerator::OnCancel(wxCommandEvent &event)
{
    Close();
}
void SqliteDBGenerator::OnCreateDatabaseButtonPressed(wxCommandEvent &event)
{
    /////////////////////////////////////////////////////////////////////////////////
    //  Getting values from the gui inputs
    ////////////////////////////////////////////////////////////////////////////////
    wxString path_to_library= library_dir_input->GetValue();
    wxString path_to_database = where_to_store_database_input ->GetValue();
    wxString path_to_full_md5_len_txt=full_md5_len_input->GetValue();
    wxString path_to_log_file= where_to_store_log_input->GetValue();

    wxFileName library_path_file_name= wxFileName (path_to_library);






    //  ############################################################################################################################
    //                        CHECKING IF FILES AND DIRS EXIST
    //  ############################################################################################################################
    if( check_for_missing_files and (!wxDirExists(library_path_file_name.GetFullPath())  or   !wxFileName::IsDirReadable(path_to_library)) )
        {
        wxMessageDialog   *err_msg =new wxMessageDialog (this,
                                    "Cannot read the Library directory \"" + path_to_library + "\". Check that this directory exists and you have permission to read it.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg->ShowModal ();
        return;
        }
    if(  !wxFileExists(path_to_full_md5_len_txt)  or   !wxFileName::IsFileReadable(path_to_full_md5_len_txt)  )
        {
        wxMessageDialog   *err_msg =new wxMessageDialog (this,
                                    "Cannot read the full_md5_len.txt file \"" + path_to_full_md5_len_txt + "\". Check that this file exists and you have permission to read it.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg->ShowModal ();
        return;
        }
    if (  !wxDirExists(wxFileName (path_to_database).GetPath())  or   !wxFileName::IsDirWritable(wxFileName (path_to_database).GetPath())   )
        {
                wxMessageDialog   *err_msg =new wxMessageDialog (this,
                "Cannot write to the specified directory \"" + wxFileName (path_to_database).GetPath() + "\" where the database is supposed to be saved. Check that this directory exists and you have permission to write into it.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg->ShowModal ();
        return;
        }
    if ( check_for_missing_files and
                        (!wxDirExists(wxFileName (path_to_log_file).GetPath())  or   !wxFileName::IsDirWritable(wxFileName (path_to_log_file).GetPath()) )
            )
        {
                wxMessageDialog   *err_msg =new wxMessageDialog (this,
                "Cannot write to the specified directory \"" + wxFileName (path_to_log_file).GetPath() + "\" where the log is supposed to be saved. Check that this directory exists and you have permission to write into it.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg->ShowModal ();
        return;
        }
    if(check_for_missing_files and wxFileExists(path_to_log_file) )
        {
        wxMessageDialog err_msg(this,
                                    "The specified log file already exists. Overwrite?",
                                    "Hi there!", wxYES_NO|wxCENTRE|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition);


        switch ( err_msg.ShowModal() )
            {
            case wxID_YES:
                break;

            case wxID_NO:
                return;
                break;
            default:
                wxLogError(wxT("Unexpected wxMessageDialog return code!"));
            }
        }

    if(wxFileExists(path_to_database))
        {
        wxMessageDialog err_msg(this,
                                    path_to_database + " already exists. Overwrite?",
                                    "Hi there!", wxYES_NO|wxCENTRE|wxICON_QUESTION|wxSTAY_ON_TOP, wxDefaultPosition);

        switch ( err_msg.ShowModal() )
            {
            case wxID_YES:
                {
                break;
                };
            case wxID_NO:
                {
                return;
                };
            default:
                wxLogError(wxT("Unexpected wxMessageDialog return code in path_to_database!"));
            }
        }
        //  ############################################################################################################################
    //                        finished with checking, now genereate database
    //  ############################################################################################################################
    //go on generating the database
        {
        sqlite3 *database;
		wxCharBuffer temp_value=Khz::Libs::DB::clear_path(path_to_database).ToUTF8();
		//std::cout << "\n original path to db is "<< temp_value.data() << "\n";
        sqlite3_open(temp_value.data(), &database);
	   //std::cout << "\nin create database function path is " << My_clear_db_path::clear_path(path_to_database) << "\n";
        sqlite3_exec(database,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
        progress_bar= new wxGenericProgressDialog ("Database generation progress", "The database is being generated. Please be patient.", 100,this, wxPD_AUTO_HIDE|wxPD_APP_MODAL|wxPD_REMAINING_TIME);
        progress_bar->Update(0);
        bool is_database_generated;
        try
            {
            create_dbtable_topics(database);
            create_dbtable_main(database);
            sqlite3_exec(database, "VACUUM;", 0, 0, 0);
            //############# old version, goes through the library dir recursively, finds files, fills the database.###########
            //long double *temp_num_pointer;
            //long double temp_num=0.0;
            //temp_num_pointer = &temp_num;

            //long double *temp_num_pointer1;
            //long double temp_num1=0.0;
            //temp_num_pointer1 = &temp_num1;

            //old version, goes through the library dir recursively, finds files, fills the database.
            //fill_database(path_to_library,database,temp_num_pointer, temp_num_pointer1);
            //################################################################################################################
            is_database_generated=fill_database_v2( path_to_library,  path_to_full_md5_len_txt, path_to_log_file,database);
            add_missing_topics(database);
            }
        catch(...)
            {
            progress_bar->Close();
             wxMessageDialog   err_msg(this,
                                    "Cannot generate the database. Ensure that you have anought disk space and try again" , "Error" ,wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
            err_msg.ShowModal ();
            sqlite3_close(database);
            return;
            };

        sqlite3_close(database);
        progress_bar->Update(100);
        progress_bar->Close();
        wxString ini_filename = ConfigManager::GetPathToConfigFile();
        wxFileConfig *config =new  wxFileConfig( "", "", ini_filename);
        if(check_for_missing_files) config->Write( wxT("PATHS/path_to_dir_with_books"), path_to_library );
        config->Write( wxT("PATHS/path_to_database"), path_to_database );
        config->Flush();
        delete config;
            //#success message dialog, is shown when database creation finished.


        if(!is_database_generated) return; // we do not show error message here. fill_database_v2 will do it.


        //if is_database_generated is true than database generation successful.
        wxString Message;
        if(errors_count==0)
            {
            Message="The database has been generated successfully.";
            }
        else
            {
            Message="The database has been generated successfully, but you have some problems with your library. See the log file \""+path_to_log_file+"\" for details. "+
                   // " There are two possible errors: the \"file not found\" error and the \"md5 mismatch\" error."+
                   //" Put missing file to the kolkhoz library directory to fix the first error."+
                   //" Replace file with the wrong md5 to fix the second. "+
                    "You do not need to regenerate the database, just solve the problems with the library.";

            }


        ///////////////////////////////////////////////////////////////////////////////////////////////////
        //
        //                    SENDING EVENT
        //
        // this event is supposed to be catched by Khz::MainWin::MyMainWindow.
        //MyMainWindow::ReconfigurePanels will be executed. This function
        // will read the kolkhoznik config file, reconnect to the databases and
        // reload left and right panels, as well as the full text search window (if necessary)
            {
             wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
             event_to_propagate.SetId(my_wx_id__RECONFIGURE_PANELS);
             event_to_propagate.SetInt(Khz::my_wx_id__RECONFIGURE_EVERYTHING);
             ProcessWindowEvent 	( 	event_to_propagate	) ;
            }
        ////////////////////////////////////////////////////////////////////////////////////////////////////
        //
        //             FINISHED WITH EVENT
        //
        /////////////////////////////////////////////////////////////////////////////////////////////////////


        wxMessageDialog *success_message =new wxMessageDialog (this,
                                    Message,
                                    "Database generation finished", wxOK|wxCENTRE|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        success_message->ShowModal ();
        wxMessageBox("Do not forget to (re)generate the full text search index", "Kolkhoznik database generator");
        this->Close();
        }

}








void SqliteDBGenerator::fill_database(wxString dir_with_books,sqlite3 *db,long double *parsed_size,  long double *total_size_of_library, wxString abs_path, long int last_id){
  	if(abs_path == "empty") //first enter
        {
        // put root dir as empty  topic to DB
        std::string query;
        sqlite3_stmt *stmt = NULL;
        query = "INSERT INTO topics(Topic) VALUES (?);";
        sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
        sqlite3_bind_text(stmt,  1, "", -1, SQLITE_TRANSIENT);
        sqlite3_step(stmt);
        sqlite3_finalize(stmt);
        //finished iserting root dir into DB

        abs_path = dir_with_books;
        }
  	  if( *total_size_of_library==0.0) *total_size_of_library=(wxDir::GetTotalSize (dir_with_books)).ToDouble();  //have to be defined only once;
    if (*parsed_size<100)
        {
        std::cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
        std::cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
        std::cout << "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
        std::cout <<  "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
        }
	wxDir dir(dir_with_books);
	if ( !dir.IsOpened() )
    {
        // deal with the error here - wxDir would already log an error message
        // explaining the exact reason of the failure
        throw("cannot open dir specified, see fill_database.cpp");
    }
    wxString filename;


     bool cont = dir.GetFirst(&filename);
    while ( cont )
        {
                                  //path          //filename
        wxFileName 	 current_file(dir_with_books,filename,wxPATH_NATIVE);


        //std::cout << "\n\n now working with " << current_file.GetFullPath() << "\n";
        if (wxDirExists(current_file.GetFullPath())) //std::cout << "dir " << current_file.GetFullPath() << "\n";
            {
            if(//dir_with_books==abs_path or
                        filename=="lost+found" || filename=="catalog" || filename=="catalog_allDVDs")
                {
                std::cout << "skipping " << current_file.GetFullPath()<< "\n";
                cont = dir.GetNext(&filename);
                continue;
                }
            wxString topic_to_db= current_file.GetFullPath(); //topic is path
            topic_to_db.Replace(abs_path, "", false);  // false indicates that only first occurence must be replaced.
            topic_to_db.Replace(wxFileName::GetPathSeparator(), "/");
            topic_to_db=this->clear_topic(topic_to_db);
            //value_to_db.Replace("'", "''");


            std::string query;
            sqlite3_stmt *stmt = NULL;
            query = "INSERT INTO topics(Topic) VALUES (?);";
            sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
            sqlite3_bind_text(stmt,  1, topic_to_db.ToStdString().c_str(), -1, SQLITE_TRANSIENT);
            sqlite3_step(stmt);
            sqlite3_finalize(stmt);


            //std::cout << "now doing " << query<< " "<<topic_to_db.ToStdString().c_str() << "\n";
            //let's cd into dhis dir and repeat everything there recursively
            fill_database(current_file.GetFullPath(),db, parsed_size,total_size_of_library, abs_path, sqlite3_last_insert_rowid(db));  // recursion, we need to put into database all of the folders.
            }
        else if(wxFileExists(current_file.GetFullPath()))
            {
            //std::cout << "\n\n file taki exists\n";
            if(//dir_with_books==abs_path or
                        filename=="TRANS.TBL") //skip all of the files in the top level dir.
                {
                std::cout << "skipping " << current_file.GetFullPath()<< "\n";
                cont = dir.GetNext(&filename);
                continue;
                }
            parse_string val_to_db(filename.ToStdString());
            wxString Topic= current_file.GetPath();
            Topic.Replace(abs_path, "", false);
            Topic.Replace(wxFileName::GetPathSeparator(), "/");
            wxString filepath_to_db=Topic;
            Topic=this->clear_topic(Topic);


            //~check that topic is in the database
            std::string if_topic_exist_query = "SELECT COUNT(*) FROM topics WHERE Topic=?;";
            sqlite3_stmt* stmt = NULL;
            int retval = sqlite3_prepare_v2(db, if_topic_exist_query.c_str(), -1, &stmt, 0);
            if(retval) throw(std::string("cannot connect to db in fill_database.cpp"));
            sqlite3_bind_text(stmt,  1, Topic.ToStdString().c_str() , -1, SQLITE_TRANSIENT);
            retval = sqlite3_step(stmt);
            int topics_number = sqlite3_column_int(stmt, 0);
            sqlite3_finalize(stmt);



            //std::cout << topics_number;
            if (topics_number==0)
                {
                std::cout << "\noh pizdec pizdec\n";
                throw(std::string("fill_database.cpp error with topics number"));
                }
            //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~finished with check            /*
            else
                {
                //std::cout << "\n\n trying to insert file into db \n";
                sqlite3_stmt *stmt = NULL;
                std::string query;
                query = std::string("INSERT INTO main(")+
					"Topic, ID_of_Topic, md5_of_file,"+
					"Author,AuthorLowerCase,Extension,Pages, DPI,"+
					"Publisher, Year, Language, ISBN,"+
					"Orientation, Color, OCR,ASIN,"+
					"Volume,Title,TitleLowerCase,Edition,"+
					"Filename,Filesize,RelPathToFile)"+
					"VALUES ("+
					+"?,?,?,"+
					"?,?,?,?,?,"+
					"?,?,?,?,"+
					"?,?,?,?,"+
					"?,?,?,?,"+
					"?,?,?"+
					");";
                sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
                sqlite3_bind_text(stmt,  1, Topic, -1, SQLITE_TRANSIENT);
                sqlite3_bind_int(stmt,  2, last_id);
                sqlite3_bind_text(stmt,  3, md5(current_file).c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  4, val_to_db.Author.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  5, (Khz::Libs::Mstr::to_lower_case(val_to_db.Author)).c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  6, val_to_db.Extension.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_int (stmt,  7, atoi(val_to_db.Pages.c_str()));
                sqlite3_bind_int (stmt,  8, atoi(val_to_db.DPI.c_str()));
                sqlite3_bind_text(stmt,  9, val_to_db.Publisher.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_int (stmt,  10, atoi(val_to_db.Year.c_str()));
                sqlite3_bind_text(stmt,  11, val_to_db.Language.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 12, val_to_db.ISBN.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 13, val_to_db.Orientation.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 14, val_to_db.Color.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 15, val_to_db.OCR.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 16, val_to_db.ASIN.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 17, val_to_db.Volume.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 18, val_to_db.Title.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 19, (Khz::Libs::Mstr::to_lower_case(val_to_db.Title)).c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 20, val_to_db.Edition.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 21, filename.ToStdString().c_str(), -1, SQLITE_TRANSIENT);
                wxULongLong file_size_current = current_file.GetSize();
                sqlite3_bind_int(stmt, 22, atoi(file_size_current.ToString().ToStdString().c_str()));
                sqlite3_bind_text(stmt, 23, filepath_to_db.c_str(), -1, SQLITE_TRANSIENT);


                sqlite3_step(stmt);
                sqlite3_finalize(stmt);
             //   std::cout <<  "current author is " << (Mstr::to_lower_case(val_to_db.Author)).c_str() << "\n";
             //   std::cout << "previous parsed size is  " << *parsed_size << "\n";
                *parsed_size=*parsed_size + (file_size_current.ToDouble());
             //   std::cout << "current file size is     " << file_size_current.ToDouble() << "\n";
                //std::cout << "parsed size increased to " << *parsed_size << "\n";


                // PROGRESS BAR UPDATING
                double files_parsed_in_procents= *parsed_size/ *total_size_of_library*100;
                int parsed_in_percents_fo_progress_bar= (int) (files_parsed_in_procents  );
              //  if(parsed_in_percents_fo_progress_bar >=100) parsed_in_percents_fo_progress_bar=99; //safty protocol
                progress_bar->Update (parsed_in_percents_fo_progress_bar);


                }
            }
        cont = dir.GetNext(&filename);
        }

}

std::string SqliteDBGenerator::md5(wxFileName 	 current_file)
    {
   // std::string filename_temp = current_file.GetFullPath().ToStdString();
    //char filename[filename_temp.length() + 1];
    //std::strcpy(filename, filename_temp.c_str());
    unsigned char c[MD5_DIGEST_LENGTH];
    int i;
    FILE *inFile = fopen (current_file.GetFullPath().mb_str(wxConvLocal).data(), "rb");
    MD5_CTX mdContext;
    int bytes;
    unsigned char data[1024];

    if (inFile == NULL) {
        printf ("%s can't be opened.\n", current_file.GetFullPath().mb_str(wxConvUTF8).data());
        return "";
    }

    MD5_Init (&mdContext);
    while ((bytes = fread (data, 1, 1024, inFile)) != 0)
        MD5_Update (&mdContext, data, bytes);
    MD5_Final (c,&mdContext);
    char result[33];
    for(i = 0; i < MD5_DIGEST_LENGTH; i++)    snprintf(result+i*2, 4, "%02x", c[i]);
    fclose (inFile);
    return result;
    }

wxString SqliteDBGenerator::clear_topic(wxString topic) const
    {
    if(topic=="") return topic;
    std::string topic_std= topic.ToStdString();
    std::vector<std::string> exploded_topic=Khz::Libs::Mstr::explode("/", topic_std);
    for(unsigned int i=0;i<exploded_topic.size();i++)
        {
        std::vector<std::string> secondary_exploded_topic= Khz::Libs::Mstr::explode("_", exploded_topic[i]);
        if (secondary_exploded_topic.size()==2)
            {
            exploded_topic[i]=secondary_exploded_topic[1];
            }
        }
    return wxString(Khz::Libs::Mstr::implode("/", exploded_topic));
    }










void SqliteDBGenerator::put_file_to_db(std::string filename, long long int topic_id, std::string Topic,
                                          std::string RelPath, std::string md5, std::string filesize, sqlite3 *db)
    {
                parse_string val_to_db(filename);
                sqlite3_stmt *stmt = NULL;
                std::string query;
                query = std::string("INSERT INTO main(")+
					"Topic, ID_of_Topic, md5_of_file,"+
					"Author,AuthorLowerCase,Extension,Pages, DPI,"+
					"Publisher, Year, Language, ISBN,"+
					"Orientation, Color, OCR,ASIN,"+
					"Volume,Title,TitleLowerCase,Edition,"+
					"Filename,Filesize,RelPathToFile,m_hash)"+
					"VALUES ("+
					+"?,?,?,"+
					"?,?,?,?,?,"+
					"?,?,?,?,"+
					"?,?,?,?,"+
					"?,?,?,?,"+
					"?,?,?,?"+
					");";
                sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
                sqlite3_bind_text(stmt,  1, Topic.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_int(stmt,  2, topic_id);
                sqlite3_bind_text(stmt,  3, md5.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  4, val_to_db.Author.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  5, (Khz::Libs::Mstr::to_lower_case(val_to_db.Author)).c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt,  6, val_to_db.Extension.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_int (stmt,  7, atoi(val_to_db.Pages.c_str()));
                sqlite3_bind_int (stmt,  8, atoi(val_to_db.DPI.c_str()));
                sqlite3_bind_text(stmt,  9, val_to_db.Publisher.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_int (stmt,  10, atoi(val_to_db.Year.c_str()));
                sqlite3_bind_text(stmt,  11, val_to_db.Language.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 12, val_to_db.ISBN.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 13, val_to_db.Orientation.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 14, val_to_db.Color.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 15, val_to_db.OCR.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 16, val_to_db.ASIN.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 17, val_to_db.Volume.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 18, val_to_db.Title.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 19, (Khz::Libs::Mstr::to_lower_case(val_to_db.Title)).c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 20, val_to_db.Edition.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_bind_text(stmt, 21, filename.c_str(), -1, SQLITE_TRANSIENT);
                //wxULongLong file_size_current = current_file.GetSize();
                sqlite3_bind_int(stmt, 22, atoi(filesize.c_str()));
                sqlite3_bind_text(stmt, 23, RelPath.c_str(), -1, SQLITE_TRANSIENT);

                std::string my_hash=Khz::Libs::Mstr::sha1(Topic+std::string("_")+val_to_db.Author+std::string("_")+val_to_db.Extension+std::string("_")+
                                            val_to_db.Publisher+std::string("_")+val_to_db.Language+std::string("_")+val_to_db.Title+std::string("_")+
                                            val_to_db.ISBN+std::string("_")+val_to_db.Orientation+std::string("_")+
                                            filename+std::string("_")+RelPath);


                sqlite3_bind_text(stmt, 24, my_hash.c_str(), -1, SQLITE_TRANSIENT);


                sqlite3_step(stmt);
                sqlite3_finalize(stmt);
    }





std::string SqliteDBGenerator::clear_relative_path(std::string relative_path_std)
    {

        wxString relative_path=wxString(relative_path_std);
        if(relative_path.Len() ==0)
            {
            std::cout << "\n\n" << "something terrible happen when parsing full_md5_len.txt. Filename is empty";
            abort();
            }
        if(relative_path.Left(1)!="/")
            {
            relative_path="/"+relative_path;
            }
        if(relative_path.Right(1)=="/")
            {
            relative_path.RemoveLast();
            }
        return relative_path.ToStdString();
    }
long long int SqliteDBGenerator::insert_topic_to_db_if_necessary(std::string topic, sqlite3 *db)
    {
            std::vector<long long int> ID_s;
            //~check that topic is in the database
            std::string if_topic_exist_query = "SELECT ID FROM topics WHERE Topic=?;";
            sqlite3_stmt* stmt = NULL;
            int retval = sqlite3_prepare_v2(db, if_topic_exist_query.c_str(), -1, &stmt, 0);
            if(retval) throw(std::string("cannot connect to db in fill_database.cpp"));
            sqlite3_bind_text(stmt,  1, topic.c_str() , -1, SQLITE_TRANSIENT);
            retval = sqlite3_step(stmt);
            //int topics_number = sqlite3_column_int(stmt, 0);
            while(  sqlite3_column_int( stmt, 0) )  //by id, it always exists and starts with 1
                {
                ID_s.push_back( sqlite3_column_int( stmt, 0)  );
                sqlite3_step( stmt );
                }
            sqlite3_finalize(stmt);



            //std::cout << topics_number;
            if (ID_s.size()>1)
                {
                std::cout << "\noh pizdec pizdec\n";
                throw(std::string("fill_database.cpp error in insert_topic_to_db_if_necessary"));
                abort();
                }
            else if(ID_s.size()==0)
                {
                std::string query;
                sqlite3_stmt *stmt2 = NULL;
                query = "INSERT INTO topics(Topic) VALUES (?);";
                sqlite3_prepare_v2( db, query.c_str(), -1, &stmt2, NULL );
                sqlite3_bind_text(stmt2,  1, topic.c_str(), -1, SQLITE_TRANSIENT);
                sqlite3_step(stmt2);
                sqlite3_finalize(stmt2);
                return sqlite3_last_insert_rowid(db);
                }
            else if (ID_s.size()==1)
                {
                return ID_s[0];
                }
            else
                {
                 std::cout << "fill_database.cpp error in insert_topic_to_db_if_necessary";

                abort();
                }

    }


bool SqliteDBGenerator::process_line(std::string line, wxString path_to_library, std::ofstream &log_file, sqlite3 *db)
    {
    boost::smatch res;
    boost::regex rx("(\\w+)\\s+(\\d+)\\s+(.+)\\s*",boost::regex_constants::icase);
    if (regex_search(line,res, rx))
        {
        std::string md5= res.str(1);
        std::string filesize=res.str(2);
        std::string full_path_to_file_temp= res.str(3);
        std::vector<std::string> exploded_full_path=Khz::Libs::Mstr::explode("/", full_path_to_file_temp);

        std::string filename=exploded_full_path[exploded_full_path.size()-1];


        exploded_full_path.pop_back();
        std::string relative_path=Khz::Libs::Mstr::implode("/", exploded_full_path);

        std::string RelPathToDb=clear_relative_path(relative_path);
        std::string topic_to_db=this->clear_topic(wxString(RelPathToDb)).ToUTF8().data();
        long long int id_in_main_table = insert_topic_to_db_if_necessary(topic_to_db, db);
        put_file_to_db(filename, id_in_main_table, topic_to_db,RelPathToDb, md5,filesize, db);

        if(check_for_missing_files)
            {

            if(path_to_library.Last()==wxFileName::GetPathSeparator())
                {
                path_to_library= path_to_library.RemoveLast();
                }
            wxString temp_str= wxString(RelPathToDb);

        //std::cout << "\n temp_str before replacing "<< temp_str.c_str()<< "\n";
            temp_str.Replace("/", wxFileName::GetPathSeparator(), true);
            wxString full_path_to_file=path_to_library + temp_str+wxFileName::GetPathSeparator()+filename;




            wxFileName current_file(full_path_to_file);
            std::string newline_char = wxString(wxTextFile::GetEOL()).ToStdString(); // \n on linux, \r\n on windows;
            if(!wxFileExists(full_path_to_file))
                {
                log_file<< "Error: file does not exist:" <<  full_path_to_file.ToStdString() << newline_char;
                errors_count++;
                }
            else if(virify_files_md5)
                {
                std::string real_md5=this->md5(current_file);
                if(real_md5!=md5)
                    {
                    log_file << "Error: md5 mismatch for file:" << full_path_to_file.ToStdString() << newline_char;
                    errors_count++;
                    }
                }
            }
        }
    else if(line=="")
        {
        }
    else
        {
        wxString message= "Bad line \"" + line +"\"in full_md5_len.txt";
        wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        return false;
        }
    return true;
    }


bool SqliteDBGenerator::fill_database_v2( wxString path_to_library,  wxString path_to_full_md5_len_txt, wxString path_to_log_file, sqlite3 *db)
    {
    this->errors_count=0;
    insert_topic_to_db_if_necessary("", db);//root



    ////////////////////////////////////////////////////////////////////////////////////
    //    open the full_md5_len.txt that contains the books list.
    //    count the lines number in the file (set total_num_of_lines).
    ////////////////////////////////////////////////////////////////////////////////
    std::ifstream full_md5_len_file(path_to_full_md5_len_txt.ToStdString().c_str());
    if(!full_md5_len_file)
        {
         wxString message= "Cannot open full_md5_len.txt. The database generation will be terminated";

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
        }

    //now we want to get the total lines number in the file.
    //We are going to update the progress bar based on ratio (processed lines)/(total lines)
    long long  int total_num_of_lines=0;
    std::string temp_str;
    std::ios::pos_type position_in_file=full_md5_len_file.tellg();
    while (std::getline(full_md5_len_file, temp_str))
    {
        if(Khz::Libs::Mstr::trim(temp_str)!="")
            {
            total_num_of_lines++;
            }
        //P.S. may be there is a better way??
    }
    this->Update();
    //std::cout << "total num " << total_num_of_lines << "\n";
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //@@
    //@@    I HAVE NO IDEA WHY, BUT fail() below always returns true even everything is ok.
    //@@    So we just disabled the checking for a while
    //@@
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

    /*
    if(full_md5_len_file.fail())  //just checking that everything is alright
        {
        wxString message= "Error happened when reading the  full_md5_len.txt file. The database generation will be terminated";
        wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
        return false;
        }
    */
    //full_md5_len_file.seekg(0, std::ios::beg); // we are going to read the file once again, so we put the get pointer to the beginning.
    full_md5_len_file.clear();// FUCK!!!!!! i have no idea why we getting errors !!!!!!
    full_md5_len_file.seekg(position_in_file);
     /////////////////////////////////////////////////////////////////////////////////////////////





    ///////////////////////////////////////////////////////////////////////////////////////////////
    //         open and prepare the log file.
    //////////////////////////////////////////////////////////////////////////////////////////////
    // we truncate the log file here. We already asked user if he wants to overwrite the file.
    std::ofstream log_file;
    if(check_for_missing_files)
        {
        log_file.open(path_to_log_file.ToStdString().c_str(), std::ios::out | std::ios::trunc);
        if(!log_file)
            {
            wxString message= "Cannot create the specified log file. The database generation will be terminated";

            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
            }
        wxDateTime now = wxDateTime::Now();
        std::string newline_char = wxString(wxTextFile::GetEOL()).ToStdString(); // \n on linux, \r\n on windows;
        log_file << "#this is the log file of the kolkhoznik database generation on " << now.Format().ToStdString() << newline_char;
        log_file << "#" << newline_char;
        log_file << "#" << newline_char;
        }
    //////////////////////////////////////////////////////////////





    ////////////////////////////////////////////////////////////////////////////////
    //      Ok, files are prepared, let's fill the database
    //      we read file line by line and put data to the database
    ///////////////////////////////////////////////////////////////////////////////

    //counts number of the processed lines.
    // In the end we compare it with the number of rows in the database just in case
    long long int counter=0;

    std::string file_string;
    while(std::getline(full_md5_len_file, file_string))
        {
        std::string str_w_spaces=file_string;
        Khz::Libs::Mstr::replaceAll(str_w_spaces, " ", "");
        if(str_w_spaces!="")
            {
            if(!process_line(file_string, path_to_library, log_file, db))
                {
                 //wxString message= "Bad full_md5_len.txt. Cannot generate the database";
                 //wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
                 //process_line will show error message
                 return false;
                };


            ////////////////////////////////////////////////////
            // Just updating the progress bar
            ///////////////////////////////////////////////////
            counter++;
            //if( total_num_of_lines and  1.0*(temp_counter2++)/total_num_of_lines*100>=1.0  ) //update progress bar every 1% of data processed
            //    {
            //    progress_bar->Update (100.0*counter/total_num_of_lines);
            //    temp_counter2=0;
            //    std::cout << "#########################################################\n\n\n\n\n";
            //    std::cout << "updating progress bar\n\n\n\n";
            //    std::cout << "##################################################################";
            //    }
            //else if (total_num_of_lines and  100.0*counter/total_num_of_lines<1 )
            //    {
                progress_bar->Update (100.0*counter/total_num_of_lines);
            //    }
            //////////////////////////////////////////////////////
            // progress bar updating fihished
            ////////////////////////////////////////////////////////


            }
        }
    //////////////////////////////////////////////////////////////////////////////////////
    //            database is filled
    /////////////////////////////////////////////////////////////////////////////





    //////////////////////////////////////////////////////////////////////////////////////
    //now we are closing files and perform final checks
    ///////////////////////////////////////////////////////////////////////////////////////
    if(check_for_missing_files)
        {
        log_file.flush();
        log_file.close();
        // CHECKING IS NOT WORKING, I HAVE NO IDEA WHY, FUCK!!!! temporarily disabled
        /* if(!log_file.good() or !full_md5_len_file.good() )
            {
            wxMessageDialog   err_msg(this,
                "Something terrible happen when generating the database. Perhaps you do not have anough disk space. Please try again later.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
            err_msg.ShowModal ();
            return false;
            }
        */
        }
    if(!check_counts(total_num_of_lines,db)) return false; // check counts will show the error message if necessary
    return true; // if we got here that everything is alright
}

bool SqliteDBGenerator::check_counts(long long int counter,sqlite3 *db)
    {
        std::string if_topic_exist_query = "SELECT COUNT(*) FROM main;";
        sqlite3_stmt* stmt = NULL;
        int retval = sqlite3_prepare_v2(db, if_topic_exist_query.c_str(), -1, &stmt, 0);
        if(retval) throw(std::string("cannot connect to db in fill_database.cpp"));
        //sqlite3_bind_text(stmt,  1, topic.c_str() , -1, SQLITE_TRANSIENT);
        retval = sqlite3_step(stmt);
        //int topics_number = sqlite3_column_int(stmt, 0);
        long long int num_of_rows_in_db= sqlite3_column_int( stmt, 0) ;
        sqlite3_finalize(stmt);
        if(num_of_rows_in_db!=counter)
            {
            wxString message= wxString("Something wrong happened when generating the database. ")+
                              "It is possible that you do not have enough disk space. "+
                              "Please try again later.";
            wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
            return false;
            }
    return true;
    }
void SqliteDBGenerator::OnCheckFilesCheckboxClicked(wxCommandEvent& event)
    {
    if(check_files_checkbox->GetValue())
        {
        choose_path_to_library_text->Enable();
        library_dir_input->Enable();
        choose_path_to_library_button->Enable();


        choose_path_to_log_text->Enable();
        where_to_store_log_input->Enable();
        choose_path_to_log_button->Enable();
        check_for_missing_files_checkbox->Enable();
        check_md5_checkbox->Enable();

        check_for_missing_files=true;
        }
    else
        {
        choose_path_to_library_text->Disable();
        library_dir_input->Disable();
        choose_path_to_library_button->Disable();
        choose_path_to_log_text->Disable();
        where_to_store_log_input->Disable();
        choose_path_to_log_button->Disable();
        check_for_missing_files_checkbox->Disable();
        check_md5_checkbox->Disable();
        check_for_missing_files=false;
        }
    }
void SqliteDBGenerator::add_missing_topics(sqlite3 *db, unsigned int min_id)
    {

    /////////////// querying the main kolkhoznik database///////////////
    std::string min_id_char =Khz::Libs::Mstr::IntToString(min_id).c_str();
    std::string query="SELECT * FROM topics WHERE(ID>="+min_id_char+" ) ORDER BY ID LIMIT 0,1";
    KHZNIK_DEBUGMESSAGE(std::cout << "SqliteDBGenerator::add_missing_topics: min_id is" << min_id << "\n";)

    sqlite3_stmt *stmt = NULL;
    int retval=sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );

    if(retval) throw(std::string("cannot connect to db in SqliteDBGenerator.cpp"));
    sqlite3_step( stmt );

    Khz::Libs::DB::mapDB result = Khz::Libs::DB::dbcontent(stmt);
    sqlite3_finalize(stmt);
    /////////////////////ok, we got everything we need from the database//////////////////


    if(result.size()==0)
        {
        return;
        }
    else if (result.size()==1)
        {
        std::string topic=result[0]["Topic"];
        if(topic=="")
            {
            }
        else
            {
            std::vector<std::string> exploded_topic=Khz::Libs::Mstr::explode("/", topic);
            std::string cur_topic;
            for (unsigned int i=0;i<exploded_topic.size();i++)
                {
                if(i==0) cur_topic=exploded_topic[0]; else cur_topic=cur_topic + "/" + exploded_topic[i];
                int cur_id=insert_topic_to_db_if_necessary(cur_topic,db);
                KHZNIK_DEBUGMESSAGE
                    (
                    std::cout << "SqliteDBGenerator::add_missing_topics: inserting topic if necessary"+cur_topic;
                    std::cout << "\n  id is "<< cur_id << "\n\n\n";
                    )
                }
            }
        add_missing_topics(db, min_id+1);
        }
    else
        {
        std::cout << "pisdec blyat' in SqliteDBGenerator::add_missing_topics. Aborted\n\n";
        abort();
        }

    }




}}// end of namespaces
