/*
Copyright (C) Ivan Ivanov, 2014-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <parse_string.h>
#include <vector>


namespace Khz{ namespace GenDB{


// class constructor
parse_string::parse_string(std::string some_string) :
Author(""),
Extension(""),
Pages(""),
DPI(""),
Publisher(""),
Year(""),
Language(""),
ISBN(""),
Orientation(""),
Color(""),
OCR(""),
ASIN(""),
Volume(""),
Title(""),
Edition("")
{
using namespace boost;
    // every variable that is used without type in this program is a string except res and temp;
    std::string string = some_string;
    //std::cout << "string is " << string << "\n";
    smatch res;
    std::vector<std::string> temp;

    std::string author_part="";
    std::string rest="";
    std::string author_not_translate="";
    std::string author3_part="";
    std::string file_extension="";
    std::string number_of_pages="";
    std::string resolution="";
    std::string isbn_current="";
    std::string asin_current="";
    std::string year_current="";
    std::string publisher_current = "";
    std::string edition_current="";
    std::string lang_code="";
    std::string lang="";
    std::string orientation_current="";
    std::string color_current="";
    std::string ocr_current="";
    std::string volume = "";
    std::string title_current="";
    std::string author_current="";

    //replace .. by : according to colhoz specs
	regex rx("([^\\.])(\\.\\.)(\\s+)",regex_constants::icase);
	string =  regex_replace(string, rx, "$1:$3");


    //divide strint to author and rest(in kolhoz specs separator is "dot space")
    temp = Khz::Libs::Mstr::explode(". ", string,1);
    if (temp.size() ==2)
        {
        author_part = temp[0]+ ".";
        rest = temp[1];
        }
    else
        {
            std::cout << "cannot find author part in string " << string << "\n";
            author_part=""; rest=string;
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    // let's check if there is english author name in russian book that shold not be anti translited
    //search for expressions like (_author_)
    rx = regex ("\\s*\\(_(.+?)_\\)");
    if (regex_search(rest,res, rx))
        {
        author_not_translate = " (" +res.str(1) + ")";  //pocket
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);

        // replace "auth1,auth2" by "auth1, auth2"  (space after comma added
        while (         regex_search(author_not_translate, res, regex("(\\S)\\,(\\S)", regex_constants::icase)))
            {
            std::string replacement = res.str(1) + ", " + res.str(2);
            author_not_translate = regex_replace(author_not_translate, regex("(\\S)\\,(\\S)", regex_constants::icase),replacement ,regex_constants::format_first_only);
            }
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    // let's check if there is piece like (ed.)
    rx=regex("^\\s*\\(([^\\(\\)]+?)\\.\\)", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        author3_part = " ("  +  res.str(1)  +  ".)";
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    else
        {author3_part = "";}
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //let's cut off extension of file
    rx = regex("\\.([^\\(\\)\\.]+?)$", regex_constants::icase); // .html, .djvu and so on
    regex rx_ext_2=regex("\\.([^\\(\\)\\.]{1,4}?\\.[^\\(\\)\\.]{1,4}?)$" , regex_constants::icase); // .html.zip, .html.gz
    if(regex_search(rest, res, rx_ext_2))
        {
        file_extension=res.str(1);
        rest=regex_replace(rest, rx_ext_2, "", regex_constants::format_first_only);
        }
    else if(regex_search(rest,res, rx))
        {
        file_extension = res.str(1);
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    else
        {
        file_extension="";
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //let's cut off rubrication part(it is not presented in each file name, so we can
    //define topic by dir name in which file is placed.
    rx = regex("_[^\\(\\)]+?_\\s*?$", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //let's cut off number of pages
    rx = regex("\\((\\d+)\\s*?s\\)", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        number_of_pages= res.str(1);
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //let's cut off resolution
    rx = regex("\\((\\d+)\\s*?dpi\\)", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        resolution= res.str(1);
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //let's cut isbn
    rx = regex("\\(\\s*?ISBN\\s*?(\\w+)\\s*\\)", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        isbn_current= res.str(1);
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //let's cut asin
    rx = regex("\\(\\s*?ASIN\\s*?(\\w{10})\\s*\\)", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        asin_current= res.str(1);
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        }
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //let's cut off publisher and year
    rx = regex("\\(([^\\(\\)]*?[12]\\d{3}\\s*?)\\)", regex_constants::icase);
    if (regex_search(rest,res, rx))
        {
        std::string publisher_year= res.str(1);
        rest= regex_replace(rest, rx, "", regex_constants::format_first_only);
        rx = regex("\\s+([12]\\d{3})$", regex_constants::icase);
        if (regex_search(publisher_year,res, rx))
            {
            year_current=res.str(1);
            publisher_current= regex_replace(publisher_year, rx, "", regex_constants::format_first_only);
            }
		else if (           regex_search(publisher_year, res, regex("\\s+([12]\\d{3})\\s+",regex_constants::icase)))
            {
            year_current=res.str(1);
            publisher_current = regex_replace(publisher_year, regex("\\s+([12]\\d{3})\\s+",regex_constants::icase),"",regex_constants::format_first_only);
            }


		else if (         regex_search(publisher_year, res,  regex("\\,([12]\\d{3})\\s+", regex_constants::icase)))
            {
            year_current=res.str(1);
            publisher_current= regex_replace(publisher_year, regex("\\,([12]\\d{3})\\s+",regex_constants::icase) ,"",regex_constants::format_first_only);
            }


		else if (           regex_search(publisher_year,res, regex("\\,([12]\\d{3})\\,", regex_constants::icase)))
            {
            year_current=res.str(1);
            publisher_current= regex_replace(publisher_year, regex("\\,([12]\\d{3})\\," ,regex_constants::icase),"",regex_constants::format_first_only);
            }



		else if (          regex_search(publisher_year, res, regex("\\,([12]\\d{3})$", regex_constants::icase)))
            {
            year_current=res.str(1);
            publisher_current= regex_replace(publisher_year, regex("\\,([12]\\d{3})$", regex_constants::icase),"",regex_constants::format_first_only);
            }
		else
            {
		    publisher_current = "";
            year_current = publisher_year;
            };
	//we need to remove last comma(if exist)
        if (regex_search(publisher_current, res, regex("(.+)\\,\\s*$", regex_constants::icase)))
        {
        publisher_current = res.str(1);
        }
    }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //let's cut a edition;
    if (publisher_current !="")
        {

        if (    regex_search(publisher_current, res, regex(           "\\s*(\\d+)\\s{0,3}ed\\.*\\,*",regex_constants::icase))     )
            {
            edition_current =  res.str(1);
            publisher_current= regex_replace(publisher_current, regex("\\s*(\\d+)\\s{0,3}ed\\.*\\,*", regex_constants::icase),"",regex_constants::format_first_only);
            }
        else if (    regex_search(publisher_current, res, regex(      "\\s*(\\d+)\\s{0,3}e{0,1}\\s{0,3}izd\\.*\\,*",regex_constants::icase))    )
            {
            edition_current =  res.str(1);
            publisher_current= regex_replace(publisher_current, regex("\\s*(\\d+)\\s{0,3}e{0,1}\\s{0,3}izd\\.*\\,*", regex_constants::icase),"",regex_constants::format_first_only);
            }
        }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //lets' cut language

     if (              regex_search(    rest, res, regex("\\(\\s*?(en|ru|de|fr|la|es|pl|ua|he|pt|bg)\\s*?\\)",regex_constants::icase)   )               )
        {
            lang_code = res.str(1);
            if     (lang_code == "en") lang="English";
            else if(lang_code == "ru") lang="Russian";
            else if(lang_code == "de") lang="German";
            else if(lang_code == "fr") lang="French";
            else if(lang_code == "la") lang="Latin";
            else if(lang_code == "es") lang="Spanish";
            else if(lang_code == "pl") lang="Polish";
            else if(lang_code == "ua") lang="Ukrainian";
            else if(lang_code == "he") lang="Hebrew";
            else if(lang_code == "pt") lang="Portuguese";
            else if(lang_code == "bg") lang="Bulgarian";
            rest= regex_replace(rest, regex("\\(\\s*?(en|ru|de|fr|la|es|pl|ua|he|pt|bg)\\s*?\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }

    if (lang_code == "")lang = "English"; //default is english
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //landscape
    if (  regex_search(rest, regex("\\(L\\)",regex_constants::icase)))
        {
        orientation_current = "landscape";
        rest= regex_replace(rest, regex("\\(L\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }
    else orientation_current = "portrait";
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //color
    if (  regex_search(rest, regex("\\(C\\)",regex_constants::icase)))
        {
        color_current = "YES";
        rest= regex_replace(rest, regex("\\(C\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }
    else color_current = "NO";


    //kromsated hell knows what is this, so just remove
    if (  regex_search(rest, regex(     "\\(KA{0,1}\\)",regex_constants::icase)))
        {
        rest= regex_replace(rest, regex("\\(KA{0,1}\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }


//has ocr?
    if (  regex_search(rest, regex(     "\\(T\\)",regex_constants::icase)))
        {
        ocr_current = "YES";
        rest= regex_replace(rest, regex("\\(T\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }
    else ocr_current = "NO";



    //outline hell knows what is this, so just remove

     if (  regex_search(rest, regex(     "\\(O\\)",regex_constants::icase)))
        {
        rest= regex_replace(rest, regex("\\(O\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }

    //something like (S), hell knows what it is, there no mention of such denotion in the kolhoz specks, just temoved.
     if (  regex_search(rest, regex(     "\\(s\\)",regex_constants::icase)))
        {
        rest= regex_replace(rest, regex("\\(s\\)", regex_constants::icase),"",regex_constants::format_first_only);
        }


    //now we cat try to get information about volume number
    //begin~ getting volume info~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


        //special case
    /*  Disabled, seems not usable
    if (     regex_search(rest, res, regex("(?:\\.|\\,)\\s+(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4}\\.{0,1}\\,{0,1}\\s{0,9}n.\\d{1,4})\\s*$",regex_constants::icase))           )
        {
        volume = res.str(1);
        rest =   regex_replace(rest, regex("(?:\\.|\\,)\\s+(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4}\\.{0,1}\\,{0,1}\\s{0,9}n.\\d{1,4})\\s*$", regex_constants::icase),"",regex_constants::format_first_only);
        }


    //beginning of the string, replace with nothing.
    else if (  regex_search(rest, res, regex("^(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+",regex_constants::icase))           )
        {
        volume = res.str(1);
        rest= regex_replace(rest, regex("^(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+", regex_constants::icase),"",regex_constants::format_first_only);
        }

    //in the middle of the string, replace with space
    else if (  regex_search(rest, res, regex("\\s+(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+([^\\s]+)",regex_constants::icase))           )
        {
        volume = res.str(1);
        std::string temp_word=res.str(2);
        rest= regex_replace(rest,      regex("\\s+(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s+([^\\s]+)", regex_constants::icase)," "+temp_word,regex_constants::format_first_only);
        }

    //end of the string, replace withe empty string
    else if (regex_search(rest, res, regex("(?:\\.|\\,)\\s+(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s*$",regex_constants::icase))           )
        {
        volume = res.str(1);
        rest =   regex_replace(rest, regex("(?:\\.|\\,)\\s+(?:vol|tom|t)\\.{0,1}\\s{0,9}(\\d{1,4})\\.{0,1}\\,{0,1}\\s*$", regex_constants::icase),"",regex_constants::format_first_only);
        }
    */
    //finished vith volume number definition ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //rest is a title (the next two lines is a analog of the php's trim.
    title_current =Khz::Libs::Mstr::trim(rest);



    //let's cut the last dot in the title and last comma
    if (regex_search(title_current, res, regex("([^\\.]+)\\.$",regex_constants::icase))  )
        {
        title_current=regex_replace(rest, regex("\\.$" , regex_constants::icase),"",regex_constants::format_first_only);
        }

    if (regex_search(title_current, res, regex("([^\\.]+)\\,$",regex_constants::icase))  )
        {
        title_current=regex_replace(rest, regex("\\,$", regex_constants::icase),"",regex_constants::format_first_only);
        }




    if (lang_code == "ru")
    {
        author_current = Khz::Libs::Mstr::trim(translate_to_ru(author_part));
        author_current = author_current  + author_not_translate;
        if (author3_part != "") author_current = author_current + " " + Khz::Libs::Mstr::trim(translate_to_ru(author3_part));
        Author = author_current;

        //next is not finished....
        Extension= Khz::Libs::Mstr::trim(file_extension);
        Pages = Khz::Libs::Mstr::trim(number_of_pages);
        DPI = Khz::Libs::Mstr::trim(resolution);
        Publisher = Khz::Libs::Mstr::trim(translate_to_ru(publisher_current));
        Year = Khz::Libs::Mstr::trim(year_current);
        Language = Khz::Libs::Mstr::trim(lang);
        ISBN = Khz::Libs::Mstr::trim(isbn_current);
        Orientation = Khz::Libs::Mstr::trim(orientation_current);
        Color = Khz::Libs::Mstr::trim(color_current);
        OCR = Khz::Libs::Mstr::trim(ocr_current);
        ASIN = Khz::Libs::Mstr::trim(asin_current);
        Volume = Khz::Libs::Mstr::trim(volume);
        Title = Khz::Libs::Mstr::trim(translate_to_ru(title_current));
        Edition = Khz::Libs::Mstr::trim(translate_to_ru(edition_current));
    }
    else
        {

        author_current = Khz::Libs::Mstr::trim(author_part);
        author_current = author_current + author_not_translate;
        if (author3_part != "") author_current = author_current + " " + Khz::Libs::Mstr::trim(author3_part);
        Author = author_current;

        //next is not finished....
        Extension=    Khz::Libs::Mstr::trim(file_extension);
        Pages =       Khz::Libs::Mstr::trim(number_of_pages);
        DPI =         Khz::Libs::Mstr::trim(resolution);
        Publisher =   Khz::Libs::Mstr::trim(publisher_current);
        Year =        Khz::Libs::Mstr::trim(year_current);
        Language =    Khz::Libs::Mstr::trim(lang);
        ISBN =        Khz::Libs::Mstr::trim(isbn_current);
        Orientation = Khz::Libs::Mstr::trim(orientation_current);
        Color =       Khz::Libs::Mstr::trim(color_current);
        OCR =         Khz::Libs::Mstr::trim(ocr_current);
        ASIN =        Khz::Libs::Mstr::trim(asin_current);
        Volume =      Khz::Libs::Mstr::trim(volume);
        Title =       Khz::Libs::Mstr::trim(title_current);
        Edition =     Khz::Libs::Mstr::trim(edition_current);

    }
    /*
    Mstr::replaceAll(Author     , "'", "''");
    Mstr::replaceAll(Extension  , "'", "''");
    Mstr::replaceAll(Pages      , "'", "''");
    Mstr::replaceAll(DPI        , "'", "''");
    Mstr::replaceAll(Publisher  , "'", "''");
    Mstr::replaceAll(Year       , "'", "''");
    Mstr::replaceAll(Language   , "'", "''");
    Mstr::replaceAll(ISBN       , "'", "''");
    Mstr::replaceAll(Orientation, "'", "''");
    Mstr::replaceAll(Color      , "'", "''");
    Mstr::replaceAll(OCR        , "'", "''");
    Mstr::replaceAll(ASIN       , "'", "''");
    Mstr::replaceAll(Volumeinfo , "'", "''");
    Mstr::replaceAll(Title      , "'", "''");
    Mstr::replaceAll(Edition    , "'", "''");
    */

}  // end of the constructor



	//translates string to russian
std::string parse_string::translate_to_ru(std::string some_string)
{
	std::vector<std::string> temp_str= Khz::Libs::Mstr::explode("__", some_string);
	bool should_be_translated=true;

	for(unsigned int i=0; i<temp_str.size();i++)
        {
        if (should_be_translated)
            {
            temp_str[i]=Khz::Libs::Mstr::replace_translit_sumbols_to_ru(temp_str[i]);
            }
        should_be_translated=!should_be_translated;

        }
    return Khz::Libs::Mstr::implode("", temp_str);
}



}} // namespaces
