/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <string>
#include <boost/regex.hpp>
#include <string_functions.h>
#include <vector>
#include <iostream>


namespace Khz{ namespace GenDB{


/**
@brief Exctracts Title, Author and so on from string like "Aldrich M. Sleep medicine (OUP, 1999)(ISBN 0195129571)(T)(O)(407s)_B_.djvu" by using regular expressions

Exctracts Title, Author and so on from string like "Aldrich M. Sleep medicine (OUP, 1999)(ISBN 0195129571)(T)(O)(407s)_B_.djvu". Uses
regular expressions for it.
**/
class parse_string
{
public:
    /**
    Constructor. Let us consider an example:
    @code
    parse_string some_name=parse_string("Aldrich M. Sleep medicine (OUP, 1999)(ISBN 0195129571)(T)(O)(407s)_B_.djvu");
    std::string title = some_name.Title; //returns "Aldrich M."
    std::string author = some_name.Author; //return "Sleep medicine"
    //and so on
    @endcode
    **/
	parse_string(std::string some_string);

public:
    /**
    Author of the book
    **/
	std::string Author;

	/**
	Extension of the file
	**/
	std::string Extension;

	/**
	Number of pages
	**/
	std::string Pages;

	/**
	Resolution
	**/
	std::string DPI;

	/**
	Publisher
	**/
	std::string Publisher;

	/**
	Year
	**/
	std::string Year;

	/**
	Language
	**/
	std::string Language;

	/**
	ISBN
	**/
	std::string ISBN;

	/**
	Orientation
	**/
	std::string Orientation;

	/**
	Color ("YES" or "NO")
	**/
	std::string Color;

	/**
	OCR, "YES" or "NO"
	**/
	std::string OCR;

	/**
	ASIN
	**/
	std::string ASIN;

	/**
	Volume
	**/
	std::string Volume;

	/**
	Title
	**/
	std::string Title;

	/**
	Edition
	**/
	std::string Edition;
private:
	//translates string to russian
	/**
	Translates string to russian. Replaces translit symbols by russian letters (calls #Khz::Libs::Mstr::replace_translit_sumbols_to_ru for it).
	Keeps latin symbols if some substring is inside of the "__" sumbols. For example, suppose that we have string
	"Programmirovanie na __C++__". This function will produce "Программирование на C++". So, first part of the initial
	string is processed by the function #Khz::Libs::Mstr::replace_translit_sumbols_to_ru. The second part of the string ("C++") does not change.
	**/
	std::string translate_to_ru(std::string some_string);
};



}}// namespaces
