/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include <iostream>
#include "Mdb_fuctions.h"
#include <string>
#include "sqlite3.h"
#include "get_books_from_db_by_query.h"
#include <cmath>
#include "string_functions.h"
//@QQQfinished commenting


namespace Khz{namespace Libs{ namespace DB{


get_books_from_db_by_query::get_books_from_db_by_query(sqlite3 *_db, std::string _dbtable,
                                                       std::string _where_statement, long _num_of_books_per_step, std::vector<std::string> _placeholder_parameters)
    {

    //this initializes the whole thing.
    //assume that you have 10 books in the database and you want to get them by 2 books per query.
    //You neeed to perform 5 queries (i.e. you should call The Step() fuction 5 times)

    //so, we set total_steps and total_lines here.
    //We query the database, get total number of rows, calculate total_steps.
    this->num_of_books_per_step=_num_of_books_per_step;
    this->where_statement = _where_statement;
    this->parameters_for_WHERE_placeholders=_placeholder_parameters;
    this->dbtable=_dbtable;
    this->db= _db;

    //Each call of the Step() function will increase this by 1;
    //We have to stop when step>total_steps
    this->step=1;
    try
        {
        std::string where;
        if(this->where_statement!="")
            {
            where= " WHERE ";
            }
        else
            {
            where="";
            }
        std::string number_query= "SELECT COUNT(*) FROM " + dbtable + where + this->where_statement + ";";
        //std::cout << "\nquery is " << number_query< "\n";
        sqlite3_stmt *stmt = NULL;
        int retval = sqlite3_prepare_v2(db, number_query.c_str(), -1, &stmt, 0);
        if(retval) throw(std::string("cannot connect to db in fill_database.cpp function show_books"));
        for(unsigned int i=0; i< parameters_for_WHERE_placeholders.size(); i++)
            {
            sqlite3_bind_text(stmt,  i+1, parameters_for_WHERE_placeholders[i].c_str(), -1, SQLITE_TRANSIENT);
            }
        retval = sqlite3_step(stmt);


        this->total_lines = sqlite3_column_int(stmt, 0);
        //std::cout << "\ntotal line is " << total_lines << "\n";
        sqlite3_finalize(stmt);


        total_steps=(long long) ceil(1.0*total_lines/num_of_books_per_step);
        }
    catch(std::string e) {std::cout << "\n" << e;total_lines=0;}
    catch(...)
        {
        total_lines=0;
      //  result= {{{"",""}}};
        return;
        }

    }

bool get_books_from_db_by_query::Step()
    {
    if(total_lines==0)
        {
        return false;
        }
    //finished
    else if(step>total_steps)
        {
        return false;
        }


    //ok, query the database
    try
        {
        std::string where;
            if(this->where_statement!="")
                {
                where= " WHERE ";
                }
            else
                {
                where="";
                }
        std::string first_limit_argument = Mstr::IntToString(      (step -1)*num_of_books_per_step     );
        std::string query= "SELECT * FROM " + dbtable + where + this->where_statement + " LIMIT "+ first_limit_argument + ","+ Mstr::IntToString( num_of_books_per_step) +";";


        sqlite3_stmt *stmt = NULL;
        sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
        for(unsigned int i=0; i< parameters_for_WHERE_placeholders.size(); i++)
            {
            sqlite3_bind_text(stmt,  i+1, parameters_for_WHERE_placeholders[i].c_str(), -1, SQLITE_TRANSIENT);
            }
                             //preparing the statement
        sqlite3_step( stmt ); //executing the statement
        current_result = Khz::Libs::DB::dbcontent(stmt) ;
        sqlite3_finalize(stmt);


        //increase step by one, see if condition in the beginning of this function.
        step++;
        return true;
        }
    catch(...)
        {
        std::cout << "\n Something terrible happen in get_books_from_db_by_query.cpp, aborted k chertovoy materi";
        abort();
        }

    }


long long int get_books_from_db_by_query::get_num_of_books_in_library(sqlite3 *db, std::string dbtable)
    {
    try
        {

        std::string query_total="select COUNT(*) from "+dbtable+";";

                //std::cout << "\n query is " << query << "\n";
        //std::string query="select * from topics;";
        //select Topic from topics where Topic like '/B_Biology/%' and substr(Topic, length('/B_Biology/')+1 ) NOT LIKE '%/%';
        // we have to select topics that are like current topic and have number of symbols "/" eqal to coorresponding number of current topic +1
        sqlite3_stmt *stmt_total = NULL;


        int retval_total=sqlite3_prepare_v2( db, query_total.c_str(), -1, &stmt_total, NULL );
        if(retval_total) throw(std::string("cannot connect to db in fulltextsearchresults.cpp"));
        sqlite3_step( stmt_total );
        long long  int total_lines_total = sqlite3_column_int(stmt_total, 0);
        sqlite3_finalize(stmt_total);
        return total_lines_total;
        }
    catch(...) {return 0;};
    }

Khz::Libs::DB::mapDB get_book_from_db_by_id(sqlite3 *db, std::string dbtable, long long int id)
    {
    try
        {
        Khz::Libs::DB::mapDB output;
        std::string query="select * from "+dbtable+ " where ID=?";

        sqlite3_stmt *stmt = NULL;
        sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
        sqlite3_bind_int(stmt,  1, id);
                            //preparing the statement
        sqlite3_step( stmt ); //executing the statement
        output = Khz::Libs::DB::dbcontent(stmt) ;
        sqlite3_finalize(stmt);
        return output;
        }
    catch(...)
        {
        Khz::Libs::DB::mapDB empty;
        return empty;
        }
    }


Khz::Libs::DB::mapDB get_books_from_db_by_query::get_book_from_db_by_id(long long int id, wxString database_path, std::string dbtable)
    {
    sqlite3 *db;
    try
        {
        wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(database_path).ToUTF8();
        sqlite3_open( temp_char_buffer, &db);
        // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
        sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
        sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
        }
    catch(...)
        {
        Khz::Libs::DB::mapDB empty;
        return empty;
        }

     try
        {
        Khz::Libs::DB::mapDB output;
        std::string query="select * from "+dbtable+ " where ID=?";

        sqlite3_stmt *stmt = NULL;
        sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
        sqlite3_bind_int(stmt,  1, id);
                            //preparing the statement
        sqlite3_step( stmt ); //executing the statement
        output = Khz::Libs::DB::dbcontent(stmt) ;
        sqlite3_finalize(stmt);

        sqlite3_close(db);
        return output;
        }
    catch(...)
        {
        sqlite3_close(db);
        Khz::Libs::DB::mapDB empty;
        return empty;
        }
    }


}}}// namespaces
