/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef KOLKHOZNIK_MYSIZEFUNCTIONS_HEADER_INCLUDED
#define KOLKHOZNIK_MYSIZEFUNCTIONS_HEADER_INCLUDED




#include <wx/window.h>
namespace Khz{namespace Libs{

/**
@brief some gui size functions
**/
class MySizeFunctions
    {
    public:
    /**
    Returns height. Assume for example that you want to get the height (in pixels)
    of the text that consists of 40 lines. Than you can get this size like this:
    @code
    int height= get_height_of_number_of_text_rows(40, window);
    @endcode

    P.S this function knows about spaces between lines, they  are counted on too.
    @param num_of_lines the number of the lines
    @param window As you could guess, the height of the string depends on the font size, font family and so on.
    The each wxWidgets window knows it's default font size, font family. This is why we need to know which window we are going to
    display the string in.
    So, this param is window that we are going to display the string in.
    **/
    static int get_height_of_number_of_text_rows(int num_of_lines,  const wxWindow *window);

    /**
    Similar to #get_height_of_number_of_text_rows, but returns width (in pixels) of the string that consists of num_of_symbols spaces.
    **/
    static int get_width_of_number_of_text_symbols(int num_of_symbols, const wxWindow *window);


    /**
    Similar to #get_height_of_number_of_text_rows, but returns width (in pixels) of the string str.
    **/
    static int get_width_of_string(wxString str, const wxWindow *window);

    /**
    Gets hight of a single string (in pixels)
    **/
    static int get_height_of_string(wxString string,  const wxWindow *window);
    };

}}//namespaces


#endif
