/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <string>
#include <vector>
//#include <regex>
#include <boost/regex.hpp>
#include <openssl/sha.h>



namespace Khz{ namespace Libs{


/**
@brief Functions for working with strings
**/
class Mstr
{
public:



    /**
    Just
    @code
    bool Mstr::Not_isalnum(int charr)
        {
        return !isalnum(charr);
        }
    @endcode
    **/
    static bool Not_isalnum(int charr);
	// analog of the php's explode function; negative i means explode everything without limit of explosions number.
    /**
    Explodes string into array of substrings by delimiter
    @param delim delimiter
    @param str string to explode
    @param i how many times it is allowed to explode. For example, if i=1
    than this function is allowed to explode only once(explode string into two substrings).
    if i=-1 than there is no restrictions.

    If string does not contain delim than this function returns vector consisting of one element.
    This element is the initial string
    **/
	static std::vector<std::string> explode(std::string delim, std::string str, int i = -1);



    /**
    Counts words number in the phrase. Words can be
    separated by a single or by multiply spaces
    **/
    static int count_words(std::string phrase);


	//replaces substring "from" by a substring "to" in "str"(replaces every occurence)

	/**
	replaces all of the occurences of the substring in the string
	@param str original string
	@param from substring to replace
	@param to replacement

	Example
	@code
	std::string str="abab";
	replaceAll(str, "b", "");
	std::cout << str; //shows "aa";
	@endcode
	**/
	static void replaceAll(std::string &str, const std::string from, const std::string to);

	/**
	removes spaces in the beginning and in the end of the string.
	**/
	static std::string trim(std::string some_string);

	/**
	computes sha1 of the string
	**/
	static std::string sha1(std::string str);

	/**
	converts int to std::string
	**/
	static std::string IntToString(int number); //convert int to string

	/**
	converst long int to std::string
	**/
	static std::string IntToString(long int number); //convert int to string

	/**
	converts long long int to std::string
	**/
	static std::string IntToString(long long number); //convert int to string


    /**
	converts unsigned int to std::string
	**/
    static std::string IntToString(unsigned int number);

    /**
    Converts std::string to long long
    **/
    static long long StringToLongLong(std::string input)
        {
        long long num;
        std::istringstream{input} >> num;
        return num;
        }

	/**
	implodes array of strings into single string
	@param separator separator of the substrings in the imploded string
	@param array_of_strings array of string

	Example
	@code
	std::vector<std::string> array;
	array.push_back("str1");
	array.push_back("str2");
	std::string imploded= implode("|", array);
	std::cout << imploded; // "str1|str2"
	@endcode
	**/
	static std::string implode(std::string separator, std::vector<std::string> array_of_strings);


    /**
    Replaces all of the symbols by the lowercase ones
    @param some_string initial string
    @param replace_bad_symbols If true, than russian "ё" will be replaced by "е",
    russian "й" will be replaced by "и"
    **/
    static std::string to_lower_case(std::string some_string, bool replace_bad_symbols=true);



    /**
	Repaces translit sumbols by russian letters.
	**/
	static std::string replace_translit_sumbols_to_ru(std::string some_string);

	/**
	Replaces non-englins characters by english characters based on replacement rules
	 ( the rules are defined in the code of the function).
	Removes all of the of non-english characters that do not have any of the
	replacement rules.
	**/
	static void translate_to_en_from_any_ONLY_ALPHABETIC_SYMBOLS(std::string &some_string);

};//class


}}// namespaces
