/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/panel.h>
#include <wx/button.h>
#include <wx/stattext.h>
#include <wx/sizer.h>
#include <wx/wrapsizer.h>
#include <wx/srchctrl.h>
#include <wx/checkbox.h>
#include <wx/notebook.h>
#include "LeftSearchPanel.h"
#include "Custom_IDs.h"
#include <wx/statline.h>
#include <wx/hyperlink.h>
#include <wx/msgdlg.h>



namespace Khz{namespace MainWin{



LeftSearchPanel::LeftSearchPanel(wxNotebook *parent)
       : wxScrolledWindow(parent, wxID_ANY)
{
wxPanel *main_panel=new wxPanel(this, wxID_ANY);



  //search input
  Search_input=new wxSearchCtrl(main_panel, my_wx_id__SEARCH_BOOKS_INPUT,wxEmptyString, wxDefaultPosition,
                                wxSize(30, 30));
  Search_input->ShowSearchButton(main_panel);
  //Search_input->SetMinSize(wxSize(30,-1));
  //search button
  Search_button = new wxButton(main_panel, my_wx_id__SEARCH_BOOKS_BUTTON, "Search");



  //full text search input
  full_text_search_input=new wxSearchCtrl(main_panel, my_wx_id__FULL_TEXT_SEARCH_BOOKS_INPUT,wxEmptyString, wxDefaultPosition,
                                wxSize(30, 30), wxTE_PROCESS_ENTER);
  //full_text_search_input->SetMinSize (wxSize(30,-1));
  full_text_search_input->ShowSearchButton(true);
  //search button
  full_text_search_button = new wxButton(main_panel, my_wx_id__FULL_TEXT_SEARCH_BOOKS_BUTTON, "Search");
  full_text_search_language_selector=new Khz::FTS::Gui::LanguageSelectorWxChoice(main_panel);

  // use stemmer checkbox
  full_text_search_use_stemmer_checkbox = new wxCheckBox(main_panel, wxID_ANY, "use stemmer");
  full_text_search_use_stemmer_checkbox->SetValue(true);
  wxHyperlinkCtrl *what_is_stemmer_hyperlink=new wxHyperlinkCtrl (main_panel, wxID_ANY, "what is this", "click");

  // restrict search by topics checkbox (full text search) together with topics list
  full_text_search_restrict_topics_checkbox=new wxCheckBox(main_panel, wxID_ANY, "restrict search by topics");
  full_text_search_restrict_topics_checkbox->SetValue(false);
  full_text_search_topics_list= new Khz::FTS::Gui::FullTextSearchTopicsList(main_panel, wxID_ANY);
  full_text_search_topics_list->Disable();
  full_text_search_topics_list->SetMinSize (wxSize(30,-1));




  //title and author checkboxes
  Title_checkbox=new wxCheckBox(main_panel, wxID_ANY, "Title");
  Author_checkbox=new wxCheckBox(main_panel, wxID_ANY, "Author");

  //search type ("AND" for all of the word in search input, "OR" for and word)
  search_type=new wxRadioButton(main_panel, -1, "AND");
  search_type->SetValue(true);
  search_type2=new wxRadioButton(main_panel, -1, "OR");


  //check title and autor checkboxes by default.
  Title_checkbox->SetValue(true);
  Author_checkbox->SetValue(true);



//fit by width with top border 10 pixels
wxBoxSizer *only_sizer = new wxBoxSizer( wxVERTICAL );



wxBoxSizer *search_type_sizer = new wxBoxSizer( wxHORIZONTAL );
search_type_sizer ->Add(search_type);
search_type_sizer->AddSpacer(10);
search_type_sizer ->Add(search_type2);

only_sizer->AddSpacer(5);
    wxStaticText *temp_text=new wxStaticText( main_panel, wxID_ANY, "Author/title search");
    wxFont font = temp_text->GetFont();
    font.SetWeight(wxFONTWEIGHT_BOLD);
    temp_text->SetFont(font);
only_sizer->Add(temp_text,0, wxALL, 5);
only_sizer->AddSpacer(10);
only_sizer->Add(search_type_sizer, 0, wxALL, 5); //radiobutton AND OR
    wxBoxSizer *search_input_sizer=new wxBoxSizer(wxVERTICAL);
        wxWrapSizer *words_divided_by_spaces_sizer=new wxWrapSizer(wxHORIZONTAL);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "word "), 0);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "or "), 0);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "few "), 0);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "words "), 0);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "divided "), 0);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "by "), 0);
        words_divided_by_spaces_sizer->Add(new wxStaticText(main_panel, wxID_ANY, "spaces"), 0);
    search_input_sizer->Add(words_divided_by_spaces_sizer, 0, wxALL|wxEXPAND);
    search_input_sizer->Add(Search_input, 0, wxALL|wxEXPAND);
only_sizer->Add(search_input_sizer,0, wxEXPAND|wxALL, 5);
     wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
        wxBoxSizer *title_author_sizer=new wxBoxSizer(wxVERTICAL);
        title_author_sizer->Add(Author_checkbox, 0, wxALL);
        title_author_sizer->AddSpacer(5);
        title_author_sizer->Add(Title_checkbox, 0, wxALL);
    button_sizer->Add(title_author_sizer, 0, wxALL);
    button_sizer->AddStretchSpacer ();
    button_sizer->Add(Search_button);
only_sizer->Add(button_sizer,0, wxEXPAND|wxALL, 5);  //button "SEARCH"
only_sizer->AddSpacer(40);

    //wxBoxSizer *lines_sizer=new wxBoxSizer(wxVERTICAL);
    //lines_sizer->AddSpacer(5);
    //lines_sizer->Add(new wxStaticLine(this), 0, wxEXPAND|wxLEFT|wxRIGHT);
    //lines_sizer->AddSpacer(2);
    //lines_sizer->Add(new wxStaticLine(this), 0, wxEXPAND|wxLEFT|wxRIGHT);
    //lines_sizer->AddSpacer(5);
//only_sizer->Add(lines_sizer, 0, wxEXPAND|wxALL, 5);
    wxStaticText *temp_text2=new wxStaticText( main_panel, wxID_ANY, "Full-text search");
    wxFont font2 = temp_text2->GetFont();
    font2.SetWeight(wxFONTWEIGHT_BOLD);
    temp_text2->SetFont(font2);
only_sizer->Add(temp_text2,0, wxALL, 5);
only_sizer->AddSpacer(10);
only_sizer->Add(full_text_search_input, 0, wxEXPAND|wxALL, 5);
    wxBoxSizer *stemmer_sizer=new wxBoxSizer(wxHORIZONTAL);
        stemmer_sizer->AddSpacer(5);
        full_text_search_use_stemmer_checkbox->SetMinSize(wxSize(-1,what_is_stemmer_hyperlink->GetSize().GetHeight()));
        stemmer_sizer->Add(full_text_search_use_stemmer_checkbox, 0, wxALL);
        stemmer_sizer->Add(what_is_stemmer_hyperlink, 0);
only_sizer->Add(stemmer_sizer, 0, wxALL);
only_sizer->Add(full_text_search_language_selector, 0, wxALL|wxLEFT,5);
only_sizer->AddSpacer(10);
    wxBoxSizer *full_text_serch_button_sizer=new wxBoxSizer(wxHORIZONTAL);
        full_text_serch_button_sizer->AddSpacer(5);
        full_text_serch_button_sizer->AddStretchSpacer();
        full_text_serch_button_sizer->Add(full_text_search_button, 0, wxALL|wxRIGHT);
        full_text_serch_button_sizer->AddSpacer(5);
only_sizer->Add(full_text_serch_button_sizer, 0, wxEXPAND|wxALL);
only_sizer->AddSpacer(10);
    wxBoxSizer *restrict_topics_checkbox_sizer=new wxBoxSizer(wxHORIZONTAL);
    restrict_topics_checkbox_sizer->AddSpacer(5);
    restrict_topics_checkbox_sizer->Add(full_text_search_restrict_topics_checkbox, 0, wxALL);
only_sizer->Add(restrict_topics_checkbox_sizer);
only_sizer->Add(full_text_search_topics_list, 1, wxEXPAND|wxALL, 5);


main_panel->SetSizer(only_sizer);
wxBoxSizer *main_sizer = new wxBoxSizer(wxVERTICAL);
main_sizer->Add(main_panel,1, wxALL|wxEXPAND);
    //main_sizer->Add(new wxStaticText(panel, wxID_ANY, "pisdec"),0);
    //main_sizer->AddStretchSpacer();


        // this part makes the scrollbars show up
        //only_sizer->FitInside(this); // ask the sizer about the needed size

SetSizer(main_sizer);



full_text_search_use_stemmer_checkbox->Bind(wxEVT_CHECKBOX, &LeftSearchPanel::OnUseStemmerCheckboxClicked, this);
what_is_stemmer_hyperlink->Bind(wxEVT_HYPERLINK, &LeftSearchPanel::OnWhatIsStemmerLinkClicked, this);
full_text_search_restrict_topics_checkbox->Bind(wxEVT_CHECKBOX, &LeftSearchPanel::OnRestrictTopicsCheckboxClicked, this);
}



wxString LeftSearchPanel::GetSearchText()
    {
    // wxAboutDialogInfo info;
    //info.SetName("My Program");
    //info.SetVersion(_("1.2.3 Beta"));
    wxString search_string = this->Search_input->GetValue();
    return search_string;
    //info.SetDescription(search_string);
    //info.SetCopyright(wxT("(C) 2007 Me <my@email.addre.ss>"));
    //wxAboutBox(info);
    }


bool LeftSearchPanel::IsAuthorChecked()
    {
    return Author_checkbox->GetValue();
    }
bool LeftSearchPanel::IsTitleChecked()
    {
    return Title_checkbox->GetValue();
    }
wxString LeftSearchPanel::GetSearchType()
    {
    if (search_type->GetValue())
        {
        return "and";
        }
    else
        {
        return "or";
        }
    }
void LeftSearchPanel::on_search_begin()
    {
    Search_button->Disable();
    //wxBitmap bitm=Search_button->GetBitmapPressed ();
    //std::cout << "\nmaking button pressed\n";
    //Search_button->SetBitmap(bitm);
    }
void LeftSearchPanel::on_search_end()
    {
    Search_button->Enable();
    //wxBitmap bitm=Search_button->GetBitmapLabel ();
    //Search_button->SetBitmap(bitm);
    }
void LeftSearchPanel::OnUseStemmerCheckboxClicked(wxCommandEvent &event)
{
if(full_text_search_use_stemmer_checkbox->GetValue())
    {
    full_text_search_language_selector->Enable();
    }
else
    {
    full_text_search_language_selector->Disable();
    };
}

void LeftSearchPanel::OnWhatIsStemmerLinkClicked(wxHyperlinkEvent& event)
    {
    wxString message=
    "Stemming is a  process of mapping related words to a common stem. "
    "This enables matching on variants of a word. For example, while "
    "using stemmer, a search for \"horse\" could return matches for "
    "\"horses\", \"horseback\", and \"horsing\", as well as \"horse\". "
    "Without stemmer only matches  for \"horse\" are gonna to be returned";
    wxMessageBox(  message,wxString("Hi there!"), wxOK|wxCENTRE|wxSTAY_ON_TOP) ;
    }

void LeftSearchPanel::OnRestrictTopicsCheckboxClicked(wxCommandEvent &event)
    {
    if(full_text_search_restrict_topics_checkbox->GetValue())
        {
        full_text_search_topics_list->Enable();
        }
    else
        {
        full_text_search_topics_list->Disable();
        }
    }


}} // end of namespace
