/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <ciso646>
#include <wx/fileconf.h>
#include <wx/stdpaths.h>
#include <wx/listctrl.h>
#include "MyBooksList.h"
#include <wx/debug.h>
#include "get_from_db.h"
#include <wx/imaglist.h>
#include <wx/icon.h>
#include <wx/msgdlg.h>
#include "string_functions.h"
#include "clear_database_path.h"
#include "Custom_IDs.h"
#include <wx/artprov.h>
#include "InfoAboutThisBook.h"
#include <wx/mimetype.h>
#include <wx/frame.h>
#include "BibtexTranslator.h"
#include <wx/dcclient.h>
#include <wx/settings.h>
#include <wx/button.h>
#include <wx/sizer.h>
#include "BibTexEntryWindow.h"
#include "OpenTheBook.h"
#include "ConfigManager.h"



namespace Khz{namespace MainWin{



MyBooksList::~MyBooksList()
    {
    delete m_pImageList;
    delete config;
    sqlite3_close(db);

    }

MyBooksList::MyBooksList(wxWindow* parent, wxWindowID id)
    : wxListCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_VIRTUAL | wxLC_SINGLE_SEL)
    {
    //##################  initializes config ###########################
    wxSetlocale(LC_ALL, ""); //this is needed to non-latin characters support in paths;
    ini_filename = ConfigManager::GetPathToConfigFile();
    config =new wxFileConfig( "", "", ini_filename);
    // ##################   finished with config initialazing.




    //@@@@@@@@@@@@@@@@@@ icons @@@@@@@@@@@@@@@@@@@
    wxString icons_size_in_config = config->Read("ICONS/books_list_icons_size","16");
    if(icons_size_in_config=="") icons_size_in_config="16";
    this->InitAndSetIconsList(wxAtoi(icons_size_in_config));
    //@@@@@@@@@@@@@@@ finished with icons @@@@@@@@@@@@@@@@@@@@@@@



    //set max number of books in list that are allowed to sort.
    wxString max_sort_num_string = config->Read("ADVANCED/max_num_of_books_to_sort","10000");
    if (max_sort_num_string=="") max_sort_num_string="10000";
    do_not_sort_list_if_books_more_than_this_number=wxAtoi(max_sort_num_string);


    //set max cache size
    wxString max_cache_size_str= config->Read("ADVANCED/max_size_of_cache", "4000");
    if(max_cache_size_str=="") max_cache_size_str="4000";
    max_size_of_cache=wxAtoi(max_cache_size_str);
    //std::cout << "\n max_size_of_cache is " << max_size_of_cache<< "\n";

    //connect to db
    wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();
    sqlite3_open( temp_char_buffer, &db);
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);


    full_rows_number=0;
    SetItemCount(full_rows_number);



    GenItemsToShow();


    number_of_sorting_item=0;
    sort_order="up";
    //users_wanded_order=" ORDER by AuthorLowerCase, TitleLowerCase, cast(Volume as integer) ";
    users_wanded_order=" ORDER by AuthorLowerCase, TitleLowerCase ";
    //UpdateOrder();


    //creating columns
    for(unsigned int i=0;i<items_to_show.size();i++)
        {

        int width = wxAtoi(    config->Read("RIGHT_PANEL_COLUMNS_POSITIONS/"+ wxString::Format(wxT("%i"),i), wxString::Format(wxT("%i"),wxLIST_AUTOSIZE))       );
        if(items_to_show[i].first=="Author")
            {
            number_of_sorting_item=i;								    //arrow up
			//InsertColumn (i ,items_to_show[i].second+wxString::FromUTF8(" \xE2\x96\xB2"),wxLIST_FORMAT_LEFT, width);

			InsertColumn (i ,items_to_show[i].second, wxLIST_FORMAT_LEFT, width);
            if(order!="")
                {
                //########arrow up ##################
                wxListItem item;
                item.SetMask(wxLIST_MASK_IMAGE);
                item.SetImage(1); //arrow up
                this->SetColumn(i, item);
                // #################################
                }
            }
        else
            {
            InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
            }
        // SetImage(0);
        }

   // std::cout << "binding";
    Bind(wxEVT_LIST_CACHE_HINT, &MyBooksList::PutDataToCache, this);
    Bind(wxEVT_LIST_ITEM_ACTIVATED, &MyBooksList::OpenThisBook, this);
    Bind(wxEVT_LIST_COL_END_DRAG, &MyBooksList::OnResizeColumn, this);
    Bind(wxEVT_LIST_COL_CLICK, &MyBooksList::UserWantsToChangeSortingOrder, this);
    Bind(wxEVT_LIST_ITEM_RIGHT_CLICK, &MyBooksList::RightClickMenu, this);
    Bind(wxEVT_COMMAND_MENU_SELECTED, &MyBooksList::OnInfoAboutBookWanted, this, my_wx_id__SHOW_INFO_ABOUT_BOOK);
    Bind(wxEVT_COMMAND_MENU_SELECTED, &MyBooksList::OnGenBibtexMenuSelected, this,my_wx_id__GEN_BIBTEX_FOR_ONE_BOOK);
    this->OnBooksInTopicWanted(1); //initialize top level
    }




void MyBooksList::PutDataToCache(wxListEvent& event)
    {
    long max_cache_size=max_size_of_cache;
    long item_min=event.GetCacheFrom();
    long item_max=event.GetCacheTo();
    if (  //all of the items are already cached
        (cache.cached_min==item_min && cache.cached_max== item_max) ||  ( cache.cached_min< item_min && cache.cached_max>item_max)
        )
        {
        return;
        }
   // std::cout << "caching items " << item_min << " " << item_max << "\n";
   //std::cout << "\n\n in putdatatocache search type is " << search_type << "\n\n";
   if(mode==SEARCH_MODE)
        {
        if(full_rows_number<=max_cache_size) //put to cache all data
            {
            cache.put_into_cache(db, "main", search_string,0, full_rows_number-1, search_type,
                          fields_to_search, order);
            }
        else if(item_max-item_min <=max_cache_size)  //put visible items, items above and below into cache
            {
            long delta = max_cache_size - (item_max-item_min);
            long min_to_cache= item_min-delta/2;
            long max_to_cache= item_max+delta/2;
            if(min_to_cache<0) min_to_cache=0;
            if(max_to_cache>(full_rows_number-1)) max_to_cache= (full_rows_number-1); // count begins from 0!!!.
            cache.put_into_cache(db, "main", search_string,min_to_cache, max_to_cache, search_type,
                          fields_to_search, order);
            //std::cout << "\n caching data" << "\n";
            }
        else  //put only visible items into cache
            {
            cache.put_into_cache(db, "main", search_string,item_min, item_max, search_type,
                          fields_to_search, order);
            }
        }
    else if (mode==TOPIC_MODE)
        {
        if(full_rows_number<=max_cache_size)
            {
            cache.put_into_cache(db, "main", Topic_id,0,full_rows_number-1, order);
            }
        else if(item_max-item_min <=max_cache_size)
            {
            long delta = max_cache_size - (item_max-item_min);
            long min_to_cache= item_min-delta/2;
            long max_to_cache= item_max+delta/2;
            if(min_to_cache<0) min_to_cache=0;
            if(max_to_cache>(full_rows_number-1)) max_to_cache= (full_rows_number-1); // count begins from 0!!!.
            cache.put_into_cache(db, "main", Topic_id,min_to_cache,max_to_cache, order);
            }
        else
            {
            cache.put_into_cache(db, "main", Topic_id,item_min,item_max,order);
            }
        }
    }



void MyBooksList::OnBooksInTopicWanted(long int _Topic_id)
    {
    cache.clear_cache();
    Topic_id=_Topic_id;
    mode=TOPIC_MODE;

    //query dabasese and get books number
    Khz::Libs::DB::show_books_in_topic temp(db, "main", Topic_id,-1, order);
    this->full_rows_number=temp.total_lines;
    this->where_statement=temp.where_statement;
    //P.S. The WHERE statement does not have any placeholders, so we leave parameters_for_WHERE_placeholders empty.
    UpdateOrder();

    //sorting is disabled as list contains too many books, remove arrow
    //indicating sorting order from header
    if(order=="" and number_of_sorting_item<this->GetColumnCount() )
        {
        //########arrow up ##################
        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        item.SetImage(-1); //no image
        this->SetColumn(number_of_sorting_item, item);
                // #################################
        }
    //if sorting was disabled before we need to redraw arrow
    //indicating sorting order.
    else if(order!="" and number_of_sorting_item<this->GetColumnCount() )
        {
        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        if(sort_order=="up")
            {
            item.SetImage(1); //arrow up
            }
        else
            {
            item.SetImage(2);//arrow down
            }
        this->SetColumn(number_of_sorting_item, item);
        }
    DeleteAllItems ();
    SetItemCount(full_rows_number);
    }






void MyBooksList::OnSearchWanted(
                            std::string _search_string,
                             std::string _search_type, std::string _fields_to_search)
    {
    cache.clear_cache();
    search_string=_search_string;
    //std::cout << "\nn search string is " << search_string;
    search_type=_search_type;
    fields_to_search=_fields_to_search;

    //this trigger is used in ongettext function, depending ot it's walue different output will be returned.
    mode=SEARCH_MODE;

    //query dabase and get books number
    Khz::Libs::DB::search_books temp(db,"main",search_string, -1, search_type,fields_to_search, order);
    this->full_rows_number=temp.total_lines;
    this->where_statement=temp.where_statement;
    this->parameters_for_WHERE_placeholders=temp.parameters_for_WHERE_placeholders;
    UpdateOrder();


    //sorting is disabled as list contains too many books, remove arrow
    //indicating sorting order from header
    if(order=="" and number_of_sorting_item<this->GetColumnCount() )
        {
        //########arrow up ##################
        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        item.SetImage(-1); //no image
        this->SetColumn(number_of_sorting_item, item);
                // #################################
        }
    //if sorting was disabled before we need to redraw arrow
    //indicating sorting order.
    else if(order!="" and number_of_sorting_item<this->GetColumnCount() )
        {
        wxListItem item;
        item.SetMask(wxLIST_MASK_IMAGE);
        if(sort_order=="up")
            {
            item.SetImage(1); //arrow up
            }
        else
            {
            item.SetImage(2);//arrow down
            }
        this->SetColumn(number_of_sorting_item, item);
        }

    DeleteAllItems ();
    SetItemCount(full_rows_number);

    }



int MyBooksList::ChooseIcon(std::string Extension) const
    {
    if(icons_size==-1) return -1; //no icons
    if (Extension=="") return 0;  //default icon

    if (File_icons_map.count(Extension) != 0)  //icon is already in the images list, return index.
        {
        auto it =File_icons_map.find(Extension);
        return  it->second;
        }
    else  //there no icon for such Extension in the m_pImageList, so we should insert icon there and return index.
        {
        wxString Ext = wxString(Extension.c_str());
        wxString temp_string;


        //folder where icon with current icons_size should be
        wxString size_folder= wxString::Format(wxT("%i"),icons_size)+"x"+wxString::Format(wxT("%i"),icons_size);
        wxString search_folder=wxString("icons")+wxFileName::GetPathSeparator()+size_folder;

        if(wxDirExists(search_folder))
            {
            temp_string=wxString("icons")+wxFileName::GetPathSeparator()+size_folder+wxFileName::GetPathSeparator()+wxString("icons_for_files_by_extension")+wxFileName::GetPathSeparator();
            }
        else
            {
            temp_string=wxString("icons")+wxFileName::GetPathSeparator()+"48x48"+wxFileName::GetPathSeparator()+wxString("icons_for_files_by_extension")+wxFileName::GetPathSeparator();
            }
        wxString Search_path=temp_string+Ext+".png";
        if (wxFileExists(Search_path))
            {
            wxIcon icon;
            try
                {
                icon.LoadFile(Search_path, wxBITMAP_TYPE_PNG);
                int index;
                if(this->icons_size==icon.GetWidth())
                    {
                    index=m_pImageList->Add(icon);
                    }
                else
                    {
                    index= m_pImageList->Add(wxBitmap(wxBitmap(icon).ConvertToImage().Rescale(icons_size, icons_size)));
                    }
                File_icons_map[Extension]=index;
                return index;
                }
            catch(...)
                {
                return 0;
                }
            }
        else
            {
            File_icons_map[Extension]=0;
            return 0;
            }
        }
    }


int MyBooksList::OnGetItemImage 	(long  item) 	const
    {
    //  std::cout<< "ongetitemtext is called";
        if ( icons_size == -1 )
        {
        return -1;
        }


    if(mode==SEARCH_MODE)  // we are in search mode
        {
        if(!cache.is_cached(item)) //item is not cached
            {
            Khz::Libs::DB::mapDB line=
                Khz::Libs::DB::search_books(db,"main",search_string, item, search_type,fields_to_search, order).result;




            std::string Extension= line[0]["Extension"];
            std::transform(Extension.begin(), Extension.end(), Extension.begin(), ::tolower);
            return  this->ChooseIcon(Extension);

            }
        else //item is cached
            {
            std::string Extension =cache.get_cached_line(item)["Extension"];
            std::transform(Extension.begin(), Extension.end(), Extension.begin(), ::tolower);
            return  this->ChooseIcon(Extension);

            }
        }

    else if(mode==TOPIC_MODE)  //we are in show books in particular topic mode.
        {
         if(!cache.is_cached(item)) //item is not cached
            {
            Khz::Libs::DB::mapDB line=Khz::Libs::DB::show_books_in_topic(db, "main", Topic_id,item, order).result;

            std::string Extension= line[0]["Extension"];
            std::transform(Extension.begin(), Extension.end(), Extension.begin(), ::tolower);
            return  this->ChooseIcon(Extension);
            }
        else
            {
            std::string Extension = cache.get_cached_line(item)["Extension"];
            std::transform(Extension.begin(), Extension.end(), Extension.begin(), ::tolower);
            return  this->ChooseIcon(Extension);
            }
        }
    return -1;
    }



wxString MyBooksList::OnGetItemText(long item, long column) const
    {


    if(items_to_show[column].first=="Num")  return Khz::Libs::Mstr::IntToString(item+1);

    if(mode==SEARCH_MODE)  // we are in search mode
        {
        if(!cache.is_cached(item)) //item is not cached
            {
            //std::cout << "\n\n in ongetitemtext search type is " << search_type << "\n\n";
            Khz::Libs::DB::mapDB line=Khz::Libs::DB::search_books(db,"main",search_string, item, search_type,fields_to_search, order).result;

            wxCHECK_MSG(line.size()==1,
                "", "Invalid number of rows in MyBooksList.cpp");
            std::string item_name=items_to_show[column].first.ToUTF8().data();
            if(items_to_show[column].first=="Title" and line[0]["Volume"]!="")
                {
                wxString temp_word;
                if( line[0]["Language"]=="Russian")
                    {
                    temp_word=wxString::FromUTF8("том"); //sources are in utf8.
                    }
                else
                    {
                    temp_word="vol";
                    }
                return wxString::FromUTF8(line[0][item_name].c_str()) +" ("+temp_word+" "+line[0]["Volume"] +")";
                }
            else
                {
                return wxString::FromUTF8(line[0][item_name].c_str());
                }
            }
        else //item is cached
            {
            // std::cout << "working with cache";
            std::string item_name=items_to_show[column].first.ToUTF8().data();
            if(items_to_show[column].first=="Title" and cache.get_cached_line(item)["Volume"]!="")
                {
                wxString temp_word;
                if( cache.get_cached_line(item)["Language"]=="Russian")
                    {
                    temp_word=wxString::FromUTF8("том"); //sources are in utf8.
                    }
                else
                    {
                    temp_word="vol";
                    }
                return wxString::FromUTF8(cache.get_cached_line(item)[item_name].c_str()) +" ("+temp_word+" "+cache.get_cached_line(item)["Volume"] +")";
                }
            else
                {
                return wxString::FromUTF8(cache.get_cached_line(item)[item_name].c_str());
                }



            }
        }

    else if(mode==TOPIC_MODE)  //we are in show books in particular topic mode.
        {
         if(!cache.is_cached(item)) //item is not cached
            {
            Khz::Libs::DB::mapDB line=Khz::Libs::DB::show_books_in_topic(db, "main", Topic_id,item, order).result;
            wxCHECK_MSG(line.size()==1,
                "", "Invalid number of rows in MyBooksList.cpp");
            std::string item_name=items_to_show[column].first.ToUTF8().data();
            if(items_to_show[column].first=="Title" and line[0]["Volume"]!="")
                {
                wxString temp_word;
                if( line[0]["Language"]=="Russian")
                    {
                    temp_word=wxString::FromUTF8("том"); //sources are in utf8.
                    }
                else
                    {
                    temp_word="vol";
                    }
                return wxString::FromUTF8(line[0][item_name].c_str()) +" ("+temp_word+" "+line[0]["Volume"] +")";
                }
            else
                {
                return wxString::FromUTF8(line[0][item_name].c_str());
                }
            }
        else  //item is cached
            {
            // std::cout << "working with cache";
            std::string item_name=items_to_show[column].first.ToUTF8().data();
            if(items_to_show[column].first=="Title" and cache.get_cached_line(item)["Volume"]!="")
                {
                wxString temp_word;
                if( cache.get_cached_line(item)["Language"]=="Russian")
                    {
                    temp_word=wxString::FromUTF8("том"); //sources are in utf8.
                    }
                else
                    {
                    temp_word="vol";
                    }
                return wxString::FromUTF8(cache.get_cached_line(item)[item_name].c_str()) +" ("+temp_word+" "+cache.get_cached_line(item)["Volume"] +")";
                }
            else
                {
                return wxString::FromUTF8(cache.get_cached_line(item)[item_name].c_str());
                }

            }
        }
    else
        {
        wxCHECK_MSG(1==1,
            "", "Bad user_wants_to_search_trigger");
        }
    abort(); // we should not get here;
    return "";
    }

void MyBooksList::OpenThisBook(wxListEvent& event)
    {
    long item =  event.GetIndex();

    Khz::Libs::DB::mapDB line;
    if(mode==SEARCH_MODE)  // we are in search mode
        {
        line=Khz::Libs::DB::search_books(db,"main",search_string, item, search_type,fields_to_search, order).result; //got just one line from db
        }
    else
        {
        line = Khz::Libs::DB::show_books_in_topic(db, "main", Topic_id, item, order).result;
        }
    if(line.size()!=1)
        {
        std::cout<< "\n error in OpenThisBook function, size of books array !=1\n";
        abort();
        }
    //std::cout << "topic blin is" <<  line[0]["Topic"] << "\n";
    wxString RelPathToFile = wxString::FromUTF8(line[0]["RelPathToFile"].c_str());  //data are saved in utf8 encoding in the database.
    wxString extension=line[0]["Extension"];
    extension =extension.Lower();
    wxString filename=line[0]["Filename"];
    RelPathToFile.Replace 	("/", wxFileName::GetPathSeparator() , true );
    wxString path_to_library= config->Read(wxString("PATHS/path_to_dir_with_books"),wxString(""));
    //std::cout << "path blin is" <<  path_to_library.mb_str(wxConvLocal).data() << "\n";


    Khz::Libs::OpentheBook(RelPathToFile, extension,filename,config );

    /*


    if (path_to_library.Right(1)== wxFileName::GetPathSeparator() )
        {
        path_to_library=path_to_library.RemoveLast();
        }
    wxString full_path_to_book = path_to_library + RelPathToFile +  wxFileName::GetPathSeparator() +filename;

    //if file not exist show error message@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
     if(  !wxFileExists(full_path_to_book)  ||   !wxFileName::IsFileReadable(full_path_to_book)  )
        {
        wxString Message ="Cannot read file \"" + full_path_to_book +"\". Check that there is such file and you have permission to read it."+
                            +wxTextFile::GetEOL()+wxTextFile::GetEOL()+
                            "P.S. Be sure that you have specified the right directory with books. Press File->Configure->Paths to check the parameter.";
        wxMessageDialog   *err_msg =new wxMessageDialog (this,
                                    Message,
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
        err_msg->ShowModal ();
        return;
        }

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    wxString app_to_open_book=config->Read("DEFAULT_APPS/"+extension, "");
    if(app_to_open_book=="")  //app for launch this file is not specified, execute defaul
        {
         // wxCharBuffer temp_char_buffer = full_path_to_book.mb_str(wxConvLocal);
         // std::cout << "\n i will try to open file " << temp_char_buffer.data() << "\n";
        if(!wxLaunchDefaultApplication("\""+ full_path_to_book + "\""))
            {
            wxMessageDialog   *err_msg =new wxMessageDialog (this,
                                    "Your system does not know which application should be used to open files with this extension. (" + full_path_to_book +"). You could either conifigure you system or configure Kolkhoznik.",
                                    "Error", wxOK|wxCENTRE|wxICON_ERROR|wxSTAY_ON_TOP|wxCENTRE, wxDefaultPosition) ;
            err_msg->ShowModal ();
            }
        }
    else if( app_to_open_book.Find("%f") ==  wxNOT_FOUND)
        {
        wxString command=app_to_open_book + " " + "\""+full_path_to_book+"\"";
         wxExecute(command);
        }
    else // app is specified
        {
        app_to_open_book.Replace("%f", "\""+full_path_to_book+"\"");
        wxExecute(app_to_open_book);
        }
    */
    }


void MyBooksList::GenItemsToShow()
    {
    std::vector< std::pair<wxString,wxString> >  temp_items_to_show;
    std::string items_string = (config->Read("COLUMNS_TO_SHOW/columns_order", "Num, Author, Title, Year, Publisher, Pages, Extension, Topic, md5_of_file")).ToStdString();
    std::vector<std::string> exploded_items_string = Khz::Libs::Mstr::explode(",", items_string);
    for(unsigned int i=0;i<exploded_items_string.size();i++)
        {
        wxString temp_name =  wxString(Khz::Libs::Mstr::trim(exploded_items_string[i]));
        wxString temp_visible_name= config->Read(wxT("COLUMNS_VISIBLE_NAMES/"+temp_name), temp_name);
        temp_items_to_show.push_back( std::pair<wxString,wxString>(temp_name,  temp_visible_name));
        }
    items_to_show=temp_items_to_show;
    }

void MyBooksList::UpdateOrder()
//reason for this is performance issue:
// if ootpub is too big than sorting becomes too slow, so just disable it.
    {
    if(
       (full_rows_number>do_not_sort_list_if_books_more_than_this_number)
       and
       (do_not_sort_list_if_books_more_than_this_number>0)
       )
        {
        order="";
        }
    else
        {
        order=users_wanded_order;
        }
    }
void MyBooksList::OnResizeColumn(wxListEvent& event)
    {

    delete config; // without this changes made by other classes that use the same config file will not be saved.
    config =new wxFileConfig( "", "", ini_filename);

    int column = event.GetColumn();
    int new_width= this->GetColumnWidth (column);
    this->config->Write( wxT("RIGHT_PANEL_COLUMNS_POSITIONS/"+ wxString::Format(wxT("%i"),column)),   wxString::Format(wxT("%i"),new_width) );
    this->config->Flush();
    event.Skip();
    }

//reads config file again and changes the control accordingly. It is called when user changes
//configuration parameters by using the configuration gui tool.
void MyBooksList::Reconfigure()
    {
    delete config;
    sqlite3_close(db);
    config =new wxFileConfig( "", "", ini_filename);

    wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();
    sqlite3_open( temp_char_buffer, &db);
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    ClearAll ();//remove everything from list
    File_icons_map.clear(); //clear icons cache.
    cache.clear_cache();
    full_rows_number=0;

    GenItemsToShow();
    SetItemCount(full_rows_number);
   number_of_sorting_item=0;
   sort_order="up";
   //users_wanded_order=" ORDER by AuthorLowerCase, TitleLowerCase, cast(Volume as integer) ";
   users_wanded_order=" ORDER by AuthorLowerCase, TitleLowerCase ";
    UpdateOrder();
    for(unsigned int i=0;i<items_to_show.size();i++)
        {

        int width = wxAtoi(    config->Read("RIGHT_PANEL_COLUMNS_POSITIONS/"+ wxString::Format(wxT("%i"),i), wxString::Format(wxT("%i"),wxLIST_AUTOSIZE))       );
        if(items_to_show[i].first=="Author")
            {
            number_of_sorting_item=i;								    //arrow up
			//InsertColumn (i ,items_to_show[i].second+wxString::FromUTF8(" \xE2\x96\xB2"),wxLIST_FORMAT_LEFT, width);

			InsertColumn (i ,items_to_show[i].second, wxLIST_FORMAT_LEFT, width);

			//########arrow up ##################
            wxListItem item;
            item.SetMask(wxLIST_MASK_IMAGE);
            item.SetImage(1); //arrow up
            this->SetColumn(i, item);
            // #################################
            }
        else
            {
            InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
            }
        // SetImage(0);
        }

   // std::cout << "binding";
    this->OnBooksInTopicWanted(1); //initialize top level
    }


void MyBooksList::UserWantsToChangeSortingOrder(wxListEvent& event)
    {
    UpdateOrder();
    if(order=="")  //show message that sorting is disabled
        {
        wxString msg =wxString("Sorting is disabled as the list contains more books than it is allowed to sort.")+
                      wxString(wxTextFile::GetEOL())+
                      "Edit File->Configure->Advanced to change the number.";
        wxMessageBox(msg, wxT("Info"), wxOK | wxICON_INFORMATION, this);
        return;
        }
    int clicked_column=event.GetColumn();


    std::string item_to_sort= items_to_show[clicked_column].first.ToStdString();
    if(item_to_sort=="Num") return;


    std::vector<std::string> default_sorting_sqlite_tables;
    default_sorting_sqlite_tables.push_back("AuthorLowerCase");
    default_sorting_sqlite_tables.push_back( "TitleLowerCase");
    //default_sorting_sqlite_tables.push_back("Volume");


    //use clicked column that is being already used for sorting, change sorting order
    if (clicked_column==number_of_sorting_item)
        {
        if(sort_order=="up")
            {
            sort_order="down";
            }
        else if(sort_order=="down")
            {
            sort_order="up";
            }
        }
    else
        {
        sort_order="up";
        }

    std::string sqlite_sorting_table;


    if (item_to_sort=="Author")
        {
        sqlite_sorting_table="AuthorLowerCase";
        }
    else if(item_to_sort=="Title")
        {
        sqlite_sorting_table="TitleLowerCase";
        }
    else
        {
        sqlite_sorting_table=item_to_sort;
        }
    std::string select_order_condition_final;


    if(sort_order=="up")
        {
        select_order_condition_final=" ORDER by " + sqlite_sorting_table + " ";
        }
    else
        {
        select_order_condition_final=" ORDER by " + sqlite_sorting_table + " DESC ";
        }

    for(unsigned int i=0; i<default_sorting_sqlite_tables.size();i++)
        {
        if(sqlite_sorting_table==default_sorting_sqlite_tables[i]) continue;
        select_order_condition_final=select_order_condition_final+","+default_sorting_sqlite_tables[i];
        }

    select_order_condition_final=select_order_condition_final+ " ";

    //###########################################set filnal values of class variables
    number_of_sorting_item=clicked_column;
    users_wanded_order=select_order_condition_final;

    ClearAll ();
    for(unsigned int i=0;i<items_to_show.size();i++)
        {

        int width = wxAtoi(    config->Read("RIGHT_PANEL_COLUMNS_POSITIONS/"+ wxString::Format(wxT("%i"),i), wxString::Format(wxT("%i"),wxLIST_AUTOSIZE))       );
        if( ((int)i)==  number_of_sorting_item  )
            {
            if(sort_order=="up")
                {                                                                //arrow up
					//InsertColumn (i ,items_to_show[i].second+wxString::FromUTF8(" \xE2\x96\xB2"),wxLIST_FORMAT_LEFT, width);
					InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
                //########arrow up ##################
                    wxListItem item;
                    item.SetMask(wxLIST_MASK_IMAGE);
                    item.SetImage(1); //arrow up
                    this->SetColumn(i, item);
                // #################################

                }
            else if(sort_order=="down")
                {																//arrow down
                 //InsertColumn (i ,items_to_show[i].second+wxString::FromUTF8(" \xE2\x96\xBC"),wxLIST_FORMAT_LEFT, width);
                 InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
                //########arrow up ##################
                    wxListItem item;
                    item.SetMask(wxLIST_MASK_IMAGE);
                    item.SetImage(2); //arrow down
                    this->SetColumn(i, item);
                // #################################
                }
            }
        else
            {
            InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
            }

        }
    cache.clear_cache();
    UpdateOrder();
    DeleteAllItems();
    SetItemCount(full_rows_number);
    //#############################################################
    //std::cout << "select_order_condition_final is" << select_order_condition_final<< "\n";

    }
void MyBooksList::RightClickMenu(wxListEvent& event)
{
    last_right_clicked_item=event.GetIndex();
    if(last_right_clicked_item==-1) return;
    wxMenu *menu = new wxMenu;
    wxMenuItem *item1 = new wxMenuItem(menu, my_wx_id__SHOW_INFO_ABOUT_BOOK,  wxT("Info"));
    wxMenuItem *item2 = new wxMenuItem(menu, my_wx_id__GEN_BIBTEX_FOR_ONE_BOOK,  wxT("Generate Bibtex"));
    //#include "CoolIcon.xpm"
    item1->SetBitmap(wxArtProvider::GetBitmap(wxART_INFORMATION, wxART_MENU));
    menu->Append(item1);
    menu->Append(item2);

    PopupMenu(menu);
    delete menu;
}

void MyBooksList::OnInfoAboutBookWanted(wxCommandEvent &event)
    {
    Khz::Libs::DB::mapDB line=GetOneBookFromDB(last_right_clicked_item);


    wxString RelPathToFile = wxString::FromUTF8(line[0]["RelPathToFile"].c_str());  //data are saved in utf8 encoding in the database.
    wxString filename=line[0]["Filename"];
    RelPathToFile.Replace 	("/", wxFileName::GetPathSeparator() , true );
    wxString path_to_library= config->Read(wxString("PATHS/path_to_dir_with_books"),wxString(""));
    //std::cout << "path blin is" <<  path_to_library.mb_str(wxConvLocal).data() << "\n";







    if (path_to_library.Right(1)== wxFileName::GetPathSeparator() )
        {
        path_to_library=path_to_library.RemoveLast();
        }
    wxString full_path_to_book = path_to_library + RelPathToFile +  wxFileName::GetPathSeparator() +filename;


    Khz::PopupWins::InfoAboutThisBook *popup_info= new Khz::PopupWins::InfoAboutThisBook(this, line, full_path_to_book);
    //popup_info->SetSize (800, -1);
    popup_info->Show();
    popup_info->Fit();
    //delete popup_info;
    }


void MyBooksList::JustReadConfigAgain()
    {
    delete config;
    config =new wxFileConfig( "", "", ini_filename);
    }

void MyBooksList::InitAndSetIconsList(int size)
{
    icons_size=size;
    int temp_size=size;
    if(temp_size!=size) return; //to avoid compiller warning;
    if(icons_size!=-1)
        {
        m_pImageList = new wxImageList(icons_size, icons_size);
        }
    else
        {
        m_pImageList = new wxImageList(16, 16);
        temp_size=16;
        }
    wxIcon icon;
    wxString temp_string;

    wxString size_folder= wxString::Format(wxT("%i"),size)+"x"+wxString::Format(wxT("%i"),size);
    wxString search_folder=wxString("icons")+wxFileName::GetPathSeparator()+size_folder;

    wxString arrows_string=wxString("icons")+wxFileName::GetPathSeparator();
    if(size==-1)
        {
        temp_string=wxString("icons")+wxFileName::GetPathSeparator()+"16x16"+wxFileName::GetPathSeparator();
        }
    else if(wxDirExists(search_folder))
        {
        temp_string=wxString("icons")+wxFileName::GetPathSeparator()+size_folder+wxFileName::GetPathSeparator();
        }
    else
        {
        temp_string=wxString("icons")+wxFileName::GetPathSeparator()+"48x48"+wxFileName::GetPathSeparator();
        }


    icon.LoadFile(temp_string+wxT("book.png"), wxBITMAP_TYPE_PNG);
    if(icons_size==icon.GetWidth() or size==-1)
        {
        m_pImageList->Add(icon);
        }
    else
        {
        m_pImageList->Add(wxBitmap(wxBitmap(icon).ConvertToImage().Rescale(icons_size, icons_size)));
        }


    icon.LoadFile(arrows_string+wxT("arrow_up.png"), wxBITMAP_TYPE_PNG);
    if(icons_size==icon.GetWidth() or size==-1)
        {
        m_pImageList->Add(icon);
        }
    else
        {
        m_pImageList->Add(wxBitmap(wxBitmap(icon).ConvertToImage().Resize(wxSize(icons_size, icons_size),wxPoint((size-16)/2, (size-16)/2))));
        }


    icon.LoadFile(arrows_string+wxT("arrow_down.png"), wxBITMAP_TYPE_PNG);
    if(icons_size==16 or size==-1)
        {
		//std::cout << "\icon size is good\n";
        m_pImageList->Add(icon);
        }
    else
        {
		//std::cout << "\nresizing arrow down\n";
		//std::cout << "icons_size is " << icons_size<< "\n";
		//std::cout << "icon width is " << icon.GetWidth()<< "\n";
        m_pImageList->Add(wxBitmap(wxBitmap(icon).ConvertToImage().Resize(wxSize(icons_size, icons_size),wxPoint((size-16)/2, (size-16)/2))));
        }


    SetImageList(m_pImageList, wxIMAGE_LIST_SMALL);
}



void MyBooksList::ReBuildIMageList(int new_size)
    {
    wxImageList *temp_pointer= m_pImageList;  //temp pointer gets adress of m_pImageList
    InitAndSetIconsList(new_size);  // m_pImageList=new wxImageList();  So, m_pImageList is different pointer now.
    delete temp_pointer; //remove ald image list.
    File_icons_map.clear(); //clear cache of extension icons;
    icons_size=new_size;
    }




void MyBooksList::ReDrawList()
    {
    delete config;
    config =new wxFileConfig( "", "", ini_filename);
    //std::cout << "\n\n first num of sort item" << number_of_sorting_item <<"\n";
    //for(int i=0;i<items_to_show.size();i++)
    //    {
    //    std::cout << "\n item_t0_show "<< items_to_show[i].first;
    //    }
    //std::cout << "\n num_of_sorting_item "<< number_of_sorting_item;
    wxString sorting_field;
    if( number_of_sorting_item < ((int) items_to_show.size()))
        {
        sorting_field=items_to_show[number_of_sorting_item].first;
        }
    else
        {
        sorting_field="huynya_ebuchay_gori_ono_vse_sinim_plamenem!!!!!!!";
        //we need sorting_field that is not in the database for sure.
        }
    wxString icons_size_in_config = config->Read("ICONS/books_list_icons_size","16");
    if(icons_size_in_config=="") icons_size_in_config="16";
    icons_size = wxAtoi(icons_size_in_config);

    wxString max_sort_num_string = config->Read("ADVANCED/max_num_of_books_to_sort","10000");
    if(max_sort_num_string=="") max_sort_num_string="10000";
    do_not_sort_list_if_books_more_than_this_number=wxAtoi(max_sort_num_string);


    wxString max_cache_size_str= config->Read("ADVANCED/max_size_of_cache", "4000");
    if(max_cache_size_str=="") max_cache_size_str="4000";
    max_size_of_cache=wxAtoi(max_cache_size_str);

    ClearAll ();
    cache.clear_cache();
    ReBuildIMageList(icons_size);


    full_rows_number=0;

    GenItemsToShow();


    bool sorting_item_is_steel_here=false;
    for(unsigned int i=0;i<items_to_show.size();i++)
        {

        int width = wxAtoi(    config->Read("RIGHT_PANEL_COLUMNS_POSITIONS/"+ wxString::Format(wxT("%i"),i), wxString::Format(wxT("%i"),wxLIST_AUTOSIZE))       );
        if(items_to_show[i].first==sorting_field)
            {
            sorting_item_is_steel_here=true;
            number_of_sorting_item=i;								    //arrow up
			if(sort_order=="up")
                {                                                                //arrow up
					//InsertColumn (i ,items_to_show[i].second+wxString::FromUTF8(" \xE2\x96\xB2"),wxLIST_FORMAT_LEFT, width);
					InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
                //########arrow up ##################
                    wxListItem item;
                    item.SetMask(wxLIST_MASK_IMAGE);
                    item.SetImage(1); //arrow up
                    this->SetColumn(i, item);
                // #################################

                }
            else if(sort_order=="down")
                {																//arrow down
                 //InsertColumn (i ,items_to_show[i].second+wxString::FromUTF8(" \xE2\x96\xBC"),wxLIST_FORMAT_LEFT, width);
                 InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
                //########arrow up ##################
                    wxListItem item;
                    item.SetMask(wxLIST_MASK_IMAGE);
                    item.SetImage(2); //arrow down
                    this->SetColumn(i, item);
                // #################################
                }
            }
        else
            {
            InsertColumn (i ,items_to_show[i].second,wxLIST_FORMAT_LEFT, width);
            }
        // SetImage(0);
        }
    if(!sorting_item_is_steel_here) number_of_sorting_item=999999999;
    if(mode==TOPIC_MODE)
        {
        this->OnBooksInTopicWanted(Topic_id);
        }
    else if (mode==SEARCH_MODE)
        {
        this->OnSearchWanted(search_string, search_type, fields_to_search);
        }
    else
        {
        std::cout << "\n\n pisdec katenku\n\n";
        abort();
        }



    }

void MyBooksList::OnGenBibtexMenuSelected(wxCommandEvent &event)
    {
    std::map<std::string, std::string>line;
    if(!cache.is_cached(last_right_clicked_item))
        {
        line=GetOneBookFromDB(last_right_clicked_item)[0];
        }
    else
        {
        line=cache.get_cached_line(last_right_clicked_item);
        }

    //string with bibtex entry
    std::string bibtex_entry=Khz::BibTex::GenBib::GenBibtexEntryFromFields(line["Author"], line["Title"],
                                    line["Publisher"], line["Year"], line["Edition"], line["ID"]);

    //prepare wxstring
    wxString Bibtex_wxString = wxString::FromUTF8(bibtex_entry.c_str());
    Khz::BibTex::BibtexEntryWindow *BibWindow= new Khz::BibTex::BibtexEntryWindow(this, Bibtex_wxString);
    BibWindow->Show();

    }



Khz::Libs::DB::mapDB  MyBooksList::GetOneBookFromDB(long long num)
    {
    Khz::Libs::DB::mapDB line;
    if(mode==SEARCH_MODE)  // we are in search mode
        {
        line=Khz::Libs::DB::search_books(db,"main",search_string, num, search_type,fields_to_search, order).result; //got just one line from db
        }
    else
        {
        line = Khz::Libs::DB::show_books_in_topic(db, "main", Topic_id, num, order).result;
        }
    if(line.size()!=1)
        {
        std::cout<< "\n error in OpenThisBook function, size of books array !=1\n";
        abort();
        }
    return line;
    }


std::string MyBooksList::GetWhere()
    {
    return this->where_statement;
    }

std::vector<std::string> MyBooksList::GetPlaceHoldersParameters()
    {
    return this->parameters_for_WHERE_placeholders;
    }




}} //end of namespaces
