/*
Copyright (C) Ivan Ivanov, 2014-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <vector>
#include <wx/listbox.h>
#include "TreeCtrlLibrary.h"
#include "MyleftPanel.h"
#include "MyMainWindow.h"
#include <wx/sizer.h>
#include "Custom_IDs.h"
#include<iostream>
#include "SqliteDBGenerator.h"
#include <wx/stdpaths.h>
#include "StatisticsWindow.h"
#include "clear_database_path.h"
#include "get_books_from_db_by_query.h"
#include "BibtexTranslator.h"
#include "BibTexGeneratorStandaloneWindow.h"
#include "FullTextSearchIndexerWindow.h"
#include <wx/dir.h>
#include "ConfigManager.h"
#include "DEBUGMESSAGE.h"


namespace Khz{namespace MainWin{


MyMainWindow::~MyMainWindow()
    {
    delete config;
    }


//shows "about kolkhoznik popup when user presses Help->About from toolbar
void MyMainWindow::OnAbout(wxCommandEvent& event)
    {
    const wxChar* NEWLINE_CHAR = wxTextFile::GetEOL();
    wxString version="0.7.1";
    wxString msg= "kolkhoznik, version "+version+ NEWLINE_CHAR+ NEWLINE_CHAR +
                "Author: Ivan Ivanov"+NEWLINE_CHAR+"License: GPL v3" +
                   NEWLINE_CHAR+NEWLINE_CHAR+"Built with wxWidgets"+
                   NEWLINE_CHAR+"The Full Text Search feature is provided by Xapian";

    wxMessageBox(msg, wxT("About kolkhoznik"), wxOK | wxICON_INFORMATION, this);
    }



//when user tries to close main window (by pressin cross on the right top of the window
// we do not exit kolkhoznik but just hide maint window to tray.
void MyMainWindow::OnQuit(wxCloseEvent& event)
    {

    Hide();

    }



MyMainWindow::MyMainWindow(const wxString& title)
    : wxFrame(NULL,wxID_ANY, title//, wxDefaultPosition,  	wxSize(800, 600)
              )
{

    is_full_text_search_window_shown=false;
	//let us look into config file
    ini_filename = ConfigManager::GetPathToConfigFile();
    config =new wxFileConfig( "", "", ini_filename);
	// config file is opened now.



	//now we read main window and splitter sizes. (they were remembered from last session)
    int window_width= wxAtoi(config->Read("Window_size/width", "800"));
    int window_height=wxAtoi(config->Read("Window_size/height", "600"));
    wxString is_maximized=config->Read("Window_size/is_expanded", "no" );
    int m_splitter_sash_pos= wxAtoi(config->Read("Window_size/sash_position", "220"));
	// reading of sizes finished.

	//let us set widh, height of the main window
    this->SetSize(window_width, window_height);
    if(is_maximized=="yes") this->Maximize();
	//finished with sizes.


	//splitter, split window to two parts, left and right, right is books list, lift is search and tree panels.
    m_splitter  =new // wxSplitterWindow(this);
    wxSplitterWindow (this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxSP_LIVE_UPDATE);

   // m_splitter.Connect(wxID_ANY, wxEVT_COMMAND_SPLITTER_SASH_POS_CHANGED, wxSplitterEventHandler(MySplitterWindow::OnPositionChanged));
  //  m_splitter.Connect(wxID_ANY, wxEVT_COMMAND_SPLITTER_SASH_POS_CHANGING, wxSplitterEventHandler(MySplitterWindow::OnPositionChanging));


    m_splitter->SetSize(GetClientSize());
    m_splitter->SetSashGravity(0); //left panel preserve it's size.



	//left panel (search and library tree)
    m_left = new MyleftPanel(m_splitter);


	//right panel (books list)
    m_right = new MyRightPanel(m_splitter);

   // Set the frame icon
   #include "CoolIcon.xpm"
   SetIcon(wxIcon(CoolIcon_xpm));


   // Create a menu bar

	// The "help" item
    wxMenu *helpMenu = new wxMenu;
    helpMenu->Append(wxID_ABOUT, wxT("&About...\tF1"),
    wxT("Show about dialog"));

	//the "file" menu
	wxMenu *fileMenu = new wxMenu;
    fileMenu->Append(wxID_EXIT, wxT("E&xit\tAlt-X"),
    wxT("Quit this program"));
    fileMenu->Append(my_wx_id__CREATE_DATABASE_GUI, wxT("Generate database"), wxT("Specify derictory with kolhoz books and generate index"));
    fileMenu->Append(my_wx_id__CONFIGURE, wxT("Configure"));
    fileMenu->Append(my_wx_id__SHOW_STATISTICS, wxT("View statistics"));

    //the "tools" menu
	wxMenu *toolsMenu = new wxMenu;
    toolsMenu->Append( my_wx_id__GENERATE_BIBTEX_FOR_MANY_FILES, wxT("Generate Bibtex"),
                      wxT("Generates bibtex database for multiply files"));

    toolsMenu->Append( my_wx_id__GENERATE_FULL_TEXT_SEARCH_DATABASE, wxT("Generate Full-text search index"),
                       wxT("Generates the full text search index database"));



    // Now append the freshly created menu to the menu bar...
    wxMenuBar *menuBar = new wxMenuBar();
    menuBar->Append(fileMenu, wxT("&File"));
    menuBar->Append(toolsMenu, wxT("&Tools"));
    menuBar->Append(helpMenu, wxT("&Help"));
	// ... and attach this menu bar to the frame

    SetMenuBar(menuBar);




	// Create a status bar just for fun
    CreateStatusBar(2);

	//split window into two parts
    m_splitter->SplitVertically(m_left, m_right, m_splitter_sash_pos);

	//bindings

	// user wants to search
    Bind(wxEVT_BUTTON, &MyMainWindow::OnSearchWanted, this,my_wx_id__SEARCH_BOOKS_BUTTON);
    Bind(wxEVT_SEARCH , &MyMainWindow::OnSearchWanted, this,my_wx_id__SEARCH_BOOKS_INPUT);


    //user wants the full text searching
    Bind(wxEVT_BUTTON, &MyMainWindow::OnFullTextSearchWanted, this,my_wx_id__FULL_TEXT_SEARCH_BOOKS_BUTTON);
    Bind(wxEVT_SEARCH , &MyMainWindow::OnFullTextSearchWanted, this,my_wx_id__FULL_TEXT_SEARCH_BOOKS_INPUT);

    //user wants create database
    Bind(wxEVT_COMMAND_MENU_SELECTED, &MyMainWindow::OnCreateDatabase, this, my_wx_id__CREATE_DATABASE_GUI);

    //user wants view stats
    Bind(wxEVT_COMMAND_MENU_SELECTED, &MyMainWindow::OnViewStatistics, this, my_wx_id__SHOW_STATISTICS);


    //user wants configure something
    Bind(wxEVT_COMMAND_MENU_SELECTED, &MyMainWindow::OnConfigure, this, my_wx_id__CONFIGURE);

    //something is being resized
    Bind(wxEVT_SIZE, &MyMainWindow::OnResize, this);
    Bind(wxEVT_MAXIMIZE, &MyMainWindow::OnMaximize, this);
    m_splitter->Bind( wxEVT_SPLITTER_SASH_POS_CHANGED, &MyMainWindow::OnSashPositionChanged, this);


    //user wants books in particular topic
    Bind(wxEVT_BUTTON, &MyMainWindow::OnBooksInTopicWanted, this,my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT);

    //only hides main window
    Bind(wxEVT_CLOSE_WINDOW , &MyMainWindow::OnQuit, this);
    Bind(wxEVT_BUTTON , &MyMainWindow::OnFullTextSearchWindowQuit, this, my_wx_id__CLOSE_FULL_TEXT_SEARCH_WINDOW);

    	//popup info about kolkhoznik
    Bind(wxEVT_COMMAND_MENU_SELECTED, &MyMainWindow::OnAbout, this, wxID_ABOUT);


    //generate bibtex window
    Bind(wxEVT_COMMAND_MENU_SELECTED,&MyMainWindow::OnGenBibtexWanted,this, my_wx_id__GENERATE_BIBTEX_FOR_MANY_FILES);


    //generate full text search database
    Bind(wxEVT_COMMAND_MENU_SELECTED,&MyMainWindow::OnGenFullTextSearchIndexWanted,this, my_wx_id__GENERATE_FULL_TEXT_SEARCH_DATABASE);

    //reconfigure something
    Bind(wxEVT_BUTTON, &MyMainWindow::ReconfigurePanels, this,my_wx_id__RECONFIGURE_PANELS);
    //P.S. some bindings are made in main.cpp.


    //status text
    SetStatusText(wxT("Welcome to Kolkhoznik!"));

}
void MyMainWindow::OnSearchWanted(wxCommandEvent &event)
    {
    m_left->on_search_begin();  //makes search button look pressed.
    this->Refresh();
    this->Update();  //updates appearance of the main window, without it the search button will not look pressed.
   // std::cout<< "event_id is " << event.GetId() << "\n";
   // std::cout<<"buttons id is " <<my_wx_id__SEARCH_BOOKS_BUTTON <<"\n";
    wxString search_text= m_left->GetSearchText();
    bool title_checked=m_left->IsTitleChecked();
    bool author_checked=m_left->IsAuthorChecked();
    wxString search_type=m_left->GetSearchType();  //or, and
    //m_left->UnselectAllTopics();
    m_right->DoSearch(search_text,title_checked, author_checked, search_type);
    m_left->on_search_end(); //makes search buttom look unpressed.
    this->Refresh();
    this->Update();
    }

void MyMainWindow::OnBooksInTopicWanted(wxCommandEvent &event)
    {
    //int is id of the topic in the database.
    m_right->ShowBooksInTopic(event.GetInt());
    }
void MyMainWindow::OnCreateDatabase(wxCommandEvent &event)
    {
    Khz::GenDB::SqliteDBGenerator *frame = new Khz::GenDB::SqliteDBGenerator(this, wxT("Generate database"));
    frame->Show(true);
    frame->Fit();
    }
void MyMainWindow::OnConfigure(wxCommandEvent &event)
    {
    config_gui= new Khz::CfgWin::ConfigureGui(this,ini_filename);
    //std::cout << "\nconfig_gui_size" <<   config_gui->GetSize().GetWidth() << " " << config_gui->GetSize().GetHeight() << "\n";
    //std::cout << "\nconfig_gui_treebook_size" <<   config_gui->main_treebook->GetSize().GetWidth() << " " << config_gui->main_treebook->GetSize().GetHeight() << "\n";
    //config_gui->PrintSomeInfo();
    //config_gui->Fit();
    //config_gui->GetSizer()->Layout();
    config_gui->Show();
    config_gui->GetSizer()->Fit(config_gui);
    config_gui->SetMinSize(config_gui->GetSize());
    //std::cout<< "\n###################################################SHOW CALLED###################################\n";
    //std::cout << "\nconfig_gui_size" <<   config_gui->GetSize().GetWidth() << " " << config_gui->GetSize().GetHeight() << "\n";
    // std::cout << "\nconfig_gui_treebook_size" <<   config_gui->main_treebook->GetSize().GetWidth() << " " << config_gui->main_treebook->GetSize().GetHeight() << "\n";
    //config_gui->PrintSomeInfo();
    //config_gui->PrintSomeInfo();
    //config_gui->RefitManually();
    /*
    wxString ini_filename = PathToConfig().GiveMePath();
    const wxChar* NEWLINE_CHAR = wxTextFile::GetEOL();


    wxString msg= wxString("We are VERY sorry, but GRAPHICAL configuration tool is not implemented")+
    wxString(" yet (but there is a hope that one day it will be...). ") +wxString(NEWLINE_CHAR)+ wxString(NEWLINE_CHAR)+
    wxString("For now you have to edit the configuration file \"") + ini_filename+"\" manually. (we recommend use Total Commander to reach hidden files on Windows)"+
	wxString(NEWLINE_CHAR)+
    wxString("The following configuration options are available:")  +wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
    wxString("1) PATH TO LIBRARY. Change key \"path_to_dir_with_books\" in section [PATHS] accordingly.") + wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
    wxString("2) PATH TO DATABASE. ")+
    wxString("Change key \"path_to_database\" in section [PATHS] accordingly.")+wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
    wxString("3) DEFAULT APPLICATIONS TO OPEN BOOKS. For example, if you want to open PDF files by")+
    wxString(" program \"BestPDFreader\" you have to put line \"pdf=BestPDFreader\" (wuthout quotes)")+
    wxString(" into section [DEFAULT_APPS].")+wxString(NEWLINE_CHAR)+ wxString(NEWLINE_CHAR)+
    wxString("4)FIELDS THAT ARE TO BE DISPLAYED ON THE RIGHT PANEL (like Author, Title, Publisher).")+
    wxString(" Edit key \"columns_order\" in section  [COLUMNS_TO_SHOW]. The default value")+
    wxString(" is \"Num, Author, Title, Year, Topic, Extension, Publisher, md5_of_file\". You can remove")+
    wxString(" fields, change fields order.")+wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
    wxString("5) VISIBLE NAMES OF THE FIELDS of the right panel. For example, if you want to display")+
    wxString(" \"Book's Author\" instead of \"Author\" you should change \"Author=Author\" ")+
    wxString("to \"Author=Book's Author\" within the section [COLUMNS_VISIBLE_NAMES].")+wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
	wxString("6) FONT SIZE OF THE BOOK'S LIST. Edit key \"books_list_font_size\" within the section [FONTS]. (for example, you can make it look like this:")+
	wxString(" \"books_list_font_size=10\" (without quotes). In this example the font size is 10 pt.)") +wxString(NEWLINE_CHAR)+ wxString(NEWLINE_CHAR)+
	wxString("7) FONT SIZE OF THE LIBRARY TREE (left panel). Edit key \"library_tree_font_size\" ")+
	wxString("within the section [FONTS] as described in the example above.")+wxString(NEWLINE_CHAR)+wxString(NEWLINE_CHAR)+
    wxString("TO APPLY CHANGES RESTART THE APPLICATION!");


    msg.Printf(msg);
    wxMessageBox(msg, wxT("Configure"), wxOK | wxICON_INFORMATION, this);
    */
    }



//remembers window size (puts data to the config file
void MyMainWindow::OnResize(wxSizeEvent &event)
    {
        delete config; // without this changes made by other classes that use the same config file will not be saved.
        config =new wxFileConfig( "", "", ini_filename);

      //  std::cout << "sizing";
        int width;
        int height;
        this->GetSize (&width, &height);
        if (!this->IsMaximized())
            {
            config->Write( wxT("Window_size/width"),wxString::Format(wxT("%i"),width) );
            config->Write( wxT("Window_size/height"), wxString::Format(wxT("%i"),height) );
            config->Write( wxT("Window_size/is_expanded"), "no" );
            }
        config->Flush();
        event.Skip();
        //testconfig->Write( wxT("Window_size/is_expanded"), "" );
    }

//remembers that window should be maximized
void MyMainWindow::OnMaximize(wxMaximizeEvent& event)
    {

    delete config; // without this changes made by other classes that use the same config file will not be saved.
    config =new wxFileConfig( "", "", ini_filename);

    if (!this->IsMaximized())
        {
        config->Write( wxT("Window_size/is_expanded"), "no" );
        }
    else
        {
        config->Write( wxT("Window_size/is_expanded"), "yes" );
        }
     config->Flush();
    event.Skip();
    }

//remembers splitter position
void MyMainWindow::OnSashPositionChanged(wxSplitterEvent& event)
    {

    delete config; // without this changes made by other classes that use the same config file will not be saved.
    config =new wxFileConfig( "", "", ini_filename);

    int sash_pos=m_splitter->GetSashPosition();
     config->Write( wxT("Window_size/sash_position"),wxString::Format(wxT("%i"),sash_pos) );
     config->Flush();
    event.Skip();
    }

//shows statistics window
void MyMainWindow::OnViewStatistics(wxCommandEvent& event)
    {
    Khz::PopupWins::StatisticsWindow *window = new Khz::PopupWins::StatisticsWindow();
    delete window ;
    }


void MyMainWindow::ReconfigurePanels(wxCommandEvent& event)
    {
    int event_int=event.GetInt();

    //database changed
    if(event_int==Khz::my_wx_id__RECONFIGURE_MAINWIN)
        {
        m_right->Reconfigure();
        m_left->ReconfigureLibraryTree();
        }

    else if(event_int==Khz::my_wx_id__RECONFIGURE_EVERYTHING)
        {
        m_right->Reconfigure();
        m_left->ReconfigureLibraryTree();

        if (is_full_text_search_window_shown)
            {
            //std::vector<wxString> temp_topics_array;
            //m_left->GetArrayOfTopicsForFullTextSearch(temp_topics_array);

            full_text_search_window->ReconnectToDB();
            //is_full_text_search_window_shown=false;
            //full_text_search_window=new Khz::FTS::Gui::FullTextSearchWindow(this);
            //full_text_search_window->Show();
            //full_text_search_window->Lower();
            //full_text_search_window->OnSearchWanted(m_left->GetFullTextSearchText(),
            //                                        m_left->GetLanguageForFullTextSearch(), temp_topics_array);
            //full_text_search_window->SetFocus();
            }
        }
    else if (event_int==Khz::my_wx_id__RECONFIGURE_XAPIAN_AND_PATH_TO_DIR_WITH_BOOKS)
        {
        m_right->JustReadConfigAgain();
        if (is_full_text_search_window_shown)
            {
            full_text_search_window->ReconnectToDB();
            }
        }
    else if (event_int==Khz::my_wx_id__RECONFIGURE_XAPIAN)
        {
        if (is_full_text_search_window_shown)
            {
            full_text_search_window->ReconnectToDB();
            }
        }


    // app to open file with particulare extension changed
    else if(event_int==Khz::my_wx_id__RECONFIGURE_JUST_READ_CONFIG_AGAIN)
        {
        m_right->JustReadConfigAgain();
        if (is_full_text_search_window_shown)
            {
            full_text_search_window->ReDraw();
            }
        }

    //number of visible fields on the right panel changed
    else if(event_int==Khz::my_wx_id__RECONFIGURE_RECONSTRUCT_RIGHT_PANEL)
        {
        m_right->RedrawList();
        }

    //fonts changed
    else if(event_int==Khz::my_wx_id__RECONFIGURE_FONTS)
        {
        m_right->SetNewFontSize();
        m_left->SetNewLibraryTreeAppearance();
        }

    //icons changed
    else if(event_int==Khz::my_wx_id__RECONFIGURE_ICONS)
        {
        m_right->RedrawList();
        m_left->SetNewLibraryTreeAppearance();//it calls ReDrawItems(), as a result the icons size will be changed too.
        }
    else if(event_int==Khz::my_wx_id__FULL_TEXT_SEARCH_REDRAW_HTML_EVENT)
        {
        if(is_full_text_search_window_shown)
            {
            full_text_search_window->ReDraw();
            }
        }
    else if (event_int==Khz::my_wx_id__RECONFIGURE_STEMMERS_ORDER)
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "MyMainWindow redrawing stememrs\n";);
        m_left->ReDrawStemers();
        if(is_full_text_search_window_shown)
            {
            full_text_search_window->ReDrawStemmers();
            }
        }
    }


void MyMainWindow::OnGenBibtexWanted(wxCommandEvent& event)
    {
    Khz::BibTex::BibTexGeneratorStandaloneWindow *gen_bibtex_window= new Khz::BibTex::BibTexGeneratorStandaloneWindow(this);
    gen_bibtex_window->Show();
    }


void MyMainWindow::OnGenFullTextSearchIndexWanted(wxCommandEvent &event)
    {
    //we use separate executable to avoid freezing of the main window
    wxString curdir=wxGetCwd();
    wxDir temp_dir(curdir);

    wxString executable =
        "\""+temp_dir.GetNameWithSep() +  "KolkhoznikFullTextSearchIndexer\"" + " \"" + ConfigManager::GetPathToConfigFile()+"\"";
    KHZNIK_DEBUGMESSAGE(std::cout << "\nexecuting command: " << executable.ToStdString() << "\n";)
    wxExecute(executable);
    //FullTextSearchIndexerWindow *window = new FullTextSearchIndexerWindow(this, "Generate the full text search index");
    //window->Show();
    }

void MyMainWindow::OnFullTextSearchWanted(wxCommandEvent &event)
    {

    m_left->on_full_text_search_begin();  //makes search button look pressed.
    this->Refresh();
    this->Update();  //updates appearance of the main window, without it the search button will not look pressed.
    std::vector<wxString> temp_topics_array;
    m_left->GetArrayOfTopicsForFullTextSearch(temp_topics_array);
    if (!is_full_text_search_window_shown)
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "\nnot shown\n";)
        is_full_text_search_window_shown=true;
        full_text_search_window=new Khz::FTS::Gui::FullTextSearchWindow(this);
        full_text_search_window->Show();

        full_text_search_window->OnSearchWanted(m_left->GetFullTextSearchText(),
                                                    m_left->GetLanguageForFullTextSearch(), temp_topics_array);
        full_text_search_window->SetFocus();
        }
    else
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "\nis shown\n";)
        full_text_search_window->OnSearchWanted(m_left->GetFullTextSearchText(),
                                                    m_left->GetLanguageForFullTextSearch(), temp_topics_array);
        //bool is_maximized=full_text_search_window->IsMaximized();
        //full_text_search_window->Hide();
        //full_text_search_window->Show();
        //if(is_maximized) full_text_search_window->Maximize();
        full_text_search_window->Raise();
        }
    m_left->on_full_text_search_end(); //makes search button look Unpressed.
    this->Refresh();
    this->Update();
    }



void MyMainWindow::OnFullTextSearchWindowQuit(wxCommandEvent& event)
    {
    //std::cout << "MyMainWindow have closed the full_text_search_window" << "\n";
    full_text_search_window->Destroy();
    is_full_text_search_window_shown=false;
    }


}}// end of namespaces
