/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/aboutdlg.h>
#include <wx/settings.h>
#include "MyRightPanel.h"
#include <wx/fileconf.h>
#include <wx/stdpaths.h>
#include "ConfigManager.h"


namespace Khz{namespace MainWin{


MyRightPanel::MyRightPanel(wxWindow* parent)
:wxPanel(parent, wxID_ANY)
{
  // let us look into config file and get font size for list of books
  wxString ini_filename = ConfigManager::GetPathToConfigFile();
  wxFileConfig *config = new wxFileConfig( "", "", ini_filename);
  wxString books_list_font_size_in_config_file=config->Read("FONTS/books_list_font_size", "");
  wxString header_font_size_in_the_config_file=config->Read("FONTS/books_list_font_size_header", "");
  // got font size

wxFont font;

//now create class that will draw books list (right panel) (wnen searching and browsing topics, both)
m_item_list =new MyBooksList(this, wxID_ANY);
//books list creation finished...


//now we get default font of books list and put it into variable "font".
font=m_item_list->GetFont();


//if font size (got from config file) is not empty than set font size for books list.
if(books_list_font_size_in_config_file!="")
	{
	font.SetPointSize(wxAtoi(books_list_font_size_in_config_file));
	m_item_list->SetFont(font);
	}
//different font fo header is not working, i have no idea why.
/*for(unsigned int i=0;i< m_item_list->GetColumnCount();i++)
	{
	if(header_font_size_in_the_config_file!="")
		{
		wxListItem item;
		m_item_list->GetColumn(i, item);
		wxFont temp_font=m_item_list->GetFont();
		std::cout << "\nsettting new font for header\n";
		temp_font.SetPointSize(25);
		item.SetFont(temp_font);
		 item.SetImage(1);
		m_item_list->Refresh();
		m_item_list->SetColumn(i, item);
		m_item_list->Refresh();
		}
	}
*/
//font size has been set.


m_item_list->EnableAlternateRowColours();

    wxBoxSizer* sizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(m_item_list,1, wxEXPAND);
    this->SetSizer(sizer);
   // m_item_list->SetScrollRate(5, 5);   // i do not know why, but does not work in windows

delete config;
}
void MyRightPanel::DoSearch(wxString search_text,bool title_checked, bool author_checked, wxString search_type)
    {
    std::string fields_to_search;
    std::string search_text_std= search_text.ToUTF8().data();
    //std::cout << "\n\n in mycanvas search string " << search_text_std << "\n\n";
    std::string search_type_std=search_type.ToStdString();
    //std::cout << "\n\n in mycanvas search type" << search_type_std << "\n\n";


    if(title_checked && author_checked)
        {
        //We want letter case undependent search.
        //But sqlite does not support this (it does only if fields are in english), so
        //we have *LowerCase fields just for searching.
        fields_to_search= "AuthorLowerCase || TitleLowerCase"; //sqlite не может искать сase notsensetive with кириллицей
        }
    else if (title_checked)
        {
        fields_to_search="TitleLowerCase";
        }
    else if(author_checked)
        {
        fields_to_search="AuthorLowerCase";
        }
    else return;
    m_item_list->OnSearchWanted(search_text_std, search_type_std, fields_to_search);

    }
void MyRightPanel::ShowBooksInTopic(long int Topic_id)
    {
    m_item_list->OnBooksInTopicWanted(Topic_id);
    }
void MyRightPanel::Reconfigure()
    {
    m_item_list->Reconfigure();
    }
void MyRightPanel::JustReadConfigAgain()
    {
    m_item_list->JustReadConfigAgain();
    }
void MyRightPanel::RedrawList()
    {
    m_item_list->ReDrawList();
    }

void MyRightPanel::SetNewFontSize()
    {
    wxFont font = m_item_list->GetFont();

      // let us look into config file and get font size for list of books
    wxString ini_filename = ConfigManager::GetPathToConfigFile();
    wxFileConfig *config = new wxFileConfig( "", "", ini_filename);
    wxString books_list_font_size_in_config_file=config->Read("FONTS/books_list_font_size", "");
	wxString header_font_size_in_the_config_file=config->Read("FONTS/books_list_font_size_header", "");
    // got font size
    //if font size (got from config file) is not empty than set font size for books list.
	m_item_list->ReDrawList();  //without this windows does not change the font size of the columns headers, i have no idea why.
    if(books_list_font_size_in_config_file!="")
        {
        font.SetPointSize(wxAtoi(books_list_font_size_in_config_file));
        m_item_list->SetFont(font);
        }
    else
        {
        int default_font_size=wxSystemSettings::GetFont(wxSYS_SYSTEM_FONT).GetPointSize();
        font.SetPointSize(default_font_size);
        m_item_list->SetFont(font);
        }
/*	for(unsigned int i=0;i< m_item_list->GetColumnCount();i++)
	{
	if(header_font_size_in_the_config_file!="")
		{
		wxListItem item;
		m_item_list->GetColumn(i, item);
		wxFont temp_font=m_item_list->GetFont();
		temp_font.SetPointSize(wxAtoi(header_font_size_in_the_config_file));
		item.SetFont(temp_font);
		m_item_list->SetColumn(i, item);
		}
	}
	this->Update();
*/
    delete config;
    }


}} // end of namespaces
