/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "wx/scrolwin.h"
#include "MyBooksList.h"
#include <wx/sizer.h>
#include <wx/panel.h>



namespace Khz{namespace MainWin{



/**
@brief The right panel of the program with the books list, governs the MyBooksList class (se the detailed description of the class)

The actual drawing of the books list is made by the MyBooksList class.
The MyBooksList class is a very complex class, it have a lot of inner
methods and variables. So, for simplicity, we provide an interface for it. As a result, this class comes.
**/
class MyRightPanel : public wxPanel
{
public:
    /**
    Constructs the right panel
    **/
    MyRightPanel(wxWindow* parent);
//    MyListCtrl *m_list_ctrl;
    /**
    Searchs books with the provided search string
    @param text search string as it was entered into the search input (several vords divided by spaces)
    @param title_checked if we should search in the Title field
    @param author_checked if we should serch in the Author field
    @param search_type Search type can be "or", "and". User can enter
    several words into the search input. If search type is "or", than
    we select rows that have one of these words from the database. If
    search type is "and", than we select only rows that have all of the words.
    **/
    void DoSearch(wxString text,bool title_checked, bool author_checked, wxString search_type);

    /**
    Shows books associated with the particular topic
    @param Topic_id is the id of the  topic in the sqlite database.
    **/
    void ShowBooksInTopic(long int Topic_id);

    /**
        Call this function if you have changed the database path in the config file. This function will read the config file
        again, reconnect to the database and rebuild the books list
    **/
    void Reconfigure();

    /**
    Reads the config and updates the current config values.
    Call this function if you have changed application associated with some file extension in the Configuration dialog.
    @note
    When user doble clicks book, we should open it.
    So we look into config to find out if some application
    is associated with the extension of the file. If it is,
    we open file with this application. Without calling the
    JustReadConfigAgain function kolkhoznik will not know that you have changed the application in the config file.


    **/
    void JustReadConfigAgain();

    /**
        you should call this if you have changed icons or visible columns(Title, Author) of the right panel. This function will read the config file
        again and refresh the books list accordingly.
     **/
    void RedrawList();

    /**
    Reads the config file again and sets the font size for the books list
    **/
    void SetNewFontSize();

    /**
    The current "WHERE" statement that is used to get books from the "main" table of the database.
    The complete query to the database is:
    @code
    "SELECT * FROM main WHERE " + where_statement + "ORDER BY..."
    @endcode

    We use this function when generating a bibtex database for books
    "that are currently displayed on the right panel" (see tools->GenBibtex in the menubar).

    P.S. When generating the bibtex database we query the main kolkhoznik database
    to get book that we want to include to the bibtex database. It's where the "WHERE"
    statement comes.

    P.P.S. The WHERE statement could have placeholders.
    There is not need to use placeholders for integer values, so you should bind a string for each of
    the placeholders (use sqlite3_bind_text for it).
    See  #GetPlaceHoldersParameters().


    P.P.P.S. We do not have something like SetWhere(). The where statements is
    constructed automatically depending on current topic, search string.





    Just calls #m_item_list->GetWhere ().
    See #Khz::MainWin::MyBooksList::GetWhere() and #Khz::MainWin::MyBooksList::GetPlaceHoldersParameters()
    **/
    std::string GetWhere()
        {
        return m_item_list->GetWhere();
        }

    /**
    Returns placeholder parameters for the WHERE statement (see #GetWhere() )


    Just calls #m_item_list->GetPlaceHoldersParameters ()
    See #Khz::MainWin::MyBooksList::GetWhere() and #Khz::MainWin::MyBooksList::GetPlaceHoldersParameters()
    **/
    std::vector<std::string> GetPlaceHoldersParameters()
        {
        return m_item_list->GetPlaceHoldersParameters();
        }
   // virtual ~MyRightPanel(){};

  //  virtual void OnDraw(wxDC& dc);

private:
    /**
    As it was mentioned before, the actual drawing of the books list is made by the MyBooksList class,
    this class is just an interface.
    So we should have a MyBooksList class object
    **/
    MyBooksList *m_item_list;
//  wxDECLARE_NO_COPY_CLASS(MyRightPanel);
};



}} // end of namespaces
