/*
Copyright (C) Ivan Ivanov, 2014-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/settings.h>
#include "TreeCtrlLibrary.h"
#include "MyleftPanel.h"
#include <wx/srchctrl.h>
#include "Custom_IDs.h"
#include <wx/fileconf.h>
#include <wx/stdpaths.h>
#include "ConfigManager.h"
#include "DEBUGMESSAGE.h"
namespace Khz{namespace MainWin{




MyleftPanel::MyleftPanel(wxWindow *parent)
       : wxPanel(parent, wxID_ANY)
{
	// let us look into config file and get font size for library tree
	 wxString ini_filename = ConfigManager::GetPathToConfigFile();
	 wxFileConfig *config = new wxFileConfig( "", "", ini_filename);
	 wxString library_tree_font_size_in_config_file=config->Read("FONTS/library_tree_font_size", "");
	// got font size


    topic_search_dlg=new SearchForTopicDialog(this, wxID_ANY,"Search for a topic");




	wxFont font;


	// create notebook (it will consist of  two tabs on left panel)
    wxNotebook *notebook = new wxNotebook(this, wxID_ANY);

	//create search panel
    search_panel = new LeftSearchPanel(notebook);


	//we put topics_tree inside of it.
	//Without wxPanel one cannot set the background color of the topics_tree.
	wxPanel *panel_for_topics_tree = new wxPanel( notebook, wxID_ANY );
	//create topics tree (left panel)
    topics_tree  = new TreeCtrlLibrary(panel_for_topics_tree,  my_wx_id__TOPIC_WITH_BOOKS_WAS_SELECTED ,wxDefaultPosition, wxDefaultSize,wxTR_TWIST_BUTTONS | wxTR_NO_LINES| wxTR_HAS_BUTTONS);


	//now we get default font of topics tree and put it into variable "font".
	font=topics_tree->GetFont();

	//if font size (got from config file) is not empty than set font size for library tree.
	if( library_tree_font_size_in_config_file!="")
		{
		font.SetPointSize(wxAtoi(library_tree_font_size_in_config_file));
		topics_tree->SetFont(font);
		}
	//the font size has been set.



    //add two tabs
    notebook->AddPage(panel_for_topics_tree, wxT("Topics"));
    notebook->AddPage(search_panel, wxT("Search"));
    search_panel->SetBackgroundColour(*wxWHITE);
    panel_for_topics_tree->SetBackgroundColour(*wxWHITE);

    //sizings
    wxSizer *sizerV = new wxBoxSizer(wxVERTICAL);
    sizerV->Add(notebook, 1, wxEXPAND|wxALL);


    wxSizer *sizerV2 = new wxBoxSizer(wxVERTICAL);
    sizerV2->Add(topics_tree, wxSizerFlags(1).Expand());
    panel_for_topics_tree->SetSizerAndFit(sizerV2);

    SetSizerAndFit(sizerV);
    search_panel->GetSizer()->FitInside(notebook);
    search_panel->SetScrollRate(5, 5);
    search_panel->SetMinSize(wxSize(0,0));
    notebook->SetMinSize(wxSize(0,0));
    this->SetMinSize(wxSize(0,0));
    this->Fit();
	delete config;

	topic_search_dlg->Bind(wxEVT_BUTTON, &MyleftPanel::display_search_results_for_topic_selected_inside_of_SearchForTopicDialog, this, my_wx_id__SEARCH_FOR_TOPIC_DIALOG_ITEM_SELECTED);
    topics_tree->Bind( wxEVT_CHAR , &MyleftPanel::OnKeyPressedInsideOfTopicsTab, this);
    this->topics_tree->SetFocus();
}

wxString MyleftPanel::GetSearchText()
    {
    return search_panel->GetSearchText();
    }



bool MyleftPanel::IsAuthorChecked()
    {
    return search_panel->IsAuthorChecked();
    }

bool MyleftPanel::IsTitleChecked()
    {
    return search_panel->IsTitleChecked();
    }

wxString MyleftPanel::GetSearchType()
    {
    return search_panel->GetSearchType();
    std::cout << "\nserch type is " << search_panel->GetSearchType() << "\n";
    }
void MyleftPanel::UnselectAllTopics()
    {
    std::cout << "\nunselecting\n";
    //topics_tree->SelectItem (topics_tree->GetRootItem());
    topics_tree->UnselectAll();
    }

void MyleftPanel::ReconfigureLibraryTree()
    {
    topics_tree->Reconfigure();
    }
void MyleftPanel::SetNewLibraryTreeAppearance()
    {
    wxFont font = topics_tree->GetFont();

      // let us look into config file and get font size for list of books
    wxString ini_filename = ConfigManager::GetPathToConfigFile();
    wxFileConfig *config = new wxFileConfig( "", "", ini_filename);
    wxString books_list_font_size_in_config_file=config->Read("FONTS/library_tree_font_size", "");
    // got font size
    //if font size (got from config file) is not empty than set font size for books list.
    if(books_list_font_size_in_config_file!="")
        {
        font.SetPointSize(wxAtoi(books_list_font_size_in_config_file));
        topics_tree->SetFont(font);
        }
    else
        {
        int default_font_size=wxSystemSettings::GetFont(wxSYS_SYSTEM_FONT).GetPointSize();
        font.SetPointSize(default_font_size);
        topics_tree->SetFont(font);
        }
    //this will redraw library tree and rescale icons if necessary.
    topics_tree->ReDrawItems();

    delete config;
    }
void MyleftPanel::on_search_begin()
    {
    search_panel->on_search_begin();
    }
void MyleftPanel::on_search_end()
    {
    search_panel->on_search_end();
    }

void MyleftPanel::OnKeyPressedInsideOfTopicsTab(wxKeyEvent &event)
    {
    if( event.IsKeyInCategory(WXK_CATEGORY_NAVIGATION )
          or event.IsKeyInCategory(WXK_CATEGORY_TAB )
            or event.IsKeyInCategory(WXK_CATEGORY_CUT)
              or event.GetKeyCode()==	WXK_EXECUTE
                or event.GetKeyCode()==WXK_RETURN
      )
        {
        event.Skip();
        return;
        }
    KHZNIK_DEBUGMESSAGE(std::cout << "\n keycode is " << event.GetKeyCode() << "\n";)
    wxString m_char=(event.GetUnicodeKey ());
    topic_search_dlg->Show();
    topic_search_dlg->search_input->SetValue(m_char);
    topic_search_dlg->search_input->SetFocus();
    topic_search_dlg->search_input->SelectNone ();
    topic_search_dlg->search_input->SetInsertionPointEnd();
    event.StopPropagation();
    }


void MyleftPanel::display_search_results_for_topic_selected_inside_of_SearchForTopicDialog(wxCommandEvent &event)
    {
    //std::cout << "\n Display_search results event caught\n";
    wxString topic=event.GetString();
    //std::cout << "\n string is " << topic << "\n";
    topics_tree->ActivateTopicIfExists(topic);
    }
}} // end of namespaces
