/*
Copyright (C) Ivan Ivanov, 2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/




#include "SearchForTopicDialog.h"
#include <wx/sizer.h>
#include "MySizeFunctions.h"
#include "ConfigManager.h"
#include "clear_database_path.h"
#include "Custom_IDs.h"
#include "string_functions.h"
#include "DEBUGMESSAGE.h"
namespace Khz{namespace MainWin{



SearchForTopicDialog::SearchForTopicDialog(wxWindow *parent, wxWindowID id, wxString title, const wxPoint &pos, const wxSize &size)
:wxDialog (parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
    {
    this->display_search_results_callback=boost::bind(&Khz::MainWin::SearchForTopicDialog::display_search_results_callback_void, this);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //let us look into config file and connet to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

    ini_filename = ConfigManager::GetPathToConfigFile();
    config =new wxFileConfig( "", "", ini_filename);
	wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();
    sqlite3_open( temp_char_buffer, &db);
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //connected to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    parallel_thread_is_running_righ_now=false;
    a_search_was_skipped=false;


    search_input=new wxSearchCtrl(this, wxID_ANY);
    search_input->SetMinSize(wxSize(-1, 2*Khz::Libs::MySizeFunctions::get_height_of_string("A", this)));
    topics_list=new wxListBox(this, wxID_ANY);

    wxBoxSizer *main_sizer=new wxBoxSizer(wxVERTICAL);
        wxBoxSizer *inner_sizer=new wxBoxSizer(wxVERTICAL);
        inner_sizer->Add(search_input, 0, wxALL|wxEXPAND,0);
        inner_sizer->AddSpacer(5);
        inner_sizer->Add(topics_list, 1, wxALL|wxEXPAND,0);
    main_sizer->Add(inner_sizer, 1, wxALL|wxEXPAND,5);



    SetSizer(main_sizer);
    /////////////////////////////////////////////////////////////////
    //            set size of the window
    /////////////////////////////////////////////////////////////////
    int this_win_height = wxAtoi(
	config->Read("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_height",
             wxString::Format(wxT("%i"), Khz::Libs::MySizeFunctions::get_height_of_number_of_text_rows(30, this))    )
                                );

    int this_win_width = wxAtoi(
    config->Read("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_width",
             wxString::Format(wxT("%i"), Khz::Libs::MySizeFunctions::get_width_of_number_of_text_symbols(200, this))  )
                               );
    wxString is_maximized=config->Read("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_is_expanded", "no" );
    SetSize(this_win_width, this_win_height);
    if(is_maximized=="yes") this->Maximize();
    //////////////////////////////////////////////////////////////////
    //          finished with size
    //////////////////////////////////////////////////////////////////
    //main_sizer->FitInside(this);

    search_input->Bind(wxEVT_IDLE, &SearchForTopicDialog::OnIdle, this);
    search_input->Bind(wxEVT_TEXT,   &SearchForTopicDialog::OnSearchInputChanged, this);
    Bind(wxEVT_BUTTON, &SearchForTopicDialog::OnSearchFinished, this, my_wx_id__SEARCH_FOR_TOPIC_FINISHED);
    topics_list->Bind(wxEVT_LISTBOX_DCLICK,  &SearchForTopicDialog::OnTopicActivated, this);

    //something is being resized
    Bind(wxEVT_SIZE, &SearchForTopicDialog::OnResize, this);
    Bind(wxEVT_MAXIMIZE, &SearchForTopicDialog::OnMaximize, this);
    topics_list->Bind( wxEVT_LEFT_UP, &SearchForTopicDialog::OnTopicClickedByMouse, this);
    Bind(wxEVT_CHAR_HOOK, &SearchForTopicDialog::OnKeyPressed, this);
    }

void SearchForTopicDialog::MainSearcherParallelThreadInternal
                    (
                    std::string search_string,
                    boost::function<void()> &finished_callback
                    )
    {
    try
        {
        //KHZNIK_DEBUGMESSAGE
        //        (
        //        // simulating thread hight load
        //        boost::this_thread::sleep( boost::posix_time::seconds(2) );
        //        )
        using namespace boost;
        search_string=Khz::Libs::Mstr::to_lower_case(search_string);

        //The search string consists of words divided by spaces.
        //Let us make an array of these (key)words.
        //byt firstly one needs to remove spaces in beginning and end of the search string and
        //replaces myltiply spaces between words by a single space.
        std::vector<std::string> keywords;
        keywords=Khz::Libs::Mstr::explode(" ",Khz::Libs::Mstr::trim(regex_replace(search_string, regex("\\s+", regex_constants::icase)," ")));



        //here we are preparing an array with equal strings, THE SIZE OF THE ARRAY IS EQUAL TO THE NUMBER OF KEYWORDS.
        std::vector<std::string> db_query_condition;
        for(unsigned int i=0;i<keywords.size(); i++)
            {
            db_query_condition.push_back("Topic LIKE ?");
            }



        //now the magic begins. Let us to prepapare the query to the database (string).
        //query part is in fact the WHERE statement for the query. (I.E. the query is something like
                                                                //SELECT ... FROM ... WHERE + query_part +
        std::string query_part = Khz::Libs::Mstr::implode(" AND ", db_query_condition);

        //And finally the database query
        sqlite3_stmt *stmt = NULL;
        std::string query_with_placeholders = "SELECT * FROM topics WHERE ("+query_part+") ORDER BY Topic COLLATE NOCASE;";

        sqlite3_prepare_v2( db, query_with_placeholders.c_str(), -1, &stmt, NULL );
        for(unsigned int i=0; i< keywords.size(); i++)
            {
            sqlite3_bind_text(stmt,  i+1, ("%"+keywords[i]+"%").c_str(), -1, SQLITE_TRANSIENT);
            }
        sqlite3_step( stmt ); //executing the statement
        this->current_result = Khz::Libs::DB::dbcontent(stmt) ;
        sqlite3_finalize(stmt);

        finished_callback();
        }
    catch(...)
        {
        this->current_result={};
         finished_callback();
        }
    }


void SearchForTopicDialog::InitSearch
                    (   std::string search_string
                    )
    {
    if(parallel_thread_is_running_righ_now)
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "\n thread busy\n";)
        a_search_was_skipped=true;
        return;
        }
    parallel_thread_is_running_righ_now=true;
    a_search_was_skipped=false;
    boost::thread
            (
            &Khz::MainWin::SearchForTopicDialog::MainSearcherParallelThreadInternal,
            this,
            search_string,
            display_search_results_callback
            );
    }




void SearchForTopicDialog::display_search_results_callback_void()
    {
    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__SEARCH_FOR_TOPIC_FINISHED);
    this->GetEventHandler()->QueueEvent 	( 	event_to_propagate.Clone()	) ;
    }


void SearchForTopicDialog::OnSearchFinished(wxCommandEvent &event)
    {
    topics_list->Clear();
    for(unsigned int i=0;i< current_result.size();i++)
        {
        wxString temp_str=current_result[i]["Topic"];
        if(temp_str=="") temp_str="Library"; // by default the top level topic is "" (empty string).
                                             // but this does not look good, so we change the string
        topics_list->Append(temp_str);
        }
    if( topics_list->GetCount()!=0)
        {
        topics_list-> SetCursor(wxCursor(wxCURSOR_HAND));
        }
    else
        {
        topics_list-> SetCursor(wxCursor(wxCURSOR_ARROW));
        }
    parallel_thread_is_running_righ_now=false;
    }
void SearchForTopicDialog::OnIdle(wxIdleEvent &event)
    {
    if(a_search_was_skipped )
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "\n SearchForTopicDialog on idle \n";)
        InitSearch(search_input->GetValue().ToUTF8().data());
        }
    }

void SearchForTopicDialog::OnSearchInputChanged(wxCommandEvent &event)
    {
    InitSearch(search_input->GetValue().ToUTF8().data());
    }

void  SearchForTopicDialog::OnTopicActivated(wxCommandEvent &event)
    {
    wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
    event_to_propagate.SetId(my_wx_id__SEARCH_FOR_TOPIC_DIALOG_ITEM_SELECTED);
    wxString selected_topic=topics_list->GetString(topics_list->GetSelection());
    if(selected_topic=="Library") selected_topic=""; // by default the top level topic is "" (empty string)
                                                     // but inside of OnSearchFinished we changed "" to "Library"
                                                     // for aesthetic purposes. Now we change it back.
    event_to_propagate.SetString(selected_topic);
    ProcessWindowEvent 	( 	event_to_propagate	) ;
    topics_list->Clear();
    search_input->ChangeValue("");
    topics_list-> SetCursor(wxCursor(wxCURSOR_ARROW));
    this->Hide();
    }



//remembers window size (puts data to the config file
void SearchForTopicDialog::OnResize(wxSizeEvent &event)
    {
    std::cout << "\nonresize catched\n";
        delete config; // without this changes made by other classes that use the same config file will not be saved.
        config =new wxFileConfig( "", "", ini_filename);
        int width;
        int height;
        this->GetSize (&width, &height);
        if (!this->IsMaximized())
            {
            config->Write( wxT("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_width"),wxString::Format(wxT("%i"),width) );
            config->Write( wxT("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_height"), wxString::Format(wxT("%i"),height) );
            config->Write( wxT("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_is_expanded"), "no" );
            }
        config->Flush();
        event.Skip();
        //testconfig->Write( wxT("SEARCH_FOR_TOPIC_DIALOG/is_expanded"), "" );
    }

//remembers that window should be maximized
void SearchForTopicDialog::OnMaximize(wxMaximizeEvent& event)
    {
    delete config; // without this changes made by other classes that use the same config file will not be saved.
    config =new wxFileConfig( "", "", ini_filename);

    if (!this->IsMaximized())
        {
        config->Write( wxT("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_is_expanded"), "no" );
        }
    else
        {
        config->Write( wxT("SEARCH_FOR_TOPIC_DIALOG/search_for_topic_window_is_expanded"), "yes" );
        }
     config->Flush();
    event.Skip();
    }

void SearchForTopicDialog::OnTopicClickedByMouse(wxMouseEvent& event)
    {
    int x;
    int y;
    event.GetPosition(&x,&y);
    int clicked_item=topics_list->HitTest (x,y);
    if(clicked_item!=wxNOT_FOUND)
        {
        wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
        event_to_propagate.SetId(my_wx_id__SEARCH_FOR_TOPIC_DIALOG_ITEM_SELECTED);
        wxString selected_topic=topics_list->GetString(topics_list->GetSelection());
        if(selected_topic=="Library") selected_topic=""; // by default the top level topic is "" (empty string)
                                                         // but inside of OnSearchFinished we changed "" to "Library"
                                                         // for aesthetic purposes. Now we change it back.
        event_to_propagate.SetString(selected_topic);
        ProcessWindowEvent 	( 	event_to_propagate	) ;
        topics_list->Clear();
        search_input->ChangeValue("");
        topics_list-> SetCursor(wxCursor(wxCURSOR_ARROW));
        this->Hide();
        }
    }

void SearchForTopicDialog::OnKeyPressed(wxKeyEvent &event)
    {
    if(event.GetKeyCode()==WXK_ESCAPE)
        {
        this->Hide();
        }
    else
        {
        event.Skip();
        }
    }
}}// namespaces
