/*
Copyright (C) Ivan Ivanov, 2014-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "sqlite3.h"
#include <wx/icon.h>
#include <wx/imaglist.h>
#include <wx/fileconf.h>
#include <wx/stdpaths.h>
#include "Mdb_fuctions.h"
#include "string_functions.h"
#include "TreeCtrlLibrary.h"
#include <wx/artprov.h>
#include "clear_database_path.h"
#include "Custom_IDs.h"
#include <wx/filename.h>
#include "ConfigManager.h"
#include "DEBUGMESSAGE.h"




namespace Khz{namespace MainWin{






TreeCtrlLibrary::~TreeCtrlLibrary()
    {
    delete config;
    sqlite3_close(db);
    delete images_list;
    }
TreeCtrlLibrary::TreeCtrlLibrary(wxWindow *parent, const wxWindowID id,
                       const wxPoint& pos, const wxSize& size,
                       long style)
          : wxTreeCtrl(parent, id, pos, size, style)
{
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //let us look into config file and connet to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

    ini_filename = ConfigManager::GetPathToConfigFile();
    config =new wxFileConfig( "", "", ini_filename);
	wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();
    sqlite3_open( temp_char_buffer, &db);
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //connected to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


    // we get icons size from config file and set images list
    wxString icons_size_in_config = config->Read("ICONS/library_tree_icons_size","16");
    if(icons_size_in_config=="") icons_size_in_config="16";
    this->SetImages(wxAtoi(icons_size_in_config));  //icon_num is set here
    //ok, finished with icons


    //add root item
    AddRoot ("Library",this->icon_num,this->icon_num);



    //root item data, the first value is the topic(empty for root item), the second
    // value is the id of the topic in the database(for the root the topic id is always 1)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    MywxTreeItemData *RootItemData=new MywxTreeItemData("",1);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //sets item data for root item
    SetItemData (this->GetRootItem(), RootItemData);

    //adds subtopics to the root item (like Physics, Mathematics and so on)
    AppendSubtopicsToItem(this->GetRootItem ()); //add topics to root (empty) topic


    //bindings
    Bind(wxEVT_TREE_ITEM_EXPANDING, &TreeCtrlLibrary::OnItemExpanding, this);
    Bind(wxEVT_TREE_SEL_CHANGED , &TreeCtrlLibrary::OnSelChanged, this);
   // Bind(wxEVT_LEFT_DOWN , &TreeCtrlLibrary::Expand_Collapse_item_onclick, this);
    Bind(wxEVT_LEFT_DOWN , &TreeCtrlLibrary::FireAdditionalShowBooksInTopicEvent, this);
    Bind(wxEVT_CHAR_HOOK , &TreeCtrlLibrary::OnKeyPressed, this);

    //expands root item
    Expand(this->GetRootItem());


    //selects root item.
    this->UnselectAll();
    SelectItem (this->GetRootItem());

    //see docs for more info.
	this->block_next_selection_event=false;
	this->SetBackgroundColour(*wxWHITE);



};



MywxTreeItemData::MywxTreeItemData(wxString topic, long int id)
    {
    AssociatedTopic=topic;
    AssociatedID=id;
    }



void TreeCtrlLibrary::AppendSubtopicsToItem(const wxTreeItemId &parent)
{
Khz::Libs::DB::mapDB result;
try
    {
     if(GetChildrenCount (parent, false)!=0) return; //subtopics already there
    std::string parent_topic=((MywxTreeItemData *)(GetItemData(parent)))->AssociatedTopic.ToUTF8().data();
    std::string query="select * from topics where Topic like ? and substr(Topic, length(?)+1 ) NOT LIKE ?;";
                //std::cout << "\n query is " << query << "\n";
    //std::string query="select * from topics;";
    //select Topic from topics where Topic like '/B_Biology/%' and substr(Topic, length('/B_Biology/')+1 ) NOT LIKE '%/%';
    // we have to select topics that are like current topic and have number of symbols "/" eqal to coorresponding number of current topic +1
    std::string separator= "/";
    sqlite3_stmt *stmt = NULL;
    int retval=sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
                //std::cout << "\n" << sqlite3_errmsg(db) << "\n";
     if(retval) throw(std::string("cannot connect to db in TreeCtrlLibrary.cpp"));
        sqlite3_bind_text(stmt,  1, (parent_topic+separator+"%").c_str(), -1, SQLITE_TRANSIENT);
                        //std::cout << "\nstmt1 " <<"'"<< (parent_topic+separator+"%").c_str()<<"'"<< "\n";
        sqlite3_bind_text(stmt,  2, (parent_topic+separator).c_str(), -1, SQLITE_TRANSIENT);
                        //std::cout << "\nstmt2 " <<"'"<< (parent_topic+separator).c_str() <<"'" << "\n";
        sqlite3_bind_text(stmt,  3, ("%"+separator+"%").c_str(), -1, SQLITE_TRANSIENT);
                        //std::cout << "\nstmt3" <<"'"<< ("%"+separator+"%").c_str() <<"'"<< "\n";
                        //std::cout << "\n" << sqlite3_errmsg(db) << "\n";
    sqlite3_step( stmt );
                        //std::cout << "\n" << sqlite3_errmsg(db) << "\n";
     result = Khz::Libs::DB::dbcontent(stmt);
                        //std::cout << "\n got result i\n";
                        //std::cout << "\n size of output in tree is "<< result.size();
                        //std::cout << "\n zero topic is " << result[0]["Topic"] << "\n";
         sqlite3_finalize(stmt);

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    //sort result by insertion algorithm, sort by topic name(visible)
    for(unsigned int i=1; i<result.size() ;i++)
        for(int    j=i;    j>0   && GetTreeTextFromTopic(result[j-1]["Topic"])    >     GetTreeTextFromTopic(result[j]["Topic"]);  j--) // пока j>0 и элемент j-1 > j, x-массив int
            std::swap(result[j-1],result[j]);        // меняем местами элементы j и j-1
    //sorting finished (successfully i hope)
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



    }
catch(...)
        {
        //result= {};
        }
    //std::cout << "\n result size is " << result.size() << "\n";

    /*  subtopic "General" is not needed
    if(result.size()!=0)
        {
        wxTreeItemId id= AppendItem (parent, "General",0,0);

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        long int topic_id = ((MywxTreeItemData *)(GetItemData(parent)))->AssociatedID;
        SetItemData (id, new MywxTreeItemData("/____huinya_ebuchaya_____", topic_id));  //we should chose name that cannot be  within the topics names.
                                                                                // /____huinya_ebuchaya_____" surely meets this requirement (sorry for bad language)
        }
    */
    for(unsigned int i=0;i<result.size();i++)
        {
        wxTreeItemId id= AppendItem (parent, GetTreeTextFromTopic(result[i]["Topic"]),0,0);

        //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        long int topic_id = atoi((result[i]["ID"]).c_str());
        SetItemData (id, new MywxTreeItemData(wxString(result[i]["Topic"]), topic_id));
        }
}



wxString TreeCtrlLibrary::GetTreeTextFromTopic(std::string topic_std)
    {
    std::vector<std::string> exploded_topic= Khz::Libs::Mstr::explode("/", topic_std);
    //std::vector<std::string> secondary_exploded_topic= Mstr::explode("_", exploded_topic[exploded_topic.size()-1]);
    //if (secondary_exploded_topic.size()==2)
     //   {
      //  return wxString(secondary_exploded_topic[1]);
     //   }
    //else
    //    {
        return wxString(exploded_topic[exploded_topic.size()-1]);  // last
      //  }
    }


void TreeCtrlLibrary::OnSelChanged(wxTreeEvent &event)
    {

    wxTreeItemId id=event.GetItem();

    if ( IsSelected(id) )
        {
        if(block_next_selection_event)
            {
            block_next_selection_event=false;
            }
        else
            {
            int topic_id=((MywxTreeItemData *)(GetItemData(id)))->AssociatedID;
            wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
            event_to_propagate.SetId(my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT);
            event_to_propagate.SetInt(topic_id);
            ProcessWindowEvent 	( 	event_to_propagate	) ;
            }

        }
    event.Skip();

    }
void TreeCtrlLibrary::OnItemExpanding(wxTreeEvent& event)
    {
        wxTreeItemId id=event.GetItem();
        size_t childs_number= GetChildrenCount (id, false);// false means not recursively.
        if(childs_number!=0)
            {
            wxTreeItemIdValue cookie;
            wxTreeItemId child_id = GetFirstChild(id, cookie);
            if(GetChildrenCount(child_id, false)==0) AppendSubtopicsToItem(child_id);
            for(;;)
                {
                child_id = GetNextChild(id, cookie);
                    if(child_id)
                        {
                        if(GetChildrenCount(child_id, false)==0)  AppendSubtopicsToItem(child_id);
                        }
                    else
                        {
                        break;
                        }
                }
            }
    }

long int TreeCtrlLibrary::GetTopic(wxTreeItemId id)
    {
   // if(GetChildrenCount(id,false)==0)
     //   {
        return ((MywxTreeItemData *)(GetItemData(id)))->AssociatedID;
   //     }
    //else
    //    {
      //  return -((MywxTreeItemData *)(GetItemData(id)))->AssociatedID; //everything from here is already in "Unsorted";
       // }

    }


wxString TreeCtrlLibrary::GetTopicString(wxTreeItemId id)
    {
    return ((MywxTreeItemData *)(GetItemData(id)))->AssociatedTopic;
    }

void TreeCtrlLibrary::SetImages(int size) //i have no idea how this works, the code is taken from wxwidgest samples
{
    if ( size == -1 )
    {
        images_list = new wxImageList(1,1, true);
        //images_list exist but not used. It should exist due to the fact that
        //when we change image we delete all imagelist and create new.
        //If we call "delete" on non-existing object than bad things can happen.
        //See TreeCtrlLibrary::ReBuildIMageList to understand why we need image_list
        //in case we do not use icons.
        SetImageList(NULL);
        icon_num=-1;
        return;
    }


    icon_num=0;
    // Make an image list containing small icons
    images_list = new wxImageList(size, size, true);

    // should correspond to TreeCtrlIcon_xxx enum
    wxBusyCursor wait;
    wxIcon icons[1];
    {
        wxSize iconSize(size, size);

       // icons[0] = wxArtProvider::GetIcon(wxART_FOLDER, wxART_LIST, iconSize);



    wxString size_folder= wxString::Format(wxT("%i"),size)+"x"+wxString::Format(wxT("%i"),size);
    wxString search_folder=wxString("icons")+wxFileName::GetPathSeparator()+size_folder;
    wxString temp_string;
    if(wxDirExists(search_folder))
        {
        temp_string=wxString("icons")+wxFileName::GetPathSeparator()+size_folder+wxFileName::GetPathSeparator();
        }
    else
        {
        temp_string=wxString("icons")+wxFileName::GetPathSeparator()+"48x48"+wxFileName::GetPathSeparator();
        }





       if(size<=16)
        {
		icons[0].LoadFile(temp_string+"folder.png", wxBITMAP_TYPE_PNG);
        }
    else
        {
        icons[0].LoadFile(wxT("icons")+wxString(wxFileName::GetPathSeparator())+"48x48"+wxString(wxFileName::GetPathSeparator())+"folder.png", wxBITMAP_TYPE_PNG);

        }
    }

    for ( size_t i = 0; i < WXSIZEOF(icons); i++ )
    {
        int sizeOrig = icons[0].GetWidth();
        if ( size == sizeOrig )
        {
            images_list->Add(icons[i]);
        }
        else
        {
            images_list->Add(wxBitmap(wxBitmap(icons[i]).ConvertToImage().Rescale(size, size)));
        }
    }

    SetImageList(images_list);
}

/*void TreeCtrlLibrary::Expand_Collapse_item_onclick(wxMouseEvent &event)
    {
    //std::cout << "got event mouse\n";
    wxPoint mouse_position=event.GetPosition();
    int temp_num=wxTREE_HITTEST_ONITEMLABEL;

    wxTreeItemId id=HitTest(mouse_position, temp_num);

    if(id)
        {
       Toggle(id);
		UnselectAll();
        SelectItem 	(id,true);
		SetFocusedItem(id);
       return;
        }
    event.Skip();
    }
*/
void TreeCtrlLibrary::FireAdditionalShowBooksInTopicEvent(wxMouseEvent &event)
//when user press "Search" buttom current topic remains selected.
//when user returns to the topics tree and clicks on ALREADY SELECTED ITEM  this
// funcion fires an event. The MyMainWindow class catches this event and shows books in this topic.

//P.S. OnSelChanged function also sends event to MyMainWindow to show books in topic.
    {
    wxPoint mouse_position=event.GetPosition();
    int temp_num=wxTREE_HITTEST_ONITEMLABEL;

    wxTreeItemId id=HitTest(mouse_position, temp_num);

    if(id)
        {

        if( !IsSelected ( id ) )
            {
            event.Skip();
            return;
            }


        int topic_id=((MywxTreeItemData *)(GetItemData(id)))->AssociatedID;
		wxCommandEvent event_to_propagate=wxCommandEvent( wxEVT_BUTTON);
		event_to_propagate.SetId(my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT);
		event_to_propagate.SetInt(topic_id);
		ProcessWindowEvent 	( 	event_to_propagate	) ;
		event.Skip();
        }
    event.Skip();
    }


void TreeCtrlLibrary::Reconfigure()
    {
    DeleteAllItems();
    delete config;
    sqlite3_close(db);

    config =new wxFileConfig( "", "", ini_filename);
	wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();
    sqlite3_open( temp_char_buffer, &db);
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);



    AddRoot ("Library",0,0);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    MywxTreeItemData *RootItemData=new MywxTreeItemData("",1);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    SetItemData (this->GetRootItem(), RootItemData);
    AppendSubtopicsToItem(this->GetRootItem ()); //add topics to root (empty) topic
    Expand(this->GetRootItem());



    this->UnselectAll();
    SelectItem (this->GetRootItem());

    }
/*void TreeCtrlLibrary::SetNewFont(wxFont font)
    {
    wxTreeItemId  root_item=this->GetRootItem();
    if(!root_item.IsOk()) return;

    My_SetNewFontRecursively(font, root_item);
    }
*/
void TreeCtrlLibrary::FindExpandedItems(wxTreeItemId item, std::vector<std::pair<int,int>> path )
    {
    size_t childs_number= GetChildrenCount (item, false);// false means not recursively.
    int counter=0;
    if(childs_number!=0)
        {
        wxTreeItemIdValue cookie;
        wxTreeItemId child_id = GetFirstChild(item, cookie);
        if(IsExpanded(child_id))
            {
            std::vector<std::pair<int,int>> temp_vector=path;
            if(IsSelected(child_id)) remember_selected_item(temp_vector,0);
            temp_vector.push_back(std::make_pair(0,1));
            FindExpandedItems(child_id, temp_vector );
            expanded_items_remember_vector.push_back(temp_vector);
            }
        else
            {
            std::vector<std::pair<int,int>> temp_vector=path;
            if(IsSelected(child_id)) remember_selected_item(temp_vector,0);
            temp_vector.push_back(std::make_pair(0,0));
            FindExpandedItems(child_id, temp_vector );
            }
        if(child_id.IsOk())
            {
            for(;;)
                {
                counter++;
                child_id = GetNextChild(item, cookie);
                    if(child_id.IsOk())
                        {
                        if(IsExpanded(child_id))
                            {
                            std::vector<std::pair<int,int>> temp_vector=path;
                            if(IsSelected(child_id)) remember_selected_item(temp_vector,counter);
                            temp_vector.push_back(std::make_pair(counter,1));
                            FindExpandedItems(child_id, temp_vector );
                            expanded_items_remember_vector.push_back(temp_vector);
                            }
                         else
                            {
                            std::vector<std::pair<int,int>> temp_vector=path;
                            if(IsSelected(child_id)) remember_selected_item(temp_vector,counter);
                            temp_vector.push_back(std::make_pair(counter,0));
                            FindExpandedItems(child_id, temp_vector );
                            }
                        }
                    else
                        {
                        break;
                        }
                }
            }
        }
    }

void TreeCtrlLibrary::ReExpandItems(wxTreeItemId item, std::vector<std::pair<int,int>> path )
    {
    if (path.size()==0) return;
    size_t childs_number= GetChildrenCount (item, false);// false means not recursively.
    if(childs_number==0) AppendSubtopicsToItem(item);
    int counter=0;
    if(0==0)
        {
        wxTreeItemIdValue cookie;
        wxTreeItemId child_id = GetFirstChild(item, cookie);
        if(path[0].first==0)
            {
            if(GetChildrenCount (child_id, false)==0) AppendSubtopicsToItem(child_id);
            std::vector<std::pair<int,int>> temp_vector=path;
            temp_vector.erase( temp_vector.begin() );
            ReExpandItems(child_id, temp_vector );
            if(path[0].second==1) Expand(child_id);
            }
        if(child_id.IsOk())
            {
            for(;;)
                {
                counter++;
                child_id = GetNextChild(item, cookie);
                    if(child_id.IsOk())
                        {
                        if(counter==path[0].first)
                            {
                            if(GetChildrenCount (child_id, false)==0) AppendSubtopicsToItem(child_id);
                            std::vector<std::pair<int,int>> temp_vector=path;
                            temp_vector.erase(temp_vector.begin());
                            ReExpandItems(child_id, temp_vector );
                            if(path[0].second==1) Expand(child_id);
                            }
                        }
                    else
                        {
                        break;
                        }
                }
            }
        }
    }

void TreeCtrlLibrary::ReDrawItems()
    {

    block_next_selection_event=true;  //without this right panel will be refreshed, but we do not want this; see OnSelChanged;
    selected_item_remember_vector.clear();
    expanded_items_remember_vector.clear();
    std::vector<std::pair<int,int>> temp_vec;
    FindExpandedItems(this->GetRootItem(), temp_vec ); //fills expanded_items_remember_vector;
    DeleteAllItems();
    std::cout << "\n\n what expanded \n";
    for(unsigned int i=0;i<expanded_items_remember_vector.size();i++)
        {
        for(unsigned int j=0;j<expanded_items_remember_vector[i].size();j++)
            {
            std::cout << expanded_items_remember_vector[i][j].first << "~" << expanded_items_remember_vector[i][j].second << "  ";
            }
        std::cout << "\n";
        }

    std::cout << "\n selected item \n";
    for(unsigned int j=0;j<selected_item_remember_vector.size();j++)
            {
            std::cout << selected_item_remember_vector[j] << "  ";
            }


    delete config;
    config =new wxFileConfig( "", "", ini_filename);


    wxString icons_size_in_config = config->Read("ICONS/library_tree_icons_size","16");
    if(icons_size_in_config=="") icons_size_in_config="16";



    this->ReBuildIMageList(wxAtoi(icons_size_in_config));  //icon_num is set here
    AddRoot ("Library",this->icon_num,this->icon_num);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    MywxTreeItemData *RootItemData=new MywxTreeItemData("",1);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    SetItemData (this->GetRootItem(), RootItemData);
    AppendSubtopicsToItem(this->GetRootItem ()); //add topics to root (empty) topic
    Expand(this->GetRootItem());
    this->UnselectAll();
    for(unsigned int i=0;i<expanded_items_remember_vector.size();i++)
        {
        ReExpandItems(this->GetRootItem(), expanded_items_remember_vector[i] );
        }
    ReStoreSelection(this->GetRootItem(), selected_item_remember_vector);
    }



void TreeCtrlLibrary::ReBuildIMageList(int new_size)
    {
    SetImageList(NULL);
    delete images_list;
    //wxImageList *temp_pointer= images_list;  //temp pointer gets adress of images_list
    SetImages(new_size);  // images_list=new wxImageList();  So, images_list is different pointer now.
    //delete temp_pointer; //remove old image list.
    }




void TreeCtrlLibrary::ReStoreSelection(wxTreeItemId item, std::vector<int> path)
    {
    if(path.size()==0) return;
    int counter=0;
    if(0==0)
        {
        wxTreeItemIdValue cookie;
        wxTreeItemId child_id = GetFirstChild(item, cookie);
        if(path[0]==0)
            {
            std::vector<int> temp_vector=path;
            temp_vector.erase( temp_vector.begin() );
            if(temp_vector.size()==0) SelectItem(child_id);
            ReStoreSelection(child_id, temp_vector );
            }
        if(child_id.IsOk())
            {
            for(;;)
                {
                counter++;
                child_id = GetNextChild(item, cookie);
                    if(child_id.IsOk())
                        {
                        if(counter==path[0])
                            {
                            std::vector<int> temp_vector=path;
                            temp_vector.erase(temp_vector.begin());
                            if(temp_vector.size()==0) SelectItem(child_id);
                            ReStoreSelection(child_id, temp_vector );
                            }
                        }
                    else
                        {
                        break;
                        }
                }
            }
        }
    }


void TreeCtrlLibrary::remember_selected_item(std::vector<std::pair<int,int>> temp_vector,int counter)
    {
    for(unsigned int i=0;i<temp_vector.size();i++)
        {
        selected_item_remember_vector.push_back(temp_vector[i].first);
        }
    selected_item_remember_vector.push_back(counter);
    }



void TreeCtrlLibrary::ActivateTopicIfExists(wxString topic)
    {
    std::string topic_std=topic.ToStdString();
    // topic is something like "/Physics/School-level/Perelman"
    std::vector<std::string> exploded_topic_std=Khz::Libs::Mstr::explode("/", topic_std);
    ActivateTopicIfExists(exploded_topic_std, GetRootItem(), 0);
    }

void TreeCtrlLibrary::ActivateTopicIfExists(std::vector<std::string> &exploded_topic, wxTreeItemId 	id, int depth)
    {
    KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: calling activate topic if exist, depth is " << depth << "\n";)
    wxString full_topic(Khz::Libs::Mstr::implode("/", exploded_topic));
    KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: full topic is " << full_topic << "\n";)
    if(this->GetTopicString (id)==full_topic)
        {
        KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: found root " << full_topic << "\n";)
        EnsureVisible (id);
        SelectItem(id);
        //##################
        // RETURN HERE!!!!
        return; // !!!!!!!!!!
        ///////////////////////
        }


    std::vector<std::string> temp_vec;
    if(  (depth+2)  >   ((int)(exploded_topic.size()))    ) return; // just in case
    for (int i=0;i<depth+2;i++)
        {
        temp_vec.push_back(exploded_topic[i]);
        }
    wxString topic_to_search_on_this_level(Khz::Libs::Mstr::implode("/", temp_vec));
    KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: topic_to_search_on_this_level " << topic_to_search_on_this_level << "\n";)


    size_t childs_number= GetChildrenCount (id, false);// false means not recursively.
    KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: childs num is  " << childs_number << "\n";)
    if(childs_number!=0)
        {
        wxTreeItemIdValue cookie;
        wxTreeItemId child_id = GetFirstChild(id, cookie);
        if(this->GetTopicString (child_id)==full_topic)
            {
            KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: found in first child " << full_topic << "\n";)
            EnsureVisible (child_id);
            SelectItem(child_id);
            return;
            }
        if(this->GetTopicString (child_id)==topic_to_search_on_this_level)
            {
            if(!IsExpanded(child_id)) Expand(child_id);
            ActivateTopicIfExists(exploded_topic, child_id, depth+1);
            return;
            }
        for(;;)
            {
            KHZNIK_DEBUGMESSAGE(std::cout << "\n ActivateTopicIfExists: cycling " << full_topic << "\n";)
            child_id = GetNextChild(id, cookie);
                if(child_id)
                    {
                    if(this->GetTopicString (child_id)==full_topic)
                        {
                        EnsureVisible (child_id);
                        SelectItem(child_id);
                        return;
                        }
                    if(this->GetTopicString (child_id)==topic_to_search_on_this_level)
                        {
                        if(!IsExpanded(child_id)) Expand(child_id);
                        ActivateTopicIfExists(exploded_topic, child_id, depth+1);
                        return;
                        }
                    }
                else
                    {
                    break;
                    }
            }
        }
    }

void TreeCtrlLibrary::OnKeyPressed(wxKeyEvent &event)
    {
    if(event.GetKeyCode()==WXK_RETURN) // enter pressed
        {
        if(GetChildrenCount (this->GetSelection(), false)!=0 )
            {
            this->Expand(this->GetSelection());
            }
        }
    else
        {
        event.Skip();
        return;
        }
    }
}} // end of namespaces
