/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include<map>
#include "Mdb_fuctions.h"





namespace Khz{

/**
@brief Classes to form the kolkhoznik main window (consisting of two panels)
**/
namespace MainWin{





/**
@brief Books cache for the MyBooksList class. See description of the MyBooksList class

Cache for the MyBooksList class. See description of the MyBooksList class.
**/
class books_cache
{
public:



    /**
    Puts data to cache. Queries database in order to find the search string.
    @param db sqlite3 database
    @param dbtable database table
    @param search_string search string as it was entered into the search input (several vords divided by spaces)
    @param num_first first item to cache. Counting starts from zero. The parameter must be <= MyBooksList.full_rows_number.
    @param num_last last item to cache. The parameter must be <= MyBooksList.full_rows_number.
    @param search_type Search type can be "or", "and". User can enter several words into the search input.
        If search type is "or", than we select rows that have one of these words from the database.
        If search type is "and", than we select only rows that have all of the words.
    @param fields_to_search fields to search (columns in the database)
        as it should be inserted into the sql query. For example, valid values are
        "TitleLowerCase", "AuthorLowerCase", "AuthorLowerCase || TitleLowerCase". So, if searching should
        be done in several fields, than these fields should be splitted by a "||" symbol.
    @param order Sorting order, for example "ORDER by AuthorLowerCase" (order goes into the sql query directly).
    @note Let us note that sqlite does not support a case-insensitive search for non-latin characters.
    To enable such search, we have columns like "TitleLowerCase". When searching, we transform a search
    string to the lowercase one, than search in the *LowerCase column.

    **/
    void put_into_cache(sqlite3 *db, std::string dbtable, std::string search_string,long num_first,long num_last, std::string search_type,
                          std::string fields_to_search, std::string order);



    /**
    Puts data to cache. Queries database in order to find books within the particular topic.
    @param db sqlite3 database
    @param dbtable database table
    @param Topic_id id of the topic in the database ("topics" table)
    @param num_first first item to cache. Counting starts from zero. The parameter must be <= MyBooksList.full_rows_number.
    @param num_last last item to cache. The parameter must be <= MyBooksList.full_rows_number.
    @param order Sorting order, for example "ORDER by AuthorLowerCase" (order goes into the sql query directly).
    @note Let us note that sqlite does not support a case-insensitive search for non-latin characters.
    To enable such search, we have columns like "TitleLowerCase". When searching, we transform a search
    string to the lowercase one, than search in the *LowerCase column.

    **/
    void put_into_cache(sqlite3 *db, std::string dbtable, long int Topic_id,long num_first,long num_last, std::string order);



    /**
    Returns cached line.
    For example, assume that we have a list of books that consists of 10 rows.
    If these books are cached, we can get an author of the fifth row (counting starts from zero) by
    @code
    cache.get_cached_line(5)["Author"]; // "Author" is column name in the database
    @endcode
    @param item item number of the MyBooksList class.
    @note Before calling this function, check that item is cached. Use #is_cached () for it.
    **/
    std::map<std::string,std::string> get_cached_line(long item) const;


    /**
    returns true if item is cached, false otherwise
    @param item item number of the MyBooksList class.
    **/
    bool is_cached(long item) const;

    /**
    clears cache
    **/
    void clear_cache();


    /**
    first cached item of the MyBooksList.
    **/
    long cached_min;

    /**
    last cached item of the MyBooksList.
    **/
    long cached_max;
private:

    /**
    array that keeps cached books
    **/
    Khz::Libs::DB::mapDB cache;
};




}} // end of namespace
