/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "InfoAboutThisBook.h"
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/dcclient.h>
#include <vector>
#include <string>
#include "string_functions.h"
#include <iso646.h>


namespace Khz{namespace PopupWins{




InfoAboutThisBook::InfoAboutThisBook(wxWindow *parent, Khz::Libs::DB::mapDB line, wxString abs_path) :
    wxDialog (parent,wxID_ANY, "Info about book", wxDefaultPosition, wxSize (600, -1),
                               wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
    {
    wxSize size;

    //get title, author,topic, filename from line.
	wxString Title =wxString::FromUTF8(line[0]["Title"].c_str());
	wxString Author=wxString::FromUTF8(line[0]["Author"].c_str());
    wxString Topic=wxString::FromUTF8(line[0]["Topic"].c_str());
    wxString FileName=wxString::FromUTF8(line[0]["Filename"].c_str());



     wxSizer * const sizerTop = new wxBoxSizer(wxVERTICAL);


     sizerTop->SetMinSize(600,-1);
        wxSizer *sizerMsgs = new wxBoxSizer(wxVERTICAL);


    sizerMsgs->Add(new wxStaticText(this, wxID_ANY, "Title"));


    m_textTitle = new wxTextCtrl(this, wxID_ANY,"",  wxDefaultPosition, wxDefaultSize, wxTE_READONLY | wxTE_MULTILINE | wxTE_NO_VSCROLL);
	m_textTitle->SetBackgroundColour(*wxWHITE);


    m_textTopic = new wxTextCtrl(this, wxID_ANY,"",  wxDefaultPosition, wxDefaultSize, wxTE_READONLY | wxTE_MULTILINE | wxTE_NO_VSCROLL);
	m_textTopic->SetBackgroundColour(*wxWHITE);
    //std::cout << "\n \n num of lines in title" <<  m_textTitle->GetNumberOfLines () << "\n";


    m_textAuthor = new wxTextCtrl(this, wxID_ANY, Author,  wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
	m_textAuthor->SetBackgroundColour(*wxWHITE);



    wxSizer *ok_button= CreateButtonSizer (wxOK);

    sizerMsgs->Add(m_textTitle, wxSizerFlags().Expand().Border(wxBOTTOM,12));

	sizerMsgs->Add(new wxStaticText(this, wxID_ANY, "Author"));
    sizerMsgs->Add(m_textAuthor, wxSizerFlags().Expand().Border(wxBOTTOM,12));



    sizerMsgs->Add(new wxStaticText(this, wxID_ANY, "Topic"));
    sizerMsgs->Add(m_textTopic, wxSizerFlags().Expand().Border(wxBOTTOM,12));


    sizerMsgs->Add(new wxStaticText(this, wxID_ANY, "Location"));

	m_textLocation=  new wxTextCtrl(this, wxID_ANY, abs_path,  wxDefaultPosition, wxDefaultSize,  wxTE_READONLY | wxTE_NO_VSCROLL);
	m_textLocation->SetBackgroundColour(*wxWHITE);
	sizerMsgs->Add(m_textLocation, wxSizerFlags().Expand().Border(wxBOTTOM,12));




    sizerMsgs->Add(new wxStaticText(this, wxID_ANY, "md5"));

	m_textMd5= new wxTextCtrl(this, wxID_ANY, wxString(line[0]["md5_of_file"]),  wxDefaultPosition, wxDefaultSize,  wxTE_READONLY | wxTE_NO_VSCROLL);
	m_textMd5->SetBackgroundColour(*wxWHITE);
	sizerMsgs->Add(m_textMd5,  wxSizerFlags().Expand().Border(wxBOTTOM,12));



    sizerMsgs->Add(new wxStaticText(this, wxID_ANY, "filename"));

	m_textFilename= new wxTextCtrl(this, wxID_ANY,FileName,  wxDefaultPosition, wxDefaultSize,  wxTE_READONLY | wxTE_NO_VSCROLL);
	m_textFilename->SetBackgroundColour(*wxWHITE);
	sizerMsgs->Add(m_textFilename,  wxSizerFlags().Expand().Border(wxBOTTOM,12));




    sizerMsgs->Add(ok_button, 0, wxALL|wxALIGN_RIGHT,0);

    sizerTop->Add(sizerMsgs, 1, wxALL|wxEXPAND,5);
    sizerTop->AddSpacer(10);
    SetSizerAndFit(sizerTop);

    int width, height;
    int width_topic, height_topic;

    m_textTitle->GetClientSize (&width, &height);
    m_textTopic->GetClientSize(&width_topic, &height_topic);

    wxString wrapped_title =wxString(Khz::Libs::Mstr::implode("\n", Wrap(Title.ToStdString(), width+5)));
    wxString wrapped_topic=wxString(Khz::Libs::Mstr::implode("\n", Wrap(Topic.ToStdString(), width+5)));

    m_textTitle->SetValue(wrapped_title);
    m_textTopic->SetValue(wrapped_topic);

    wxWindowDC dc(m_textTitle);
    size = m_textTitle->GetSizeFromTextSize(dc.GetMultiLineTextExtent(wrapped_title));
    size.SetHeight(size.GetHeight()+10);
    m_textTitle->SetMinSize(size);


    wxWindowDC dc_topic(m_textTopic);
    size = m_textTopic->GetSizeFromTextSize(dc.GetMultiLineTextExtent(wrapped_topic));
    size.SetHeight(size.GetHeight()+10);
    m_textTopic->SetMinSize(size);


   // SetSizerAndFit(sizerTop);
    //sizerTop->Fit	( 	this) ;
    sizerMsgs->RecalcSizes();
    sizerTop->RecalcSizes ();
    sizerTop->SetSizeHints 	(this) ;
    }


std::vector<std::string> InfoAboutThisBook::Wrap(std::string some_string, int Length)
    {
    std::vector<std::string> result;

    int textwidth = 0, textheight = 0;
    int Index = 0;
    wxString FullStr = wxString(some_string);
    wxString str= wxString(some_string);


    wxClientDC dc(this);
     bool allow_not_space_dividing=0;
    do
    {
        //gets width, height of string in pixels
        dc.GetTextExtent(str, &textwidth, &textheight);
        //std::cout << "\n width is " << textwidth << " length is "<< Length << "\n" ;
        //std::cout << "\n length of string is "<< str.Length();
        //std::cout << "\n cropped string is "<< str.Left(str.Length()-1) << "\n";
        if(textwidth>Length)
        {
            Index = 0;
            do
            {
                str = str.Left(str.Length()-1);
               // std::cout << "\n new str inside do is " << str << "\n";
                Index++;
                dc.GetTextExtent(str, &textwidth, &textheight);

                wxString c = wxString(str.Last());
               //   std::cout << "\n width is " << textwidth << " length is "<< Length << "\n" ;

                if(  !allow_not_space_dividing and (textwidth<Length && c == _(" ")) ) break;
                if(  allow_not_space_dividing and textwidth<Length  ) break;
                if (textwidth==0) allow_not_space_dividing=1;
          //  std::cout << "\n doing";
            }while(1);
            if(str!="")
                {
                result.push_back(str.ToStdString());
                allow_not_space_dividing=0;
                }
            str = FullStr.Right(Index);
        }
        else
            break;
    //std::cout << "\n doing 2\n";
    }while(1);
    result.push_back(str.ToStdString());
    return result;
    }

}}// end of namespaces
