/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/





#include <wx/dialog.h>
#include "Mdb_fuctions.h"
#include <wx/textctrl.h>




namespace Khz{

/**
@brief Popup dialogs
**/
namespace PopupWins{






/**
@brief A window with information about the book. The window appears when user right clicks book(right panel) and chooses the "Info" entry in the popup menu.

Creates a window with information about the book. The window appears when user right clicks book(right panel) and selects the "Info" entry in the popup menu
**/
class InfoAboutThisBook : public wxDialog
{
public:
    /**
    Constructs  the info window
    @param parent standard wxWidgets parent
    @param line Mdb::mapDB is std::vector< std::map< std::string, std::string > >.
    <strong class="paramname">line</strong>'s vector must have only one element. <strong class="paramname">line</strong>[0]["Author"]
    should give the Author of the book,
    <strong class="paramname">line</strong>[0]["Title"] should give the title of the book, and so on.<br><br>
    NOTE: <strong class="paramname">line</strong> is not not constructed by hands. It is constructed by the Mdb::dbcontent function.
    This function constructs <strong class="paramname">line</strong> based on the database content
    @param abs_path full path to the book file
    **/
    InfoAboutThisBook(wxWindow *parent, Khz::Libs::DB::mapDB line, wxString abs_path);
private:
    /**
    Input with title of the book
    **/
    wxTextCtrl *m_textTitle;
    /**
    Input with topic of the book
    **/
    wxTextCtrl *m_textTopic;
    /**
    Input with author of the book
    **/
    wxTextCtrl *m_textAuthor;
    /**
    Input with full path to the book
    **/
    wxTextCtrl *m_textLocation;
    /**
    Input with md5 of the book
    **/
    wxTextCtrl *m_textMd5;
    /**
    Input with filename
    **/
    wxTextCtrl *m_textFilename;
    /**
    Splits long string into several substrings, length of each of the substrings is less than <strong class="paramname">Length</strong>.
    If we put these substrings into some input, it will look like that the initial string is wrapped.
    @param some_string string to wrap
    @param Length in pixels
    @return vector of string, length of each is less than <strong class="paramname">Length</strong>
    **/
    std::vector<std::string> Wrap(std::string some_string, int Length);
};




}}// end of namespaces
