/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/frame.h>
#include <wx/button.h>
#include <wx/filectrl.h>

namespace Khz{namespace PopupWins{


/**
@brief Similar to the wxFileDialog, but we have more control here.



This class is  combination of three elements: wxButton #cancel_button, wxButton #ok_button, wxFileCtrl #file_ctrl.
These elements are public, so you can access all of their methods. It means that you can set the buttons's labels text, you can access all
of the methods of the wxFileCtrl class.

Usage Example:
@code
    wxString home_dir = wxStandardPaths::Get().GetDocumentsDir();
    MyFileDialog dlg(this, "Select where to save the generated bibtex database",
                     home_dir, "database.bib",
                     "bib files (*.bib)|*.bib", wxFC_SAVE);
    if(dlg.ShowModal()==wxID_CANCEL) return;

    wxString path= dlg.file_ctrl->GetPath();
    //here do whatever you want with path.
@endcode
**/
class MyFileDialog : public wxDialog
    {
    public:
        /**
        Constructor
        @param parent standard wxwidgets parent
        @param message caption of the window
        @param defaultDirectory see wxFileCtrl class (wxWidgets library)
        @param defaultFilename  see wxFileCtrl class (wxWidgets library)
        @param wildCard         see wxFileCtrl class (wxWidgets library)
        @param style            see wxFileCtrl class (wxWidgets library)
        @param pos window position
        @param size window size
        @param name i have no idea what is this
        **/
        MyFileDialog
                (
                wxWindow *parent,
                const wxString &message=wxFileSelectorPromptStr,
                const wxString &defaultDirectory=wxEmptyString,
                const wxString &defaultFilename =wxEmptyString,
                const wxString &wildCard=wxFileSelectorDefaultWildcardStr,
                long style=wxFC_DEFAULT_STYLE,
                const wxPoint &pos=wxDefaultPosition,
                const wxSize &size=wxDefaultSize,
                const wxString &name=wxFileDialogNameStr
                );

        /**
        Cancel button. This button is public, so you can change it's label an so on.
        **/
        wxButton *cancel_button;
        /**
        Ok button. This button is public, so you can change it's label an so on.
        **/
        wxButton *ok_button;

        /**
         Draws the tree of the directories
        **/
        wxFileCtrl *file_ctrl;
    private:
        /**
        wxFileCtrl has a bug/feature: If a directory is currently highlighted, than wxFilCtrl->GetPath()
        returns the currently highlighted directory, not path+filename.
        So, here come the fix: when user presses the Ok button, we check if any of the directories is highlighted.
        If it is, than we change the working directory to the currently highighted one and allow user to select any of the inner directories.
        Otherwise, we return path+filename.
        So, this function changes the working directory if there is a highlited one. At the same time, this function blocks the event
        that was passed as a parametr. As a result, this dialog keeps running.
        If no one directory is highlighted, than this function lets the event to pass. As a result, this dialog returns path+filename and gets closed.
        **/
        void OnOkButtonPressed(wxCommandEvent& event);
    protected:
        /**
        See wxWindow::DoGetBestClientSize() (wxWidgets library)
        **/
        virtual wxSize DoGetBestClientSize( )const;
    };

}}// namespaces
