/*
Copyright (C) Ivan Ivanov, 2014-2016

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <wx/button.h>
#include <wx/sizer.h>
#include <wx/defs.h>
#include "Custom_IDs.h"
#include <wx/dialog.h>
#include "TreeCtrlLibrary.h"
#include "MyTopicSelectorDialog.h"
#include <wx/stdpaths.h>
#include "clear_database_path.h"
#include <wx/msgdlg.h>
#include "Mdb_fuctions.h"
#include "ConfigManager.h"



namespace Khz{namespace PopupWins{









MyTopicSelectorDialog::MyTopicSelectorDialog(wxWindow *parent, wxWindowID id, wxString title, const wxPoint &pos, const wxSize &size,  bool show_subtopics_checkbox)
:wxDialog (parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER)
    {
    cancel_button = new wxButton(this, wxID_CANCEL);
    ok_button = new wxButton(this, wxID_OK);
    ok_button->Disable();
    library_tree= new Khz::MainWin::TreeCtrlLibrary(this, wxID_ANY ,wxDefaultPosition, wxDefaultSize,wxTR_TWIST_BUTTONS | wxTR_NO_LINES| wxTR_HAS_BUTTONS);
    if(show_subtopics_checkbox)
        {
        include_subtopics_checkbox=new wxCheckBox(this, wxID_ANY, "include subtopics");
        }


    Bind(wxEVT_BUTTON, &MyTopicSelectorDialog::BlockSelectionEvent, this,my_wx_id__SHOW_BOOKS_IN_TOPIC_COMMAND_EVENT);
    Bind(wxEVT_TREE_SEL_CHANGED, &MyTopicSelectorDialog::EnableDisableOkButton,this);

    wxBoxSizer *only_sizer= new wxBoxSizer(wxVERTICAL);


    only_sizer->Add(library_tree, 1, wxEXPAND|wxALL, 5);
    if(show_subtopics_checkbox)
        {
        only_sizer->Add(include_subtopics_checkbox, 0, wxALL, 5);
        }
    //ok cancel buttons
        wxBoxSizer *buttons_sizer=new wxBoxSizer(wxHORIZONTAL);
        buttons_sizer->AddStretchSpacer ();
        buttons_sizer->Add(cancel_button);
        buttons_sizer->AddSpacer(3);
        buttons_sizer->Add(ok_button);
    only_sizer->Add(buttons_sizer,0,wxEXPAND|wxALL,5);
    this->SetSizer(only_sizer);
    }



void MyTopicSelectorDialog::BlockSelectionEvent(wxCommandEvent &event)
    {
    event.StopPropagation();
    }


void MyTopicSelectorDialog::EnableDisableOkButton(wxTreeEvent& event)
    {
    if(library_tree-> GetSelection() == library_tree->GetRootItem())
        {
        ok_button->Disable();
        }
    else
        {
        ok_button->Enable();
        }
    }

void MyTopicSelectorDialog::GetArrayOfSubtopics(wxString topic, std::vector<wxString> &output)
    {


    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //let us look into config file and connet to the main kolkhoznik database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    sqlite3 *db;
    wxString ini_filename = ConfigManager::GetPathToConfigFile();
    wxFileConfig *config =new wxFileConfig( "", "", ini_filename);
	wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();
	delete config;
	try
        {
        sqlite3_open( temp_char_buffer, &db);
        }
    catch(...)
        {
        wxMessageBox("Cannot connect to the main kolkhoznik database", "Error", wxOK | wxICON_ERROR);
        return;
        }
   // std::cout << "\n path to db " << config->Read("PATHS/path_to_database","") << "\n";
    sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
    sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
                           //connected to the database
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
     Khz::Libs::DB::mapDB result;
    try
        {
        std::string query="select ID, Topic from topics where Topic like ?;";
        sqlite3_stmt *stmt = NULL;
        int retval=sqlite3_prepare_v2( db, query.c_str(), -1, &stmt, NULL );
                //std::cout << "\n" << sqlite3_errmsg(db) << "\n";
        if(retval) throw(std::string("cannot connect to db in MyTopicSelectorDialog.cpp"));
        sqlite3_bind_text(stmt,  1, (topic.ToUTF8().data()+std::string("/%")).c_str(), -1, SQLITE_TRANSIENT);

                        //std::cout << "\n" << sqlite3_errmsg(db) << "\n";
        sqlite3_step( stmt );
                        //std::cout << "\n" << sqlite3_errmsg(db) << "\n";
        result = Khz::Libs::DB::dbcontent(stmt);
                        //std::cout << "\n got result i\n";
                        //std::cout << "\n size of output in tree is "<< result.size();
                        //std::cout << "\n zero topic is " << result[0]["Topic"] << "\n";
        sqlite3_finalize(stmt);
        }
    catch(...)
        {
        wxMessageBox("Cannot connect to the main kolkhoznik database", "Error", wxOK | wxICON_ERROR);
        sqlite3_close(db);
        return;
        }
    sqlite3_close(db);

    for (unsigned int i=0;i<result.size();i++)
        {
        //std::cout << "pushing topic " << result[i]["Topic"] << "\n";
        output.push_back(wxString::FromUTF8(result[i]["Topic"].c_str()));
        }
    }




}} // end of namespaces
