/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include <wx/dialog.h>
#include "TreeCtrlLibrary.h"
#include <wx/checkbox.h>



namespace Khz{namespace PopupWins{



/**
@brief Dialog that allows to select topic.



This class is  combination of four elements:
wxButton #cancel_button, wxButton #ok_button,
TreeCtrlLibrary #library_tree, wxCheckBox #include_subtopics_checkbox.
These elements are public, so you can access all of their methods.
It means that you can set the buttons's labels text, you can access all
of the methods of the TreeCtrlLIbrary class.


Usage example (without "include subtopics" checkbox):

@code
    MyTopicSelectorDialog dlg(this, wxID_ANY, "Select topic");
    if(dlg.ShowModal()==wxID_CANCEL) return;
    wxString topic = dlg.library_tree->GetTopicString(dlg.library_tree->GetSelection());
    // do whatever you want with topic
@endcode

Usage example with "include subtopics" checkbox

@code
    wxArrayString temp_wx_array= wxArrayString();

    MyTopicSelectorDialog dlg(this, wxID_ANY, "Select topic", wxDefaultPosition, wxDefaultSize, true);
    if(dlg.ShowModal()==wxID_CANCEL) return;
    wxString topic = dlg.library_tree->GetTopicString(dlg.library_tree->GetSelection());
    temp_wx_array.Add(topic);
    if(dlg.include_subtopics_checkbox->GetValue())
        {
        std::vector<wxString> subtopics;
        dlg.GetArrayOfSubtopics(topic, subtopics);
        for(unsigned int i=0;i<subtopics.size();i++) temp_wx_array.Add(subtopics[i]);
        }
    wxListBox *some_listbox = new wxListBox(this, wxID_ANY);
    some_listbox->InsertItems(temp_wx_array, 0);
    }
@endcode
**/
class MyTopicSelectorDialog : public wxDialog
    {
    public:
        /**
        Constructor
        @param parent standard wxwidgets parent
        @param id standard wxwidgets id
        @param title the window title
        @param pos the window position
        @param size the window size
        @param show_subtopics_checkbox if true, than dialog will contain the "include subtopics" checkbox

        **/
        MyTopicSelectorDialog
                (
                wxWindow *parent,
                wxWindowID id,
                wxString title,
                const wxPoint &pos=wxDefaultPosition,
                const wxSize &size=wxDefaultSize,
                bool show_subtopics_checkbox=false
                );


        /**
        Cancel button. This button is public, so you can change it's label an so on.
        **/
        wxButton *cancel_button;


        /**
        Ok button. This button is public, so you can change it's label an so on.
        **/
        wxButton *ok_button;


        /**
         Draws the topics tree of the library
        **/
        Khz::MainWin::TreeCtrlLibrary *library_tree;

        /**
        "include subtopics" checkbox
        **/
        wxCheckBox *include_subtopics_checkbox;

        /**Generates array of subtopics for the topic**/
        void GetArrayOfSubtopics(wxString topic, std::vector<wxString> &output);

    private:
        /**
        In order to display the topics tree of the library, we use a TreeCtrlLibrary class object inside of this dialog.
        When user selects a topic inside of the TreeCtrlLibrary class object, the
         TreeCtrlLibrary class object  generates event with special id. This events gets caught by the
                        #Khz::MainWin::MyMainWindow class object, as a result #Khz::MainWin::MyMainWindow makes #Khz::MainWin::MyRightPanel to show books belonging to the particular topic.
        Here we want just to select topic, we do not want to refresh the books list on the right panel.
        So, this function blocks the event.
        **/
        void BlockSelectionEvent(wxCommandEvent &event);

        /**
        When the dalog is just launched, the "Ok" button is disabled and the root item of the library tree is selected.
        This function enables the "Ok" button when user changes the selection.

         The function executes each time when a user changes the selection. If the user selects the root item again, than this function
         disables the "Ok" button.
        **/
        void EnableDisableOkButton(wxTreeEvent& event);
    };



}} // end of namespaces
