/*
Copyright (C) Ivan Ivanov, 2014-2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/



#include "StatisticsWindow.h"
#include <wx/stdpaths.h>
#include "Mdb_fuctions.h"
#include "clear_database_path.h"
#include <iso646.h>
#include "ConfigManager.h"







namespace Khz{namespace PopupWins{





StatisticsWindow::~StatisticsWindow()
    {
    delete config;
    sqlite3_close(db);

    }
StatisticsWindow::StatisticsWindow()
    {
     wxSetlocale(LC_ALL, ""); //this is needed to non-latin characters support in paths;
    wxString ini_filename = ConfigManager::GetPathToConfigFile();
    config =new wxFileConfig( "", "", ini_filename);


    wxCharBuffer temp_char_buffer=Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database","")).ToUTF8();

    //wxMessageDialog *msg_dialog = new wxMessageDialog(NULL, "Please wait a little", "Statistics");
    //msg_dialog->Show();
    try
        {
        sqlite3_open( temp_char_buffer, &db);

        sqlite3_exec(db,"PRAGMA encoding = \"UTF-8\"" , 0, 0, 0);
        sqlite3_exec(db, "PRAGMA case_sensitive_like=OFF", 0, 0, 0);








        std::string query_total="select COUNT(*) from main;";
        std::string query_pdf="select COUNT(*) from main where  Extension='pdf' COLLATE NOCASE;";
        std::string query_djvu="select COUNT(*) from main where  (Extension='djv' COLLATE NOCASE or Extension='djvu' COLLATE NOCASE);";
        std::string query_rus="select COUNT(*) from main where Language='Russian' COLLATE NOCASE;";
        std::string query_eng="select COUNT(*) from main where Language ='English' COLLATE NOCASE;";
                //std::cout << "\n query is " << query << "\n";
        //std::string query="select * from topics;";
        //select Topic from topics where Topic like '/B_Biology/%' and substr(Topic, length('/B_Biology/')+1 ) NOT LIKE '%/%';
        // we have to select topics that are like current topic and have number of symbols "/" eqal to coorresponding number of current topic +1
        sqlite3_stmt *stmt_total = NULL;
        sqlite3_stmt *stmt_pdf = NULL;
        sqlite3_stmt *stmt_djvu = NULL;
        sqlite3_stmt *stmt_rus = NULL;
        sqlite3_stmt *stmt_eng = NULL;

        int retval_total=sqlite3_prepare_v2( db, query_total.c_str(), -1, &stmt_total, NULL );
        if(retval_total) throw(std::string("cannot connect to db in statisticswindow.cpp"));
        sqlite3_step( stmt_total );
        long long  total_lines_total = sqlite3_column_int(stmt_total, 0);
        sqlite3_finalize(stmt_total);

        int retval_pdf=sqlite3_prepare_v2( db, query_pdf.c_str(), -1, &stmt_pdf, NULL );
        if(retval_pdf) throw(std::string("cannot connect to db in statisticswindow.cpp"));
        sqlite3_step( stmt_pdf );
        long long total_lines_pdf = sqlite3_column_int(stmt_pdf, 0);
        sqlite3_finalize(stmt_pdf);

        int retval_djvu=sqlite3_prepare_v2( db, query_djvu.c_str(), -1, &stmt_djvu, NULL );
        if(retval_djvu) throw(std::string("cannot connect to db in statisticswindow.cpp"));
        sqlite3_step( stmt_djvu);
        long long total_lines_djvu = sqlite3_column_int(stmt_djvu, 0);
        sqlite3_finalize(stmt_djvu);



        int retval_rus=sqlite3_prepare_v2( db, query_rus.c_str(), -1, &stmt_rus, NULL );
        if(retval_rus) throw(std::string("cannot connect to db in statisticswindow.cpp"));
        sqlite3_step( stmt_rus );
        int total_lines_rus = sqlite3_column_int(stmt_rus, 0);
        sqlite3_finalize(stmt_rus);


        int retval_eng=sqlite3_prepare_v2( db, query_eng.c_str(), -1, &stmt_eng, NULL );
        if(retval_eng) throw(std::string("cannot connect to db in statisticswindow.cpp"));
        sqlite3_step( stmt_eng );
        int total_lines_eng = sqlite3_column_int(stmt_eng, 0);
        sqlite3_finalize(stmt_eng);












        const wxChar* NEWLINE_CHAR = wxTextFile::GetEOL();
        wxString msg;
        msg = wxString("Total files: ") +  wxString::Format(wxT("%i"),total_lines_total)+NEWLINE_CHAR;
        msg+= wxString("Pdf files: ") + wxString::Format(wxT("%i"),total_lines_pdf)+NEWLINE_CHAR;
        msg+= wxString("Djvu files: ") + wxString::Format(wxT("%i"),total_lines_djvu)+NEWLINE_CHAR;
        msg+= wxString(NEWLINE_CHAR)+ wxString("Languages: ")  +NEWLINE_CHAR+NEWLINE_CHAR;
        msg+= wxString("Russian: ") + wxString::Format(wxT("%i"),total_lines_rus)+NEWLINE_CHAR;
        msg+= wxString("English: ") + wxString::Format(wxT("%i"),total_lines_eng)+NEWLINE_CHAR;
        msg+= wxString("Other: ") + wxString::Format(wxT("%i"),total_lines_total-total_lines_eng-total_lines_rus)+NEWLINE_CHAR;
        //wxMessageBox(msg, wxT("Statistics"), wxOK | wxICON_INFORMATION);

        //delete msg_dialog;
        wxMessageDialog *msg_dialog = new wxMessageDialog(NULL, msg, "Statistics");
        msg_dialog->ShowModal();
        delete msg_dialog;
        }
    catch(...)
        {
        //delete msg_dialog;
        ShowErrorMessage();
        }
    }

void StatisticsWindow::ShowErrorMessage()
    {
    wxString path_to_db = Khz::Libs::DB::clear_path(config->Read("PATHS/path_to_database",""));
    wxString message= "It seems that database \"" + path_to_db + "\" does not exist or is damaged. You have to generate new database first.";
    wxMessageBox(message, wxT("Error"), wxOK | wxICON_ERROR);
    }


}}// end of namespaces
