/*
Copyright (C) Ivan Ivanov, 2015

This file is part of kolkhoznik.
kolkhoznik is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

kolkhoznik is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with kolkhoznik.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <vector>
#include <wx/string.h>

namespace Khz{

/**
@brief Stores default (hardcoded) config params
**/
class default_config_params
{
public:
    /**
    constructor
    **/
    default_config_params();


    /**
    text layer extractors. the first element of the std::pair is extension,
    the second element of the std::pair is command.
    For example, for the djvu text layer extractor we have the following pair:
    @code
    std::make_pair("djvu", "%edjvutxt %f")
    @endcode
    Executable starts with \%e, \%f represents the filename (full path to the file).
    **/
    std::vector<std::pair<wxString, wxString> > text_layer_extractors;




    /**
    archive extractors. the first element of the std::pair is extension,
    the second element of the std::pair is command.
    For example, for the zip archive extractor we have the following pair:
    @code
    std::make_pair("zip", "%e7z x %f -o%o")
    @endcode
    Executable starts with \%e, \%f represents  the archive filename (full path to the file).
    \%o represents the output dir.
    **/
    std::vector<std::pair<wxString, wxString> > archive_extractors;
private:
    /**
    fills #text_layer_extractors;
    **/
    void fill_text_layer_extractors_params();


    /**
    fills #archive_extractors
    **/
    void fill_archive_extractors_commands();
};

}// namespace
